/*
 * com_hik_mcrsdk_rtsp_RtspClient.cpp
 *
 *  Created on: 2014-3-20
 *      Author: weilinfeng
 */


#include "com_hik_mcrsdk_rtsp_RtspClient.h"
#include "MCRPrint.h"
#include "Define.h"
#include "RtspClientSDK.h"
#include "HPR_Guard.h"
#include "RtspDefine.h"
#include "HPR_Hpr.h"
#include <map>

using namespace std;

//java虚拟机
extern JavaVM *gJavaVM;

// Jni环境变量
JNIEnv* gEnv[ENGINE_MAX_NUM];

//回调管理map
map<jint, jobject> oCallBack_Map;
//全局锁
HPR_Mutex oGuardLock;
HPR_Mutex oJniEngineLock[ENGINE_MAX_NUM];
//
bool bJniInitLib = false;
bool bBackStopeds[ENGINE_MAX_NUM];
bool bRealStopeds[ENGINE_MAX_NUM];

int gGlobalError;

int RTSPDataCallback(int handle, int dataType, char* data, int len, unsigned int timestamp, int packetNo, void* pUser, bool bRealse)
{
	//CRtspClientPrint::Print("Jni::DataCallback->engine[%d] data len:%d", handle, len);

	if (bRealse && gEnv[handle] != NULL)
	{
		// 取消联接当前线程
		gJavaVM->DetachCurrentThread();
		gEnv[handle] = NULL;
		return 0;
	}

	if(len <= 0 || data == NULL) {
		LOGD("jni RTSPDataCallback->data len less than 0\n");
		return 0;
	}

	if (!bRealse && gEnv[handle] == NULL)
	{
		// 联接当前线程
		jint ret = gJavaVM->AttachCurrentThread(&gEnv[handle], NULL);
		if (ret < 0)
		{
			LOGD("jni RTSPDataCallback->g_JavaVM->AttachCurrentThreaRtspClientail\n");
			return 0;
		}
	}

	jbyteArray data_array = gEnv[handle]->NewByteArray(len);
	if (data_array == NULL)
	{
		LOGD("jni RTSPDataCallback->gEnv[handle]->NewByteArray fail");
		gJavaVM->DetachCurrentThread();
		gEnv[handle] = NULL;
		return 0;
	}

	gEnv[handle]->SetByteArrayRegion(data_array, 0, len, (const jbyte*)data);

	if(oCallBack_Map[handle] == NULL)
	{
		jbyte* jpointer = gEnv[handle]->GetByteArrayElements(data_array,0);
		gEnv[handle]->ReleaseByteArrayElements(data_array,jpointer,0);

		gEnv[handle]->DeleteLocalRef(data_array);
		data_array = NULL;

		gJavaVM->DetachCurrentThread();
		gEnv[handle] = NULL;
		LOGD("jni RTSPDataCallback->gCallBack_Map[%d] is NULL", handle);
		return 0;
	}

	map<jint, jobject>::iterator iter = oCallBack_Map.find(handle);
	if(iter == oCallBack_Map.end())
	{
		LOGD("jni RTSPDataCallback->not find data call back fail");
		gJavaVM->DetachCurrentThread();
		gEnv[handle] = NULL;
		return 0;
	}

	jclass dataCallback = gEnv[handle]->GetObjectClass(iter->second);
	jmethodID dataCallbackMethod = gEnv[handle]->GetMethodID(dataCallback, "onDataCallBack", "(II[BIIII)V");
	gEnv[handle]->CallVoidMethod(oCallBack_Map[handle], dataCallbackMethod, (jint)handle, (jint)dataType, data_array, (jint)len, (jint)timestamp, (jint)packetNo, handle);

	jbyte* jpointer = gEnv[handle]->GetByteArrayElements(data_array, 0);
	gEnv[handle]->ReleaseByteArrayElements(data_array, jpointer,0);
	gEnv[handle]->DeleteLocalRef(data_array);
	data_array = NULL;

	gEnv[handle]->DeleteLocalRef(dataCallback);
	dataCallback = NULL;

	//gJavaVM->DetachCurrentThread();

	return 0;
}

int RTSPMsgCallback(int handle, int opt, void* param1, void* param2, void* pUser)
{
	// 方法锁
	HPR_Guard gGuard(&oJniEngineLock[handle]);
	JNIEnv* env = NULL;

	// 获取虚拟机资源
	if (gJavaVM->AttachCurrentThread(&env, NULL) < 0)
	{
		LOGD("jni RTSPMsgCallback->g_JavaVM->AttachCurrentThread fail");
		return 0;
	}

	map<jint, jobject>::iterator iter = oCallBack_Map.find(handle);
	if(iter == oCallBack_Map.end())
	{
		LOGD("jni RTSPMsgCallback->not find msg call back fail");
		return 0;
	}

	jclass callbackClass = env->GetObjectClass(iter->second);
	jmethodID msgcallbackMethod = env->GetMethodID(callbackClass, "onMessageCallBack", "(IIIII)V");
	env->CallVoidMethod(oCallBack_Map[handle], msgcallbackMethod, (jint)handle, (jint)opt, 0, 0, handle);
	LOGD("jni_RTSPDataCallback->engine[%d],callback value:%d", handle, opt);

	// 释放局部引用
	env->DeleteLocalRef(callbackClass);
	callbackClass = NULL;

	// 释放虚拟机资源
	gJavaVM->DetachCurrentThread();

	return 0;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    initLib
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_initLib
(JNIEnv * env, jclass obj)
{
	if (env == NULL) {
		LOGD("Java_com_hik_RtspClient_initLib->env == NULL");
		return false;
	}

	HPR_Guard gGuard(&oGuardLock);

	if (bJniInitLib)
	{
		LOGD("Java_com_hik_RtspClient_initLib->have been inited");
		return true;
	}

	if(!RtspClientInitLib())
	{
		LOGD("Java_com_hik_RtspClient_initLib->init RtspClientInitLib fail!");
		gGlobalError = RtspClientGetLastError();
		return false;
	}

	for (int i = 0; i < ENGINE_MAX_NUM; i++) {
		bBackStopeds[i] = true;
		bRealStopeds[i] = true;
		gEnv[i] = NULL;
	}

	oCallBack_Map.clear();

	bJniInitLib = true;

	LOGD("Java_com_hik_RtspClient_initLib->init lib success");

	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    finiLib
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_finiLib
(JNIEnv * env, jclass obj)
{
	if(env == NULL) {
		LOGD("Java_com_hik_mcrsdk_rtsp_RtspClient_finiLib->env == NULL");
		return true;
	}

	HPR_Guard gGuard(&oGuardLock);

	if(!bJniInitLib)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_finiLib->not lib\n");
		return true;
	}

	for (int i = 0; i<ENGINE_MAX_NUM; i++) {
		bBackStopeds[i] = true;
		bRealStopeds[i] = true;
		gEnv[i] = NULL;
	}

	oCallBack_Map.clear();

	RtspClientFiniLib();

	bJniInitLib = false;
	LOGD("Java_com_hik_RtspClient_RtspClient_finiLib->fini success\n");
	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    createRtspClientEngine
 * Signature: (Lcom/hik/mcrsdk/rtsp/RtspClientCallback;I)I
 */
JNIEXPORT jint JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_createRtspClientEngine
(JNIEnv *env, jobject obj, jobject callcack, jint tansmethod)
{
	if(env == NULL) {
		LOGD("Java_com_hik_mcrsdk_rtsp_RtspClient_createRtspClientEngine->env == NULL");
		return true;
	}

	HPR_Guard gGuard(&oGuardLock);
	if(!bJniInitLib)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_createRtspClientEngine->not init Lib");
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return RTSPCLIENT_INVALIDATE_ENGINEID;
	}

	if(callcack == NULL) {
		LOGD("Java_com_hik_RtspClient_RtspClient_createRtspClientEngine->callcack == NULL");
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return RTSPCLIENT_INVALIDATE_ENGINEID;
	}

	if(tansmethod == -1) {
		LOGD("Java_com_hik_RtspClient_RtspClient_createRtspClientEngine->tansmethod == -1");
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return RTSPCLIENT_INVALIDATE_ENGINEID;
	}

	int engineid  = RtspClientCreateEngine(tansmethod, RTSPDataCallback, RTSPMsgCallback, NULL);
	if(engineid == RTSPCLIENT_INVALIDATE_ENGINEID)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_createRtspClientEngine->createRtspClientEngine[%d] fail!", engineid);
		gGlobalError = RtspClientGetLastError();
		return RTSPCLIENT_INVALIDATE_ENGINEID;
	}

	// 如果存在键i返回1，如果不存在键i则返回0
	if (oCallBack_Map.count(engineid))
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_createRtspClientEngine-> engine[%d] existed in gCallBack_Map", engineid);
		RtspClientReleaseEngine(engineid);
		gGlobalError = RTSPCLIENT_ENGINE_NOT_EXSIT;
		return RTSPCLIENT_INVALIDATE_ENGINEID;
	}

	pair<map<jint, jobject>::iterator, bool> ret= oCallBack_Map.insert(map<jint, jobject>::value_type(engineid,env->NewGlobalRef(callcack)));
	if (!ret.second)
	{
		RtspClientReleaseEngine(engineid);
		LOGD("Java_com_hik_RtspClient_RtspClient_createRtspClientEngine-> engine[%d] insert gCallBack_Map fail", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_INSERT_ENGINE_FAIL;
		return RTSPCLIENT_INVALIDATE_ENGINEID;
	}

	LOGD("Java_com_hik_RtspClient_RtspClient_createRtspClientEngine-> engine[%d] create success", engineid);
	return (jint)engineid;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    releaseRtspClientEngineer
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_releaseRtspClientEngineer
(JNIEnv *env, jobject obj, jint engineid)
{
	if(env == NULL) {
		LOGD("Java_com_hik_mcrsdk_rtsp_RtspClient_releaseRtspClientEngineer->env == NULL");
		return true;
	}

	HPR_Guard gGuard(&oGuardLock);

	if(!bJniInitLib)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_releaseRtspClientEngineer-> engine[%d] not init lib", engineid);
		return true;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return false;
	}

	if(!RtspClientReleaseEngine((int)engineid))
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_releaseRtspClientEngineer-> engine[%d] release fail", engineid);
		gGlobalError = RtspClientGetLastError();
		return false;
	}

	// 如果存在键i返回1，如果不存在键i则返回0
	if (!oCallBack_Map.count(engineid))
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_releaseRtspClientEngineer-> engine[%d] not existed", engineid);
		gGlobalError = RTSPCLIENT_ENGINE_NOT_EXSIT;
		return false;
	}

	env->DeleteGlobalRef(oCallBack_Map[engineid]);
	oCallBack_Map[engineid] = NULL;
	oCallBack_Map.erase(engineid);

	LOGD("Java_com_hik_RtspClient_RtspClient_releaseRtspClientEngineer-> engine[%d] release success\n", engineid);
	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    generateLiveUrl
 * Signature: (Lcom/hik/mcrsdk/rtsp/LiveInfo;Ljava/lang/String;)Z
 */
JNIEXPORT jstring JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_generateLiveUrl
  (JNIEnv *env, jobject jobj, jobject jliveInfo)
{
	LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generateLiveUrl");
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generateLiveUrl -> getLiveInfo env == NULL");
		return NULL;
	}

	if(jliveInfo == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generateLiveUrl -> jliveInfo == NULL");
		return NULL;
	}

	LiveInfo liveInfo = {0};
	jstring jstr = NULL;
	jclass jLiveInfoClass = env->GetObjectClass(jliveInfo);

	jmethodID id_getMagIp;
	id_getMagIp = env->GetMethodID(jLiveInfoClass, "getMagIp", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jliveInfo, id_getMagIp));
	if(jstr != NULL)
	{
		liveInfo.magIp = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		liveInfo.magIp = NULL;
	}

	jmethodID id_getMagPort;
	id_getMagPort = env->GetMethodID(jLiveInfoClass, "getMagPort", "()I");
	liveInfo.magPort = env->CallIntMethod(jliveInfo, id_getMagPort);

	jmethodID id_getCameraIndexCode;
	id_getCameraIndexCode = env->GetMethodID(jLiveInfoClass, "getCameraIndexCode", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jliveInfo, id_getCameraIndexCode));
	if(jstr != NULL)
	{
		liveInfo.cameraIndexCode = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		liveInfo.cameraIndexCode = NULL;
	}

	jmethodID id_getCascadeFlag;
	id_getCascadeFlag = env->GetMethodID(jLiveInfoClass, "getCascadeFlag", "()I");
	liveInfo.cascadeFlag = env->CallIntMethod(jliveInfo, id_getCascadeFlag);

	jmethodID id_getStreamType;
	id_getStreamType = env->GetMethodID(jLiveInfoClass, "getStreamType", "()I");
	liveInfo.streamType = env->CallIntMethod(jliveInfo, id_getStreamType);

	jmethodID id_getMcuNetID;
	id_getMcuNetID = env->GetMethodID(jLiveInfoClass, "getMcuNetID", "()I");
	liveInfo.mcuNetID = env->CallIntMethod(jliveInfo, id_getMcuNetID);

	jmethodID id_getDeviceNetID;
	id_getDeviceNetID = env->GetMethodID(jLiveInfoClass, "getDeviceNetID", "()I");
	liveInfo.deviceNetID = env->CallIntMethod(jliveInfo, id_getDeviceNetID);

	jmethodID id_getToken;
	id_getToken = env->GetMethodID(jLiveInfoClass, "getToken", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jliveInfo, id_getToken));
	if(jstr != NULL)
	{
		liveInfo.token = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		liveInfo.token == NULL;
	}

	jmethodID id_getiPriority;
	id_getiPriority = env->GetMethodID(jLiveInfoClass, "getiPriority", "()I");
	liveInfo.iPriority = env->CallIntMethod(jliveInfo, id_getiPriority);

	jmethodID id_getIsInternet;
	id_getIsInternet = env->GetMethodID(jLiveInfoClass, "getIsInternet", "()I");
	liveInfo.isInternet = env->CallIntMethod(jliveInfo, id_getIsInternet);

	jmethodID id_isbTranscode;
	id_isbTranscode = env->GetMethodID(jLiveInfoClass, "isbTranscode", "()Z");
	liveInfo.bTranscode = env->CallBooleanMethod(jliveInfo, id_isbTranscode);

	jmethodID id_getResolution;
	id_getResolution = env->GetMethodID(jLiveInfoClass, "getResolution", "()I");
	liveInfo.resolution = env->CallIntMethod(jliveInfo, id_getResolution);

	jmethodID id_getBitrate;
	id_getBitrate = env->GetMethodID(jLiveInfoClass, "getBitrate", "()I");
	liveInfo.bitrate = env->CallIntMethod(jliveInfo, id_getBitrate);

	jmethodID id_getFramerate;
	id_getFramerate = env->GetMethodID(jLiveInfoClass, "getFramerate", "()I");
	liveInfo.framerate = env->CallIntMethod(jliveInfo, id_getFramerate);

	jmethodID id_getVideotype;
	id_getVideotype = env->GetMethodID(jLiveInfoClass, "getVideotype", "()I");
	liveInfo.videotype = env->CallIntMethod(jliveInfo, id_getVideotype);

	jmethodID id_getSystemformat;
	id_getSystemformat = env->GetMethodID(jLiveInfoClass, "getSystemformat", "()I");
	liveInfo.systemformat = env->CallIntMethod(jliveInfo, id_getSystemformat);

	env->DeleteLocalRef(jLiveInfoClass);
	jLiveInfoClass = NULL;

	char liveUrl[MAX_URL_LEN];
	memset(liveUrl, 0, MAX_URL_LEN);
	bool bRet = GenerateLiveUrl(&liveInfo, liveUrl);
	if(!bRet)
	{
		LOGD("Java_com_hik_mcrsdk_rtsp_RtspClient_generateLiveUrl GenerateLiveUrl fail");
		return NULL;
	}
	jstring jliveUrl = env->NewStringUTF(liveUrl);
	return jliveUrl;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    startRtspProc
 * Signature: (ILjava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2
(JNIEnv *env, jobject obj, jint engineid, jstring rtspurl)
{
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2 -> env == NULL");
		return false;
	}

	if(rtspurl == NULL) {
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2 -> rtspurl == NULL");
		return false;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if (!bJniInitLib){
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		return false;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return false;
	}

	if (!bRealStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2-> engine[%d] real playing", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_REAL_PLAY;
		return true;
	}

	if (!bBackStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2-> engine[%d] playing back", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_STOP;
		return false;
	}

	const char* url = env->GetStringUTFChars(rtspurl, NULL);
	if (!RtspClientStartRtspProc((int)engineid, url)) {
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2-> engine[%d] start real play fail", engineid);
		gGlobalError = RtspClientGetLastError();
		return false;
	}

	env->ReleaseStringUTFChars(rtspurl, url);
	bRealStopeds[engineid] = false;
	LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2-> engine[%d] start success", engineid);
	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    startRtspProc
 * Signature: (ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2
(JNIEnv *env, jobject obj, jint engineid, jstring rtspurl, jstring deviceName, jstring devicePsw)
{
	LOGD("enter Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2.");
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2 -> env == NULL");
		LOGD("leave Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2.");
		return false;
	}

	if(rtspurl == NULL) {
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2 -> rtspurl == NULL");
		LOGD("leave Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2.");
		return false;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if(!bJniInitLib) {
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		LOGD("leave Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2.");
		return false;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		LOGD("leave Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2.");
		return false;
	}

	if (!bRealStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2-> engine[%d] real playing", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_REAL_PLAY;
		LOGD("leave Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2.");
		return true;
	}

	if (!bBackStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2-> engine[%d] playing back", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_STOP;
		LOGD("leave Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2.");
		return false;
	}

	char* url = NULL;
	if (NULL != rtspurl)
	{
		url = (char*)env->GetStringUTFChars(rtspurl, NULL);
	}
	if (NULL == url)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2-> engine[%d] play url is null", engineid);
		LOGD("leave Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2.");
		return false;
	}
	char* devName = NULL;
	if (NULL != deviceName)
	{
		devName = (char*)env->GetStringUTFChars(deviceName, NULL);
	}
	char* devPsw = NULL;
	if (NULL != devicePsw)
	{
		devPsw = (char*)env->GetStringUTFChars(devicePsw, NULL);
	}
	bool ret = RtspClientStartRtspProc((int)engineid, url, devName, devPsw);
	if (!ret) {
		LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2-> engine[%d] start real play fail", engineid);
		gGlobalError = RtspClientGetLastError();
		LOGD("leave Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2.");
		return false;
	}

	env-> ReleaseStringUTFChars(rtspurl, url);
	if (deviceName != NULL)
	{
		env-> ReleaseStringUTFChars(deviceName, devName);
	}
	if (devicePsw != NULL)
	{
		env-> ReleaseStringUTFChars(devicePsw, devPsw);
	}

	bRealStopeds[engineid] = false;
	LOGD("Java_com_hik_RtspClient_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2-> engine[%d] start success", engineid);
	LOGD("leave Java_com_hik_mcrsdk_rtsp_RtspClient_startRtspProc__ILjava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2.");
	return true;
}


/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    getRtspSession
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_getRtspSession
(JNIEnv *env, jobject obj, jint engineid)
{
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_getRtspSession -> env == NULL");
		return NULL;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if(!bJniInitLib) {
		LOGD("Java_com_hik_RtspClient_RtspClient_getRtspSessionID-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		return NULL;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_getRtspSessionID-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return NULL;
	}

	if (bRealStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_getRtspSessionID-> engine[%d] already stop", engineid);
		return NULL;
	}

	LOGD("Java_com_hik_RtspClient_RtspClient_getRtspSessionID->engine[%d] get Rtsp Session success", engineid);
	char* session = GetRtspClientSession(engineid);
	if(session == NULL) {
		LOGD("Java_com_hik_RtspClient_RtspClient_getRtspSessionID-> engine[%d] GetRtspClientSession fail", engineid);
		return NULL;
	}
	jstring jsession = env->NewStringUTF(session);

	return jsession;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    stopRtspProc
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_stopRtspProc
(JNIEnv *env, jobject obj, jint engineid)
{
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_stopRtspProc -> env == NULL");
		return false;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if(!bJniInitLib) {
		LOGD("Java_com_hik_RtspClient_RtspClient_stopRtspProc-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		return false;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_stopRtspProc-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return false;
	}

	if (bRealStopeds[engineid] && bBackStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_stopRtspProc-> engine[%d] already stop", engineid);
		return true;
	}

	if(!RtspClientStopRtspProc((int)engineid))
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_stopRtspProc-> engine[%d] stop fail", engineid);
		gGlobalError = RtspClientGetLastError();
		return false;
	}

	bRealStopeds[engineid] = true;
	bBackStopeds[engineid] = true;

	LOGD("Java_com_hik_RtspClient_RtspClient_stopRtspProc->engine[%d] stop success", engineid);
	return true;
}

bool getABStime(JNIEnv *env, pABS_TIME time, jobject jtimeobj)
{
	if(env == NULL)
	{
		LOGD("jni -> getABStime env == NULL");
		return false;
	}

	if(time == NULL)
	{
		LOGD("jni -> getABStime time == NULL");
		return false;
	}

	if(jtimeobj == NULL)
	{
		LOGD("jni -> getABStime jtimeobj == NULL");
		return false;
	}

	jclass jtimeTClass = env->GetObjectClass(jtimeobj);
	if(jtimeTClass == NULL) {
		LOGD("jni -> getABStime jtimeTClass == NULL");
		return false;
	}

	jmethodID id_getYear = env->GetMethodID(jtimeTClass, "getYear", "()I");
	if(id_getYear == NULL) {
		LOGD("jni -> getABStime id_getYear == NULL");
		env->DeleteLocalRef(jtimeTClass);
		return false;
	}
	time->dwYear = env->CallIntMethod(jtimeobj, id_getYear);

	jmethodID id_getMonth = env->GetMethodID(jtimeTClass, "getMonth", "()I");
	if(id_getMonth == NULL) {
		LOGD("jni -> getABStime id_getMonth == NULL");
		env->DeleteLocalRef(jtimeTClass);
		return false;
	}
	time->dwMonth = env->CallIntMethod(jtimeobj, id_getMonth);

	jmethodID id_getDay = env->GetMethodID(jtimeTClass, "getDay", "()I");
	if(id_getDay == NULL) {
		LOGD("jni -> getABStime id_getDay == NULL");
		env->DeleteLocalRef(jtimeTClass);
		return false;
	}
	time->dwDay = env->CallIntMethod(jtimeobj, id_getDay);

	jmethodID id_getHour = env->GetMethodID(jtimeTClass, "getHour", "()I");
	if(id_getHour == NULL) {
		LOGD("jni -> getABStime id_getHour == NULL");
		env->DeleteLocalRef(jtimeTClass);
		return false;
	}
	time->dwHour = env->CallIntMethod(jtimeobj, id_getHour);

	jmethodID id_getMinute = env->GetMethodID(jtimeTClass, "getMinute", "()I");
	if(id_getMinute == NULL) {
		LOGD("jni -> getABStime id_getMinute == NULL");
		env->DeleteLocalRef(jtimeTClass);
		return false;
	}
	time->dwMinute = env->CallIntMethod(jtimeobj, id_getMinute);

	jmethodID  id_getSecond = env->GetMethodID(jtimeTClass, "getSecond", "()I");
	if(id_getSecond == NULL) {
		LOGD("jni -> getABStime id_getSecond == NULL");
		env->DeleteLocalRef(jtimeTClass);
		return false;
	}
	time->dwSecond = env->CallIntMethod(jtimeobj, id_getSecond);
	env->DeleteLocalRef(jtimeTClass);
	jtimeTClass = NULL;
	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    generatePlaybackUrl
 * Signature: (Lcom/hik/mcrsdk/rtsp/PlaybackInfo;Ljava/lang/String;)Z
 */
JNIEXPORT jstring JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_generatePlaybackUrl
  (JNIEnv *env, jobject, jobject jplaybackInfo)
{
	LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generatePlaybackUrl");
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generatePlaybackUrl -> getPlaybackInfo env == NULL");
		return NULL;
	}

	if(jplaybackInfo == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generatePlaybackUrl -> getPlaybackInfo jplaybackInfo == NULL");
		return NULL;
	}

	PlaybcakInfo playbackInfo = {0};
	jstring jstr = NULL;
	jclass jPlaybackInfoClass = env->GetObjectClass(jplaybackInfo);

	jmethodID id_getMagIp;
	id_getMagIp = env->GetMethodID(jPlaybackInfoClass, "getMagIp", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getMagIp));
	if(jstr != NULL)
	{
		playbackInfo.magIp = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		playbackInfo.magIp = NULL;
	}

	jmethodID id_getMagPort;
	id_getMagPort = env->GetMethodID(jPlaybackInfoClass, "getMagPort", "()I");
	playbackInfo.magPort = env->CallIntMethod(jplaybackInfo, id_getMagPort);

	jmethodID id_getmcuNetID;
	id_getmcuNetID = env->GetMethodID(jPlaybackInfoClass, "getmcuNetID", "()I");
	playbackInfo.mcuNetID = env->CallIntMethod(jplaybackInfo, id_getmcuNetID);

	jmethodID id_getPlaybackUrl;
	id_getPlaybackUrl = env->GetMethodID(jPlaybackInfoClass, "getPlaybackUrl", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getPlaybackUrl));
	if(jstr != NULL)
	{
		playbackInfo.playbackUrl = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		playbackInfo.playbackUrl = NULL;
	}

	jmethodID id_getToken = env->GetMethodID(jPlaybackInfoClass, "getToken", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getToken));
	if(jstr != NULL)
	{
		playbackInfo.token = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		playbackInfo.token = NULL;
	}

	jmethodID id_getMagVersion = env->GetMethodID(jPlaybackInfoClass, "getMagVersion", "()I");
	int iMagVersion = env->CallIntMethod(jplaybackInfo, id_getMagVersion);
	playbackInfo.magVersion = iMagVersion;

	jmethodID id_getBegin;
	id_getBegin = env->GetMethodID(jPlaybackInfoClass, "getBegin", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getBegin));
	if(jstr != NULL)
	{
		playbackInfo.begin = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		playbackInfo.begin = NULL;
	}

	jmethodID id_getEnd;
	id_getEnd = env->GetMethodID(jPlaybackInfoClass, "getEnd", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getEnd));
	if(jstr != NULL)
	{
		playbackInfo.end = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		playbackInfo.end = NULL;
	}

	env->DeleteLocalRef(jPlaybackInfoClass);
	jPlaybackInfoClass = NULL;

	char playbackUrl[MAX_URL_LEN];
	memset(playbackUrl, 0, MAX_URL_LEN);
	bool bRet = GeneratePlaybackUrl(&playbackInfo, playbackUrl);
	if(!bRet)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generatePlaybackUrl -> GeneratePlaybackUrl fail");
		return NULL;
	}
	jstring jplaybackUrl = env->NewStringUTF(playbackUrl);
	return jplaybackUrl;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    generateNCGPlaybackUrl
 * Signature: (Lcom/hik/mcrsdk/rtsp/NCGPlaybackInfo;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_generateNCGPlaybackUrl
  (JNIEnv *env, jobject, jobject jplaybackInfo)
{
	LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generateNCGPlaybackUrl");
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generateNCGPlaybackUrl -> getPlaybackInfo env == NULL");
		return NULL;
	}

	if(jplaybackInfo == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generateNCGPlaybackUrl -> getNCGPlaybackInfo jplaybackInfo == NULL");
		return NULL;
	}

	NCGPlaybcakInfo playbackInfo = {0};
	jstring jstr = NULL;
	jclass jNCGPlaybackInfoClass = env->GetObjectClass(jplaybackInfo);

	jmethodID id_getMagIp;
	id_getMagIp = env->GetMethodID(jNCGPlaybackInfoClass, "getMagIp", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getMagIp));
	if(jstr != NULL) {
		playbackInfo.magIp = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else {
		playbackInfo.magIp = NULL;
	}

	jmethodID id_getMagPort;
	id_getMagPort = env->GetMethodID(jNCGPlaybackInfoClass, "getMagPort", "()I");
	playbackInfo.magPort = env->CallIntMethod(jplaybackInfo, id_getMagPort);

	jmethodID id_getCameraIndexCode;
	id_getCameraIndexCode = env->GetMethodID(jNCGPlaybackInfoClass, "getCameraIndexCode", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getCameraIndexCode));
	if(jstr != NULL)
	{
		playbackInfo.cameraIndexCode = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		playbackInfo.cameraIndexCode = NULL;
	}

	jmethodID id_getmcuNetID;
	id_getmcuNetID = env->GetMethodID(jNCGPlaybackInfoClass, "getmcuNetID", "()I");
	playbackInfo.mcuNetID = env->CallIntMethod(jplaybackInfo, id_getmcuNetID);

	jmethodID id_getToken = env->GetMethodID(jNCGPlaybackInfoClass, "getToken", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getToken));
	if(jstr != NULL)
	{
		playbackInfo.token = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		playbackInfo.token = NULL;
	}

	jmethodID id_getFileName;
	id_getFileName = env->GetMethodID(jNCGPlaybackInfoClass, "getFileName", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getFileName));
	if(jstr != NULL)
	{
		playbackInfo.fileName = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		playbackInfo.fileName = NULL;
	}

	jmethodID id_getRecordPos;
	id_getRecordPos = env->GetMethodID(jNCGPlaybackInfoClass, "getRecordPos", "()I");
	playbackInfo.recordPos = env->CallIntMethod(jplaybackInfo, id_getRecordPos);

	jmethodID id_getRecordType;
	id_getRecordType = env->GetMethodID(jNCGPlaybackInfoClass, "getRecordType", "()I");
	playbackInfo.recordType = env->CallIntMethod(jplaybackInfo, id_getRecordType);

	jmethodID id_getBegin;
	id_getBegin = env->GetMethodID(jNCGPlaybackInfoClass, "getBegin", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getBegin));
	if(jstr != NULL)
	{
		playbackInfo.begin = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		playbackInfo.begin = NULL;
	}

	jmethodID id_getEnd;
	id_getEnd = env->GetMethodID(jNCGPlaybackInfoClass, "getEnd", "()Ljava/lang/String;");
	jstr = static_cast<jstring>(env->CallObjectMethod(jplaybackInfo, id_getEnd));
	if(jstr != NULL)
	{
		playbackInfo.end = (char*)env->GetStringUTFChars(jstr, NULL);
	}
	else
	{
		playbackInfo.end = NULL;
	}

	env->DeleteLocalRef(jNCGPlaybackInfoClass);
	jNCGPlaybackInfoClass = NULL;

	char playbackUrl[MAX_URL_LEN];
	memset(playbackUrl, 0, MAX_URL_LEN);
	bool bRet = GenerateNCGPlaybackUrl(&playbackInfo, playbackUrl);
	if(!bRet)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_generateNCGPlaybackUrl -> GenerateNCGPlaybackUrl fail");
		return NULL;
	}
	jstring jplaybackUrl = env->NewStringUTF(playbackUrl);
	return jplaybackUrl;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    playbackByTime
 * Signature: (ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;Lcom/hik/mcrsdk/rtsp/ABS_TIME;Lcom/hik/mcrsdk/rtsp/ABS_TIME;)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_playbackByTime
(JNIEnv * env, jobject obj, jint engineid, jstring playbackUrl, jstring deviceName, jstring devicePsw, jobject jbeginTimeobj, jobject jendTimeobj)
{
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_playbackByTime -> env == NULL");
		return false;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if(!bJniInitLib) {
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackByTime-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		return false;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackByTime-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return false;
	}

	if (!bRealStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_playbackByTime->engine[%d] real playing\n", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_REAL_PLAY;
		return false;
	}

	if(!bBackStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_playbackByTime->engine[%d] playing back\n", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_STOP;
		return true;
	}

	if(jbeginTimeobj == NULL || jendTimeobj == NULL) {
		LOGD("Java_com_hik_RtspClient_playbackByTime->engine[%d] param error\n", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_INET_PARAM_NULL;
		return false;
	}

	if(playbackUrl == NULL) {
		LOGD("Java_com_hik_RtspClient_playbackByTime->engine[%d] playbackUrl == NULL\n", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_INET_PARAM_NULL;
		return false;
	}

	const char* devName = NULL;
	if(deviceName != NULL) {
		devName = env->GetStringUTFChars(deviceName, NULL);
	}
	else
	{
		devName = "admin";
	}

	const char* devPsw = NULL;
	if(devicePsw != NULL) {
		devPsw = env->GetStringUTFChars(devicePsw, NULL);
	}
	else
	{
		devPsw = "12345";
	}
	const char* url = env->GetStringUTFChars(playbackUrl, NULL);

	if (jbeginTimeobj == NULL || jendTimeobj == NULL) {
		LOGD("Java_com_hik_RtspClient_playbackByTime->engine[%d] param error\n", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_INET_PARAM_NULL;
		return false;
	}

	ABS_TIME beginTime;
	bool bRet = getABStime(env, &beginTime, jbeginTimeobj);
	if(!bRet)
	{
		LOGD("Java_com_hik_RtspClient_playbackByTime->engine[%d] getABStime beginTime fail\n", engineid);
		return false;
	}

	ABS_TIME endtime;
	bRet = getABStime(env, &endtime, jendTimeobj);
	if(!bRet)
	{
		LOGD("Java_com_hik_RtspClient_playbackByTime->engine[%d] getABStime endtime fail\n", engineid);
		return false;
	}

	if (!RtspClientPlaybackByTime((int)engineid, url, devName, devPsw, &beginTime, &endtime)) {
		LOGD("Java_com_hik_RtspClient_playbackByTime->engine[%d] start play back fail\n", engineid);
		gGlobalError = RtspClientGetLastError();
		return false;
	}

	bBackStopeds[engineid] = false;
	LOGD("Java_com_hik_RtspClient_playbackByTime->engine[%d] start play back success\n", engineid);
	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    setPlaybackPos
 * Signature: (ILcom/hik/mcrsdk/rtsp/ABS_TIME;Lcom/hik/mcrsdk/rtsp/ABS_TIME;)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_setPlaybackPos
(JNIEnv * env , jobject obj, jint engineid, jobject fromT, jobject toT)
{
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_setPlaybackPos -> env == NULL");
		return false;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if(!bJniInitLib) {
		LOGD("Java_com_hik_RtspClient_RtspClient_setPlaybackPos-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		return false;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_setPlaybackPos-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return false;
	}

	if (bBackStopeds[engineid] && bRealStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_setPlaybackPos-> engine[%d] play back already stop", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_STOP;
		return false;
	}

	if (fromT == NULL) {
		LOGD("Java_com_hik_RtspClient_RtspClient_setPlaybackPos-> engine[%d] fromT is null", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_INET_PARAM_NULL;
		return false;
	}

	jmethodID id_getYear, id_getMonth, id_getDay, id_getHour, id_getMinute, id_getSecond;
	ABS_TIME fromTime, toTime;
	jclass fromTClass, toTClass;

	fromTClass = env->GetObjectClass(fromT);
	id_getYear = env->GetMethodID(fromTClass, "getYear", "()I");
	fromTime.dwYear = env->CallIntMethod(fromT, id_getYear);
	id_getMonth = env->GetMethodID(fromTClass, "getMonth", "()I");
	fromTime.dwMonth = env->CallIntMethod(fromT, id_getMonth);
	id_getDay = env->GetMethodID(fromTClass, "getDay", "()I");
	fromTime.dwDay = env->CallIntMethod(fromT, id_getDay);
	id_getHour = env->GetMethodID(fromTClass, "getHour", "()I");
	fromTime.dwHour = env->CallIntMethod(fromT, id_getHour);
	id_getMinute = env->GetMethodID(fromTClass, "getMinute", "()I");
	fromTime.dwMinute = env->CallIntMethod(fromT, id_getMinute);
	id_getSecond = env->GetMethodID(fromTClass, "getSecond", "()I");
	fromTime.dwSecond = env->CallIntMethod(fromT, id_getSecond);

	if (toT != NULL) {
		toTClass = env->GetObjectClass(toT);
		id_getYear = env->GetMethodID(toTClass, "getYear", "()I");
		toTime.dwYear = env->CallIntMethod(toT, id_getYear);
		id_getMonth = env->GetMethodID(toTClass, "getMonth", "()I");
		toTime.dwMonth = env->CallIntMethod(toT, id_getMonth);
		id_getDay = env->GetMethodID(toTClass, "getDay", "()I");
		toTime.dwDay = env->CallIntMethod(toT, id_getDay);
		id_getHour = env->GetMethodID(toTClass, "getHour", "()I");
		toTime.dwHour = env->CallIntMethod(toT, id_getHour);
		id_getMinute = env->GetMethodID(toTClass, "getMinute", "()I");
		toTime.dwMinute = env->CallIntMethod(toT, id_getMinute);
		id_getSecond = env->GetMethodID(toTClass, "getSecond", "()I");
		toTime.dwSecond = env->CallIntMethod(toT, id_getSecond);

		if (!RtspClientSetPlaybackPos((int)engineid, &fromTime, &toTime))
		{
			LOGD("Java_com_hik_RtspClient_RtspClient_setPlaybackPos-> engine[%d] RtspClientSetPlaybackPos fail", engineid);
			return false;
		}
	}
	else {
		if (!RtspClientSetPlaybackPos((int)engineid, &fromTime, NULL))
		{
			LOGD("Java_com_hik_RtspClient_RtspClient_setPlaybackPos-> engine[%d] RtspClientSetPlaybackPos fail", engineid);
			gGlobalError = RtspClientGetLastError();
			return false;
		}
	}

	env->DeleteLocalRef(fromTClass);
	fromTClass = NULL;
	env->DeleteLocalRef(toTClass);
	toTClass = NULL;
	LOGD("Java_com_hik_RtspClient_RtspClient_setPlaybackPos-> engine[%d] set pos success\n", engineid);
	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    pause
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_pause
(JNIEnv * env , jobject obj, jint engineid)
{
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_pause -> env == NULL");
		return false;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if(!bJniInitLib) {
		LOGD("Java_com_hik_RtspClient_RtspClient_pause-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		return false;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_pause-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return false;
	}

	if (bBackStopeds[engineid] && bRealStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_pause-> engine[%d] play back and realplay already stop", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_STOP;
		return false;
	}

	if (!RtspClientPause((int)engineid))
	{
		gGlobalError = RtspClientGetLastError();
		LOGD("Java_com_hik_RtspClient_RtspClient_pause-> engine[%d] pause fail", engineid);
		return false;
	}

	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    resume
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_resume
(JNIEnv * env , jobject obj, jint engineid)
{
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_resume -> env == NULL");
		return false;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if(!bJniInitLib) {
		LOGD("Java_com_hik_RtspClient_RtspClient_pause-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		return false;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_pause-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return false;
	}

	if (bBackStopeds[engineid] && bRealStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_resume-> engine[%d] play back already stop", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_STOP;
		return false;
	}

	if (!RtspClientResume((int)engineid))
	{
		gGlobalError = RtspClientGetLastError();
		LOGD("Java_com_hik_RtspClient_RtspClient_resume-> engine[%d] resume", engineid);
		return false;
	}

	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    playbackFast
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_playbackFast
(JNIEnv * env , jobject obj, jint engineid)
{
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_playbackFast -> env == NULL");
		return false;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if(!bJniInitLib) {
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackFast-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		return false;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackFast-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return false;
	}

	if (bBackStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackFast-> engine[%d] play back already stop", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_STOP;
		return false;
	}

	if (!RtspClientPlaybackFast((int)engineid))
	{
		gGlobalError = RtspClientGetLastError();
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackFast-> engine[%d] play back fast", engineid);
		return false;
	}

	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    playbackSlow
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_playbackSlow
(JNIEnv * env , jobject obj, jint engineid)
{
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_playbackSlow -> env == NULL");
		return false;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if(!bJniInitLib) {
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackSlow-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		return false;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackSlow-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return false;
	}

	if (bBackStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackSlow-> engine[%d] play back already stop", engineid);
		gGlobalError = RTSPCLIENT_ERR_CODE_CAN_NOT_STOP;
		return false;
	}

	if (!RtspClientPlaybackSlow((int)engineid))
	{
		gGlobalError = RtspClientGetLastError();
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackSlow-> engine[%d] play back slow", engineid);
		return false;
	}

	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    playbackNormal
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_playbackNormal
(JNIEnv * env , jobject obj, jint engineid)
{
	if(env == NULL)
	{
		LOGD(" Java_com_hik_mcrsdk_rtsp_RtspClient_playbackNormal -> env == NULL");
		return false;
	}

	HPR_Guard gGuard(&oJniEngineLock[engineid]);

	if(!bJniInitLib) {
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackNormal-> engine[%d] not init lib", engineid);
		gGlobalError = RTSPCLIENT_NO_INIT;
		return false;
	}

	if (engineid < 0 || engineid >= ENGINE_MAX_NUM)
	{
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackNormal-> %d is invalid engine", engineid);
		gGlobalError = RTSPCLIENT_ENGINEID_INVALID;
		return false;
	}

	if (bBackStopeds[engineid]) {
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackNormal-> engine[%d] play back already stop", engineid);
		gGlobalError = RTSPCLIENT_NO_PLAY_STATE;
		return false;
	}

	if (!RtspClientPlaybackNormal((int)engineid))
	{
		gGlobalError = RtspClientGetLastError();
		LOGD("Java_com_hik_RtspClient_RtspClient_playbackNormal-> engine[%d] play back normal", engineid);
		return false;
	}

	return true;
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    getRtspClientVision
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_getRtspClientVision
(JNIEnv * env , jobject obj)
{
	return env->NewStringUTF(GetRtspClientVersion());
}

/*
 * Class:     com_hik_mcrsdk_rtsp_RtspClient
 * Method:    getLastError
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_hik_mcrsdk_rtsp_RtspClient_getLastError
(JNIEnv * env , jobject obj)
{
	return (jint)gGlobalError;
}

