/*
 * CAudioBuffer.h
 *
 *  Created on: 2014-6-4
 *      Author: weilinfeng
 */

#ifndef AUDIOBUFFER_H
#define AUDIOBUFFER_H

#include "HPR_Mutex.h"

#ifndef AUDIO_BUFFER_SIZE
#define AUDIO_BUFFER_SIZE    500*1024  // 音频数据缓冲区大小(500K)
#endif

class CAudioBuffer
{
public:

    ~CAudioBuffer();

    CAudioBuffer();

    // 初始化函数
    bool Initialize();

    // 释放资源
    void Release();

    // 安全填充数据
    int WriteDataToBuffer(const unsigned char* pData, int iWriteSize);

    // 安全读取数据
    int ReadDataFromBuffer(int iReadSize, unsigned char* pData);

    // 重新设置缓冲区和一些变量
    void ReSet();

    // 获取错误码
    int GetLastError() {return m_iLastError;};

private:

    // 填充数据
    int WriteData(const unsigned char* pData, int iDataLen);

    // 从缓冲区中读数据
    int ReadData(int iReadSize, unsigned char* pData);

private:
    // 缓冲区首地址的指针
    unsigned char* m_pBufferBeginPtr;

    // 缓冲区尾地址的指针
    unsigned char* m_pBufferEndPtr;

    // 缓冲区无效空间的地址
    unsigned char* m_pInvalidPtr;

    // 开始读缓冲区的地址
    unsigned char* m_pReadPtr;

    // 开始写缓冲区的地址
    unsigned char* m_pWritePtr;

    // 表示写和读之间是否差了一轮
    bool m_bOneRound;

    // 标识已初始化
    bool m_bInit;

    // 错误码
    int m_iLastError;

    int m_iCount;

    // 缓冲区中总的数据大小
    int m_iTotleSize;

    // 互斥体
    HPR_MUTEX_T m_lock;
};

#endif // AUDIOBUFFER_H

