﻿/*
 * AudioCodec.h
 *
 *  Created on: 2014-6-4
 *      Author: weilinfeng
 */


#ifndef AUDIOCODEC_H_
#define AUDIOCODEC_H_
#include "G711codec.h"
#include "g722codec.h"
#include "G726_Codec.h"
#include "AudioCodecInferface.h"
#include "AudioBuffer.h"
#include "audio_codec_common.h"
#include "HPR_Types.h"
#include "Define.h"
#include "audio_de_noise.h"

/*
 * G711 压缩比是固定的:PCM = G711*2
 * G726 压缩比需要根据音频参数计算，一般为四倍左右
 * G722 压缩比需要根据音频参数计算，一般为十几倍左右,G722解码需要设置比特率，其他不需要设置
 */

#define LAW_U 0
#define LAW_A 1

#define PER_G722_DATA_SIZE		80			// 现阶音频数据长度写死，为80（G722）
#define PER_G711_DATA_SIZE		320
#define TMP_BUF_SIZE            1024        // 音频缓冲大小

// 回音消除参数
typedef struct
{
    // 回音消除参数
	DENOISE_PARAM denoiseParam;						// 回音消除参数
	MEM_TAB       denoiseMemTab[1];					// 回音消除处理内存
	DENOISE_PROCESS_PARAM denoiseProcessParam;		// 回音消除处理参数
}DenoiseParam;

typedef void (*pAudioCodecPCMDataCallBack)(char* pPCMData, unsigned int nDataSize, void* pUser);
typedef void (*pAudioCodecXXXDataCallBack)(char* pXXData, unsigned int nDataSize, void* pUser);

class AudioCodec
{
public:
    AudioCodec();

    virtual ~AudioCodec();

    // 初始化
    bool Init(pAudioCodecPCMDataCallBack pcmDataCallBack, pAudioCodecXXXDataCallBack xxxDataCallBack, void* pUser);

    // 反初始化
    void Fini();

    // 开始编码
    bool StartEncode(int aEncType, EncodeParams& encodeParams);

    // 停止编码
    bool StopEncode();

    // 输入PCM数据
   	bool InputPCMData(const char* pData, HPR_INT32 nDataSize);

    // 正在编码
    bool isEncoding();

    // 开始解码
    bool StartDecode(int aDecType, DecodeParams& decodeParams);

    // 停止解码
    bool StopDecode();

	// 正在编码
	bool isDecoding();

    // xxx
	bool InputXXData(const char* pData, HPR_INT32 nDataSize);

	int GetLastError();

    // 音频编码编码线程
	HPR_VOIDPTR static AudioEncodeThread(HPR_VOID* pParam);

	// 音频解码线程
	HPR_VOIDPTR static AudioDecodeThread(HPR_VOID* pParam);

	pAudioCodecPCMDataCallBack pPCMDataCallBack;

	pAudioCodecXXXDataCallBack pXXXDataCallBack;


private:
    // 初始化编解码库
    bool InitEncode();

    // 初始化解码库
    bool InitDecode();

    // 初始化编解码库
    void FiniEncode();

    void FiniDecode();

    bool InitDenoise();

    void FiniDenoise();

	bool InitG711Encode();

	bool InitG711Decode();

	bool InitG722Encode();

	bool InitG722Decode();

	bool InitG726Encode();

	bool InitG726Decode();

    // 分配缓冲区
    bool MallocBuffer();

    // 释放缓冲区
    void FreeBuffer();

    // 音频数据编码
    void Encode();

    // 音频数据解码
    void Decode();

    // G711编码
    void G711Encode(int lowType);

    // G711解码
    void G711Decode(int lowType);

    // G722编码
	void G722Encode();

    // G722解码
    void G722Decode();

    // G722编码
	void G726Encode();

    // G722解码
    void G726Decode();

private:
	// 编码参数
	EncodeParams mEncParams;

	// 解码参数
	DecodeParams mDecParams;

	void* pUser;
    // 标志是否初始化
    bool mInit;

    // 标志是否正在编码
    bool mEncoding;

    // 标志是否正在解码
    bool mDecoding;

    // 编码线程退出标志
    bool mEncThreadExit;

    // 解码线程退出标志
    bool mDecThreadExit;

    // 编码线程句柄
    HPR_HANDLE mEncThread;

    // 解码线程句柄
    HPR_HANDLE mDecThread;

    // 编码句柄
    void* mEncHandle;

    // 解码句柄
    void* mDecHandle;

    // 编码类型
    int mEncType;

    // 解码类型
    int mDecType;
    
    // 编码前的PCM数据缓冲区
    CAudioBuffer* mEncPCMBuf;
    // 音频编码参数
    AUDIOENC_INFO mEncInfo;
    // 音频编码参数
	AUDIOENC_PARAM mEncParam;
	// 音频编码内存块
	MEM_TAB	mEncMemTab[1];
	// 编码数据处理参数
	AUDIOENC_PROCESS_PARAM mEncProParam;

    // 解码前的XX数据缓冲区
    CAudioBuffer* mDecXXBuf;
    // 音频解码参数
	AUDIODEC_PARAM mDecParam;

	// 音频解码内存块
	MEM_TAB mDecMemTab[1];
	// 音频解码数据处理参数
	AUDIODEC_PROCESS_PARAM mDecProParam;

    // 缓冲音频数据大小
    unsigned int mTmpBufSize;

    int mLastError;

    // 测试文件
    //FILE* mTestFile;

    // 回音消除句柄
    void *mDenoiseHandle;

    // 回音消除参数
    DenoiseParam mDenoiseParam;
};

#endif /* AUDIOCODEC_H_ */
