/*
 * AudioCodecInferface.h
 *
 *  Created on: 2014-8-6
 *      Author: weilinfeng
 */

#ifndef AUDIOCODECINFERFACE_H_
#define AUDIOCODECINFERFACE_H_
#ifdef __cpluscplus
#define AUDIOCODECINFERFACE_API extern
#else
#define  AUDIOCODECINFERFACEAPI
#endif

# define AUDIOCODEC_ERR_SUCCESS 0
# define AUDIOCODEC_ERR_NOT_SUPPORT_ENCODE 1
# define AUDIOCODEC_ERR_NOT_SUPPORT_DECODE 2

#define TYPE_G711_U 1
#define TYPE_G711_A 2
#define TYPE_G722 3
#define TYPE_G726 4

#ifndef NULL
#define NULL 0
#endif


typedef struct _EncodeParams_ {
	int sampleRate;
	int bitRate;
	int channels;
}EncodeParams;

typedef struct _DecodeParams_ {
	int bitRate;
}DecodeParams;

typedef void (*pAudioInterfacePCMDataListener)(char* buffer, unsigned int buflen, void* pUser);
typedef void (*pAudioInterfaceXXXDataListener)(char* buffer, unsigned int buflen, void* pUser);

/** @fn bool Init
 *   @brief 初始化
 *   @return true - 初始化成功; false - 初始化库失败
 */
AUDIOCODECINFERFACEAPI bool AudioInit(void* pUser);

/** @fn bool Fini
 *   @brief 反初始化
 *   @return 无
 */
AUDIOCODECINFERFACEAPI void AudioFini();


/** @fn bool SetPCMDataListener
 *   @brief 设置解码后的音频数据回调
 *   @param [in] pcmDataListener - 数据回调函数指针
 *   @return true - 成功, false - 失败
 */
AUDIOCODECINFERFACEAPI bool SetPCMAudioDataListener(pAudioInterfacePCMDataListener pcmDataListener);


/** @fn bool SetXXXDataListener
 *   @brief 设置编码后的音频数据回调
 *   @param [in] xxxDataListener - 数据回调函数指针
 *   @return true - 成功, false - 失败
 */
AUDIOCODECINFERFACEAPI bool SetXXXAudioDataListener(pAudioInterfaceXXXDataListener xxxDataListener);

/** @fn bool StartEncode
 *   @brief 开始编码
 *   @param [in] iEncodeType - 编码类型
 *   @param [in] encodeParams - 编码参数
 *   @param [out] 无
 *   @return true 表示成功， false 表示失败
 */
AUDIOCODECINFERFACEAPI bool StartAudioEncode(int iEncodeType, EncodeParams& encodeParams);

/** @fn bool isEncoding
 *   @brief 是否正在编码
 *   @return  true - 是; false - 否
 */
AUDIOCODECINFERFACEAPI bool isEncoding();

/** @fn bool InputAudioData
 *   @brief 塞入音频数据
 *   @param [in] InputAudioData - PCM音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
AUDIOCODECINFERFACEAPI bool InputPCMAudioData(const char* pData, unsigned int nDataSize);

/** @fn bool StopEncode
 *   @brief 停止编码
 *   @return 无
 */
AUDIOCODECINFERFACEAPI void StopAudioEncode();

/** @fn bool StartDecode
 *   @brief 开始解码
 *   @param [in] iDecodeType - 解码类型
 *   @param [in] decodeParams - 解码参数
 *   @param [out] 无
 *   @return true 表示连接成功， false 表示连接失败
 */
AUDIOCODECINFERFACEAPI bool StartAudioDecode(int iDecodeType, DecodeParams& decodeParams);

/** @fn bool isDecoding
 *   @brief 是否正在解码
 *   @return  true - 是; false - 否
 */
AUDIOCODECINFERFACEAPI bool isDecoding();

/** @fn bool InputXXXData
 *   @brief 塞入音频数据
 *   @param [in] InputAudioData - XXX音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
AUDIOCODECINFERFACEAPI bool InputXXXAudioData(const char* pData, unsigned int nDataSize);

/** @fn bool StopDecode()
 *   @brief 停止解码
 *   @return 无
 */
AUDIOCODECINFERFACEAPI void StopAudioDecode();

/** @fn bool GetLastError()
 *   @brief 停止解码
 *   @return 无
 */
AUDIOCODECINFERFACEAPI int GetAidioLastError();

#endif /* AUDIOCODECINFERFACE_H_ */
