/*******************************************************************************************************************************
**  版权信息：     版权所有 (c) 2010, 杭州海康威视软件有限公司, 保留所有权利
**
**  文件名称：     G711codec.h
**
**  文件描述:      G711音频编解码库接口函数声明文件
**
**  当前版本：     3.2.0
**  作    者：     陈扬坤
**  日    期：     2011年10月15日
**  备    注：     更改接口，改有包含audio_common.h
**
**  历史版本：     3.0.0
**  作    者:      陈扬坤
**  日    期:      2011年5月10号
**  备    注：     按照代码规范整理G711CODEC接口
**
**  历史版本：     2.3
**  作    者:      陈扬坤
**  日    期:      2010年11月18号
**   
**  历史版本：     2.2
**  作    者:      沈林杰
**  日    期:      2010年4月20号
**           
**  历史版本：     2.1
**  作    者:      黄添喜
**  日    期:      2007年12月21号
**
**  修改历史:        
*******************************************************************************************************************************/
#ifndef _HIK_G711CODEC_LIB_H_
#define _HIK_G711CODEC_LIB_H_

/*******************************************************************************************************************************
* 作用域控制宏 (必须使用，以便C++语言能调用)
*******************************************************************************************************************************/
#ifdef __cplusplus
extern "C" {
#endif

#include "datatypedef.h"
#include "mem_tab.h"
#include "audio_codec_common.h"

/*******************************************************************************************************************************
*文件版本和时间宏声明
*******************************************************************************************************************************/
/* 当前版本号*/
/* 主版本号，接口改动、功能增加、架构变更时递增，最大63 */
#define HIK_G711CODEC_MAJOR_VERSION      3
/* 子版本号，性能优化、局部结构调整、模块内集成其他库的主版本提升时递增，最大31 */
#define HIK_G711CODEC_SUB_VERSION        1
/* 修正版本号，修正bug后递增，最大31 */
#define HIK_G711CODEC_REVISION_VERSION   0
/* 版本日期*/
#define HIK_G711CODEC_VER_YEAR           11           /* 年 */
#define HIK_G711CODEC_VER_MONTH          10           /* 月 */
#define HIK_G711CODEC_VER_DAY            15           /* 日 */


#define	HIK_G711CODEC_LIB_S_NEED_MORE_DATA          0x00000002
#define HIK_G711CODEC_LIB_E_TYPE_ERR                0x86000000   /* 输入A, U law类型错误 */
#define HIK_G711CODEC_MTAB_NUM                      1


/*******************************************************************************************************************************
* 接口函数声明
*******************************************************************************************************************************/

/*******************************************************************************************************************************
* 功  能：获取编码一帧输入数据大小
*
* 参  数：info_param   -IO   信息结构指针
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G711ENC_GetInfoParam(AUDIOENC_INFO *info_param);

/*******************************************************************************************************************************
* 功  能：G711ENC获取所需内存大小
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G711ENC_GetMemSize(AUDIOENC_PARAM *param, MEM_TAB mem_tab[HIK_G711CODEC_MTAB_NUM]);

/*******************************************************************************************************************************
* 功  能：创建G711ENC模块
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*         handle     -O 返回G711CODEC模块句柄         
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G711ENC_Create(AUDIOENC_PARAM *param, MEM_TAB mem_tab[HIK_G711CODEC_MTAB_NUM], void **handle);

/*******************************************************************************************************************************
* 功  能: 函数调用一次,编码sample_num个样点
*
* 参  数: handle           -I     句柄(handle由Create返回)
*         enc_proc_param   -IO    编码处理参数
*
* 返回值: 状态码
*******************************************************************************************************************************/
HRESULT HIK_G711ENC_Encode(void *handle, AUDIOENC_PROCESS_PARAM *enc_proc_param);

/*******************************************************************************************************************************
* 功  能：G711DEC获取所需内存大小
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G711DEC_GetMemSize(AUDIODEC_PARAM *param, MEM_TAB mem_tab[HIK_G711CODEC_MTAB_NUM]);

/*******************************************************************************************************************************
* 功  能：创建G711DEC模块
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*         handle     -O 返回G711CODEC模块句柄
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G711DEC_Create(AUDIODEC_PARAM *param, MEM_TAB mem_tab[HIK_G711CODEC_MTAB_NUM], void **handle);

/*******************************************************************************************************************************
* 功  能: 函数调用一次,解码出sample_num个PCM样点
*
* 参  数: handle           -IO    句柄(handle由Create返回)
*         enc_proc_param   -IO    编码处理参数
*
* 返回值: 状态码
*******************************************************************************************************************************/
HRESULT HIK_G711DEC_Decode(void *handle, AUDIODEC_PROCESS_PARAM *dec_proc_param);

/*******************************************************************************************************************************
* 功  能：获取当前编码版本信息
*
* 参  数：无
*
* 返回值：返回版本信息
*
* 备  注：版本信息格式为：版本号＋年（7位）＋月（4位）＋日（5位）
*         其中版本号为：主版本号（6位）＋子版本号（5位）＋修正版本号（5位）
*******************************************************************************************************************************/
U32 HIK_G711CODEC_GetVersion();

#ifdef __cplusplus
}
#endif


#endif