﻿/*******************************************************************************************************************************
** 版权信息：Copyright (c) 2008, 杭州海康威视数字技术股份有限公司
**
** 文件名称：G726_Codec.h
**
** 文件标识：HIKVISION_G726_CODEC_H
**
** 文件摘要：g726编解码接口程序
**
** 当前版本：1.0.1
** 作    者：陈扬坤
** 日    期：2014年3月3日
** 备    注：增加解码库的处理长度
**
** 当前版本：0.9.9
** 作    者：陈扬坤
** 日    期：2011年10月13日
** 备    注：更改接口，改有包含audio_codec_common.h
**
** 历史版本：0.9
** 作    者：陈扬坤
** 日    期：2011.6.25
** 备    注：优化g726编解码代码，规范代码
**
** 历史版本：0.8
** 作    者：沈林杰
** 日    期：2008.12.2
** 备    注：
*******************************************************************************************************************************/

#ifndef _HIK_G726_CODEC_LIB_H_
#define _HIK_G726_CODEC_LIB_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "datatypedef.h"
#include "mem_tab.h"
#include "audio_codec_common.h"


/*******************************************************************************************************************************
*文件版本和时间宏声明
*******************************************************************************************************************************/
/* 当前版本号*/
/* 主版本号，接口改动、功能增加、架构变更时递增，最大63 */
#define HIK_G726CODEC_MAJOR_VERSION      1
/* 子版本号，性能优化、局部结构调整、模块内集成其他库的主版本提升时递增，最大31 */
#define HIK_G726CODEC_SUB_VERSION        0
/* 修正版本号，修正bug后递增，最大31 */
#define HIK_G726CODEC_REVISION_VERSION   1
/* 版本日期*/
#define HIK_G726CODEC_VER_YEAR           14           /* 年*/
#define HIK_G726CODEC_VER_MONTH          3            /* 月*/
#define HIK_G726CODEC_VER_DAY            3            /* 日*/

#define	HIK_G726CODEC_LIB_S_NEED_MORE_DATA          0x00000002
#define HIK_G726CODEC_MTAB_NUM                      1

/******************************************************************************************************************************/
/* G726每个采样点编码后bit数bits_per_sample与码率对应如下，标准采样率8000：                                                   */
/* bits_per_sample            bitrate                                                                                         */
/*       2                     16000                                                                                          */  
/*       3                     24000                                                                                          */
/*       4                     32000                                                                                          */
/*       5                     40000                                                                                          */
/******************************************************************************************************************************/


/*******************************************************************************************************************************
* 接口函数声明
*******************************************************************************************************************************/

/*******************************************************************************************************************************
* 功  能：获取编码一帧输入数据大小
*
* 参  数：info_param   -IO   信息结构指针
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G726ENC_GetInfoParam(AUDIOENC_INFO *info_param);

/*******************************************************************************************************************************
* 功  能：G726ENC获取所需内存大小
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G726ENC_GetMemSize(AUDIOENC_PARAM *param, MEM_TAB mem_tab[HIK_G726CODEC_MTAB_NUM]);

/*******************************************************************************************************************************
* 功  能：创建G726ENC模块 
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*         handle     -O 返回G726CODEC模块句柄         
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G726ENC_Create(AUDIOENC_PARAM *param, MEM_TAB mem_tab[HIK_G726CODEC_MTAB_NUM], void **handle);

/*******************************************************************************************************************************
* 函数名: G726_Encode
*
* 功  能：编码一帧
*
* 参  数：param		-IO 编解码参数结构体指针
*
* 返回值：成功 HIK_G726CODEC_LIB_S_OK
*         错误 其它状态码
*
* 备  注：
*******************************************************************************************************************************/
HRESULT HIK_G726ENC_Encode(void *handle, AUDIOENC_PROCESS_PARAM *prc_param);

/*******************************************************************************************************************************
* 功  能：G726DEC获取所需内存大小
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G726DEC_GetMemSize(AUDIODEC_PARAM *param, MEM_TAB mem_tab[HIK_G726CODEC_MTAB_NUM]);

/*******************************************************************************************************************************
* 功  能：创建G726DEC模块
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*         handle     -O 返回G726CODEC模块句柄         
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G726DEC_Create(AUDIODEC_PARAM *param, MEM_TAB mem_tab[HIK_G726CODEC_MTAB_NUM], void **handle);

/*******************************************************************************************************************************
* 函数名: G726_Decode
*
* 功  能：编码一帧
*
* 参  数：param		-IO 编解码参数结构体指针
*
* 返回值：成功 HIK_G726CODEC_LIB_S_OK
*         错误 其它状态码
*******************************************************************************************************************************/
HRESULT HIK_G726DEC_Decode(void *handle, AUDIODEC_PROCESS_PARAM *prc_param);

/*******************************************************************************************************************************
* 功  能：获取当前编码版本信息
*
* 参  数：无
*
* 返回值：返回版本信息
*
* 备  注：版本信息格式为：版本号＋年（7位）＋月（4位）＋日（5位）
*         其中版本号为：主版本号（6位）＋子版本号（5位）＋修正版本号（5位）
*******************************************************************************************************************************/
U32 G726CODEC_GetVersion();

#ifdef __cplusplus
}
#endif

#endif

