/*******************************************************************************************************************************
* 
* 版权信息：版权所有 (c) 2010, 杭州海康威视软件有限公司, 保留所有权利
* 
* 文件名称：g722codec.h
* 摘    要：海康威视 G722音频文件编解码库的接口函数声明文件

** 当前版本：2.1.0
** 作    者：陈扬坤
** 日    期：2011年10月13日
** 备    注：更改接口，改有包含audio_codec_common.h

* 历史版本：2.0.0
* 作    者：陈扬坤
* 日    期：2011年5月10日
* 备    注：整理G722编解码接口

* 历史版本：1.2.3
* 作    者：陈扬坤
* 日    期：2010年12月15日
* 备    注：合并G722编解码库

* 历史版本：1.2.2.n5.3.20100905
* 作    者：马先哲
* 日    期：2010年09月05日
* 备    注：按照C静态处理库开发规范，修改宏定义、变量、结构体、以及接口函数
*
* 版本更新：
* 版    本：1.2.1
* 作    者：黄添喜
* 日    期：2007年08月14日
* 备    注：1.2-1.n5.2.20070814 黄添喜修改,更新了接口函数。
*			1.1-1.n5.1.20070808 黄添喜修改,将编码和解码的工程分别做成了编码库和解码库
*
* 版    本：0.0.n5.0.20070808
* 作    者：贾永华
* 日    期：2007年08月08日
* 备    注：0.0.n5.0.20070808.  原始版本 贾永华
*
*******************************************************************************************************************************/
#ifndef _HIK_G722CODEC_LIB_H_
#define _HIK_G722CODEC_LIB_H_

#ifdef __cplusplus
extern "C" {
#endif 

#include "datatypedef.h"
#include "mem_tab.h"
#include "audio_codec_common.h"

/*******************************************************************************************************************************
//文件版本和时间宏声明
*******************************************************************************************************************************/
/* 当前版本号*/
/* 主版本号，接口改动、功能增加、架构变更时递增，最大63 */
#define HIK_G722CODEC_MAJOR_VERSION       2
/* 子版本号，性能优化、局部结构调整、模块内集成其他库的主版本提升时递增，最大31 */
#define HIK_G722CODEC_SUB_VERSION         1
/* 修正版本号，修正bug后递增，最大31 */
#define HIK_G722CODEC_REVISION_VERSION    0
/* 版本日期*/
#define HIK_G722CODEC_VER_YEAR            11            /* 年 */
#define HIK_G722CODEC_VER_MONTH           10            /* 月 */
#define HIK_G722CODEC_VER_DAY             13            /* 日 */


#define HIK_G722CODEC_MTAB_NUM                 1
#define HIK_G722CODEC_IN_SAMPLE_RATE           16000

#define HIK_G722CODEC_IN_BIT_RATE_16000 16000
#define HIK_G722CODEC_IN_BIT_RATE_24000 24000
#define HIK_G722CODEC_IN_BIT_RATE_32000 32000

#define HIK_G722CODEC_LIB_S_NEED_MORE_DATA     0x00000002

/*G722编码的信号源为16KHz 16位采样的PCM码，不可更改，支持的码率有16000，24000，32000                                          */

/*******************************************************************************************************************************
* 接口函数声明
*******************************************************************************************************************************/

/*******************************************************************************************************************************
* 功  能：获取编码一帧输入数据大小
*
* 参  数：info_param   -IO   信息结构指针
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G722ENC_GetInfoParam(AUDIOENC_INFO *info_param);

/*******************************************************************************************************************************
* 功  能：编码获取所需内存大小
*
* 参  数：param      -I  参数结构指针
*         mem_tab    -IO 内存申请表
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G722ENC_GetMemSize(AUDIOENC_PARAM *param, MEM_TAB mem_tab[HIK_G722CODEC_MTAB_NUM]);

/*******************************************************************************************************************************
* 功  能：g722编码创建G722编码模块
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*         handle     -O 返回G722CODEC模块句柄         
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G722ENC_Create(AUDIOENC_PARAM *param, MEM_TAB mem_tab[HIK_G722CODEC_MTAB_NUM], void **handle);

/*******************************************************************************************************************************
* 功  能: G722编码
*
* 参  数: handle           -IO    句柄(handle由Create返回)
*         proc_param       -IO    编码处理参数
*
* 返回值: 状态码
*******************************************************************************************************************************/
HRESULT HIK_G722ENC_Encode(void *handle, AUDIOENC_PROCESS_PARAM *enc_proc_param);

/*******************************************************************************************************************************
* 功  能：解码获取所需内存大小
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G722DEC_GetMemSize(AUDIODEC_PARAM * param, MEM_TAB mem_tab[HIK_G722CODEC_MTAB_NUM]);

/*******************************************************************************************************************************
* 功  能：g722解码创建G722解码模块
*
* 参  数：param      -I 参数结构指针
*         mem_tab    -I 内存申请表
*         handle     -O 返回G722CODEC模块句柄         
*
* 返回值：返回状态码
*******************************************************************************************************************************/
HRESULT HIK_G722DEC_Create(AUDIODEC_PARAM *param, MEM_TAB mem_tab[HIK_G722CODEC_MTAB_NUM], void **handle);

/*******************************************************************************************************************************
* 功  能: G722解码
*
* 参  数: handle           -IO    句柄(handle由Create返回)
*         proc_param       -IO    编码处理参数
*
* 返回值: 状态码
*******************************************************************************************************************************/
HRESULT HIK_G722DEC_Decode(void *handle, AUDIODEC_PROCESS_PARAM *dec_proc_param);	

/*******************************************************************************************************************************
* 功  能：获取当前编码版本信息
*
* 参  数：无
*
* 返回值：返回版本信息
*
* 备  注：版本信息格式为：版本号＋年（7位）＋月（4位）＋日（5位）
*         其中版本号为：主版本号（6位）＋子版本号（5位）＋修正版本号（5位）
*******************************************************************************************************************************/
U32 G722CODEC_GetVersion();

#ifdef __cplusplus
}
#endif

#endif
