﻿/*
 * AudioCodec.cpp
 *
 *  Created on: 2014-6-4
 *      Author: weilinfeng
 */


#include "AudioCodec.h"
#include "HPR_Thread.h"
#include "HPR_Utils.h"
#include "MCRPrint.h"
#include "hik_malloc.h"

AudioCodec::AudioCodec()
: pPCMDataCallBack(NULL)
, pXXXDataCallBack(NULL)
, mInit(false)
, mEncoding(false)
, mDecoding(false)
, mEncThreadExit(false)
, mDecThreadExit(false)
, mEncThread(NULL)
, mDecThread(NULL)
, mEncHandle(NULL)
, mDecHandle(NULL)
, mEncType(TYPE_G711_A)
, mDecType(TYPE_G711_A)
, mEncPCMBuf(NULL)
, mDecXXBuf(NULL)
, mTmpBufSize(0)
, mLastError(0)
, pUser(NULL)
//, mTestFile(NULL)
{
	// 音频解码内存块
	memset(&mDecMemTab[0], 0, sizeof(MEM_TAB));
	mDecMemTab[0].base = NULL;
	memset(&mEncMemTab[0], 0, sizeof(MEM_TAB));
	mEncMemTab[0].base = NULL;
	memset(&mEncParam, 0 , sizeof(AUDIOENC_PARAM));
	memset(&mEncProParam, 0 , sizeof(AUDIOENC_PROCESS_PARAM));
	memset(&mDecParam, 0 , sizeof(AUDIODEC_PARAM));
	memset(&mDecProParam, 0 , sizeof(AUDIODEC_PROCESS_PARAM));
	memset(&mEncInfo, 0 , sizeof(AUDIOENC_INFO));
	mDenoiseParam.denoiseMemTab->base = NULL;
}

AudioCodec::~AudioCodec()
{
    // TODO Auto-generated destructor stub
}


/** @fn Init
 *  @brief 初始化
 *  @param aEncType - 编码类型(IN)
 *  @param aDecType - 解码类型(IN)
 *  @return true - 成功
 *          false - 失败
 */
bool AudioCodec::Init(pAudioCodecPCMDataCallBack pcmDataCallBack, pAudioCodecXXXDataCallBack xxxDataCallBack, void* pUser)
{
    if (mInit)
    {
    	LOGD("AudioCodec->Init() have inited");
        return true;
    }

    // 回调监听
    pPCMDataCallBack = pcmDataCallBack;

    pXXXDataCallBack = xxxDataCallBack;

    this->pUser = pUser;

    // 分配缓冲区
    bool ret = MallocBuffer();
    if (!ret)
    {
        LOGD("AudioCodec->Init() malloc buffer fail");
        return false;
    }

	// 初始化回音消除库
    ret = InitDenoise();
    if (!ret)
    {
        LOGD("AudioCodec Init InitDenoise codec fail");
        return false;
    }

    // 初始化成功标识
    mInit = true;

    return true;
}

/** @fn Fini
 *  @brief 反初始化
 *  @param
 *  @return
 */
void AudioCodec::Fini()
{
	LOGD("AudioCodec->Fini()\n");
    // 停止编解码
    StopEncode();

    StopDecode();

    // 释放缓冲区
    FreeBuffer();

    mInit = false;
}

/** @fn StartEncode
 *  @brief 开始编码,每次调用开始编码，必须对应停止编码
 *  @param 无
 *  @return true - 成功
 *          false - 失败
 */
bool AudioCodec::StartEncode(int aEncType, EncodeParams& encodeParams)
{
    if (mEncoding)
    {
        LOGD("AudioCodec->StartEncode() have start encode ");
        return true;
    }

    // 设置编码\解码类型
    mEncType = aEncType;
    memcpy(&mEncParams, &encodeParams, sizeof(EncodeParams));
    LOGD("AudioCodec->StartEncode() encodeParams sampleRate:%d", encodeParams.sampleRate);
    LOGD("AudioCodec->StartEncode() encodeParams bitRate:%d", encodeParams.bitRate);
    LOGD("AudioCodec->StartEncode() encodeParams channels:%d", encodeParams.channels);
    LOGD("AudioCodec->StartEncode() mEncParams sampleRate:%d", mEncParams.sampleRate);
    LOGD("AudioCodec->StartEncode() mEncParams bitRate:%d", mEncParams.bitRate);
    LOGD("AudioCodec->StartEncode() mEncParams channels:%d", mEncParams.channels);

    bool bRet = InitEncode();
    if(!bRet) {
    	 LOGD("AudioCodec->StartEncode() InitCodec fail");
    	return false;
    }

//    mTestFile = fopen("/storage/emulated/0/AudioTest/audio.aac", "wb");
//	if (mTestFile == NULL) {
//		LOGD("AudioCodec->StartEncode() open mTestFile fail");
//		StopEncode();
//		return false;
//	}

	mEncThreadExit = false;
	// 以挂起的方式创建编码线程
	mEncThread = HPR_Thread_Create(AudioEncodeThread, this, 0);
	if (NULL == mEncThread)
	{
		mEncThreadExit = true;
		StopEncode();
		return false;
	}

	LOGD("AudioCodec->StartEncode() success");
    return true;
}


/** @fn StopEncode
 *  @brief 停止编码
 *  @param 无
 *  @return true - 成功
 *          false - 失败
 */
bool AudioCodec::StopEncode()
{
	LOGD("AudioCodec->StopEncode()\n");
	mEncThreadExit = true;
	if (NULL != mEncThread)
	{
		HPR_Thread_Wait(mEncThread);
		mEncThread = HPR_INVALID_HANDLE;
	}

	FiniEncode();

	LOGD("AudioCodec->StopEncode() suucess");

	return true;
}

// 正在编码
bool AudioCodec::isEncoding() {
	return mEncoding;
}


/** @fn StartDecode
 *  @brief 开始解码
 *  @param 无
 *  @return true - 成功
 *          false - 失败
 */
bool AudioCodec::StartDecode(int aDecType, DecodeParams& decodeParams)
{
    if (mDecoding) {
        LOGD("AudioCodec->StartDecode() be decodeing");
        return true;
    }

    mDecType = aDecType;
    memcpy(&mDecParams, &decodeParams, sizeof(DecodeParams));
    LOGD("AudioCodec->StartEncode() decodeParams bitRate:%d", decodeParams.bitRate);
    LOGD("AudioCodec->StartEncode() mDecParams bitRate:%d", mDecParams.bitRate);

    bool bRet = InitDecode();
	if(!bRet) {
		 LOGD("AudioCodec->StartEncode() InitDecode fail");
		 StopDecode();
		return false;
	}

	mDecThreadExit = false;
	// 以挂起的方式创建解码线程
	mDecThread = HPR_Thread_Create(AudioDecodeThread, this, 0, true, 0);
	if (NULL == mDecThread)
	{
		mDecThreadExit = true;
		StopDecode();
		return false;
	}

	LOGD("AudioCodec->StartDecode() success");
    return true;
}

/** @fn StopEncode
 *  @brief 停止解码
 *  @param 无
 *  @return true - 成功
 *          false - 失败
 */
bool AudioCodec::StopDecode()
{
	LOGD("AudioCodec->StopDecode()\n");
    mDecThreadExit = true;
	if (NULL != mDecThread)
	{
		HPR_Thread_Wait(mDecThread);
		mDecThread = HPR_INVALID_HANDLE;
	}

	FiniDecode();

	LOGD("AudioCodec->StopDecode() success\n");
    return true;
}

// 正在编码
bool AudioCodec::isDecoding() {
	return mDecoding;
}

/** @fn InputPCMData
 *  @brief 输入PCM数据
 *  @param pData - 输入的数据(IN)
 *  @param nDataSize - 输入的数据长度(IN)
 *  @return true - 成功
 *          false - 失败
 */
bool AudioCodec::InputPCMData(const char* pData, HPR_INT32 nDataSize)
{
	//LOGD("AudioCodec->InputPCMData() data size:%d\n", nDataSize);
    // 不在编码,直接返回
    if (!mEncoding)
    {
        LOGD("AudioCodec->InputPCMData() not Encoding\n");
        return false;
    }

    if (pData == NULL || nDataSize <= 0)
    {
        LOGD("AudioCodec->InputPCMData() param error\n");
        return false;
    }

    if (mEncPCMBuf == NULL)
    {
        LOGD("AudioCodec->InputPCMData() buffer null\n");
        return false;
    }

    // 消回音处理
    HPR_INT32 ret;

    // 向G7XX缓冲区写数据  test
    ret = mEncPCMBuf->WriteDataToBuffer((unsigned char*)pData, nDataSize);

    /*if (nDataSize > mDenoiseParam.denoiseProcessParam.in_buf_size)
    {
        // 向G7XX缓冲区写数据
        ret = mEncPCMBuf->WriteDataToBuffer((unsigned char*)pData, nDataSize);
    }
    else
    {
    	memset(mDenoiseParam.denoiseProcessParam.in_buf, 0, mDenoiseParam.denoiseProcessParam.in_buf_size);
    	memcpy(mDenoiseParam.denoiseProcessParam.in_buf, pData, nDataSize);
    	int iRet = HIK_DENOISE_Proc(mDenoiseHandle, &mDenoiseParam.denoiseProcessParam);
    	if (iRet != HIK_AUDIOCODEC_LIB_S_OK)
    	{
            LOGD("AudioCodec AudioCodec::InputPCMData->HIK_DENOISE_Proc is failed, err:0x%x\n", iRet);
            return false;
    	}

        // 向G7XX缓冲区写数据
        ret = mEncPCMBuf->WriteDataToBuffer((unsigned char*)mDenoiseParam.denoiseProcessParam.out_buf, nDataSize);
    }*/

    if (ret > 0)
    {
        return true;
    }
    else
    {
        LOGD("AudioCodec->InputPCMData() WriteDataToBuffer is failed\n");
        return false;
    }
}

/** @fn InputG7XXData
 *  @brief 输入G7XX数据
 *  @param pData - 输入的数据(IN)
 *  @param nDataSize - 输入的数据长度(IN)
 *  @return true - 成功
 *          false - 失败
 */
bool AudioCodec::InputXXData(const char* pData, HPR_INT32 nDataSize)
{
	//LOGD("AudioCodec->InputG7XXData data size:%d\n", nDataSize);

    if (!mDecoding)
    {
        LOGD("AudioCodec->InputXXData\n");
        return false;
    }

    if (pData == NULL || nDataSize <= 0 || nDataSize > TMP_BUF_SIZE)
    {
        LOGD("AudioCodec->InputXXData pData == NULL\n");
        return false;
    }

    if (mDecXXBuf == NULL)
    {
        LOGD("AudioCodec->InputXXData data size:%d\n", nDataSize);
        return false;
    }

    mDecXXBuf->WriteDataToBuffer((unsigned char*)pData, nDataSize);
    return true;
}

int AudioCodec::GetLastError() {
	return mLastError;
}

// 初始化编解码库
bool AudioCodec::InitEncode() {
	bool ret = false;
	 if(mEncType == TYPE_G711_U){
		ret = InitG711Encode();
	}
	else if(mEncType == TYPE_G711_A) {
		ret = InitG711Encode();
	}
	else if(mEncType == TYPE_G722) {
		ret = InitG722Encode();
	}
	else if(mEncType == TYPE_G726) {
		ret = InitG726Encode();
	}
	else {
		mLastError = AUDIOCODEC_ERR_NOT_SUPPORT_ENCODE;
		ret = false;
	}

	return ret;
}

   // 初始化解码库
bool AudioCodec::InitDecode() {
	bool ret = false;

	if(mDecType == TYPE_G711_U) {
		ret = InitG711Decode();
	}
	else if(mDecType == TYPE_G711_A) {
		ret = InitG711Decode();
	}
	else if(mDecType == TYPE_G722) {
		ret = InitG722Decode();
	}
	else if(mDecType == TYPE_G726) {
		ret = InitG726Decode();
	}
	else {
		mLastError = AUDIOCODEC_ERR_NOT_SUPPORT_DECODE;
		ret = false;
	}

	return ret;
}

bool AudioCodec::InitG711Encode() {
	// G711采样率固定为8000
	mEncParam.sample_rate = mEncParams.sampleRate;
	mEncParam.num_channels = mEncParams.channels;
	mEncParam.bitrate = mEncParams.bitRate;

	LOGD("AudioCodec->InitG711Encode() mEncParam.sample_rate:%d\n", mEncParam.sample_rate);
	LOGD("AudioCodec->InitG711Encode() mEncParam.num_channels:%d\n", mEncParam.num_channels);
	LOGD("AudioCodec->InitG711Encode() mEncParam.bitrate:%d \n", mEncParam.bitrate);

	// 数据块处理参数(mG711CodecProParam)初始化
	HRESULT hr =  HIK_G711ENC_GetInfoParam(&mEncInfo);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		LOGD("AudioCodec->InitCodec HIK_G711ENC_GetInfoParam failed hr：%d\n", hr);
		FiniEncode();
		return false;
	}

	// 初始化G711编码库
	hr = HIK_G711ENC_GetMemSize(&(mEncParam), mEncMemTab);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		FiniEncode();
		LOGD("AudioCodec->InitCodec() HIK_G711ENC_GetMemSize is failed hr：%d\n", hr);
		return false;
	}

	if(mEncMemTab[0].size > 0 && mEncMemTab[0].alignment > 0) {
		// 分配内存
		mEncMemTab[0].base = (unsigned char *)_aligned_malloc(mEncMemTab[0].size, mEncMemTab[0].alignment);
		if (NULL == mEncMemTab[0].base)
		{
			FiniEncode();
			LOGD("AudioCodec->InitCodec() g711EncMemTab base malloc is failed\n");
			return false;
		}
	}

	// 创建G711编码库
	hr = HIK_G711ENC_Create(&mEncParam, mEncMemTab, &mEncHandle);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		FiniEncode();
		LOGD("AudioCodec->InitCodec() HIK_G711ENC_Create is failed hr：%d\n", hr);
		return false;
	}

	LOGD("AudioCodec->InitCodec() *********************");
	LOGD("AudioCodec->InitCodec() InitCodec g711 encode mem_Tab size:%d\n", mEncMemTab[0].size);
	LOGD("AudioCodec->InitCodec()  g711 encode mem_Tab alignment:%d\n", mEncMemTab[0].alignment);
	LOGD("AudioCodec->InitCodec()  g711 Info frame size:%d\n", mEncInfo.in_frame_size);
	LOGD("AudioCodec->InitCodec() *********************");
	return true;
}

bool AudioCodec::InitG711Decode() {
	// G711 最少码率为64Kbps, 可以不填
	//mDecParam.gdec_bitrate = 64000;

	// 获取内存大小
	HRESULT hr = HIK_G711DEC_GetMemSize(&mDecParam, mDecMemTab);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		LOGD("AudioCodec->InitG711Decode() HIK_G711DEC_GetMemSize is failed hr：%d\n", hr);
		return false;
	}

	LOGD("AudioCodec->InitG711Decode() *********************");
	LOGD("AudioCodec->InitG711Decode() g711 decode mem_Tab size:%d\n", mDecMemTab[0].size);
	LOGD("AudioCodec->InitG711Decode() g711 decode mem_Tab alignment:%d\n", mDecMemTab[0].alignment);
	LOGD("AudioCodec->InitG711Decode() *********************");

	// 分配内存
	mDecMemTab[0].base = (unsigned char *)_aligned_malloc(mDecMemTab[0].size, mDecMemTab[0].alignment);
	if (NULL == mDecMemTab[0].base)
	{
		FiniDecode();
		LOGD("AudioCodec->InitG711Decode() aligned_malloc is failed \n");
		return false;
	}

	// 创建G711解码库
	hr = HIK_G711DEC_Create(&mDecParam, mDecMemTab, &mDecHandle);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		FiniDecode();
		LOGD("AudioCodec->InitG711Decode() HIK_G711DEC_Create is failed hr：%d\n", hr);
		return false;
	}
	return true;
}

bool AudioCodec::InitG722Encode() {
	mEncParam.sample_rate = mEncParams.sampleRate;
	mEncParam.num_channels = mEncParams.channels;
	mEncParam.bitrate = mEncParams.bitRate;

	LOGD("AudioCodec->InitG722Encode() mEncParam.sample_rate:%d\n", mEncParam.sample_rate);
	LOGD("AudioCodec->InitG722Encode() mEncParam.num_channels:%d\n", mEncParam.num_channels);
	LOGD("AudioCodec->InitG722Encode() mEncParam.bitrate:%d \n", mEncParam.bitrate);

	HRESULT hr =  HIK_G722ENC_GetInfoParam(&mEncInfo);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr) {
		LOGD("AudioCodec->InitCodec() HIK_G722ENC_Create is failed hr：%d\n", hr);
		return false;
	}

	// 初始化G722编码库
	hr = HIK_G722ENC_GetMemSize(&(mEncParam), mEncMemTab);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		FiniEncode();
		LOGD("AudioCodec->InitCodec() G722ENC_GetMemSize is failed \n");
		return false;
	}

	// 分配内存
	mEncMemTab[0].base = (unsigned char *)_aligned_malloc(mEncMemTab[0].size, mEncMemTab[0].alignment);
	if (NULL == mEncMemTab[0].base)
	{
		FiniEncode();
		LOGD("AudioCodec->InitCodec() malloc is failed \n");
		return false;
	}

	// 创建G722编码库
	hr = HIK_G722ENC_Create(&mEncParam, mEncMemTab, &mEncHandle);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		FiniEncode();
		LOGD("AudioCodec->InitCodec() G722ENC_Create is failed \n");
		return false;
	}

	LOGD("AudioCodec->InitCodec() *********************");
	LOGD("AudioCodec->InitCodec() g722encode mem_Tab size:%d\n", mEncMemTab[0].size);
	LOGD("AudioCodec->InitCodec() g722 encode mem_Tab alignment:%d\n", mEncMemTab[0].alignment);
	LOGD("AudioCodec->InitCodec() g722 Info frame size:%d\n", mEncInfo.in_frame_size);
	LOGD("AudioCodec->InitCodec() *********************");
	return true;
}

bool AudioCodec::InitG722Decode() {
	mDecParam.gdec_bitrate = mDecParams.bitRate;

	// 获取内存大小
	HRESULT hr = HIK_G722DEC_GetMemSize(&mDecParam, mDecMemTab);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		LOGD("AudioCodec->InitG722Decode() G722DEC_GetMemSize is failed \n");
		return false;
	}

	LOGD("AudioCodec->InitG722Decode() *********************");
	LOGD("AudioCodec->InitG722Decode() g722 decode mem_Tab size:%d\n", mDecMemTab[0].size);
	LOGD("AudioCodec->InitG722Decode() g722 decode mem_Tab alignment:%d\n", mDecMemTab[0].alignment);
	LOGD("AudioCodec->InitG722Decode() *********************");

	// 分配内存
	mDecMemTab[0].base = (unsigned char *)_aligned_malloc(mDecMemTab[0].size, mDecMemTab[0].alignment);
	if (NULL == mDecMemTab[0].base)
	{
		FiniDecode();
		LOGD("AudioCodec->InitG722Decode() aligned_malloc is failed \n");
		return false;
	}

	// 创建G722解码库
	hr = HIK_G722DEC_Create(&mDecParam, mDecMemTab, &mDecHandle);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		FiniDecode();
		LOGD("AudioCodec->InitG722Decode() G722DEC_Create is failed \n");
		return false;
	}
	return true;
}

bool AudioCodec::InitG726Encode() {
	mEncParam.sample_rate = mEncParams.sampleRate;
	mEncParam.num_channels = mEncParams.channels;
	mEncParam.bitrate = mEncParams.bitRate;

	LOGD("AudioCodec->InitG726Encode() mEncParam.sample_rate:%d\n", mEncParam.sample_rate);
	LOGD("AudioCodec->InitG726Encode() mEncParam.num_channels:%d\n", mEncParam.num_channels);
	LOGD("AudioCodec->InitG726Encode() mEncParam.bitrate:%d \n", mEncParam.bitrate);

	HRESULT hr =  HIK_G726ENC_GetInfoParam(&mEncInfo);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr) {
		LOGD("AudioCodec->InitG726Encode() HIK_G722ENC_Create is failed hr：%d\n", hr);
		return false;
	}

	// 初始化G722编码库
	hr = HIK_G726ENC_GetMemSize(&(mEncParam), mEncMemTab);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		FiniEncode();
		LOGD("AudioCodec->InitG726Encode() G722ENC_GetMemSize is failed \n");
		return false;
	}

	// 分配内存
	mEncMemTab[0].base = (unsigned char *)_aligned_malloc(mEncMemTab[0].size, mEncMemTab[0].alignment);
	if (NULL == mEncMemTab[0].base)
	{
		FiniEncode();
		LOGD("AudioCodec->InitG726Encode() malloc is failed \n");
		return false;
	}

	// 创建G722编码库
	hr = HIK_G726ENC_Create(&mEncParam, mEncMemTab, &mEncHandle);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		FiniEncode();
		LOGD("AudioCodec->InitG726Encode() HIK_G726ENC_Create is failed \n");
		return false;
	}

	LOGD("AudioCodec->InitG726Encode() *********************");
	LOGD("AudioCodec->InitG726Encode() g726encode mem_Tab size:%d\n", mEncMemTab[0].size);
	LOGD("AudioCodec->InitG726Encode() g726 encode mem_Tab alignment:%d\n", mEncMemTab[0].alignment);
	LOGD("AudioCodec->InitG726Encode() g726 Info frame size:%d\n", mEncInfo.in_frame_size);
	LOGD("AudioCodec->InitG726Encode() *********************");
	return true;
}

bool AudioCodec::InitG726Decode() {
	mDecParam.gdec_bitrate = mDecParams.bitRate;

	// 获取内存大小
	HRESULT hr = HIK_G726DEC_GetMemSize(&mDecParam, mDecMemTab);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		LOGD("AudioCodec->InitG726Decode() HIK_G726DEC_GetMemSize is failed \n");
		return false;
	}

	LOGD("AudioCodec->InitG726Decode() *********************");
	LOGD("AudioCodec->InitG726Decode() g726 decode mem_Tab size:%d\n", mDecMemTab[0].size);
	LOGD("AudioCodec->InitG726Decode() g726 decode mem_Tab alignment:%d\n", mDecMemTab[0].alignment);
	LOGD("AudioCodec->InitG726Decode() *********************");

	// 分配内存
	mDecMemTab[0].base = (unsigned char *)_aligned_malloc(mDecMemTab[0].size, mDecMemTab[0].alignment);
	if (NULL == mDecMemTab[0].base)
	{
		FiniDecode();
		LOGD("AudioCodec->InitG726Decode() aligned_malloc is failed \n");
		return false;
	}

	// 创建G726解码库
	hr = HIK_G726DEC_Create(&mDecParam, mDecMemTab, &mDecHandle);
	if (HIK_AUDIOCODEC_LIB_S_OK != hr)
	{
		FiniDecode();
		LOGD("AudioCodec->InitG726Decode() HIK_G726DEC_Create is failed \n");
		return false;
	}
	return true;
}

/** @fn FiniCodec
 *  @brief 反初始化编解码库
 *  @param 无
 *  @return true - 成功
 *          false - 失败
 */
void AudioCodec::FiniEncode()
{
	LOGD("AudioCodec->FiniEncode()\n");
	// 反初始化编码库内存
	if (mEncMemTab[0].base != NULL)
	{
		LOGD("AudioCodec->FiniEncode()1\n");
		_aligned_free(mEncMemTab[0].base);
		LOGD("AudioCodec->FiniEncode()2\n");
		mEncMemTab[0].base = NULL;
		LOGD("AudioCodec->FiniEncode()3\n");
	}
	LOGD("AudioCodec->FiniEncode() success\n");
}

/** @fn FiniCodec
 *  @brief 反初始化编解码库
 *  @param 无
 *  @return true - 成功
 *          false - 失败
 */
void AudioCodec::FiniDecode()
{
	LOGD("AudioCodec->FiniDecode()\n");

	// 反初始化解码库内存
	if (mDecMemTab[0].base != NULL)
	{
		LOGD("AudioCodec->FiniDecode()1 \n");
		_aligned_free(mDecMemTab[0].base);
		LOGD("AudioCodec->FiniDecode()2\n");
		mDecMemTab[0].base = NULL;
		LOGD("AudioCodec->FiniDecode()3\n");
	}

	LOGD("AudioCodec->FiniDecode() success\n");
}

/** @fn InitDenoise
 *  @brief 初始化回音消除库
 *  @param 无
 *  @return true - 成功
 *          false - 失败
 */
bool AudioCodec::InitDenoise()
{
	 // 初始化回音消除库
	if (mDecType == TYPE_G711_A || mDecType == TYPE_G711_U)
	{
		mDenoiseParam.denoiseParam.sample_rate = 8000;
	}
	else if (mDecType == TYPE_G722)
	{
		mDenoiseParam.denoiseParam.sample_rate = HIK_G722CODEC_IN_SAMPLE_RATE;
	}
	else
	{
		mDenoiseParam.denoiseParam.sample_rate = 8000;
	}
	mDenoiseParam.denoiseParam.frame_size  = HIK_DENOISE_FRAME_SIZE;
	mDenoiseParam.denoiseParam.de_level    = DENOISE_LEVEL1;

    mDenoiseParam.denoiseProcessParam.in_buf      	= new short[HIK_DENOISE_FRAME_SIZE];
    mDenoiseParam.denoiseProcessParam.in_buf_size 	= HIK_DENOISE_FRAME_SIZE;
    mDenoiseParam.denoiseProcessParam.out_buf     	= new short[HIK_DENOISE_FRAME_SIZE];
    mDenoiseParam.denoiseProcessParam.out_buf_size 	= HIK_DENOISE_FRAME_SIZE;

	int iRet = HIK_DENOISE_GetMemSize(&mDenoiseParam.denoiseParam, mDenoiseParam.denoiseMemTab); // 得到回声消除需要分配的内存大小
	if (iRet != HIK_DENOISE_LIB_S_OK)
	{
		LOGD("HIK_DENOISE_Create is failed,err:0X%x", iRet);
		FiniDenoise();
		return false;
	}

    // 分配内存
	mDenoiseParam.denoiseMemTab[0].base = (unsigned char *)_aligned_malloc(mDenoiseParam.denoiseMemTab[0].size, mDenoiseParam.denoiseMemTab[0].alignment);
    if (NULL == mDenoiseParam.denoiseMemTab[0].base)
    {
		LOGD("Memory malloc failed!\n");
		FiniDenoise();
		return false;
    }

	memset(mDenoiseParam.denoiseMemTab[0].base , 0, mDenoiseParam.denoiseMemTab[0].size);
	iRet = HIK_DENOISE_Create(&mDenoiseParam.denoiseParam, mDenoiseParam.denoiseMemTab, &mDenoiseHandle); // 回声消除处理器创建
	if (iRet != HIK_DENOISE_LIB_S_OK)
	{
		LOGD("HIK_DENOISE_Create is failed,err:0X%x", iRet);
		FiniDenoise();
		return false;
	}
	return true;
}

/** @fn FiniCodec
 *  @brief 反初始化回音消除库
 *  @param 无
 *  @return true - 成功
 *          false - 失败
 */
void AudioCodec::FiniDenoise()
{
    if (mDenoiseParam.denoiseMemTab[0].base != NULL)
    {
    	_aligned_free(mDenoiseParam.denoiseMemTab[0].base);
    	mDenoiseParam.denoiseMemTab[0].base = NULL;
    }

    if (mDenoiseParam.denoiseProcessParam.in_buf != NULL)
    {
    	delete mDenoiseParam.denoiseProcessParam.in_buf;
    	mDenoiseParam.denoiseProcessParam.in_buf = NULL;
    }

    if (mDenoiseParam.denoiseProcessParam.out_buf != NULL)
    {
    	delete mDenoiseParam.denoiseProcessParam.out_buf;
    	mDenoiseParam.denoiseProcessParam.out_buf = NULL;
    }
}

/** @fn AudioEncodeThread
 *  @brief 音频编码编码线程
 *  @param pParam - 线程传入参数(IN)
 *  @return
 */

HPR_VOIDPTR AudioCodec::AudioEncodeThread(HPR_VOID* pParam)
{
    AudioCodec* pAudioCodec = (AudioCodec*)pParam;
    if (NULL != pAudioCodec)
    {
        pAudioCodec->Encode();
    }

    return NULL;
}

/** @fn AudioDecodeThread
 *  @brief 音频解码线程
 *  @param pParam - 线程传入参数(IN)
 *  @return
 */
HPR_VOIDPTR AudioCodec::AudioDecodeThread(HPR_VOID* pParam)
{
    AudioCodec* pAudioCodec = (AudioCodec*)pParam;
    if (NULL != pAudioCodec)
    {
        pAudioCodec->Decode();
    }

    return NULL;
}

/** @fn MallocBuffer
 *  @brief 分配缓冲区
 *  @param 无
 *  @return true - 成功
 *          false - 失败
 */
bool AudioCodec::MallocBuffer()
{
    // 分配PCM\G7XX缓冲区
    mEncPCMBuf = new CAudioBuffer();
    mDecXXBuf = new CAudioBuffer();
    if (NULL == mEncPCMBuf || NULL == mDecXXBuf)
    {
        FreeBuffer();
        return false;
    }

    // 缓冲区初始化
    bool ret1 = mEncPCMBuf->Initialize();
    bool ret2 = mDecXXBuf->Initialize();
    if (!ret1 || !ret2)
    {
        FreeBuffer();
        return false;
    }

    return true;
}

/** @fn FreeBuffer
 *  @brief 释放缓冲区
 *  @param 无
 *  @return 无
 */
void AudioCodec::FreeBuffer()
{
    if (NULL != mEncPCMBuf)
    {
        mEncPCMBuf->Release();
        mEncPCMBuf = NULL;
    }

    if (NULL != mDecXXBuf)
    {
        mDecXXBuf->Release();
        mDecXXBuf = NULL;
    }
}

/** @fn G7XXEncode
 *  @brief G7XX编码
 *  @param
 *  @return
 */
void AudioCodec::Encode()
{
	if(mEncType == TYPE_G711_A) {
		G711Encode(LAW_A);
	}
	else if(mEncType == TYPE_G711_U) {
		G711Encode(LAW_U);
	}
	else if(mEncType == TYPE_G722) {
		G722Encode();
	}
	else if(mEncType == TYPE_G726) {
		G726Encode();
	}
	else {

	}
}

/** @fn G7XXEncode
 *  @brief G7XX解码
 *  @param
 *  @return
 */
void AudioCodec::Decode()
{
	if(mDecType == TYPE_G711_A) {
		G711Decode(LAW_A);
	}
	else if(mDecType == TYPE_G711_U) {
		G711Decode(LAW_U);
	}
	else if(mDecType == TYPE_G722) {
		G722Decode();
	}
	else if(mDecType == TYPE_G726) {
		G726Decode();
	}
	else {

	}
}

/** @fn G711Encode
 *  @brief G711编码
 *  @param
 *  @return
 */
void AudioCodec::G711Encode(int lowType)
{
    if (NULL == mEncPCMBuf || mEncThread == NULL)
    {
        return;
    }
    
    // 为编码输入输出分配内存
    unsigned char* pPCMBuf = (unsigned char*)malloc(AUDIO_CODEC_BUF_SIZE);
    if (pPCMBuf == NULL)
    {
        return;
    }
    memset(pPCMBuf, 0, AUDIO_CODEC_BUF_SIZE);
    
    unsigned char* pG711Buf = (unsigned char*)malloc(AUDIO_CODEC_BUF_SIZE);
    if (pG711Buf == NULL)
    {
        free(pPCMBuf);
        pPCMBuf = NULL;
        return;
    }
    memset(pG711Buf, 0, AUDIO_CODEC_BUF_SIZE);

    // 标识正在解码
    mEncoding = true;
    
    while (!mEncThreadExit)
    {
        // 从缓冲区读取数据
        HPR_INT32 nReadSize = mEncPCMBuf->ReadDataFromBuffer(mEncInfo.in_frame_size, pPCMBuf);
        if (nReadSize <= 0)
        {
            // 停留20ms,然后再读
            HPR_Sleep(20);
            continue;
        }
        //LOGD("AudioCodec->G711Encode HIK_G711ENC_Encode in_frame_size:%d\n", nReadSize);

        mEncProParam.in_buf = pPCMBuf;
		mEncProParam.out_buf = pG711Buf;
		//g711编码类型,0 - U law, 1- A law
		mEncProParam.g711_type = lowType;

        // 编码
        HRESULT hr = HIK_G711ENC_Encode(mEncHandle, &mEncProParam);
        if (HIK_AUDIOCODEC_LIB_S_OK == hr)
        {
        	// 以160个长度回调
			int remainder = mEncProParam.out_frame_size % 160;
			int n = mEncProParam.out_frame_size / 160;
			for(int i = 0; i < n;)
			{
				if (pXXXDataCallBack != NULL)
				{
					(*pXXXDataCallBack)((char*)mEncProParam.out_buf + (i*160), 160, pUser);
				}
				i++;
			}

			// 如果不是160的整数倍，怎余数单独回调
			if(remainder > 0)
			{
				if (pXXXDataCallBack != NULL)
				{
					(*pXXXDataCallBack)((char*)mEncProParam.out_buf + (n*160), remainder, pUser);
				}
			}
        }
        else
        {
            LOGD("AudioCodec->G711Encode HIK_G711ENC_Encode is failed.Err:%d\n", hr);
        }
        
        // 停留10ms
        HPR_Sleep(10);
    }
    
    free(pPCMBuf);
    free(pG711Buf);
    pPCMBuf = NULL;
    pG711Buf = NULL;
    
    // 标识正在编码
    mEncoding = false;
    
    LOGD("AudioCodec->AudioEncode thread stop");
}

/** @fn G711Decode
 *  @brief G711解码
 *  @param
 *  @return
 */
void AudioCodec::G711Decode(int lowType)
{
    if (NULL == mDecXXBuf || mDecThread == NULL)
    {
        //LOGD("AudioCodec->AudioCodec::G711Decode 444444");
        return;
    }

    // 为解码输入输出分配内存，适合为PER_G711_DATA_SIZE*2 = 320
    unsigned char* pPCMBuf = (unsigned char*)malloc(AUDIO_CODEC_BUF_SIZE);
    if (pPCMBuf == NULL)
    {
        LOGD("AudioCodec->G711Decode pPCMBuf == NULL");
        return;
    }
    memset(pPCMBuf, 0, AUDIO_CODEC_BUF_SIZE);

    unsigned char* pG711Buf = (unsigned char*)malloc(AUDIO_CODEC_BUF_SIZE);
    if (pG711Buf == NULL)
    {
        LOGD("AudioCodec->G722Decode pG711Buf == NULL");
        free(pPCMBuf);
        return;
    }
    memset(pG711Buf, 0, AUDIO_CODEC_BUF_SIZE);

    // 数据块处理参数(mG711CodecProParam)初始化
    mDecProParam.in_buf = pG711Buf;
    mDecProParam.in_data_size = 0;
    mDecProParam.out_buf = pPCMBuf;
    // 0 - u lay
    mDecProParam.g711_type = lowType;

    // 消耗的数据长度
    int proc_data_size = AUDIO_DEC_DATA_SIZE;

    // 标识正在解码
    mDecoding = true;

    while (!mDecThreadExit)
    {
        // 从缓冲区读取数据
        HPR_INT32 nReadSize = mDecXXBuf->ReadDataFromBuffer(proc_data_size, mDecProParam.in_buf + mDecProParam.in_data_size);
        if (nReadSize <= 0)
        {
            // 停留20ms,然后再读
            HPR_Sleep(20);
            continue;
        }

        // 解码
        //LOGD("AudioCodec->G711Decode HIK_G711DEC_Decode before in_data_size:%d\n", mDecProParam.in_data_size);
        mDecProParam.in_data_size += nReadSize;
        //LOGD("AudioCodec->G711Decode HIK_G711DEC_Decode after in_data_size:%d\n", mDecProParam.in_data_size);
        HRESULT hr = HIK_G711DEC_Decode(mDecHandle, &mDecProParam);
        if (HIK_AUDIOCODEC_LIB_S_OK == hr)
        {
        	// 回调解码出来的PCM数据
			if (pPCMDataCallBack != NULL)
			{
				(*pPCMDataCallBack)((char*)mDecProParam.out_buf, mDecProParam.out_frame_size, pUser);
			}

            // 消耗的数据长度
            proc_data_size = mDecProParam.proc_data_size;
            //LOGD("AudioCodec->G711Decode HIK_G711DEC_Decode proc_data_size:%d\n", proc_data_size);
			// 计算剩余的G711数据
            mDecProParam.in_data_size -= proc_data_size;
			// 移动数据
			memmove(mDecProParam.in_buf, mDecProParam.in_buf+proc_data_size, mDecProParam.in_data_size);
        }
        else
		{
			LOGD("AudioCodec->G711Decode HIK_G711DEC_Decode is failed.Err:%d\n", hr);
		}

        HPR_Sleep(5);
    }

    free(pG711Buf);
    free(pPCMBuf);
    pG711Buf = NULL;
    pPCMBuf = NULL;

    // 标识正在解码
    mDecoding = false;

    LOGD("AudioCodec->G711Decode thread stop");
}


/** @fn G722Encode
 *  @brief G722编码
 *  @param
 *  @return
 */
void AudioCodec::G722Encode()
{
    if (NULL == mEncPCMBuf || mEncThread == NULL)
    {
        return;
    }

    // 为编码输入输出分配内存
    unsigned char* pPCMBuf = (unsigned char*)malloc(mEncInfo.in_frame_size);
    if (pPCMBuf == NULL)
    {
        return;
    }
    memset(pPCMBuf, 0, mEncInfo.in_frame_size);

    unsigned char* pG722Buf = (unsigned char*)malloc(mEncInfo.in_frame_size);
    if (pG722Buf == NULL)
    {
        free(pPCMBuf);
        pPCMBuf = NULL;
        return;
    }
    memset(pG722Buf, 0, mEncInfo.in_frame_size);

    // 数据块处理参数(mG722CodecProParam)初始化
    mEncProParam.in_buf = pPCMBuf;
    mEncProParam.out_buf = pG722Buf;
    mEncProParam.g711_type = LAW_U;

    // 标识正在解码
    mEncoding = true;

    while (!mEncThreadExit)
    {
        // 从缓冲区读取数据
        HPR_INT32 nReadSize = mEncPCMBuf->ReadDataFromBuffer(mEncInfo.in_frame_size, pPCMBuf);
        if (nReadSize <= 0)
        {
            // 停留20ms,然后再读
            HPR_Sleep(20);
            continue;
        }

        // 编码
        HRESULT hr = HIK_G722ENC_Encode(mEncHandle, &mEncProParam);
        if (HIK_AUDIOCODEC_LIB_S_OK == hr)
        {
        	// 以160个长度回调
			int remainder = mEncProParam.out_frame_size % 160;
			int n = mEncProParam.out_frame_size / 160;
			for(int i = 0; i < n;)
			{
				if (pXXXDataCallBack != NULL)
				{
					(*pXXXDataCallBack)((char*)mEncProParam.out_buf + (i*160), 160, pUser);
				}
				i++;
			}

			// 如果不是160的整数倍，怎余数单独回调
			if(remainder > 0)
			{
				if (pXXXDataCallBack != NULL)
				{
					(*pXXXDataCallBack)((char*)mEncProParam.out_buf + (n*160), remainder, pUser);
				}
			}
        }
        else
		{
			LOGD("AudioCodec->G722Encode HIK_G722ENC_Encode is failed.Err:%d\n", hr);
		}

        // 停留10ms
        HPR_Sleep(10);
    }

    free(pPCMBuf);
    free(pG722Buf);
    pPCMBuf = NULL;
    pG722Buf = NULL;

    // 标识正在编码
    mEncoding = false;
    LOGD("AudioCodec->AudioEncode thread stop");
}

/** @fn G722Decode
 *  @brief G722解码
 *  @param
 *  @return
 */
void AudioCodec::G722Decode()
{
    if (NULL == mDecXXBuf || mDecThread == NULL)
    {
        //LOGD("AudioCodec->G722Decode 444444");
        return;
    }

    // 为解码输入输出分配内存
    unsigned char* pPCMBuf = (unsigned char*)malloc(AUDIO_CODEC_BUF_SIZE);
    if (pPCMBuf == NULL)
    {
        LOGD("AudioCodec->G722Decode() pPCMBuf == NULL");
        return;
    }
    memset(pPCMBuf, 0, AUDIO_CODEC_BUF_SIZE);

    unsigned char* pG7XXBuf = (unsigned char*)malloc(AUDIO_CODEC_BUF_SIZE);
    if (pG7XXBuf == NULL)
    {
        LOGD("AudioCodec->G722Decode() G722Decode pG7XXBuf == NULL");
        free(pPCMBuf);
        return;
    }
    memset(pG7XXBuf, 0, AUDIO_CODEC_BUF_SIZE);

    // 数据块处理参数(mG722CodecProParam)初始化
    //mDecProParam.in_data_size = PER_G722_DATA_SIZE;//HIK_G722CODEC_IN_SAMPLE_RATE/25/8 = 80;
    mDecProParam.in_buf = pPCMBuf;
    mDecProParam.in_data_size = 0;
    mDecProParam.out_buf = pG7XXBuf;

    // 标识正在解码
    mDecoding = true;

    // 消耗的G7xx数据
    int proc_data_size = AUDIO_DEC_DATA_SIZE;

    while (!mDecThreadExit)
    {
    	// 从缓冲区读取数据
		HPR_INT32 nReadSize = mDecXXBuf->ReadDataFromBuffer(proc_data_size, mDecProParam.in_buf+ mDecProParam.in_data_size);
		if(nReadSize <= 0)
		{
			// 停留20ms,然后再读
			HPR_Sleep(20);
			continue;
		}

		mDecProParam.in_data_size += nReadSize;
    	// 如果解析失败，标识in_buf数据为空
    	HRESULT hr = HIK_G722DEC_Decode(mDecHandle, &mDecProParam);
		if (HIK_AUDIOCODEC_LIB_S_OK == hr)
		{
			// 回调解码出来的PCM数据
			if (pPCMDataCallBack != NULL)
			{
				(*pPCMDataCallBack)((char*)mDecProParam.out_buf, mDecProParam.out_frame_size, pUser);
			}

			proc_data_size = mDecProParam.proc_data_size;

            // 计算剩余的G711数据
            mDecProParam.in_data_size -= proc_data_size;
            // 移动数据
            memmove(mDecProParam.in_buf, mDecProParam.in_buf+proc_data_size, mDecProParam.in_data_size);
		}
		else
		{
			LOGD("AudioCodec->G722Decode() HIK_G722DEC_Decode is failed.Err:%d\n", hr);
		}

        // 停留5ms
        HPR_Sleep(5);
    }

    free(pG7XXBuf);
    free(pPCMBuf);
    pG7XXBuf = NULL;
    pPCMBuf = NULL;

    // 标识正在解码
    mDecoding = false;

    LOGD("AudioCodec->G722Decode() thread stop");
}

// G726编码
void AudioCodec::G726Encode() {
	if (NULL == mEncPCMBuf || mEncThread == NULL)
	{
		return;
	}

	// 为编码输入输出分配内存
	unsigned char* pPCMBuf = (unsigned char*)malloc(mEncInfo.in_frame_size);
	if (pPCMBuf == NULL)
	{
		return;
	}
	memset(pPCMBuf, 0, mEncInfo.in_frame_size);

	unsigned char* pG726Buf = (unsigned char*)malloc(mEncInfo.in_frame_size);
	if (pG726Buf == NULL)
	{
		free(pPCMBuf);
		pPCMBuf = NULL;
		return;
	}
	memset(pG726Buf, 0, mEncInfo.in_frame_size);

	// 数据块处理参数(mG726CodecProParam)初始化
	mEncProParam.in_buf = pPCMBuf;
	mEncProParam.out_buf = pG726Buf;

	// 标识正在解码
	mEncoding = true;

	while (!mEncThreadExit)
	{
		// 从缓冲区读取数据
		HPR_INT32 nReadSize = mEncPCMBuf->ReadDataFromBuffer(mEncInfo.in_frame_size, pPCMBuf);
		if (nReadSize <= 0)
		{
			// 停留20ms,然后再读
			HPR_Sleep(20);
			continue;
		}

		// 编码
		HRESULT hr = HIK_G726ENC_Encode(mEncHandle, &mEncProParam);
		if (HIK_AUDIOCODEC_LIB_S_OK == hr)
		{
			// 以160个长度回调
			int remainder = mEncProParam.out_frame_size % 160;
			int n = mEncProParam.out_frame_size / 160;
			for(int i = 0; i < n;)
			{
				if (pXXXDataCallBack != NULL)
				{
					(*pXXXDataCallBack)((char*)mEncProParam.out_buf + (i*160), 160, pUser);
				}
				i++;
			}

			// 如果不是160的整数倍，怎余数单独回调
			if(remainder > 0)
			{
				if (pXXXDataCallBack != NULL)
				{
					(*pXXXDataCallBack)((char*)mEncProParam.out_buf + (n*160), remainder, pUser);
				}
			}
		}
		else
		{
			LOGD("AudioCodec->G726Encode HIK_G726ENC_Encode is failed.Err:%d\n", hr);
		}

		// 停留10ms
		HPR_Sleep(10);
	}

	free(pPCMBuf);
	free(pG726Buf);
	pPCMBuf = NULL;
	pG726Buf = NULL;

	// 标识正在编码
	mEncoding = false;
	LOGD("AudioCodec->G726Encode thread stop");
}

// G726解码
void AudioCodec::G726Decode() {
	if (NULL == mDecXXBuf || mDecThread == NULL)
	{
		//LOGD("AudioCodec->G726Decode 444444");
		return;
	}

	// 为解码输入输出分配内存
	unsigned char* pPCMBuf = (unsigned char*)malloc(AUDIO_CODEC_BUF_SIZE);
	if (pPCMBuf == NULL)
	{
		LOGD("AudioCodec->G726Decode() pPCMBuf == NULL");
		return;
	}
	memset(pPCMBuf, 0, AUDIO_CODEC_BUF_SIZE);

	unsigned char* pG7XXBuf = (unsigned char*)malloc(AUDIO_CODEC_BUF_SIZE);
	if (pG7XXBuf == NULL)
	{
		LOGD("AudioCodec->G726Decode() pG7XXBuf == NULL");
		free(pPCMBuf);
		return;
	}
	memset(pG7XXBuf, 0, AUDIO_CODEC_BUF_SIZE);

	// 数据块处理参数(mG726CodecProParam)初始化
	//mDecProParam.in_data_size = PER_G726_DATA_SIZE;
	mDecProParam.in_buf = pPCMBuf;
	mDecProParam.in_data_size = 0;
	mDecProParam.out_buf = pG7XXBuf;

	// 标识正在解码
	mDecoding = true;

	// 消耗的G7xx数据
	int proc_data_size = AUDIO_DEC_DATA_SIZE;

	while (!mDecThreadExit)
	{
		// 从缓冲区读取数据
		HPR_INT32 nReadSize = mDecXXBuf->ReadDataFromBuffer(proc_data_size, mDecProParam.in_buf+ mDecProParam.in_data_size);
		if(nReadSize <= 0)
		{
			// 停留20ms,然后再读
			HPR_Sleep(20);
			continue;
		}

		mDecProParam.in_data_size += nReadSize;
		// 如果解析失败，标识in_buf数据为空
		HRESULT hr = HIK_G726DEC_Decode(mDecHandle, &mDecProParam);
		if (HIK_AUDIOCODEC_LIB_S_OK == hr)
		{
			// 回调解码出来的PCM数据
			if (pPCMDataCallBack != NULL)
			{
				(*pPCMDataCallBack)((char*)mDecProParam.out_buf, mDecProParam.out_frame_size, pUser);
			}

			proc_data_size = mDecProParam.proc_data_size;

			// 计算剩余的G726数据
			mDecProParam.in_data_size -= proc_data_size;
			// 移动数据
			memmove(mDecProParam.in_buf, mDecProParam.in_buf+proc_data_size, mDecProParam.in_data_size);
		}
		else
		{
			LOGD("AudioCodec->G726Decode() HIK_G726DEC_Decode is failed.Err:%d\n", hr);
		}

		// 停留5ms
		HPR_Sleep(5);
	}

	free(pG7XXBuf);
	free(pPCMBuf);
	pG7XXBuf = NULL;
	pPCMBuf = NULL;

	// 标识正在解码
	mDecoding = false;

	LOGD("AudioCodec->G726Decode() thread stop");
}

