/*
 * AudioCodecInferface.cpp
 *
 *  Created on: 2014-8-6
 *      Author: weilinfeng
 */

#include "AudioCodecInferface.h"
#include "AudioCodec.h"
#include "MCRPrint.h"

// MediaSDK初始化标识
extern bool gInitLib;
// 编解码对象
AudioCodec * pAudioCodec = NULL;
// 初始化标识
bool bAudioInit = false;
// 登陆标识
bool bStartEncode = false;

bool bStartDecode = false;

pAudioInterfacePCMDataListener pAudioCMDataCallback;

pAudioInterfaceXXXDataListener pAudioXXXDataCallback;

void AudioPCMDataCallback(char* buffer, unsigned int buflen, void* pUser) {
	if(pAudioCMDataCallback != NULL) {
		(*pAudioCMDataCallback)(buffer, buflen, pUser);
	}
}

void AudioXXXDataCallback(char* buffer, unsigned int buflen, void* pUser) {
	if(pAudioXXXDataCallback != NULL) {
		(*pAudioXXXDataCallback)(buffer, buflen, pUser);
	}
}


/** @fn bool Init()
 *   @brief 初始化
 *   @return true - 初始化成功; false - 初始化库失败
 */
AUDIOCODECINFERFACEAPI bool AudioInit(void* pUser) {
	if(!gInitLib) {
		LOGD("AudioCodecInferface->AudioInit() MediaSDK not init\n");
		return false;
	}

	if(bAudioInit) {
		LOGD("AudioCodecInferface->AudioInit() have been init\n");
		return true;
	}

	if(pAudioCodec == NULL) {
		pAudioCodec = new AudioCodec();
	}

	pAudioCodec->Init(AudioPCMDataCallback, AudioXXXDataCallback, pUser);


	bAudioInit = true;

	LOGD("AudioCodecInferface->AudioInit() success\n");
	return true;
}

/** @fn bool Fini()
 *   @brief 反初始化
 *   @return 无
 */
AUDIOCODECINFERFACEAPI void AudioFini() {
	LOGD("AudioCodecInferface->AudioFini()\n");
	if(!gInitLib) {
		LOGD("AudioCodecInferface->AudioFini() MCRSDK not init\n");
		return;
	}

	if(pAudioCodec != NULL) {
		pAudioCodec->Fini();
		delete pAudioCodec;
		pAudioCodec = NULL;
	}

	bAudioInit = false;

	LOGD("AudioCodecInferface->Fini() success\n");
}

/** @fn bool SetPCMDataListener
 *   @brief 设置解码后的音频数据回调
 *   @param [in] pcmDataListener - 数据回调函数指针
 *   @return true - 成功, false - 失败
 */
AUDIOCODECINFERFACEAPI bool SetPCMAudioDataListener(pAudioInterfacePCMDataListener pcmDataListener) {
	if(pcmDataListener == NULL) {
		LOGD("AudioCodecInferface->SetPCMDataListener() pcmDataListener == NULL\n");
		return false;
	}

	pAudioCMDataCallback = pcmDataListener;
	LOGD("AudioCodecInferface->SetPCMDataListener() success\n");
	return true;
}


/** @fn bool SetXXXDataListener
 *   @brief 设置编码后的音频数据回调
 *   @param [in] xxxDataListener - 数据回调函数指针
 *   @return true - 成功, false - 失败
 */
AUDIOCODECINFERFACEAPI bool SetXXXAudioDataListener(pAudioInterfaceXXXDataListener xxxDataListener) {
	if(xxxDataListener == NULL) {
		LOGD("AudioCodecInferface->SetXXXDataListener() xxxDataListener == NULL\n");
		return false;
	}

	pAudioXXXDataCallback = xxxDataListener;
	LOGD("AudioCodecInferface->SetXXXDataListener() success\n");
	return true;
}

/** @fn bool StartEncode
 *   @brief 开始编码
 *   @param [in] iEncodeType - 编码类型
 *   @param [in] encodeParams - 编码参数
 *   @param [out] 无
 *   @return true 表示成功， false 表示失败
 */
AUDIOCODECINFERFACEAPI bool StartAudioEncode(int iEncodeType, EncodeParams& encodeParams) {
	if(!bAudioInit) {
		LOGD("AudioCodecInferface->StartAudioEncode() not init\n");
		return false;
	}

	if(bStartEncode) {
		LOGD("AudioCodecInferface->StartAudioEncode() be StartEncode\n");
		return true;
	}

	bool bRet = pAudioCodec->StartEncode(iEncodeType, encodeParams);
	if(!bRet) {
		LOGD("AudioCodecInferface->StartAudioEncode() StartEncode fail\n");
		return false;
	}

	bStartEncode = true;
	LOGD("AudioCodecInferface->StartAudioEncode() success\n");
	return true;
}

/** @fn bool isEncoding
 *   @brief 是否正在编码
 *   @return  true - 是; false - 否
 */
AUDIOCODECINFERFACEAPI bool isEncoding() {
	if(!bAudioInit) {
		LOGD("AudioCodecInferface->isEncoding() not init\n");
		return false;
	}

	return pAudioCodec->isEncoding();
}

/** @fn bool InputAudioData
 *   @brief 塞入音频数据
 *   @param [in] InputAudioData - PCM音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
AUDIOCODECINFERFACEAPI bool InputPCMAudioData(const char* pData, unsigned int nDataSize) {
	if(!bAudioInit) {
		LOGD("AudioCodecInferface->InputPCMData() not init\n");
		return false;
	}

	if(!bStartEncode) {
		LOGD("AudioCodecInferface->InputPCMData() not bStartEncode\n");
		return false;
	}

	LOGD("AudioCodecInferface->InputPCMData() nDataSize:%d\n", nDataSize);
	return pAudioCodec->InputPCMData(pData, nDataSize);
}

/** @fn bool StopEncode
 *   @brief 停止编码
 *   @return 无
 */
AUDIOCODECINFERFACEAPI void StopAudioEncode() {
	if(!bAudioInit) {
		LOGD("AudioCodecInferface->StopAudioEncode() not init\n");
		return;
	}

	pAudioCodec->StopEncode();

	bStartEncode = false;
	LOGD("AudioCodecInferface->StopAudioEncode() success\n");
}

/** @fn bool StartDecode
 *   @brief 开始解码
 *   @param [in] iDecodeType - 解码类型
 *   @param [in] decodeParams - 解码参数
 *   @param [out] 无
 *   @return true 表示连接成功， false 表示连接失败
 */
AUDIOCODECINFERFACEAPI bool StartAudioDecode(int iDecodeType, DecodeParams& decodeParams) {
	if(!bAudioInit) {
		LOGD("AudioCodecInferface->StartAudioDecode() not init\n");
		return false;
	}

	if(bStartDecode) {
		LOGD("AudioCodecInferface->StartAudioDecode() be StartDecode\n");
		return true;
	}

	bool bRet = pAudioCodec->StartDecode(iDecodeType, decodeParams);
	if(!bRet) {
		LOGD("AudioCodecInferface->StartAudioDecode() StartDecode fail\n");
		return false;
	}

	bStartDecode = true;
	LOGD("AudioCodecInferface->StartAudioDecode() success iDecodeType:%d\n", iDecodeType);
	return true;
}

/** @fn bool isDecoding
 *   @brief 是否正在解码
 *   @return  true - 是; false - 否
 */
AUDIOCODECINFERFACEAPI bool isDecoding() {
	if(!bAudioInit) {
		LOGD("AudioCodecInferface->isEncoding() not init\n");
		return false;
	}

	return pAudioCodec->isDecoding();
}

/** @fn bool InputXXXData
 *   @brief 塞入音频数据
 *   @param [in] InputAudioData - XXX音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
AUDIOCODECINFERFACEAPI bool InputXXXAudioData(const char* pData, unsigned int nDataSize) {
	if(!bAudioInit) {
		LOGD("AudioCodecInferface->InputXXXData() not init\n");
		return false;
	}

	if(!bStartDecode) {
		LOGD("AudioCodecInferface->InputXXXData() not bStartEncode\n");
		return false;
	}

	//LOGD("AudioCodecInferface->InputXXXData() nDataSize:%d\n", nDataSize);
	return pAudioCodec->InputXXData(pData, nDataSize);
}

/** @fn bool StopDecode()
 *   @brief 停止解码
 *   @return 无
 */
AUDIOCODECINFERFACEAPI void StopAudioDecode() {
	if(!bAudioInit) {
		LOGD("AudioCodecInferface->StopAidioDecode() not init\n");
		return;
	}

	pAudioCodec->StopDecode();

	bStartDecode = false;
	LOGD("AudioCodecInferface->StopAidioDecode() success\n");
}

/** @fn bool GetLastError()
 *   @brief 停止解码
 *   @return 无
 */
AUDIOCODECINFERFACEAPI int GetAidioLastError() {
	if(!bAudioInit) {
		LOGD("AudioCodecInferface->StopAidioDecode() not init\n");
		return AUDIOCODEC_ERR_SUCCESS;
	}

	return pAudioCodec->GetLastError();
}
