/*
 * MCRSDK.cpp
 *
 *  Created on: 2014-3-17
 *      Author: weilinfeng
 */
#include "MCRSDK.h"
#include "MCRPrint.h"
#include "HPR_Hpr.h"

// 初始化标识
bool gInitLib = false;
int glogflag = 0;
const char* glogpath = NULL;

//捕获IOS管道破裂的异常
#ifdef __linux__ //send error:SIGPIPE
#include <signal.h>
void handle_pipe(int sig);
void handle_pipe(int sig)
{
    // do nothing
}
#endif

/** @fn bool InitLib()
 *   @brief 初始化所有SDK共有部分
 *   @param [in] 无
 *   @param [out] 无
 *   @return true - 初始化成功; false - 初始化库失败
 */
MCRSDKAPI bool InitLib()
{
    if(gInitLib)
    {
        LOGD("MCRSDK->InitLib() have inited lib\n");
        return true;
    }

    if (HPR_Init() == -1)
    {
        LOGD("MCRSDK->InitLib() hpr init fail\n");
        return false;
    }

    gInitLib = true;

    //捕获IOS管道破裂的异常
	#ifdef __linux__
	#ifdef  __APPLE__
		signal(SIGPIPE, SIG_IGN);
	#else
		struct sigaction action;
		action.sa_handler = handle_pipe;
		sigemptyset(&action.sa_mask);
		action.sa_flags = 0;
		sigaction(SIGPIPE, &action, NULL);
	#endif
	#endif

    return true;
}

/** @fn bool RtspClientFiniLib()
 *   @brief 释放RTSP库
 *   @param [in] 无
 *   @param [out] 无
 *   @return 无
 */
MCRSDKAPI bool FiniLib()
{
    if (!gInitLib)
    {
        LOGD("MCRSDK->InitLib() not init lib\n");
        return true;
    }

    HPR_Fini();

    gInitLib = false;

    return true;
}

/** @fn bool SetPrint()
 *   @brief 设置日志 打印
 *   @param [in] iflag无 0-关闭日志，1-打印日志，2-将日志写到logPath的文件中
 *   @param [out] logPath- 日志文件路径
 *   @return 无
 */
MCRSDKAPI bool SetPrint(int iflag, const char* logPath)
{
	glogflag = iflag;
	glogpath = logPath;
	return true;
}


