/*
 * PushManager.h
 *
 *  Created on: 2014-3-4
 *      Author: weilinfeng
 */

#ifndef PUSHMANAGER_H_
#define PUSHMANAGER_H_

#include "PushClientSDK.h"
#include "PushDefine.h"
#include "HPR_Socket.h"
#include "HPR_Thread.h"
#include "HPR_Mutex.h"

#define SESSION_SIZE 256
#define PUSH_BUF_SIZE 1024*500
#define CONNECT_TIMEOUT 10*1000
#define RECV_TIMEOUT 2

class CPushManager {
public:
	// 构造函数
	CPushManager();
	// 析构函数
	~CPushManager();
	// 建立长连接
	int Connect(const char *IP, int port, const char* sessionId, void* pUser);
	// 断开长连接
	void DisConnect();
	// 发送心跳
	bool SendHeartBeat();
	// 透明通道发送的数据
	// bool InputData(PLucency pLucency);
	// 设置数据包回调函数
	void SetPacketLister(pPacketLister packetLister);
	// 设置连接回调函数
	void SetConnectionLister(pConnectionLister connectionLister);
public:
    // 数据接收线程退出标识
    bool m_bQuit;
private:
	// 初始化
    void Init();
	// 饭初始化
	void Deinit();
    // 接收Push数据
    bool RecvPushData();
    // 生成心跳包
    int GenerateHeartBeatPacket(char* buf);
    // 发送心跳包
    bool SendPacket(char* buf , int len);
    // 开启Push数据交互
    bool StartPushTrans();
    // 停止Push数据交互
    void StopPushTrans();
    // 连接服务器
    bool ConnectServer();
    bool ConnectServer(const char *ip, int port);
    // 断开服务器
    void DisConnectServer();
    // 处理推送数据
    void ProcPushData(char* buf, unsigned int readlen);
    // 回调数据
    void CallDataBack(char* buf, int packlen);
    // 判断push头
    bool IsHeadWithPush(char const *src);
	// Push Data接收线程
    static HPR_VOIDPTR CALLBACK RecvPushDataProc(HPR_VOIDPTR pParam);
    // Push Data发送线程
    static HPR_VOIDPTR CALLBACK SendPushDataProc(HPR_VOIDPTR pParam);

    // 测试消息解析
	void test_ProcPushData();
private :
    //
    unsigned long m_ip;
    //
	unsigned int m_port;
    //
    void* m_pUser;
    //
    char* m_sSession;
    // 套接字
	HPR_SOCK_T m_iSocket;
	// 心跳的标识
	HPR_BOOL m_bStop;
	// 读描述符集合
	fd_set m_fdread;
	// 写描述符集合
	fd_set m_fdwrite;
	// 延迟时间
	struct timeval m_Timeout;
	// Push数据接收缓冲
	char* m_pRecvBuf;
	// 发送数据缓冲区
	char* m_pSendBuf;
	//数据包回调
	pPacketLister m_pPacketLister;
	// 连接回调
	pConnectionLister m_pConnectionLister;
    //push发送线程句柄
    HPR_HANDLE m_hSendThreadHandle;
    //push接收线程句柄
    HPR_HANDLE m_hRecvThreadHandle;

    // 数据处理缓冲
    char* m_pBuffer;
    // 数据缓存写入位置
    HPR_UINT32 m_nWritePos;
};

#endif /* PUSHMANAGER_H_ */
