/*
 * RtpPacket.h
 *
 *  Created on: 2014-5-15
 *      Author: weilinfeng
 */


#ifndef __RTP_PACKET_H__
#define __RTP_PACKET_H__

#include "HPR_Addr.h"

#include "RtpRtcpDefine.h"
#include "Rtp_Rtcp.h"

struct rtp_param {
	unsigned int ssrc;
	unsigned short sequence;
	unsigned int timestamp;
	int marker;
	int payload;
	int extension;
};

struct rtp_buf {
	void* buf;
	int len;
};

class CRtpPacket {
public:
	// 构造函数
	CRtpPacket();
	// 析构函数
	~CRtpPacket();

public:
	// 初始化资源
	bool Init();
	// 释放资源
	void Release();
	// 打包RTP头
	rtp_buf* pack(rtp_param* param);
	// 拆包rtp头
	rtp_buf* unpack(void* pRtpBuf, int iLen);
	// 获取
	rtp_buf* data();

	rtp_ext_hdr_t* ext_hdr();
	unsigned char* ext_data();

	int version();
	int padding();
	int extension();
	int count();
	int marker();
	int payload();
	unsigned short sequence();
	unsigned int timestamp();
	unsigned int ssrc();
	unsigned int csrc(int index);

	bool IsValid();

	static int RtpHeaderSize();

public:
	rtp_hdr_t* m_rtphdr;
	int m_iLength;
	bool m_bValid;
	rtp_buf m_data;
	char* m_pRtpBuf;
};

inline int CRtpPacket::version() {
	return m_rtphdr->version;
}

inline int CRtpPacket::padding() {
	return m_rtphdr->padding;
}

inline int CRtpPacket::extension() {
	return m_rtphdr->extension;
}

inline int CRtpPacket::count() {
	return m_rtphdr->count;
}

inline unsigned int CRtpPacket::csrc(int index) {
	unsigned int* pCSRC = (unsigned int*) (((unsigned char*) m_rtphdr) + sizeof(rtp_hdr_t));
	return pCSRC[index];
}

inline int CRtpPacket::marker() {
	return m_rtphdr->marker;
}

inline int CRtpPacket::payload() {
	return m_rtphdr->payload;
}

inline unsigned short CRtpPacket::sequence() {
	return m_rtphdr->sequence;
}

inline unsigned int CRtpPacket::timestamp() {
	return m_rtphdr->timestamp;
}

inline unsigned int CRtpPacket::ssrc() {
	return m_rtphdr->ssrc;
}

inline rtp_buf* CRtpPacket::data() {
	return &m_data;
}

inline unsigned char* CRtpPacket::ext_data() {
	int offset = sizeof(rtp_hdr_t) + count() * sizeof(unsigned int) + sizeof(rtp_ext_hdr_t);
	return ((unsigned char*) m_rtphdr) + offset;
}

inline rtp_ext_hdr_t* CRtpPacket::ext_hdr() {
	int offset = sizeof(rtp_hdr_t) + count() * sizeof(unsigned int);
	return reinterpret_cast<rtp_ext_hdr_t*>(((unsigned char*) m_rtphdr) + offset);
}

inline bool CRtpPacket::IsValid() {
	return m_bValid;
}

inline int CRtpPacket::RtpHeaderSize() {
	return sizeof(rtp_hdr_t);
}

#endif

