/*
 * RtpRtcpTransEngine.h
 *
 *  Created on: 2013-3-4
 *      Author: weilinfeng
 */

#ifndef __RTPRTCPTRANSWRAPPER_H__
#define __RTPRTCPTRANSWRAPPER_H__

#include "HPR_Addr.h"
#include "HPR_Thread.h"
#include "HPR_Utils.h"
#include "HPR_Select.h"
#include "HPR_Socket.h"
#include "HPR_Mutex.h"
#include "Rtp_Rtcp.h"

#include "EventWrapperEx.h"
#include "RtpRtcpDefine.h"

#include "RtspClientSDK.h"
#include "RtpPacket.h"
#include "RtspDefine.h"

class CRtspClientEngine;
class CRtpRtcpTransEngine
{
public:
	CRtpRtcpTransEngine();
	~CRtpRtcpTransEngine();

public:
	HPR_BOOL Init(int iIndex, pRtspDataCallback pDataCallback, CRtspClientEngine* pRtspEngine);
	HPR_BOOL Release();
	
	HPR_BOOL StartRtpRtcpTrans();
	HPR_BOOL StopRtpRtcpTrans();

	inline int GetLostPacketNum();

protected:
	static HPR_VOIDPTR CALLBACK RtpRtcpRecvTrans(HPR_VOIDPTR pParam);
	static HPR_VOIDPTR CALLBACK RtpRtcpSendTrans(HPR_VOIDPTR pParam);

private:
	HPR_BOOL SendRtcpPacket();
	HPR_BOOL RecvRtpRtcpPacket();

	void GenerateRtcpPacket();
	int GenerateRtcpRRPacket();
	int GenerateRtcpSDESPacket();
	int GenerateRtcpAPPPacket();

	bool MallocMemory();

	void FreeMemory();

	bool HandleMediaInfo();

	HPR_BOOL ProcRtpData(char* pbuffer, HPR_UINT32 readlen);

	inline HPR_BOOL IsEndFlag( char const *src ) const;
	inline HPR_BOOL IsHeadWithRtp( char const *src ) const;

	void CalculateLostPacketNum(int palyload);

public:
	// 初始化标识
	HPR_BOOL m_bInit;
	// 开始数据交互标识
	HPR_BOOL m_bStartTrans;
	int m_handle;
	// 数据回调函数指针
	pRtspDataCallback m_pDataCallback;
	// rtsp引擎指针
	CRtspClientEngine* m_pRtspEngine;
    // 数据流标识
    bool m_bNoData;
    // 延迟时间
    struct timeval m_Timeout;
    // 读描述符集合
    fd_set m_fdread;
    // 写描述符集合
    fd_set m_fdwrite;
    // 超时次数等待时间
    int m_TimeoutCount;
    // 链路重置事件
    CEventWrapperEx m_ReseteEvent;
    // rtprtcp数据接收缓冲
	char* m_pRtpRtcpRecvBuf;
	//  rtprtcp数据发送缓冲
	char* m_pRtpRtcpSendBuf;
	// 粘包处理数据缓冲
    char* m_pBuffer;
    // 数据处理位置
    HPR_UINT32 m_nWritePos;
    //
	int m_iRtcpSendLen;
	//
	int m_iServerSsrc;
	//
	int m_iClientSsrc;
	//
	int m_iPalyload[4];
	//
	int m_iCurSeq[4];
	//
	int m_iTotalLostPacketNum;
	//
	int m_iLastPacketSeq;
	//
	int m_iLastPacketTimeStamp;
	//
	bool m_bQuit;
	//
	HPR_HANDLE m_hSendTHandle;
	//
	HPR_HANDLE m_hRecvTHandle;
	//
	rtcp_t	m_rtcp_packet;
	//
	CRtpPacket* m_pRtpPacket;
	// 海康流头信息
	HIK_MEDIAINFO m_MediaInfo;

	// 国标码流标识
	bool bTrans;
	// rtp拆包标识
	bool bUnpack;
};

inline int CRtpRtcpTransEngine::GetLostPacketNum()
{
	return m_iTotalLostPacketNum;
}

#endif
