/*
 * RtpTcpTransEngine.h
 *
 *  Created on: 2013-3-4
 *      Author: weilinfeng
 */

#ifndef __RTPTRANSWRAPPER_H__
#define __RTPTRANSWRAPPER_H__

#include <string>
#include <fstream>
#include "HPR_Addr.h"
#include "HPR_Thread.h"
#include "HPR_Utils.h"
#include "HPR_Socket.h"
#include "HPR_Select.h"
#include "RtpPacket.h"
#include "RtpRtcpDefine.h"
#include "EventWrapperEx.h"
#include "RtspDefine.h"
#include "UdpTcpPortPairs.h"

using namespace std;

class CRtspClientEngine;
class CRtpTcpTransEngine
{
public:
    CRtpTcpTransEngine();
    ~CRtpTcpTransEngine();

public:
    // 初始化
    HPR_BOOL Init(int handle, pRtspDataCallback pDataCallbackFunc, CUdpTcpPortPairs *portPair, CRtspClientEngine* rtspEngine);
    // 释放资源
    void Release();							
    // 开始RTP交互
    HPR_BOOL StartRtpTrans();
    // 停止RTP交互
    void StopRtpTrans();

    inline int GetLostPacketNum();

private:
    //
    bool HandleMediaInfo();
    // 分配内存
    bool MallocMemory();
    // 释放内存
    void FreeMemory();
    // 连接服务器
    HPR_INT32 ConnectServer();
    //断开服务器
    void DisConnectServer();

    // rtp发送线程
    static HPR_VOIDPTR CALLBACK SendRtpDataProc(HPR_VOIDPTR pParam);
    // 生成RTP报文
    void GenerateRtpPacket();
    // 发送RTP报文
    HPR_BOOL SendRtpPacket();


    // rtp接收线程
   static HPR_VOIDPTR CALLBACK RecvRtpDataProc(HPR_VOIDPTR pParam);
    // 接收RTP报文
    HPR_BOOL RecvRtpData();
    // rtp处理函数
	HPR_BOOL ProcRtpData(char* pbuffer, HPR_UINT32 readLen);
	// 判断Rtp头
    inline HPR_BOOL IsHeadWithRtp(char const *src) const;
    // 判断回放结束标识
	HPR_BOOL IsEndFlag(char const *src) const;
public:
    // 初始化标识
    HPR_BOOL m_bInit;		
    // 开始交互标识
    HPR_BOOL m_bStartTrans;	
    // 标识是否要退出线程
    HPR_BOOL m_bQuit;	
    // RTP socket
    HPR_SOCK_T m_iRtpSocket;
    // 引擎句柄
    int m_handle;
    // 数据回调函数
    pRtspDataCallback m_pDataCallback;
    //rtp接收线程句柄
    HPR_HANDLE m_hRecvRtpHandle;
    //rtp发送线程句柄
    HPR_HANDLE m_hSendRtpHandle;
    //rtsp引擎
    CRtspClientEngine* m_pRtspEngine;
    //
    CUdpTcpPortPairs *m_pPortPair;
    //数据流标识
    bool m_bNoData;
    // 延迟时间
    struct timeval m_Timeout;
    // 读描述符集合
    fd_set m_fdread;
    // 写描述符集合
    fd_set m_fdwrite;
    //超时次数等待时间
    int m_TimeoutCount;
    //链路重置事件
    CEventWrapperEx m_ReseteEvent;

    /*******黏包处理************/
    char* m_pBuffer;
    //
    HPR_UINT32 m_nWritePos;
    //
    char* m_pRtpRecvBuf;
    //
    char* m_pRtpSendBuf;
    //
    int m_iRtpSendLen;
    //
	CRtpPacket* m_pRtpPacket;
	//
	int m_iPalyload[4];
	//
	int m_iCurSeq[4];
	//
	int m_iTotalLostPacketNum;
	//
	int m_iLastPacketSeq;
	//
	int m_iLastPacketTimeStamp;

	// 海康流头信息
	HIK_MEDIAINFO m_MediaInfo;

	// 国标码流标识
	bool bTrans;
	// rtp拆包标识
	bool bUnpack;

    //FILE *m_file;
};

inline int CRtpTcpTransEngine::GetLostPacketNum()
{
	return m_iTotalLostPacketNum;
}
#endif
