/**	@file 	RtspClientEngine.h
 *	@note 	HangZhou Hikvision System Technology Co., Ltd. All Right Reserved.
 *	@brief 	RtspClientEngine
 *
 *	@author	weilinfeng
 *	@date	2013-05-20
 *
 *
 */
#ifndef __RTSPCLIENT_ENGINE_H__
#define __RTSPCLIENT_ENGINE_H__

#include "HPR_Addr.h"
#include "HPR_Mutex.h"
#include "HPR_String.h"
#include "HPR_Socket.h"
#include "HPR_Utils.h"
#include "HPR_Error.h"
#include "RtspClientSDK.h"
#include "RtspDefine.h"
#include "RtspParser.h"
#include "UdpTcpPortPairs.h"
#include "TCPInterleavedPairs.h"
#include "RtpUdpTransEngine.h"
#include "RtcpUdpTransEngine.h"
#include "RtpTcpTransEngine.h"
#include "RtcpTcpTransEngine.h"
#include "RtpRtcpTransEngine.h"
#include <string>

#define IPADDR_LEN	16	// IPv4 地址长度定义

using namespace std;

class CRtspClientEngine
{
public:
	CRtspClientEngine();
	~CRtspClientEngine();

	// 初始化资源
	HPR_BOOL Init(int iIndex, int iBasePort, int iRtpProtocol, pRtspDataCallback pDataCallback, pRtspMsgCallback pMsgCallback, void* userId);
	// 释放资源
	HPR_BOOL Release();
	//开始实时预览
	HPR_BOOL StartRtspProc(const char* pRtspURL, const char* devicename = NULL, const char* devicepsw = NULL);
	// 停止rtsp交互
	HPR_BOOL StopRtspProc();
	// 返回rtsp回话ID
	char* GetSession();
    // 按时间回放
    HPR_BOOL PlaybackByTime(const char* pRtspURL, const char* devicename, const char* devicepsw, pABS_TIME from, pABS_TIME to);
    // 在绝对时间范围内，随机定位
    HPR_BOOL RandomPlay(pABS_TIME pfrom, pABS_TIME pto = NULL);
    // 改变播放速率
    HPR_BOOL ChangeRate(int scale);
    // 暂停回放
    HPR_BOOL PlayBackPause();
    // 恢复回放
    HPR_BOOL PlayBackResume();
    const char *fetchMediaHeader(int *len);
public:
    // 用户数据
    void *m_UserId;
	
protected:
	static HPR_VOIDPTR OptionsProc(HPR_VOIDPTR pParam);

private:
	HPR_BOOL Options();
	HPR_BOOL Describe();
	HPR_BOOL Setup();
	HPR_BOOL Play();
	HPR_BOOL Play(pABS_TIME pfrom, pABS_TIME pto = NULL);
	HPR_BOOL RandomPlay(unsigned long lTimePos);
	HPR_BOOL Pause();
	HPR_BOOL Resume();
	HPR_BOOL ForceIFrame();
	HPR_BOOL TearDown();
	HPR_BOOL GetRedirectUrl();
	
	HPR_BOOL MallocMemory();
	void FreeMemory();

	HPR_BOOL RecognVodVersion(const char* pRtspURL);
	HPR_BOOL RecognRealVersion(const char* pRtspURL);

	HPR_BOOL ParseURL(char* pRtspURL, char* deviceName, char* devicePassword);

	HPR_BOOL ConnectServer();
	HPR_BOOL DisConnectServer();
	
	HPR_BOOL SendRtspRequest();
	HPR_BOOL RecvRtspResponse();
	
	HPR_BOOL ParseDescribePacket();
	HPR_BOOL ParseSetupPacket(int mediaIndex);
	HPR_BOOL ParsePlayPacket();
	HPR_BOOL ParseRandomPlayPacket();
	HPR_BOOL ParseChanageRatePacket();
	HPR_BOOL ParsePausePacket();
	HPR_BOOL ParseResumePacket();
	HPR_BOOL ParseForceIFramePacket();
	HPR_BOOL ParseTearDownPacket();
	HPR_BOOL ParseOptionsPacket();

	// Added by hyf 2015-3-23
	HPR_BOOL ParseErrStackPacket();

	void StartOptionsThread();
	void StopOptionsThread();
	
	HPR_BOOL StartRtpTcpTrans();
	HPR_BOOL StopRtpTcpTrans();
	HPR_BOOL StartRtpUdpTrans();
	HPR_BOOL StopRtpUdpTrans();
	HPR_BOOL StartRtpRtspTrans();
	HPR_BOOL StopRtpRtspTrans();
	

	HPR_BOOL GenerateAuth( const char* devicename, const char* devicepsw);

	HPR_BOOL GenerateOption();
	HPR_BOOL GenerateDescribe(PRTSPCLIENT_DESCRIBEPARAM param);
	HPR_BOOL GenerateSetup(PRTSPCLIENT_SETUPPARAM param);
	HPR_BOOL GeneratePlay(PRTSPCLIENT_PLAYPARAM param);
	HPR_BOOL GenerateForceIFrame();
	HPR_BOOL GeneratePause();
	HPR_BOOL GenerateResume();
	HPR_BOOL GenerateHeartBeat();
	HPR_BOOL GenerateTearDown();

	HPR_BOOL ConvertToStdTime(pABS_TIME pTime, string & stdTime);
	HPR_BOOL DealStreamErrInfo(unsigned short totalLen, unsigned short packageLen);
	HPR_BOOL GetErrorStackInfoFromBuf(char* errbuf, int errbuflen, const char* buf, int buflen);

	void PritfMediaHeader(const char * mediaheader, int len);

private:
	HPR_BOOL m_bInit;
	HPR_BOOL m_bPlaying;
	int	m_iEngineerIndex;
	int m_iRtpProtocol;

	HPR_SOCK_T m_iRTSPSocket;
	HPR_BOOL m_bStopOptions;
	HPR_BOOL m_bVTDUStackErr;

	pRtspDataCallback m_pDataCallback;
	pRtspMsgCallback m_pMsgCallback;

	int m_iVersion;

	int m_iSeq;
	unsigned long m_iRTSPServerAddr;
	unsigned int m_iRTSPServerPort;
	HPR_UINT8 m_sRTSPServerAddr[IPADDR_LEN];

	status_code m_statusCode;
	int m_iRecvTotalLen;
	char* m_sSendBuf;
	char* m_sRecvBuf;
	char* m_sRtspUrl;
	char* m_sLocationUrl;
	char* m_sSession;
	char* m_Auth;

	char* m_sTransportBuf;
	char* m_sServerPortBuf;
	char* m_sInterleavedBuf;
	int m_iSSRC;

	const char* m_UserAgent;
	SDP_INFO m_oSdpInfo;

	CRtpUdpTransEngine* m_pRtpUdpTransEngine;
	CRtcpUdpTransEngine* m_pRtcpUdpTransEngine;
	CRtpTcpTransEngine* m_pRtpTcpTransEngine;
	CRtpRtcpTransEngine* m_pRtpRtcpTransEngine;
	//CRtcpTransEngine* m_pRtcpTransEngine;

	CUdpTcpPortPairs m_oUdpTcpPortPair[MEDIA_INFO_MAX_NUM];
	CTcpInterleavedPairs m_oTCPInterleavedPair;
	CRtspParser m_oRtspParser;
	
	HPR_HANDLE m_hOptionsThread;

    RTSPSESSION_STATE m_state;

	bool m_bretrans;

	friend class CRtpRtcpTransEngine;
    friend class CRtpTcpTransEngine;
    friend class CRtcpTcpTransEngine;
    friend class CRtpUdpTransEngine;
    friend class CRtcpUdpTransEngine;
};

#endif
