/*
 * RtpTcpTransEngine.cpp
 *
 *  Created on: 2013-3-4
 *      Author: weilinfeng
 */

#include "HPR_Error.h"

#include "RtpTcpTransEngine.h"
#include "RtspClientEngine.h"

#include "MCRPrint.h"
#include "RtpRtcpDefine.h"
#include "MsgManage.h"


extern CMsgManage oMsgManage;
extern bool bBackFinish[ENGINE_MAX_NUM];

CRtpTcpTransEngine::CRtpTcpTransEngine()
: m_bInit(HPR_FALSE)
, m_bStartTrans(HPR_FALSE)
, m_bQuit(HPR_TRUE)
, m_iRtpSocket(HPR_INVALID_SOCKET)
, m_handle(-1)
, m_pDataCallback(NULL)
, m_hRecvRtpHandle(HPR_INVALID_HANDLE)
, m_hSendRtpHandle(HPR_INVALID_HANDLE)
, m_pRtspEngine(NULL)
, m_pPortPair(NULL)
, m_bNoData(HPR_TRUE)
, m_TimeoutCount(0)
, m_pBuffer(NULL)
, m_nWritePos(0)
, m_pRtpRecvBuf(NULL)
, m_pRtpSendBuf(NULL)
, m_iRtpSendLen(0)
, m_pRtpPacket(NULL)
, m_iLastPacketSeq(0)
, m_iLastPacketTimeStamp(0)
, m_iTotalLostPacketNum(-1)
, bTrans(false)
, bUnpack(false)
{
    m_Timeout.tv_sec = 2;
    m_Timeout.tv_usec = 0;
	for(int i = 0; i < 4; i++)
	{
		m_iCurSeq[i] = -1;
		m_iPalyload[i] = -1;
	}
}

CRtpTcpTransEngine::~CRtpTcpTransEngine()
{
}

/**	@fn		Init
 * 	@author	Weilinfeng
 *	@brief	初始化
 *	@param 	iIndex - RTSP引擎的index [in]
 *	@param 	pCallbackFunc -	数据回调函数 [in]
 *	@param 	iServeraddr - 服务器端地址 [in]
 *	@param	iServerport - 服务器端端口 [in]
 *	@param	iClientport - 客户端端口 [in]
 *	@param	iServerssrc - 同步标识 [in]
 *	@param	sSession - 连接的Session [in]
 *	@return	HPR_TRUE - 成功
 *			HPR_FALSE - 失败
 */
HPR_BOOL CRtpTcpTransEngine::Init(int handle, pRtspDataCallback pDataCallbackFunc, CUdpTcpPortPairs *portPair, CRtspClientEngine* rtspEngine)
{
    if (m_bInit)
    {
    	LOGD("CRtpTcpTransEngine[%d]::Init() have been init\n", handle);
        return HPR_TRUE;
    }

    if (pDataCallbackFunc == NULL || rtspEngine == NULL || portPair == NULL)
    {
        LOGD("CRtpTcpTransEngine[%d]::Init() Callback Param Error\n", handle);
        return HPR_FALSE;
    }

    if(rtspEngine->m_oSdpInfo.header ==NULL || rtspEngine->m_oSdpInfo.headlen != 40)
    {
        LOGD("CRtpTcpTransEngine[%d]::Init() sdpInfo is null\n", handle);
        return HPR_FALSE;
    }

    // 初始化参数
   m_handle = handle;
   m_pDataCallback	= pDataCallbackFunc;
   m_pRtspEngine = rtspEngine;
   m_pPortPair = portPair;
   m_handle = rtspEngine->m_iEngineerIndex;

   bool ret = HandleMediaInfo();
   	if(!ret) {
   		LOGD("CRtpTcpTransEngine[%d]::Init() HandleMediaInfo fail\n", m_handle);
   		return HPR_FALSE;
   	}

   	ret = MallocMemory();
    if (!ret)
    {
        LOGD("CRtpTcpTransEngine[%d]::Init() MallocMemory fail\n", m_handle);
        return HPR_FALSE;
    }

    m_bStartTrans = HPR_FALSE;
    m_bInit = HPR_TRUE;

    //m_file = fopen("/storage/emulated/0/data.ps", "wb+");

    LOGD("CRtpTcpTransEngine[%d]::Init() success\n", m_handle);
    return HPR_TRUE;
}

bool CRtpTcpTransEngine::HandleMediaInfo() {
	if(m_pRtspEngine == NULL) {
		LOGD("CRtpTcpTransEngine[%d]::HandleMediaInfo() m_pRtspEngine == NULL\n", m_handle);
		return false;
	}

	if(m_pRtspEngine->m_oSdpInfo.header == NULL) {
		LOGD("CRtpTcpTransEngine[%d]::HandleMediaInfo() m_pRtspEngine->m_oSdpInfo.header == NULL\n", m_handle);
		return false;
	}

	memset(&m_MediaInfo, 0, sizeof(HIK_MEDIAINFO));

	unsigned int media_fourcc;
	memcpy(&media_fourcc, m_pRtspEngine->m_oSdpInfo.header, 4);
	if(media_fourcc == HKMI) {
		memcpy(&m_MediaInfo, m_pRtspEngine->m_oSdpInfo.header, sizeof(HIK_MEDIAINFO));
		if(m_MediaInfo.system_format != SYSTEM_MPEG2_PS) {
			LOGD("CRtpTcpTransEngine::HandleMediaInfo HKMI system format: 不是PS封装格式\n");
			//bTrans = true;
		}
	}
	else if(media_fourcc == HVAG) {
		HVAG_MEDIA_HEAD hvagMediaInfo;
		memset(&hvagMediaInfo, 0, sizeof(HVAG_MEDIA_HEAD));
		memcpy(&hvagMediaInfo, m_pRtspEngine->m_oSdpInfo.header, sizeof(HVAG_MEDIA_HEAD));

		unsigned short system_format = ntohs(hvagMediaInfo.system_format);
		if(system_format == HVAG_SYSTEM_RTP_PS) {
			bUnpack = true;
			LOGD("CRtpTcpTransEngine::HandleMediaInfo HVAG system format: RTP+PS（国标）封装格式 , 对应转化成PS封装, bUnpack = true\n");
		}
		else if(system_format == HVAG_SYSTEM_PS) {
			LOGD("CRtpTcpTransEngine::HandleMediaInfo HVAG system format: HVAG_SYSTEM_PS 封装格式\n");
			//bTrans = true;
		}
		else {
			LOGD("CRtpTcpTransEngine::HandleMediaInfo HVAG 未知封装格式, system format:%d\n", system_format);
		}

		// 设置为海康头
		m_MediaInfo.media_fourcc = HKMI;

		if(system_format == HVAG_SYSTEM_NULL) {
			m_MediaInfo.system_format = SYSTEM_NULL;
			LOGD("CRtpTcpTransEngine::HandleMediaInfo system format: 无帧边界原始封装格式\n");
		}
		else if(system_format == HVAG_SYSTEM_PS) {
			m_MediaInfo.system_format = SYSTEM_MPEG2_PS;
			LOGD("CRtpTcpTransEngine::HandleMediaInfo system format: PS封装格式\n");
		}
		else if(system_format == HVAG_SYSTEM_RTP) {
			m_MediaInfo.system_format = SYSTEM_RTP;
			LOGD("CRtpTcpTransEngine::HandleMediaInfo system format: RTP封装格式\n");
		}
		else if(system_format == HVAG_SYSTEM_RTP_PS) {
			m_MediaInfo.system_format = SYSTEM_MPEG2_PS;
			LOGD("CRtpTcpTransEngine::HandleMediaInfo system format: RTP+PS（国标）封装格式 , 对应转化成PS封装\n");
		}
		else if(system_format == HVAG_SYSTEM_RAW) {
			m_MediaInfo.system_format = SYSTEM_HIK;
			LOGD("CRtpTcpTransEngine::HandleMediaInfo system format: 有帧边界原始封装格式 \n");
		}


		unsigned short video_format = ntohs(hvagMediaInfo.video_format);
		if(video_format == HVAG_VIDEO_H264) {
			m_MediaInfo.video_format = VIDEO_HIK264;
			LOGD("CRtpTcpTransEngine::HandleMediaInfo video format: H264编码\n");
		}
		else if(video_format == HVAG_VIDEO_MPEG4) {
			m_MediaInfo.video_format = VIDEO_MPEG4;
			LOGD("CRtpTcpTransEngine::HandleMediaInfo video format: MPEG4编码\n");
		}
		else {
			LOGD("CRtpTcpTransEngine::HandleMediaInfo video format: unkown\n");
		}
	}

	LOGD("CRtpTcpTransEngine[%d]::HandleMediaInfo() success\n", m_handle);
	return true;
}

/**	@fn		Release
 * 	@author	Weilinfeng
 *	@brief	释放资源
 *	@param 	无 [in]
 *	@param 	无 [out]
 *	@return	HPR_TRUE - 成功
 *			HPR_FALSE - 失败
 */
void CRtpTcpTransEngine::Release()
{
	// fclose(m_file);
    // 释放内存
    FreeMemory();

    m_bStartTrans = HPR_FALSE;
    m_bInit = HPR_FALSE;
}


bool CRtpTcpTransEngine::MallocMemory()
{
	m_pRtpPacket = new CRtpPacket();
	if(m_pRtpPacket == NULL) {
		 FreeMemory();
		 return false;
	}

	if(!m_pRtpPacket->Init()) {
		 FreeMemory();
		 return false;
	}

    m_pBuffer = new char[10*RTP_RTPBUF_SIZE];
    if(m_pBuffer == NULL) {
		 FreeMemory();
		 return false;
	}
    memset(m_pBuffer, 0, 10*RTP_RTPBUF_SIZE);

    m_pRtpRecvBuf = new char[RTP_RTPBUF_SIZE];
    if(m_pRtpRecvBuf == NULL) {
		 FreeMemory();
		 return false;
	}
    memset(m_pRtpRecvBuf, 0, RTP_RTPBUF_SIZE);

    m_pRtpSendBuf = new char[RTP_RTPBUF_SIZE];
    if(m_pRtpSendBuf == NULL) {
		 FreeMemory();
		 return false;
	}
    memset(m_pRtpSendBuf, 0, RTP_RTPBUF_SIZE);
    
    return true;
}

void CRtpTcpTransEngine::FreeMemory()
{
	if(m_pRtpPacket != NULL) {
		m_pRtpPacket->Release();
		delete m_pRtpPacket;
		m_pRtpPacket = NULL;
	}

    if (m_pBuffer != NULL) {
    	delete[] m_pBuffer;
    	m_pBuffer = NULL;
    }

    if (m_pRtpRecvBuf != NULL) {
    	delete[] m_pRtpRecvBuf;
    	m_pRtpRecvBuf = NULL;
    }

    if (m_pRtpSendBuf != NULL) {
		delete[] m_pRtpSendBuf;
		m_pRtpSendBuf = NULL;
	}
}

HPR_INT32 CRtpTcpTransEngine::ConnectServer()
{
	 LOGD("CRtpTcpTransEngine[%d]::ConnectServer()\n", m_handle);
    // 创建套接字
    m_iRtpSocket = HPR_CreateSocket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (m_iRtpSocket == HPR_INVALID_SOCKET)
    {
        LOGD("CRtpTcpTransEngine[%d]::ConnectServer() Create RTP socket fail, last error:%u and system error:%u!\n", \
								m_handle, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());

        //LOGD("CRtpTcpTransEngine::ConnectServer() Create RTP socket fail\n");
        return HPR_ERROR;
    }

    // 设置套接字接收缓冲大小和发送缓冲大小
    HPR_INT32 ret = HPR_SetBuffSize(m_iRtpSocket, RTP_RTPBUF_SIZE, RTP_RTPBUF_SIZE);
    if (ret != HPR_OK)
    {
		LOGD("CRtpTcpTransEngine[%d]::ConnectServer() HPR_SetBuffSize fail, last error:%u and system error:%u!\n", \
								m_handle, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());

		//LOGD("CRtpTcpTransEngine::ConnectServer() HPR_SetBuffSize fail\n");
        HPR_CloseSocket(m_iRtpSocket);
        m_iRtpSocket = HPR_INVALID_SOCKET;
        return HPR_ERROR;
    }

    // 设置地址重用
    ret = HPR_SetReuseAddr(m_iRtpSocket, true);
    if (ret != HPR_OK)
    {
    	LOGD("CRtpTcpTransEngine::ConnectServer() HPR_SetReuseAddr fail\n");
        HPR_CloseSocket(m_iRtpSocket);
        m_iRtpSocket = HPR_INVALID_SOCKET;
        return HPR_ERROR;
    }

    // 通过地址族类型,字符串格式的地址及端口号组建HPR网络地址结构
    HPR_ADDR_T remoteaddr;
    ret = HPR_MakeAddr4ByInt(m_pPortPair->GetServerAddr(), m_pPortPair->GetRTPServerPort(), &remoteaddr);
    if (ret != HPR_OK)
    {
    	LOGD("CRtpTcpTransEngine::ConnectServer() HPR_MakeAddrByString fail\n");
        HPR_CloseSocket(m_iRtpSocket);
        m_iRtpSocket = HPR_INVALID_SOCKET;
        return HPR_ERROR;
    }

    //2013-01-28 将rtsp连接超时时间改为10s
    // 连接服务器，超时时间是毫秒
    ret = HPR_ConnectWithTimeOut(m_iRtpSocket, &remoteaddr, 20000);
    if (ret != HPR_OK)
    {
    	HPR_UINT32 lastErrorCode = HPR_GetLastError();
		HPR_UINT32 systemErrorCode = HPR_GetSystemLastError();
		//LOGD(m_pPortPair->GetServerAddr());
		LOGD("CRtpTcpTransEngine::ConnectServer() connect fail, ip:%u and port:%u\n", m_pPortPair->GetServerAddr(), m_pPortPair->GetRTPServerPort());
		LOGD("CRtpTcpTransEngine::ConnectServer() connect fail, last error code:%u and system error code2:%u\n", lastErrorCode, systemErrorCode);

		LOGD("CRtpTcpTransEngine::ConnectServer() connect fail");
		HPR_CloseSocket(m_iRtpSocket);
        m_iRtpSocket = HPR_INVALID_SOCKET;
        return HPR_ERROR;
    }

    LOGD("CRtpTcpTransEngine[%d]::ConnectServer() success\n", m_handle);

    return HPR_OK;
}

void CRtpTcpTransEngine::DisConnectServer()
{
    // 释放Socket
    if (m_iRtpSocket != HPR_INVALID_SOCKET)
    {
        // 关闭套接字，即：套接字不在做收发数据
        HPR_INT32 iRet = HPR_ShutDown(m_iRtpSocket, HPR_SHUT_RDWR);
        if(iRet != HPR_OK)
        {
        	LOGD("CRtpTcpTransEngine[%d]::DisConnectServer() HPR_ShutDown fail, last error:%u and system error:%u!\n", \
									m_handle, \
									HPR_GetLastError(), \
									HPR_GetSystemLastError());
        }
        iRet = HPR_CloseSocket(m_iRtpSocket);
        if(iRet != HPR_OK)
		{
        	LOGD("CRtpTcpTransEngine[%d]::DisConnectServer() HPR_CloseSocket fail, last error:%u and system error:%u!\n", \
									m_handle, \
									HPR_GetLastError(), \
									HPR_GetSystemLastError());
		}
        m_iRtpSocket = HPR_INVALID_SOCKET;
    }
}

/**	@fn		StartRtpTrans
 * 	@author	Weilinfeng
 *	@brief	开始RTP交互
 *	@param 	无 [in]
 *	@param 	无 [out]
 *	@return	HPR_TRUE - 成功
 *			HPR_FALSE - 失败
 */
HPR_BOOL CRtpTcpTransEngine::StartRtpTrans()
{
	LOGD("CRtpTcpTransEngine[%d]::StartRtpTrans()success\n", m_handle);
    if (!m_bInit)
    {
        LOGD("CRtpTcpTransEngine[%d]::StartRtpTrans() No Init\n", m_handle);
        return HPR_FALSE;
    }

    // 如果正在交互,则直接返回
    if (m_bStartTrans)
    {
        LOGD("CRtpTcpTransEngine[%d]::StartRtpTrans() has started\n", m_handle);
        return HPR_TRUE;
    }

    // 断开连接
    if (ConnectServer() == HPR_ERROR)
    {
        LOGD("CRtpTcpTransEngine[%d]::StartRtpTrans() connect server fail\n", m_handle);
        StopRtpTrans();
        return HPR_FALSE;
    }

    // 发送会话ID给服务器
	int len = HPR_Sendn(m_iRtpSocket, m_pRtspEngine->m_sSession, strlen(m_pRtspEngine->m_sSession), 4000);
	if (len <= 0)
	{
		StopRtpTrans();
		LOGD("CRtpTcpTransEngine[%d]::StartRtpTrans() HPR_Sendn fail, last error:%u and system error:%u!\n", \
								m_handle, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());
		return HPR_FALSE;
	}

	m_bQuit = HPR_FALSE;
	m_hSendRtpHandle = HPR_Thread_Create(SendRtpDataProc, this, 0);
	if (m_hSendRtpHandle == HPR_INVALID_HANDLE)
	{
		LOGD("CRtpTransEngine::Init() create rtp send thread fail\n");
		StopRtpTrans();
		return HPR_FALSE;
	}

    // 创建RTP发送线程
    m_hRecvRtpHandle = HPR_Thread_Create(RecvRtpDataProc, this, 0);
    if (m_hRecvRtpHandle == HPR_INVALID_HANDLE)
    {
        LOGD("CRtpTcpTransEngine[%d]::Init() create rtp send thread fail\n", m_handle);
        StopRtpTrans();
        return HPR_FALSE;
    }

    m_bStartTrans = HPR_TRUE;

    LOGD("CRtpTcpTransEngine[%d]::StartRtpTrans()success\n", m_handle);
    return HPR_TRUE;
}

/**	@fn		StopRtpTrans
 * 	@author	Weilinfeng
 *	@brief	停止RTP交互
 *	@param 	无 [in]
 *	@param 	无 [out]
 *	@return	HPR_TRUE - 成功
 *			HPR_FALSE - 失败
 */
void CRtpTcpTransEngine::StopRtpTrans()
{
    m_bQuit = HPR_TRUE;

    // 设置重置等待事件有信
    m_ReseteEvent.Signal();

    // 停止RTP发送和接收线程
    if (m_hSendRtpHandle != (HPR_HANDLE)HPR_INVALID_THREAD)
    {
        HPR_Thread_Wait(m_hSendRtpHandle);
        LOGD("CRtpTcpTransEngine[%d]::StopRtpTrans() wait  end\n", m_handle);
        m_hSendRtpHandle = (HPR_HANDLE)HPR_INVALID_THREAD;
    }

    // 停止RTP发送和接收线程
    if (m_hRecvRtpHandle != (HPR_HANDLE)HPR_INVALID_THREAD)
    {
        HPR_Thread_Wait(m_hRecvRtpHandle);
        LOGD("CRtpTcpTransEngine[%d]::StopRtpTrans() wait  end\n", m_handle);
        m_hRecvRtpHandle = (HPR_HANDLE)HPR_INVALID_THREAD;
    }

    DisConnectServer();

    m_bStartTrans = HPR_FALSE;
}

/**	@fn		RecvDataByTCPProc
 * 	@author	Weilinfeng
 *	@brief	RTP发送线程
 *	@param 	pParam - CRtpTransEngine对象指针 [in]
 *	@return	NULL
 */
HPR_VOIDPTR CALLBACK CRtpTcpTransEngine::RecvRtpDataProc(HPR_VOIDPTR pParam)
{	
    CRtpTcpTransEngine *pRTPTrans = (CRtpTcpTransEngine*)pParam;
    if (!pRTPTrans) {
        LOGD("CRtpTcpTransEngine[%d]::RecvDataByTCPProc() NULL Error\n", pRTPTrans->m_handle);
        return NULL;
    }

    if (pRTPTrans->m_pDataCallback) {
		if(pRTPTrans->m_MediaInfo.system_format == SYSTEM_NULL){
			LOGD("CRtpTcpTransEngine::RecvRtpDataProc system format: 没有封装 ");
		}
		else if(pRTPTrans->m_MediaInfo.system_format == SYSTEM_HIK) {
			LOGD("CRtpTcpTransEngine::RecvRtpDataProc system format: 海康封装");
		}
		else if(pRTPTrans->m_MediaInfo.system_format == SYSTEM_MPEG2_PS) {
			LOGD("CRtpTcpTransEngine::RecvRtpDataProc system format: PS");
		}
		else if(pRTPTrans->m_MediaInfo.system_format == SYSTEM_MPEG2_TS) {
			LOGD("CRtpTcpTransEngine::RecvRtpDataProc system format: TS");
		}
		else if(pRTPTrans->m_MediaInfo.system_format == SYSTEM_RTP) {
			LOGD("CRtpTcpTransEngine::RecvRtpDataProc system format: RTP");
		}
		else {
			LOGD("CRtpTcpTransEngine::RecvRtpDataProc system format:%d", pRTPTrans->m_MediaInfo.system_format);
		}

        // 回调RTP包(拆RTP包方式)
        (*pRTPTrans->m_pDataCallback)(pRTPTrans->m_handle, DATATYPE_HEADER, (char*)&pRTPTrans->m_MediaInfo, sizeof(HIK_MEDIAINFO), 0, 0, pRTPTrans->m_pRtspEngine->m_UserId, false);
    }

    for(;;){
        if (pRTPTrans->m_bQuit) {
            LOGD("CRtpTcpTransEngine[%d]::RecvDataByTCPProc() Exit\n", pRTPTrans->m_handle);
            (*pRTPTrans->m_pDataCallback)(pRTPTrans->m_handle, DATATYPE_HEADER, (char*)&pRTPTrans->m_MediaInfo, sizeof(HIK_MEDIAINFO), 0, 0, pRTPTrans->m_pRtspEngine->m_UserId, true);
            break;
        }

        // 接收RTP报文
        pRTPTrans->RecvRtpData();

    }

    return NULL;
}

/**	@fn		RecvDataByTCP
 * 	@author	Weilinfeng
 *	@brief	TCP方式接收RTP报文
 *	@param 	无 [in]
 *	@param 	无 [out]
 *	@return	HPR_TRUE - 成功
 *			HPR_FALSE - 失败
 */
HPR_BOOL CRtpTcpTransEngine::RecvRtpData()
{
	//LOGD("CRtpTcpTransEngine[%d]::RecvRtpData\n", m_handle);

    if (m_pRtspEngine->m_state == RTSPSTATE_PAUSE && m_bNoData)
    {
        LOGD("CRtpTcpTransEngine::RecvRtpData->engine pause state\n");
        HPR_Sleep(1000);
    }
    else
    {
        FD_ZERO(&m_fdread);
        FD_SET(m_iRtpSocket, &m_fdread);
        HPR_UINT32 ret = HPR_Select(m_iRtpSocket+1, &m_fdread, NULL, NULL, &m_Timeout);
        if ( ret > 0)
        {
            HPR_INT32 readLen = HPR_Recv(m_iRtpSocket, m_pRtpRecvBuf, RTP_RTPBUF_SIZE);
            if (readLen > 0)
            {
            	m_TimeoutCount = 0;
            	//ProcRtpData(readLen);
                ProcRtpData(m_pRtpRecvBuf, readLen);
            	//LOGD("CRtpTcpTransEngine[%d]::RecvRtpData->recv data len %d\n", m_handle, readLen);
                m_bNoData = HPR_FALSE;
            }// 返回0代表网络中断，Unix下，进程会收到一个SIGPIPE信号，进程对该信号的默认处理是进程终止
            else
            {
            	LOGD("CRtpTcpTransEngine[%d]::RecvRtpData->recv fail, last error:%u and system error:%u!\n", \
            											m_handle, \
            											HPR_GetLastError(), \
            											HPR_GetSystemLastError());


            	m_bQuit = HPR_TRUE;
                m_pRtspEngine->m_bStopOptions = HPR_TRUE;
                m_ReseteEvent.WaitTimeout(5000);
                //HPR_Sleep(5000);
                oMsgManage.InsertMsg(m_handle, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
                //m_pMsgCallback(m_handle, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
                LOGD("CRtpTcpTransEngine[%d]::RecvRtpData->recv - data\n", m_handle);
            }
        }// 超时
        else if (ret == 0)
        {
        	LOGD("CRtpTcpTransEngine[%d]::RecvRtpData->slect 0 data, last error:%u and system error:%u!\n", \
        											m_handle, \
        											HPR_GetLastError(), \
        											HPR_GetSystemLastError());

             m_bNoData = HPR_TRUE;
             //wlf-20130125 添加超时判断，如果超时60s，SDK向上层抛出连接异常消息，通知上层重连
			 if(m_TimeoutCount > 20)
			 {
				 //m_pMsgCallback(m_handle, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
				 oMsgManage.InsertMsg(m_handle, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
				 m_bQuit = HPR_TRUE;
				 m_pRtspEngine->m_bStopOptions = HPR_TRUE;
				 LOGD("CRtpTcpTransEngine[%d]::RecvRtpData->Select timeout > 15s\n", m_handle);
			 }
			 m_TimeoutCount++;

			 LOGD("CRtpTcpTransEngine[%d]::RecvRtpData->Select timeout!\n", m_handle);
             //HPR_Sleep(500);
        } // 出错了
        else
        {

        	LOGD("CRtpTcpTransEngine[%d]::RecvRtpData->slect -1 data, last error:%u and system error:%u!\n", \
        											m_handle, \
        											HPR_GetLastError(), \
        											HPR_GetSystemLastError());

        	m_bQuit = HPR_TRUE;
        	m_pRtspEngine->m_bStopOptions = HPR_TRUE;
        	m_ReseteEvent.WaitTimeout(5000);
        	//HPR_Sleep(5000);
        	oMsgManage.InsertMsg(m_handle, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
            //m_pMsgCallback(m_handle, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
            LOGD("CRtpTcpTransEngine[%d]::RecvRtpData->select - \n", m_handle);

        }
    }

    return HPR_TRUE;
}

// rtp处理函数
HPR_BOOL CRtpTcpTransEngine::ProcRtpData(char* pbuffer, HPR_UINT32 readlen)
{
	if(m_pBuffer == NULL)
	{
		LOGD("CRtpTcpTransEngine::ProcRtpData-> buffer is null");
		return HPR_FALSE;
	}

	LOGD("CRtpTcpTransEngine::ProcRtpData-> readlen:%d", readlen);

	if(readlen < 10*RTP_RTPBUF_SIZE - m_nWritePos)
	{
		memcpy(m_pBuffer + m_nWritePos, pbuffer, readlen);
		m_nWritePos += readlen;

		HPR_UINT16 packlen = 0;
		HPR_BOOL keeploop = HPR_TRUE;

		char *pTmpPtr = m_pBuffer;

		LOGD("CRtpTcpTransEngine::ProcRtpData-> into while, and m_nWritePos:%d", m_nWritePos);

		//因此缓存内必须至少有16字节才足以解析，所以限制调整为大于等于16
		while (keeploop && m_nWritePos >= 16)
		{
			//判断起始位置是否是rtp头，如果有且数据足够则回调
			if (IsHeadWithRtp(pTmpPtr))
			{
				memcpy(&packlen, pTmpPtr + 2, 2);
				if(m_pRtspEngine->m_iVersion == 3)
				{
					packlen = packlen*4;
					LOGD("CRtpTcpTransEngine::ProcRtpData->m_pRtspEngine->m_iVersion == 3, rtp packet len:%d\n", packlen);
				}
				else
				{
					packlen = htons(packlen);
					LOGD("CRtpTcpTransEngine::ProcRtpData->m_pRtspEngine->m_iVersion != 3, rtp packet len:%d\n", packlen);
				}

				LOGD("CRtpTcpTransEngine::ProcRtpData-> rtp packet len:%d, and m_nWritePos:%d\n", packlen, m_nWritePos);

				//2013-07-02 对buffer中剩余数据是否满足rtp包条件做修改，添加等于判断，解决公安仅发一次回访标识引起的解析失败的问题
				if ((packlen + 4) <= m_nWritePos) {
					char* ptr;
					unsigned long len = 0;
					if(m_pRtspEngine->m_iVersion == 3) {
						ptr = pTmpPtr+16;
						len = packlen-12;

						rtp_buf* stream = m_pRtpPacket->unpack(ptr, len);
						if (stream != NULL) {
							unsigned short seq = m_pRtpPacket->sequence();
							unsigned int timeStamp = m_pRtpPacket->timestamp();
							rtp_buf* dataBuf = m_pRtpPacket->data();
						}
					}
					else {
						ptr = pTmpPtr+4;
						len = packlen;
					}

					if (len == 8) {
						if(IsEndFlag(ptr)) {
							return HPR_TRUE;
						}
					}

					// 如果是国标码流直接去掉rtp头
					if(bUnpack) {
						rtp_buf* stream = m_pRtpPacket->unpack(ptr, len);
						if (stream != NULL) {
							unsigned short seq = m_pRtpPacket->sequence();
							unsigned int timeStamp = m_pRtpPacket->timestamp();
							rtp_buf* dataBuf = m_pRtpPacket->data();
							if(dataBuf != NULL && m_pDataCallback != NULL) {
								LOGD("CRtpTcpTransEngine::ProcRtpData data len:%d\n", dataBuf->len);
								LOGD("CRtpTcpTransEngine::ProcRtpData timeStamp:%d\n", timeStamp);
								LOGD("CRtpTcpTransEngine::ProcRtpData seq:%d\n", seq);
								(*m_pDataCallback)(m_handle, DATATYPE_STREAM, (char*)dataBuf->buf, dataBuf->len, timeStamp, seq, m_pRtspEngine->m_UserId, false);
							}
						}
					}
					else {
						if(m_pDataCallback != NULL) {
							(*m_pDataCallback)(m_handle, DATATYPE_STREAM, (char*)ptr, len, 0, 0, m_pRtspEngine->m_UserId, false);
//							LOGD("CRtpTcpTransEngine::ProcRtpData");
//							FILE *fp = fopen("/mnt/sdcard/rtcp.ps", "ab+");
//							fwrite(ptr, len, 0, fp);
//							fclose(fp);
						}
					}

					//写入位置需要减去整个内容的长度
					m_nWritePos = m_nWritePos - packlen - 4;
					pTmpPtr = pTmpPtr + packlen + 4;

					if (m_nWritePos < 16) {
						LOGD("CRtpTcpTransEngine::ProcRtpData-> write position < 16\n");
						memcpy(m_pBuffer, pTmpPtr, m_nWritePos);
					}
				}
				else
				{
					if (pTmpPtr != m_pBuffer)
					{
						LOGD("CRtpTcpTransEngine::ProcRtpData-> skip,and  m_nWritePos:%d ,packetlen:%d \n", m_nWritePos, packlen);
						memcpy(m_pBuffer, pTmpPtr, m_nWritePos);
					}

					//在数据不足以回调时退出循环，等待下一批数据填入缓存后重新解析
					keeploop = HPR_FALSE;
				}
			}
			else
			{
				LOGD("CRtpTcpTransEngine::ProcRtpData-> not find rtp TPKT and m_nWritePos:%d, in front\n", m_nWritePos);
				HPR_UINT32 headpos = 1;
				//循环到最后16字节之前，因为可能最后是一个不完整的头，留在后面数据补齐后再判断
				for ( ; headpos < (m_nWritePos - 15); ++headpos)
				{
					if (IsHeadWithRtp(pTmpPtr + headpos))
					{
						break;
					}
				}

				LOGD("CRtpTcpTransEngine::ProcRtpData-> headpos:%d, and m_nWritePos:%d\n", headpos, m_nWritePos);

				//无论找到与否这里均作一次拷贝，是由于存在以下两种情况
				//1、找到了rtp头
				if (headpos < (m_nWritePos - 15))
				{
					m_nWritePos -= headpos;
					pTmpPtr += headpos;
					LOGD("CRtpTcpTransEngine::ProcRtpData-> find rtp header");
				}//2、没有找到rtp头
				else
				{
					//此处的写位置应置为15，即之前排查没有查到的15个字节应该保留
					m_nWritePos = 15;

					memcpy(m_pBuffer, pTmpPtr + headpos, m_nWritePos);
					//缓存数据中没有rtp头，则退出循环
					keeploop = HPR_FALSE;
					LOGD("CRtpTcpTransEngine::ProcRtpData-> not find rtp header");
				}
				HPR_Sleep(1);
			}
		}

		LOGD("CRtpTcpTransEngine::ProcRtpData-> out while, and m_nWritePos:%d", m_nWritePos);
	}
	else
	{
		LOGD("CRtpTcpTransEngine::ProcRtpData-> buffer overflow");
		oMsgManage.InsertMsg(m_handle, RTSPCLIENT_MSG_BUFFER_OVERFLOW, 0, 0, m_pRtspEngine->m_UserId);
	}

	return HPR_TRUE;
}

inline HPR_BOOL CRtpTcpTransEngine::IsHeadWithRtp( char const *src ) const
{
    if (((*(unsigned char *)src) == 0x24 && (*(unsigned char *)(src + 1)) == 0x00))
    {
        return HPR_TRUE;
    }
    else
    {
        return HPR_FALSE;
    }
}

HPR_BOOL CRtpTcpTransEngine::IsEndFlag( char const *src ) const
{
	DOWNLOAD_BYE sByte;
	memset(&sByte, 0, sizeof(DOWNLOAD_BYE));
	memcpy(&sByte, src, sizeof(DOWNLOAD_BYE));
    if (!bBackFinish[m_handle]) {
        if (sByte.iByte == HEADER_FLAG && sByte.iByteEx == HEADER_FLAG_EX) {
            LOGD("++++++++++++++DataCallback->play bcak finish mark\n");
            oMsgManage.InsertMsg(m_handle, RTSPCLIENT_MSG_PLAYBACK_FINISH, 0, 0, m_pRtspEngine->m_UserId);
        }
    }
	
	return HPR_FALSE;
}

/**	@fn		GenerateRtpPacket
 * 	@author	weilinfeng
 *	@brief	生成rtp包
 *	@param 	无 [in]
 *	@param 	无 [out]
 */
void CRtpTcpTransEngine::GenerateRtpPacket()
{
	memset(m_pRtpSendBuf, 0, RTP_RTPBUF_SIZE);

	unsigned long iSessionLen = strlen(m_pRtspEngine->m_sSession);
	m_iRtpSendLen		= sizeof(rtp_hdr_t) + 4 + 64;

	rtp_hdr_t rtphead;
	rtphead.version		= 2;
	rtphead.extension	= 1;
	rtphead.padding		= 0;
	rtphead.count		= 0;
	rtphead.marker		= 0;
	rtphead.payload		= 0;
	rtphead.sequence	= 0;
	rtphead.timestamp	= 0;
	rtphead.ssrc		= 0;

#ifdef __EHOME_PLATFORM__
	char* holeStr = (char*)"ClientRTPPunching";
	memcpy(m_pRtpSendBuf, (char*)(&(rtphead)), sizeof(rtphead));
	memcpy(m_pRtpSendBuf + sizeof(rtphead), holeStr, strlen(holeStr));
	m_iRtpSendLen = sizeof(rtphead) + strlen(holeStr);

#else
	#ifdef __ASB_PLATFORM__
		m_pRtpSendBuf[0] = 0;
		m_pRtpSendBuf[1] = 0;
		m_pRtpSendBuf[2] = 0;
		m_pRtpSendBuf[3] = 1;
		memcpy(m_pRtpSendBuf + 4, &m_iServerSsrc, 4);

		m_iRtpSendLen = 8;
	#else
		memcpy(m_pRtpSendBuf, (char*)(&(rtphead)), sizeof(rtphead));
		memset(m_pRtpSendBuf + sizeof(rtphead), 0, 2);

		short int iNetSessionLen = htons(16);//htons(iSessionLen);
		memcpy(m_pRtpSendBuf + sizeof(rtphead) + 2, (char*)(&iNetSessionLen), 2);
		memcpy(m_pRtpSendBuf + sizeof(rtphead) + 4, (char*)(m_pRtspEngine->m_sSession), iSessionLen);
		m_iRtpSendLen = sizeof(rtp_hdr_t) + 4 + 64;
	#endif
#endif
    
}

/**	@fn		SendRtpPacket
 * 	@author	weilinfeng
 *	@brief	发送rtp包
 *	@param 	无 [in]
 *	@param 	无 [out]
 */
HPR_BOOL CRtpTcpTransEngine::SendRtpPacket()
{
	struct timeval timeout;
	timeout.tv_sec  = 3;
	timeout.tv_usec = 0;

	fd_set fdwrite;
	FD_ZERO(&fdwrite);
	FD_SET(m_iRtpSocket, &fdwrite);
	HPR_INT32 iRet = HPR_Select(m_iRtpSocket + 1, NULL, &fdwrite, NULL, &timeout);
	if (iRet > 0)
	{
//		if (!HPR_FdIsSet(m_iRtpSocket, &fdwrite) > 0)
//		{
//			CRtspClientPrint::Print("CRtpTcpTransEngine[%d]::SendRtpPacket->HPR_FdIsSet fail, last error:%u and system error:%u!\n", \
//									m_handle, \
//									HPR_GetLastError(), \
//									HPR_GetSystemLastError());
//
//			m_bQuit = HPR_TRUE;
//		}

		LOGD("CRtpTcpTransEngine::SendRtpPacket send rtp packet");
		HPR_INT32 iLen = HPR_Send(m_iRtpSocket, m_pRtpSendBuf, m_iRtpSendLen);
		if(iLen > 0)
		{

		}
		else
		{
			LOGD("CRtpTcpTransEngine[%d]::SendRtpPacket->send fail, last error:%u and system error:%u!\n", \
									m_handle, \
									HPR_GetLastError(), \
									HPR_GetSystemLastError());

			m_bQuit = HPR_TRUE;

		}

	}
	else if(iRet == 0)
	{
		LOGD("CRtpTcpTransEngine[%d]::SendRtpPacket->slect 0 data, last error:%u and system error:%u!\n", \
								m_handle, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());
	}
	else
	{
		LOGD("CRtpTcpTransEngine[%d]::SendRtpPacket->slect 0 datafail, last error:%u and system error:%u!\n", \
								m_handle, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());

						m_bQuit = HPR_TRUE;
	}

	return HPR_TRUE;
}

/**	@fn		SendRtpDataProc
 * 	@author	weilinfeng
 *	@brief	发送rtp线程
 *	@param 	无 [in]
 *	@param 	无 [out]
 */
HPR_VOIDPTR CALLBACK CRtpTcpTransEngine::SendRtpDataProc( HPR_VOIDPTR pParam )
{
	CRtpTcpTransEngine *pRTPTrans = (CRtpTcpTransEngine*)pParam;
	if (pRTPTrans == NULL)
	{
		return NULL;
	}

	pRTPTrans->GenerateRtpPacket();

	int iCount = 20;

	for (;;)
	{
		if (pRTPTrans->m_bQuit)
		{
			LOGD("CRtpTransEngine::RtpSendProc() Exit\n");
			break;
		}

		if (iCount >= 20)
		{
			iCount = 0;
			pRTPTrans->SendRtpPacket();
		}
		else
		{
			HPR_Sleep(500);
			iCount++;
		}
	}

	return NULL;
}
