/*
 * RtpUdpTransEngine.cpp
 *
 *  Created on: 2013-5-10
 *      Author: weilinfeng
 */

#include "HPR_Select.h"
#include "HPR_Socket.h"
#include "RtpUdpTransEngine.h"
#include "RtspClientEngine.h"
#include "MCRPrint.h"
#include "RtpUtil.h"
#include "MsgManage.h"

extern CMsgManage oMsgManage;

CRtpUdpTransEngine::CRtpUdpTransEngine()
: m_pDataCallback(NULL)
, m_pRtspEngine(NULL)
, m_bInit(HPR_FALSE)
, m_bStartTrans(HPR_FALSE)
, m_bQuit(HPR_TRUE)
, m_hSendTHandle(HPR_INVALID_HANDLE)
, m_hRecvTHandle(HPR_INVALID_HANDLE)
, m_iIndex(-1)
, m_iRTPSocket(HPR_INVALID_SOCKET)
, m_iServerSsrc(0)
, m_pRTPSendBuf(NULL)
, m_iRTPSendLen(0)
, m_pRTPRecvBuf(NULL)
, m_iRTPRecvLen(0)
, m_iLastPacketSeq(0)
, m_iLastPacketTimeStamp(0)
, m_iPacketCount(0)
, m_pPortPair(NULL)
, m_TimeoutCount(0)
, m_bNoData(HPR_TRUE)
, m_iTotalLostPacketNum(-1)
, bTrans(false)
, bUnpack(false)
{
	memset(&m_MediaInfo, 0, sizeof(HIK_MEDIAINFO));

	memset(&m_remoteaddr, 0, sizeof(m_remoteaddr));
	for(int i = 0; i < 4; i++) {
		m_iCurSeq[i] = 0;
		m_iPalyload[i] = -1;
	}
}

CRtpUdpTransEngine::~CRtpUdpTransEngine() {
}

HPR_BOOL CRtpUdpTransEngine::Init(int iIndex, pRtspDataCallback pDataCallback, unsigned int& iClientSsrc,
		CUdpTcpPortPairs* portPair, CRtspClientEngine* pRtspEngine) {
	if (m_bInit) {
		LOGD("CRtpTransEngine::Init() inited\n");
		return HPR_TRUE;
	}

	if (iIndex < 0) {
		LOGD("CRtpTransEngine::Init() iIndex < 0\n");
		return HPR_FALSE;
	}

	if (pDataCallback == NULL) {
		LOGD("CRtpTransEngine::Init() pDataCallback == NULL\n");
		return HPR_FALSE;
	}

	if (portPair == NULL) {
		LOGD("CRtpTransEngine::Init() portPair == NULL\n");
		return HPR_FALSE;
	}

	if (pRtspEngine == NULL){
		LOGD("CRtpTransEngine::Init() pRtspEngine == NULL\n");
		return HPR_FALSE;
	}

	m_iIndex		= iIndex;
	m_pDataCallback	= pDataCallback;
	m_pRtspEngine 	= pRtspEngine;

	m_pPortPair 	= portPair;
	m_iServerSsrc	= htonl(0);
	iClientSsrc		= generateSRC();

	bool ret = HandleMediaInfo();
	if(!ret) {
		LOGD("CRtpTransEngine::Init() HandleMediaInfo fail\n");
		return HPR_FALSE;
	}

	ret = MallocMemory();
	if (!ret) {
		LOGD("CRtpTransEngine::Init() MallocMemory fail\n");
		return HPR_FALSE;
	}

	m_bStartTrans = HPR_FALSE;

	m_bInit = HPR_TRUE;

	return HPR_TRUE;
}

bool CRtpUdpTransEngine::HandleMediaInfo() {
	if(m_pRtspEngine == NULL) {
		return false;
	}

	if(m_pRtspEngine->m_oSdpInfo.header == NULL) {
		return false;
	}

	memset(&m_MediaInfo, 0, sizeof(HIK_MEDIAINFO));

	unsigned int media_fourcc;
	memcpy(&media_fourcc, m_pRtspEngine->m_oSdpInfo.header, 4);
	if(media_fourcc == HKMI) {
		memcpy(&m_MediaInfo, m_pRtspEngine->m_oSdpInfo.header, sizeof(HIK_MEDIAINFO));
		if(m_MediaInfo.system_format != SYSTEM_MPEG2_PS) {
			LOGD("HKMI system format: It is not PS封装格式");
			//bTrans = true;
		}
	}
	else if(media_fourcc == HVAG) {
		HVAG_MEDIA_HEAD hvagMediaInfo;
		memset(&hvagMediaInfo, 0, sizeof(HVAG_MEDIA_HEAD));
		memcpy(&hvagMediaInfo, m_pRtspEngine->m_oSdpInfo.header, sizeof(HVAG_MEDIA_HEAD));

		unsigned short system_format = ntohs(hvagMediaInfo.system_format);
		if(system_format == HVAG_SYSTEM_RTP_PS) {
			bUnpack = true;
			LOGD("HVAG system format: RTP+PS（国标）封装格式 , 对应转化成PS封装, bUnpack = true");
		}
		else if(system_format == HVAG_SYSTEM_PS) {
			LOGD("HVAG system format: HVAG_SYSTEM_PS PS封装格式");
			//bTrans = true;
		}

		// 设置为海康头
		m_MediaInfo.media_fourcc = HKMI;

		if(system_format == HVAG_SYSTEM_NULL) {
			m_MediaInfo.system_format = SYSTEM_NULL;
			LOGD("system format: 无帧边界原始封装格式");
		}
		else if(system_format == HVAG_SYSTEM_PS) {
			m_MediaInfo.system_format = SYSTEM_MPEG2_PS;
			LOGD("system format: PS封装格式");
		}
		else if(system_format == HVAG_SYSTEM_RTP) {
			m_MediaInfo.system_format = SYSTEM_RTP;
			LOGD("system format: RTP封装格式");
		}
		else if(system_format == HVAG_SYSTEM_RTP_PS) {
			m_MediaInfo.system_format = SYSTEM_MPEG2_PS;
			LOGD("system format: RTP+PS（国标）封装格式 , 对应转化成PS封装");
		}
		else if(system_format == HVAG_SYSTEM_RAW) {
			m_MediaInfo.system_format = SYSTEM_HIK;
			LOGD("system format: 有帧边界原始封装格式 ");
		}


		unsigned short video_format = ntohs(hvagMediaInfo.video_format);
		if(video_format == HVAG_VIDEO_H264) {
			m_MediaInfo.video_format = VIDEO_HIK264;
			LOGD("video format: H264编码 ");
		}
		else if(video_format == HVAG_VIDEO_MPEG4) {
			m_MediaInfo.video_format = VIDEO_MPEG4;
			LOGD("video format: MPEG4编码 ");
		}
	}

	return true;
}

HPR_BOOL CRtpUdpTransEngine::Release() {

    // 释放内存
    FreeMemory();

    m_bStartTrans = HPR_FALSE;

    m_bInit = HPR_FALSE;

    return HPR_TRUE;
}

HPR_BOOL CRtpUdpTransEngine::StartRtpTrans() {

	if (!m_bInit)
	{
		LOGD("CRtpTransEngine::StartRtpTrans() No Init\n");
		return HPR_FALSE;
	}

	if (m_bStartTrans)
	{
		LOGD("CRtpTransEngine::StartRtpTrans() has started\n");
		return HPR_TRUE;
	}

	// 断开连接
	if (ConnectServer() == HPR_ERROR)
	{
		LOGD("CRtpTcpTransEngine[%d]::StartRtpTrans() connect server fail\n", m_iIndex);
		DisConnectServer();
		return HPR_FALSE;
	}

	m_bQuit = HPR_FALSE;
	m_hSendTHandle = HPR_Thread_Create(RtpSendProc, this, 0);
	if (m_hSendTHandle == HPR_INVALID_HANDLE)
	{
		LOGD("CRtpTransEngine::Init() create rtp send thread fail\n");
		return HPR_FALSE;
	}

	m_hRecvTHandle = HPR_Thread_Create(RtpRecvProc, this, 0);
	if (m_hRecvTHandle == HPR_INVALID_HANDLE)
	{
		LOGD("CRtpTransEngine::Init() create rtp recv thread fail\n");

		m_bQuit = HPR_TRUE;
		if (m_hSendTHandle != HPR_INVALID_HANDLE)
		{
			HPR_Thread_Wait(m_hSendTHandle);
			m_hSendTHandle = HPR_INVALID_HANDLE;
		}

		return HPR_FALSE;
	}

	m_bStartTrans = HPR_TRUE;

	return HPR_TRUE;
}

HPR_BOOL CRtpUdpTransEngine::StopRtpTrans() {

	LOGD("CRtpTransEngine::StopRtpTrans()\n");

	m_bQuit = HPR_TRUE;

	if (m_hSendTHandle != (HPR_HANDLE)HPR_INVALID_THREAD)
	{
		HPR_Thread_Wait(m_hSendTHandle);
		LOGD("CRtcpUdpTransEngine[%d]::StopRtpTrans() send thread wait  end\n", m_iIndex);
		m_hSendTHandle = (HPR_HANDLE)HPR_INVALID_THREAD;
	}

	if (m_hRecvTHandle != (HPR_HANDLE)HPR_INVALID_THREAD)
	{
		HPR_Thread_Wait(m_hRecvTHandle);
		LOGD("CRtcpUdpTransEngine[%d]::StopRtpTrans() recv thread wait  end\n", m_iIndex);
		m_hRecvTHandle = (HPR_HANDLE)HPR_INVALID_THREAD;
	}

	DisConnectServer();

	m_bStartTrans = HPR_FALSE;

	return HPR_TRUE;
}

void CRtpUdpTransEngine::GenerateRTPPacket() {
	memset(m_pRTPSendBuf, 0, RTP_RTPBUF_SIZE);

	int iSessionLen		= strlen(m_pRtspEngine->m_sSession);
	m_iRTPSendLen		= sizeof(rtp_hdr_t) + 4 + 64;

	rtp_hdr_t rtphead;
	rtphead.version		= 2;
	rtphead.extension	= 1;
	rtphead.padding		= 0;
	rtphead.count		= 0;
	rtphead.marker		= 0;
	rtphead.payload		= 0;
	rtphead.sequence	= 0;
	rtphead.timestamp	= 0;
	rtphead.ssrc		= 0;

#ifdef __EHOME_PLATFORM__
	char* holeStr = (char*)"ClientRTPPunching";
	memcpy(m_pRTPSendBuf, (char*)(&(rtphead)), sizeof(rtphead));
	memcpy(m_pRTPSendBuf + sizeof(rtphead), holeStr, strlen(holeStr));
	m_iRTPSendLen = sizeof(rtphead) + strlen(holeStr);

#else
	#ifdef __ASB_PLATFORM__
		m_pRTPSendBuf[0] = 0;
		m_pRTPSendBuf[1] = 0;
		m_pRTPSendBuf[2] = 0;
		m_pRTPSendBuf[3] = 1;
		memcpy(m_pRTPSendBuf + 4, &m_iServerSsrc, 4);

		m_iRTPSendLen = 8;
	#else
		memcpy(m_pRTPSendBuf, (char*)(&(rtphead)), sizeof(rtphead));
		memset(m_pRTPSendBuf + sizeof(rtphead), 0, 2);

		short int iNetSessionLen = htons(16);//htons(iSessionLen);
		memcpy(m_pRTPSendBuf + sizeof(rtphead) + 2, (char*)(&iNetSessionLen), 2);
		memcpy(m_pRTPSendBuf + sizeof(rtphead) + 4, (char*)(m_pRtspEngine->m_sSession), iSessionLen);
		m_iRTPSendLen = sizeof(rtp_hdr_t) + 4 + 64;
	#endif
#endif
}

HPR_BOOL CRtpUdpTransEngine::SendRtpPacket()
{
	struct timeval timeout;
	timeout.tv_sec  = 3;
	timeout.tv_usec = 0;

	fd_set fdwrite;
	FD_ZERO(&fdwrite);
	FD_SET(m_iRTPSocket, &fdwrite);
	HPR_INT32 ret = HPR_Select(m_iRTPSocket + 1, NULL, &fdwrite, NULL, &timeout);
	if ( ret > 0)
	{
		HPR_INT32 iRet = HPR_SendTo(m_iRTPSocket, (char*)m_pRTPSendBuf, m_iRTPSendLen, (HPR_ADDR_T*)&m_remoteaddr);
		if(iRet > 0)
		{
			LOGD("CRtpUdpTransEngine::SendRtpPacket select timeout");

		}
		else
		{
			LOGD("CRtpUdpTransEngine[%d]::SendRtpPacket->send fail, last error:%u and system error:%u!\n", \
												m_iIndex, \
												HPR_GetLastError(), \
												HPR_GetSystemLastError());

			m_bQuit = HPR_TRUE;

		}

	}
	else if(ret == 0)
	{
		LOGD("CRtpUdpTransEngine[%d]::SendRtpPacket->select 0, last error:%u and system error:%u!\n", \
															m_iIndex, \
															HPR_GetLastError(), \
															HPR_GetSystemLastError());
	}
	else
	{
		LOGD("CRtpUdpTransEngine[%d]::SendRtpPacket->select fail, last error:%u and system error:%u!\n", \
															m_iIndex, \
															HPR_GetLastError(), \
															HPR_GetSystemLastError());

		m_bQuit = HPR_TRUE;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtpUdpTransEngine::RecvRtpPacket() {
	if (m_pRtspEngine->m_state == RTSPSTATE_PAUSE && m_bNoData)
	{
		LOGD("CRtpUdpTransEngine::RecvRtpData->engine pause state\n");
		HPR_Sleep(1000);
	}
	else {
		int iReadLen = 0;
		struct timeval timeout;
		timeout.tv_sec = 1;
		timeout.tv_usec = 0;

		HPR_ADDR_T tmpaddr;

		fd_set fdread;
		FD_ZERO(&fdread);
		FD_SET(m_iRTPSocket, &fdread);
		HPR_INT32 ret = HPR_Select(m_iRTPSocket + 1, &fdread, NULL, NULL, &timeout);
		if (ret > 0) {
			memset(m_pRTPRecvBuf, 0, RTP_RTPBUF_SIZE);
			iReadLen = HPR_RecvFrom(m_iRTPSocket, m_pRTPRecvBuf, RTP_RTPBUF_SIZE, &tmpaddr);
			if (iReadLen > 0) {

				ProcRtpData(m_pRTPRecvBuf, iReadLen);
				m_TimeoutCount = 0;
				m_bNoData = HPR_FALSE;
			}
			else {
				LOGD("CRtpUdpTransEngine[%d]::RecvRtpData->recv - data, last error:%u and system error:%u!\n", \
						m_iIndex, \
						HPR_GetLastError(), \
						HPR_GetSystemLastError());

				m_bQuit = HPR_TRUE;
				m_pRtspEngine->m_bStopOptions = HPR_TRUE;
				m_ReseteEvent.WaitTimeout(5000);
				//HPR_Sleep(5000);
				oMsgManage.InsertMsg(m_iIndex, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
				//m_pMsgCallback(m_handle, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
				//CRtspClientPrint::Print("CRtpUdpTransEngine[%d]::RecvRtpData->recv - data\n", m_iIndex);
			}
		}
		else if (ret == 0)
		{
			 LOGD("CRtpUdpTransEngine[%d]::RecvRtpData->Select timeout, last error:%u and system error:%u!\n", \
								 m_iIndex,\
								 HPR_GetLastError(), \
								 HPR_GetSystemLastError());

			 m_bNoData = HPR_TRUE;
			 //wlf-20130125 添加超时判断，如果超时60s，SDK向上层抛出连接异常消息，通知上层重连
			 if(m_TimeoutCount > 30)
			 {
				 //m_pMsgCallback(m_handle, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
				 oMsgManage.InsertMsg(m_iIndex, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
				 m_bQuit = HPR_TRUE;
				 m_pRtspEngine->m_bStopOptions = HPR_TRUE;
				 //CRtspClientPrint::Print("CRtpTcpTransEngine[%d]::RecvRtpData->Select timeout > 15s\n", m_handle);
			 }
			 m_TimeoutCount++;
		} // 出错了
		else
		{
			LOGD("CRtpUdpTransEngine[%d]::RecvRtpData->Select - data, last error:%u and system error:%u!\n", \
					m_iIndex, HPR_GetLastError(), \
					HPR_GetSystemLastError());

			m_bQuit = HPR_TRUE;
			m_pRtspEngine->m_bStopOptions = HPR_TRUE;
			m_ReseteEvent.WaitTimeout(5000);
			//HPR_Sleep(5000);
			oMsgManage.InsertMsg(m_iIndex, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
			//m_pMsgCallback(m_handle, RTSPCLIENT_MSG_CONNECTION_EXCEPTION, 0, 0, m_pRtspEngine->m_UserId);
			//CRtspClientPrint::Print("CRtpUdpTransEngine[%d]::RecvRtpData->select - \n", m_iIndex);

		}
	}

	return HPR_TRUE;
}

void CRtpUdpTransEngine::ProcRtpData(char*  rtpRecvBuf, int len)
{
	int iCallbackPos = 0;
	int iMinSeq = 0;
	int iCurrentSeq = 0;

	if (PACKET_BUFFER_NUM == 1)
	{
		rtp_buf* stream = m_pRtpPacketPool[0]->unpack(rtpRecvBuf, len);
		if (stream && m_pDataCallback)
		{
			(*m_pDataCallback)(m_iIndex, \
					DATATYPE_STREAM, \
					(char*)m_pRtpPacketPool[0]->m_pRtpBuf, \
					m_pRtpPacketPool[0]->m_iLength, \
					m_pRtpPacketPool[0]->timestamp(), \
					(int)m_pRtpPacketPool[0]->sequence(), \
					m_pRtspEngine->m_UserId, false);

			//CRtspClientPrint::Print("CRtpUdpTransEngine[%d]::ProcRtpData->rtp data len: %d and time stamp:%d \n", m_iIndex, m_pRtpPacketPool[0]->m_iLength, (int)m_pRtpPacketPool[0]->sequence());
			//CRtspClientPrint::Print("rtp packet sequence:%d\n",(int)m_pRtpPacketPool[0]->sequence());
			m_iLastPacketSeq = m_pRtpPacketPool[0]->sequence();
			m_iLastPacketTimeStamp = m_pRtpPacketPool[0]->timestamp();


		}
	}
	else
	{
		// UDP排序对双轨流无法支持，所以暂时不做UDP排序
		if (m_iPacketCount < PACKET_BUFFER_NUM)
		{
			m_pRtpPacketPool[m_iPacketCount]->unpack(rtpRecvBuf, len);
			m_iPacketCount++;
		}
		else
		{
			iMinSeq = m_pRtpPacketPool[0]->sequence();
			if (iMinSeq >> 8 == 0xFF)
			{
				iMinSeq -= 65536;
			}

			for (int i = 1; i < PACKET_BUFFER_NUM; i++)
			{
				iCurrentSeq = m_pRtpPacketPool[i]->sequence();
				if (iCurrentSeq >> 8 == 0xFF)
				{
					iCurrentSeq -= 65536;
				}

				if (iCurrentSeq < iMinSeq)
				{
					iCallbackPos = i;
					iMinSeq = iCurrentSeq;
				}
			}

			if (m_pDataCallback) {
				(*m_pDataCallback)(m_iIndex, \
						DATATYPE_STREAM, \
						(char*)m_pRtpPacketPool[iCallbackPos]->m_pRtpBuf, \
						m_pRtpPacketPool[iCallbackPos]->m_iLength, \
						m_pRtpPacketPool[iCallbackPos]->timestamp(), \
						(int)m_pRtpPacketPool[iCallbackPos]->sequence(), \
						m_pRtspEngine->m_UserId, false);

				//CRtspClientPrint::Print("CRtpUdpTransEngine[%d]::ProcRtpData->rtp data len: %d and time stamp:%d \n", m_iIndex, m_pRtpPacketPool[iCallbackPos]->m_iLength, (int)m_pRtpPacketPool[iCallbackPos]->sequence());
				//CRtspClientPrint::Print("rtp packet sequence:%d\n",(int)m_pRtpPacketPool[iCallbackPos]->sequence());
				m_iLastPacketSeq = m_pRtpPacketPool[iCallbackPos]->sequence();
				m_iLastPacketTimeStamp = m_pRtpPacketPool[iCallbackPos]->timestamp();



				m_pRtpPacketPool[iCallbackPos]->unpack(m_pRTPRecvBuf, len);
			}
		}
	}
}

HPR_VOIDPTR CRtpUdpTransEngine::RtpRecvProc(HPR_VOIDPTR pParam) {

	CRtpUdpTransEngine *pRTPTrans = (CRtpUdpTransEngine*)pParam;

	if (pRTPTrans == NULL)
	{
		LOGD("CRtpTransEngine::RtpRecvProc() NULL Error\n");
		return NULL;
	}

	CRtspClientEngine* pRtspEngine = pRTPTrans->m_pRtspEngine;
	if (pRTPTrans == NULL)
	{
		LOGD("CRtpUdpTransEngine::RtpRecvProc() pRtspEngine NULL fail\n");
		return NULL;
	}

	if (pRTPTrans->m_pDataCallback)
	{
		// 回调RTP包(拆RTP包方式)
		(*pRTPTrans->m_pDataCallback)(pRTPTrans->m_iIndex, DATATYPE_HEADER, pRtspEngine->m_oSdpInfo.header, pRtspEngine->m_oSdpInfo.headlen, 0, 0, pRtspEngine->m_UserId, false);
	}

	for (;;)
	{
		if (pRTPTrans->m_bQuit)
		{
			LOGD("CRtpTransEngine::RtpRecvProc() Exit\n");
			(*pRTPTrans->m_pDataCallback)(pRTPTrans->m_iIndex, DATATYPE_HEADER, pRtspEngine->m_oSdpInfo.header, pRtspEngine->m_oSdpInfo.headlen, 0, 0, pRtspEngine->m_UserId, true);
			break;
		}

		pRTPTrans->RecvRtpPacket();
	}

	return NULL;
}



HPR_VOIDPTR CRtpUdpTransEngine::RtpSendProc(HPR_VOIDPTR pParam) {
	CRtpUdpTransEngine *pRTPTrans = (CRtpUdpTransEngine*)pParam;
	if (pRTPTrans == NULL)
	{
		return NULL;
	}

	pRTPTrans->GenerateRTPPacket();

	int iCount = 20;

	for (;;)
	{
		if (pRTPTrans->m_bQuit)
		{
			LOGD("CRtpTransEngine::RtpSendProc() Exit\n");
			break;
		}

		if (iCount >= 20)
		{
			iCount = 0;
			pRTPTrans->SendRtpPacket();
		}
		else
		{
			HPR_Sleep(500);
			iCount++;
		}
	}

	return NULL;
}


bool CRtpUdpTransEngine::MallocMemory() {

	for (int i = 0; i < PACKET_BUFFER_NUM; i++) {
		m_pRtpPacketPool[i] = new CRtpPacket();
		if (m_pRtpPacketPool[i] == NULL) {
			LOGD("CRtpTransEngine::MallocMemory() Create RtpPacket fail\n");
			FreeMemory();
			return false;
		}

		if (!m_pRtpPacketPool[i]->Init()) {
			LOGD("CRtpTransEngine::MallocMemory() Init RtpPacket fail\n");
			FreeMemory();
			return false;
		}
	}

	m_pRTPSendBuf	= new char[RTP_RTPBUF_SIZE];
	if (m_pRTPSendBuf == NULL){
		LOGD("CRtpTransEngine::MallocMemory() m_pRTPSendBuf == NULL\n");
		FreeMemory();
		return false;
	}
	memset(m_pRTPSendBuf, 0, RTP_RTPBUF_SIZE);

	m_pRTPRecvBuf	= new char[RTP_RTPBUF_SIZE];
	if (m_pRTPRecvBuf == NULL){
		LOGD("CRtpTransEngine::MallocMemory() m_pRTPRecvBuf == NULL\n");
		FreeMemory();
		return false;
	}
	memset(m_pRTPRecvBuf, 0, RTP_RTPBUF_SIZE);

	return true;
}

void CRtpUdpTransEngine::FreeMemory() {
	if(m_pRTPSendBuf != NULL)
	{
		delete[] m_pRTPSendBuf;
		m_pRTPSendBuf = NULL;
	}

	if(m_pRTPRecvBuf != NULL)
	{
		delete[] m_pRTPRecvBuf;
		m_pRTPRecvBuf = NULL;
	}

	for (int i = 0; i < PACKET_BUFFER_NUM; i++)
	{
		if (m_pRtpPacketPool[i] != NULL)
		{
			m_pRtpPacketPool[i]->Release();
			delete m_pRtpPacketPool[i];
			m_pRtpPacketPool[i] = NULL;
		}
	}
}

HPR_INT32 CRtpUdpTransEngine::ConnectServer() {

	// 创建套接字
	m_iRTPSocket = HPR_CreateSocket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (m_iRTPSocket == HPR_INVALID_SOCKET)
	{
		LOGD("CRtpUdpTransEngine[%d]::ConnectServer() Create RTP socket fail, errorCode:%d and system errorCode:%d\n", \
				m_iIndex, \
				HPR_GetLastError(), \
				HPR_GetSystemLastError());

		return HPR_ERROR;
	}

	// 设置套接字接收缓冲大小和发送缓冲大小
	HPR_INT32 bufSize = 5120 * 100;
	HPR_INT32 ret = HPR_SetBuffSize(m_iRTPSocket, bufSize, bufSize);
	if (ret != HPR_OK)
	{
		LOGD("CRtpUdpTransEngine[%d]::ConnectServer() HPR_SetBuffSize fail, errorCode:%d and system errorCode:%d\n", \
				m_iIndex, \
				HPR_GetLastError(), \
				HPR_GetSystemLastError());

		HPR_CloseSocket(m_iRTPSocket);
		m_iRTPSocket = HPR_INVALID_SOCKET;
		return HPR_ERROR;
	}

	ret = HPR_SetReuseAddr(m_iRTPSocket, true);
	if(ret != HPR_OK)
	{
		LOGD("CRtpUdpTransEngine[%d]::ConnectServer() HPR_SetReuseAddr fail, errorCode:%d and system errorCode:%d\n", \
						m_iIndex, \
						HPR_GetLastError(), \
						HPR_GetSystemLastError());
	}

	HPR_ADDR_T m_addr;
	m_addr.SA.sin4.sin_family = AF_INET;
	m_addr.SA.sin4.sin_addr.s_addr = htonl(INADDR_ANY);
	m_addr.SA.sin4.sin_port = htons(m_pPortPair->GetRTPClientPort());

	m_remoteaddr.SA.sin4.sin_family = AF_INET;
	m_remoteaddr.SA.sin4.sin_addr.s_addr = m_pPortPair->GetServerAddr();
	m_remoteaddr.SA.sin4.sin_port = htons(m_pPortPair->GetRTPServerPort());

	if (HPR_Bind(m_iRTPSocket, &m_addr) == HPR_ERROR)
	{
		LOGD("CRtpUdpTransEngine[%d]::ConnectServer() rtp bind socket fail, errorCode:%d and system errorCode:%d\n", \
				m_iIndex, \
				HPR_GetLastError(), \
				HPR_GetSystemLastError());

		return HPR_ERROR;
	}

	LOGD("CRtpUdpTransEngine[%d]::ConnectServer() success, errorCode:%d and system errorCode:%d\n", \
							m_iIndex, \
							HPR_GetLastError(), \
							HPR_GetSystemLastError());

	LOGD("CRtpUdpTransEngine::ConnectServer() rtp bind socket success, rtp socket:%d, client port:%d, server port:%d\n", \
			m_iRTPSocket, m_pPortPair->GetRTPClientPort(), m_pPortPair->GetRTPServerPort());



	return HPR_OK;
}

void CRtpUdpTransEngine::DisConnectServer() {
	// 释放Socket
	if (m_iRTPSocket != HPR_INVALID_SOCKET)
	{
		// 关闭套接字，即：套接字不在做收发数据
		HPR_INT32 iRet = HPR_ShutDown(m_iRTPSocket, HPR_SHUT_RDWR);
		if(iRet != HPR_OK)
		{
			LOGD("CRtpUdpTransEngine[%d]::DisConnectServer() HPR_ShutDown fail, errorCode:%d and system errorCode:%d\n", \
							m_iIndex, \
							HPR_GetLastError(), \
							HPR_GetSystemLastError());
		}

		iRet = HPR_CloseSocket(m_iRTPSocket);
		if(iRet != HPR_OK)
		{
			LOGD("CRtpUdpTransEngine[%d]::DisConnectServer() HPR_CloseSocket fail, errorCode:%d and system errorCode:%d\n", \
							m_iIndex, \
							HPR_GetLastError(), \
							HPR_GetSystemLastError());
		}
		m_iRTPSocket = HPR_INVALID_SOCKET;
	}
}
