/** @file RtspClient.cpp
 *   @note HangZhou Hikvision System Technology Co., Ltd. All Right Reserved.
 *   @brief RTSP通信库实现文件
 *
 *   @author     Weilinfeng
 *   @date       2012-05-028
 *
 *   @note Created By Weilinfeng 2012-05-028
 *
 *   @warning
 */

#include "PushManager.h"
#include "RtspClientEngine.h"
#include "RtspClientSDK.h" 
#include "MCRPrint.h"
#include "HPR_Guard.h"
#include "MsgManage.h"
#include <new>
#include <map>

using namespace std;
extern bool gInitLib;

bool bTest = false;
FILE *fp = NULL;

// 初始化标识
bool bInitLib = false;

// 是否云存储标识
bool bCloudDevice = false;
char cloudDeviceName[MAX_USERNAME_LEN];
char cloudDevicePwd[MAX_USERNAME_LEN];

bool bBackFinish[ENGINE_MAX_NUM];
//对象管理map
map<int, CRtspClientEngine*> oRtspEngine_Map;
//全局锁
HPR_Mutex oClientLock;
// 每个引擎对应的锁
HPR_Mutex oEngineLock[ENGINE_MAX_NUM];
// 播放速率
int iScales[ENGINE_MAX_NUM];

pRtspMsgCallback pMsgCallback;

pRtspDataCallback pDataCallback;

CMsgManage oMsgManage;

int ibasePort = 10000;

int DataCallback(int handle, int dataType, char* data, int len, unsigned int timestamp, int packetNo, void* pUser, bool bRealse)
{
	//LOGD("RtspClientSDK::DataCallback->engine[%d] data len:%d", handle, len);
	if (pDataCallback != NULL)
	{
		(*pDataCallback)(handle, dataType, data, len, timestamp, packetNo, pUser, bRealse);
	}

	// 写文件测试
	if (bTest)
	{
		if (NULL != fp)
		{
			fwrite(data, len, 1, fp);
		}
	}

	if (len == 8) {
		if (!bBackFinish[handle]) {
			DOWNLOAD_BYE sByte;
			memset(&sByte, 0, sizeof(DOWNLOAD_BYE));
			memcpy(&sByte, data, sizeof(DOWNLOAD_BYE));
			if (sByte.iByte == HEADER_FLAG && sByte.iByteEx == HEADER_FLAG_EX) {
				LOGD("DataCallback->play bcak finish mark\n");
				oMsgManage.InsertMsg(handle, RTSPCLIENT_MSG_PLAYBACK_FINISH, 0, 0, pUser);
				bBackFinish[handle] = true;
			}
		}
	}

	return 0;
}

int MsgCallback(int handle, int opt, void* param1, void* param2, void* pUser)
{
    if (handle < 0)
    {
        return 0;
    }

    if (opt == STREAM_CLIENT_ERR_FROM_SERVER || opt == STREAM_CLIENT_VTM_ERR)   // 流媒体服务器或VTM产生错误信息
    {
        if (NULL == param1)
        {
            return -1;
        }
        PERRORSTACKINFO1 pErrInfo = (PERRORSTACKINFO1)param1;
        int iErr = pErrInfo->perrorInfo->errorCode + STREAM_CLIENT_ERRNO_START;
        LOGD("RtspClientSDK::DataCallback->engine[%d] send Message:%d", handle, iErr);
        (*pMsgCallback)(handle, iErr, param1, param2, pUser);
    }
	else if (pMsgCallback != NULL) {
		LOGD("RtspClientSDK::DataCallback->engine[%d] send Message:%d", handle, opt);
		(*pMsgCallback)(handle, opt, param1, param2, pUser);
	}
	return 0;
}

/** @fn bool RtspClientInitLib()
 *   @brief 初始化RTSP库
 *   @param [in] 无
 *   @param [out] 无
 *   @return true - 初始化成功; false - 初始化库失败
 */
RTSPCLIENTSDKAPI bool RtspClientInitLib()
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientInitLib() MCRSDK not init\n");
		return false;
	}

	HPR_Guard g_guard(&oClientLock);

    if(bInitLib)
    {
        LOGD("RtspClient->RtspClientInitLib() have inited lib\n");
        return true;
    }

    for(int i = 0; i < ENGINE_MAX_NUM; i++)
	{
		iScales[i] = NORMAL_SCALE;
		bBackFinish[i] = false;
	}

    if(!oMsgManage.Run(MsgCallback))
    {
    	LOGD("Java_com_hik_RtspClient_initLib-> g_MsgManage run fail!");
    	g_nRtspClientErrorCode = RTSPCLIENT_MSG_MANAGE_RUN_FAIL;
    	return false;
    }

    bInitLib = true;

    return true;
}

/** @fn bool RtspClientFiniLib()
 *   @brief 释放RTSP库
 *   @param [in] 无
 *   @param [out] 无
 *   @return 无
 */
RTSPCLIENTSDKAPI bool RtspClientFiniLib()
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientFiniLib() MCRSDK not init\n");
		return false;
	}

	HPR_Guard g_guard(&oClientLock);

    if (!bInitLib)
    {
        LOGD("RtspClient->RtspClientFiniLib() not init lib\n");
        return true;
    }

    for(int i = 0; i < ENGINE_MAX_NUM; i++)
    {
    	iScales[i] = NORMAL_SCALE;
    	bBackFinish[i] = false;
    }

    oMsgManage.Stop();

    bInitLib = false;

    return true;
}

/** @fn bool RtspClientCreateEngine()
 *   @brief 创建Rtsp引擎
 *   @param [in] rtpProtocol - RTP传输协议
 *   @param [in] dataCallBack - 数据回调函数
 *   @param [in] msgCallback - 消息回调函数
 *   @param [out] handle	- 引擎句柄
 *   @return true - 初始化成功; false - 初始化库失败
 */
RTSPCLIENTSDKAPI int RtspClientCreateEngine(int tansmethod, pRtspDataCallback dataCallBack, pRtspMsgCallback msgCallback, void* pUser)
{
	LOGD("enter RtspClient->RtspClientCreateEngine()\n");
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientCreateEngine() MCRSDK not init\n");
		LOGD("leave RtspClient->RtspClientCreateEngine()\n");
		return RTSPCLIENT_INVALIDATE_ENGINEID;
	}

    HPR_Guard g_guard(&oClientLock);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientCreateEngine() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        LOGD("leave RtspClient->RtspClientCreateEngine()\n");
        return RTSPCLIENT_INVALIDATE_ENGINEID;
    }

    if(tansmethod != RTPTCP_TRANSMODE && tansmethod != RTPUDP_TRANSMODE && tansmethod != RTPRTSP_TRANSMODE) {
    	LOGD("RtspClient->RtspClientCreateEngine() callback is null\n");
		g_nRtspClientErrorCode = RTSPCLIENT_PROTOCOL_NOTSUPPORT;
		LOGD("leave RtspClient->RtspClientCreateEngine()\n");
		return RTSPCLIENT_INVALIDATE_ENGINEID;
    }

    if (dataCallBack == NULL || msgCallback == NULL) {
    	LOGD("RtspClient->RtspClientCreateEngine() callback is null\n");
    	g_nRtspClientErrorCode = RTSPCLIENT_CALLBACK_PARAM_NULL;
    	LOGD("leave RtspClient->RtspClientCreateEngine()\n");
    	return RTSPCLIENT_INVALIDATE_ENGINEID;
    }
    LOGD("RtspClient->RtspClientCreateEngine(): engine_max_num:%d\n", ENGINE_MAX_NUM);
    for (int i = 0; i<ENGINE_MAX_NUM; i++)
    {
        // 如果存在键i返回1，如果不存在键i则返回0
        if (!oRtspEngine_Map.count(i))
        {
        	LOGD("RtspClient->RtspClientCreateEngine() not exist.\n");
            CRtspClientEngine* pRtspClientEngine = new (std::nothrow) CRtspClientEngine();
            if(pRtspClientEngine == NULL)
            {
                LOGD("RtspClient->RtspClientCreateEngine() crete rtsp client engine fail\n");
                g_nRtspClientErrorCode = RTSPCLIENT_MOLLOC_RTSPENGINE_FAIL;
                LOGD("leave RtspClient->RtspClientCreateEngine()\n");
                return RTSPCLIENT_INVALIDATE_ENGINEID;
            }

            pair<map<int, CRtspClientEngine*>::iterator, bool> ret= oRtspEngine_Map.insert(map<int, CRtspClientEngine*>::value_type(i,pRtspClientEngine));
            // 当g_RtspEngine_Map中存在i引擎时，insert失败
            if (ret.second)
            {
                if(pRtspClientEngine->Init(i, ibasePort, tansmethod, DataCallback, MsgCallback, pUser) == HPR_FALSE)
                {
                    //从map中删除引擎对
                    oRtspEngine_Map.erase(i);
                    //释放RtspclientEngine分配的内存
                    delete pRtspClientEngine;
                    pRtspClientEngine = NULL;
                    LOGD("RtspClient->RtspClientCreateEngine() init engine fail\n");
                    return RTSPCLIENT_INVALIDATE_ENGINEID;
                }
                else
                {
                    pMsgCallback = msgCallback;
                	pDataCallback = dataCallBack;
                	LOGD("RtspClient->RtspClientCreateEngine() init engine success[%d]\n", i);
                    return i;
                }
            }
            else {
                LOGD("RtspClient->RtspClientCreateEngine() insert engine fail\n");
                if (pRtspClientEngine!= NULL)
                {
                    delete pRtspClientEngine;
                    pRtspClientEngine = NULL;
                } 
                continue;  
            }
        }
        else
        {
        	LOGD("RtspClient->RtspClientCreateEngine() exist[%d].\n", i);
//        	return i;
        }
    }
    g_nRtspClientErrorCode = RTSPCLIENT_RTSPENGINE_EXCEED_ERROR;
    LOGD("leave RtspClient->RtspClientCreateEngine()\n");
    return RTSPCLIENT_INVALIDATE_ENGINEID;
}

/** @fn bool RtspClientReleaseEngine(void* handle)
 *   @brief 释放RTSP库
 *   @param [in] handle - 引擎句柄
 *   @param [out] 无
 *   @return true - 成功; false - 失败
 */
RTSPCLIENTSDKAPI bool RtspClientReleaseEngine( int engineId )
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientReleaseEngine() MCRSDK not init\n");
		return false;
	}

    HPR_Guard g_guard(&oClientLock);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientReleaseEngine() not init lib\n");
        return true;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return false;
    }

    //查找指向键engineId的元素，返回指向元素的指针，如果元素不存在，则指向g_RtspEngine_Map.end()
    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine!= NULL)
    {
        pRtspClientEngine->Release();
        delete pRtspClientEngine;
        pRtspClientEngine = NULL;
    }

    LOGD("RtspClient->RtspClientReleaseEngine() success\n");
    oRtspEngine_Map.erase(iter);

    return true;
}

/** @fn bool RtspClientStartRtspProc()
 *   @brief 开始Rtsp流程, 向服务器发送Describe\Setup\Play信令
 *   @param [in] handle - 引擎句柄
 *   @param [in] pszRtspUrl - Rtsp地址
 *   @param [out] 无
 *   @return true - 打开成功, false - 打开失败
 */
RTSPCLIENTSDKAPI bool RtspClientStartRtspProc( int engineId, const char* pszRtspUrl)
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientStartRtspProc() MCRSDK not init\n");
		return false;
	}

    HPR_Guard g_guard(&oEngineLock[engineId]);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientStartRtspProc() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        return false;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
        LOGD("RtspClient->RtspClientStartRtspProc() engine id error\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return false;
    }

    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        LOGD("RtspClient->RtspClientStartRtspProc() not find rtsp engine\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine == NULL)
    {
        LOGD("RtspClient->RtspClientStartRtspProc() rtsp engine is null\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
        return false;
    }

    if(pRtspClientEngine->StartRtspProc(pszRtspUrl) == HPR_FALSE)
    {
        LOGD("RtspClient->RtspClientStartRtspProc() start rtsp proc fail\n");
        return false;
    }

	// 开启写文件测试
	if (bTest)
	{
		FILE *fp = fopen("/mnt/sdcard/rtcp.ps", "ab+");
		if (NULL == fp)
		{
			LOGD("RtspClient->RtspClientStartRtspProc() fopen test file failed\n");
			return false;
		}
	}

    return true;
}

/** @fn bool RtspClientStartRtspProc()
 *   @brief 开始Rtsp流程, 向服务器发送Describe\Setup\Play信令
 *   @param [in] handle - 引擎句柄
 *   @param [in] pszRtspUrl - Rtsp地址
*    @param [in] deviceName - 设备名称
 *   @param [in] devicePsw - 设备密码
 *   @param [out] 无
 *   @return true - 打开成功, false - 打开失败
 */
RTSPCLIENTSDKAPI bool RtspClientStartRtspProc( int engineId, const char* pszRtspUrl, const char* deviceName, const char* devicePsw )
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientStartRtspProc() MCRSDK not init\n");
		return false;
	}

    HPR_Guard g_guard(&oEngineLock[engineId]);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientStartRtspProc() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        return false;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
        LOGD("RtspClient->RtspClientStartRtspProc() engine id error\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return false;
    }

    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        LOGD("RtspClient->RtspClientStartRtspProc() not find rtsp engine\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine == NULL)
    {
        LOGD("RtspClient->RtspClientStartRtspProc() rtsp engine is null\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
        return false;
    }

    if(pRtspClientEngine->StartRtspProc(pszRtspUrl, deviceName, devicePsw) == HPR_FALSE)
    {
        LOGD("RtspClient->RtspClientStartRtspProc() start rtsp proc fail\n");
        return false;
    }

	// 开启写文件测试
	if (bTest)
	{
		FILE *fp = fopen("/mnt/sdcard/rtcp.ps", "ab+");
		if (NULL == fp)
		{
			LOGD("RtspClient->RtspClientStartRtspProc() fopen test file failed\n");
			return false;
		}
	}

    return true;
}

RTSPCLIENTSDKAPI bool GenerateLiveUrl(PLiveInfo liveInfo, char* liveUrl)
{
	LOGD("RtspClient->GenerateLiveUrl()");
	if(liveInfo == NULL || liveUrl == NULL)
	{
		LOGD("RtspClient->GenerateLiveUrl() liveInfo == null || liveUrl == NULL\n");
		return false;
	}

	if(liveInfo->magIp == NULL) {
		LOGD("RtspClient->GenerateLiveUrl() magIp == null\n");
		return false;
	}

	if(liveInfo->cameraIndexCode == NULL) {
		LOGD("RtspClient->GenerateLiveUrl() cameraIndexCode == null\n");
		return false;
	}

	memset(liveUrl, 0, MAX_URL_LEN);
	const char * streamType = NULL;
	if(liveInfo->streamType == STREAM_TYPE_MAIN) {
		streamType = "MAIN";
	}
	else if(liveInfo->streamType == STREAM_TYPE_SUB)
	{
		streamType = "SUB";
	}

	/** 级联标识，0-非级联*/
	if(liveInfo->cascadeFlag == CASCADE_TYPE_NO)
	{
		LOGD("RtspClient->GenerateLiveUrl() cascadeFlag:false\n");
		LOGD("RtspClient->GenerateLiveUrl() magIp:%s\n", liveInfo->magIp);
		LOGD("RtspClient->GenerateLiveUrl() magPort:%d\n", liveInfo->magPort);
		LOGD("RtspClient->GenerateLiveUrl() cameraIndexCode:%s\n", liveInfo->cameraIndexCode);
		LOGD("RtspClient->GenerateLiveUrl() mcuNetID:%d\n", liveInfo->mcuNetID);
		LOGD("RtspClient->GenerateLiveUrl() deviceNetID:%d\n", liveInfo->deviceNetID);
		LOGD("RtspClient->GenerateLiveUrl() token:%s\n", liveInfo->token);
		LOGD("RtspClient->GenerateLiveUrl() streamType:%d\n", liveInfo->streamType);
		LOGD("RtspClient->GenerateLiveUrl() isInternet:%d\n", liveInfo->isInternet);

		if(liveInfo->token != NULL) {
			sprintf(liveUrl, "rtsp://%s:%d/realplay://%s:%s:TCP?cnid=%d&pnid=%d&token=%s&auth=%d", liveInfo->magIp, liveInfo->magPort, liveInfo->cameraIndexCode, streamType, liveInfo->mcuNetID, liveInfo->deviceNetID, liveInfo->token, liveInfo->iPriority);
		}
		else {
			sprintf(liveUrl, "rtsp://%s:%d/realplay://%s:%s:TCP?cnid=%d&pnid=%d&auth=%d", liveInfo->magIp, liveInfo->magPort, liveInfo->cameraIndexCode, streamType, liveInfo->mcuNetID, liveInfo->deviceNetID, liveInfo->iPriority);
		}

		/** MCU是否在公网 */
		if(liveInfo->isInternet == NETWORK_TYPE_INTERNET)
		{
			//重定向标识（redirect）：1表示重定向，0或其它值代表不重定向；
			strcat(liveUrl, "&redirect=0");
		}
		else if(liveInfo->isInternet == NETWORK_TYPE_LOCAL)
		{
			strcat(liveUrl, "&redirect=1");
		}
	}/** 级联标识，1-级联 */
	else if(liveInfo->cascadeFlag == CASCADE_TYPE_YES)
	{
		LOGD("RtspClient->GenerateLiveUrl() cascadeFlag:true\n");
		if(liveInfo->token != NULL) {
			sprintf(liveUrl, "rtsp://%s:%d/ncg://%s:%s?token=%s&cnid=%d", liveInfo->magIp, liveInfo->magPort, liveInfo->cameraIndexCode, streamType, liveInfo->token,liveInfo->mcuNetID);
		}
		else {
			sprintf(liveUrl, "rtsp://%s:%d/ncg://%s:%s?cnid=%d", liveInfo->magIp, liveInfo->magPort, liveInfo->cameraIndexCode, streamType,liveInfo->mcuNetID);
		}
	}
	else
	{
		return false;
	}

	if(liveInfo->bTranscode)
	{
		// 转码标识（transcode）：1表示转码，0或其它值代表不转码。
        if(strchr(liveUrl, '?') != NULL)
        {
            strcat(liveUrl, "&transcode=1");
        }
        else
        {
            strcat(liveUrl, "?transcode=1");
        }
		sprintf(liveUrl + strlen(liveUrl), "&resolution=%d", liveInfo->resolution);
		sprintf(liveUrl + strlen(liveUrl), "&bitrate=%d", liveInfo->bitrate);
		sprintf(liveUrl + strlen(liveUrl), "&framerate=%d", liveInfo->framerate);
		sprintf(liveUrl + strlen(liveUrl), "&videotype=%d", liveInfo->videotype);
		sprintf(liveUrl + strlen(liveUrl), "&systemformat=%d", liveInfo->systemformat);

		LOGD("RtspClient->GenerateLiveUrl() bTranscode:true\n");
		LOGD("RtspClient->GenerateLiveUrl() iPriority:%d\n", liveInfo->iPriority);
		LOGD("RtspClient->GenerateLiveUrl() resolution:%d\n", liveInfo->resolution);
		LOGD("RtspClient->GenerateLiveUrl() bitrate:%d\n", liveInfo->bitrate);
		LOGD("RtspClient->GenerateLiveUrl() framerate:%d\n", liveInfo->framerate);
		LOGD("RtspClient->GenerateLiveUrl() videotype:%d\n", liveInfo->videotype);
		LOGD("RtspClient->GenerateLiveUrl() systemformat:%d\n", liveInfo->systemformat);
	}
	else
	{
		// 转码标识（transcode）：1表示转码，0或其它值代表不转码。
        if(strchr(liveUrl, '?') != NULL)
        {
            strcat(liveUrl, "&transcode=0");
        }
        else
        {
            strcat(liveUrl, "?transcode=0");
        }

		LOGD("RtspClient->GenerateLiveUrl() bTranscode:false\n");
	}
	LOGD(liveUrl);
    LOGD("\n");
	return true;
}

/** @fn bool RtspClientStopRtspProc(void* handle )
 *   @brief 停止RTSP库
 *   @param [in] handle - 引擎句柄
 *   @param [out] 无
 *   @return true - 停止成功; false - 停止失败
 */
RTSPCLIENTSDKAPI bool RtspClientStopRtspProc( int engineId )
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientStopRtspProc() MCRSDK not init\n");
		return false;
	}

    HPR_Guard g_guard(&oEngineLock[engineId]);
    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientStopRtspProc() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        return false;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
    	LOGD("RtspClientSDK::RtspClientStopRtspProc->engine[%d] engineId invalid", engineId);
    	g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
    	return false;
    }

    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
    	LOGD("RtspClientSDK::RtspClientStopRtspProc->engine[%d] no find rtspclient engine", engineId);
    	g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
    	return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine == NULL)
    {
    	LOGD("RtspClientSDK::RtspClientStopRtspProc->engine[%d] rtspclient engine is null", engineId);
    	g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
    	return false;
    }

    pRtspClientEngine->StopRtspProc();

    LOGD("RtspClientSDK::RtspClientStopRtspProc->engine[%d] stop success", engineId);

	// 停止写文件测试
	if (bTest)
	{
		fclose(fp);
	}

    return true;
}

/** @fn bool GetRtspClientSession(int handle )
 *   @brief 获取Rtsp回话
 *   @param [in] handle - 引擎ID
 *   @param [out] 无
 *   @return true - 停止成功; false - 停止失败
 */
RTSPCLIENTSDKAPI char* GetRtspClientSession(int engineId)
{
	if(!gInitLib) {
		LOGD("RtspClient->GetRtspClientSession() MCRSDK not init\n");
		return NULL;
	}

	HPR_Guard g_guard(&oEngineLock[engineId]);
	if(!bInitLib)
	{
		LOGD("RtspClient->GetRtspClientSession() not init lib\n");
		g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
		return NULL;
	}

	if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
	{
		LOGD("RtspClientSDK::GetRtspClientSession->engine[%d] engineId invalid", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
		return NULL;
	}

	map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
	if(iter == oRtspEngine_Map.end())
	{
		LOGD("RtspClientSDK::GetRtspClientSession->engine[%d] no find rtspclient engine", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
		return NULL;
	}

	CRtspClientEngine* pRtspClientEngine = iter->second;
	if (pRtspClientEngine == NULL)
	{
		LOGD("RtspClientSDK::GetRtspClientSession->engine[%d] rtspclient engine is null", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
		return NULL;
	}

	LOGD("RtspClientSDK::GetRtspClientSession->engine[%d] Get Session success", engineId);

	return pRtspClientEngine->GetSession();
}

/** @fn bool GeneratePlaybackUrl
 *   @brief 生成回放地址
 *   @param [in] playbcakInfo - 远程回放信息
 *   @param [out] buf - 生成的远程回放地址
 *   @param [out] 无
 *   @return true - 播放成功, false - 播放失败
 */
RTSPCLIENTSDKAPI bool GeneratePlaybackUrl(PPlaybcakInfo playbcakInfo, char* buf)
{
	if(playbcakInfo == NULL || buf == NULL) {
		LOGD("RtspClient->GeneratePlaybackUrl() pszRtspUrl == NULL\n");
		return false;
	}

	if(playbcakInfo->magIp == NULL) {
		LOGD("RtspClient->GeneratePlaybackUrl() magIp == null\n");
		return false;
	}

	if(playbcakInfo->playbackUrl == NULL) {
		LOGD("RtspClient->GeneratePlaybackUrl() playbackUrl == null\n");
		return false;
	}

	if(playbcakInfo->begin == NULL) {
		LOGD("RtspClient->GeneratePlaybackUrl() begin == null\n");
	}

	if(playbcakInfo->end == NULL) {
		LOGD("RtspClient->GeneratePlaybackUrl() end == null\n");
	}

	LOGD("RtspClient->GeneratePlaybackUrl() cameraIndexCode:%s\n", playbcakInfo->magIp);
	LOGD("RtspClient->GeneratePlaybackUrl() cameraIndexCode:%d\n", playbcakInfo->magPort);
	LOGD("RtspClient->GeneratePlaybackUrl() playbackUrl:%s\n", playbcakInfo->playbackUrl);
	LOGD("RtspClient->GeneratePlaybackUrl() magVersion:%d\n", playbcakInfo->magVersion);

	size_t len = strlen(playbcakInfo->playbackUrl);
	if (len < 9)
	{
		LOGD("RtspClient->GeneratePlaybackUrl() playbackUrl len invalid\n");
		return false;
	}

	if (HPR_Strncasecmp(playbcakInfo->playbackUrl, "rtsp://", 7) != 0)
	{
		LOGD("RtspClient->GeneratePlaybackUrl() playbackUrl format invalid\n");
		return false;
	}
    
    memset(buf, 0, MAX_URL_LEN);
    char *pStr = HPR_Strstr(playbcakInfo->playbackUrl, "@");
    if (pStr!= NULL)
    {
        // 云存储
        size_t len1 = strlen(playbcakInfo->playbackUrl);
        size_t len2 = strlen(pStr);
        size_t len3 = len1 - len2;
        char strURL[len3 + 1];
        char strTem[len3 + 1];
        memset(strURL, 0, len3 + 1);
        memset(strTem, 0, len3 + 1);
        HPR_Strncpy(strURL, playbcakInfo->playbackUrl, (int)len3);
        HPR_Strcpy(strTem, strURL + 7);
        
        char *p = HPR_Strstr(strTem, ":");
        if (p != NULL)
        {
            memset(cloudDeviceName, 0, MAX_USERNAME_LEN);
            memset(cloudDevicePwd, 0, MAX_PASSWD_LEN);
            HPR_Strcpy(cloudDevicePwd, p + 1);
            HPR_Strncpy(cloudDeviceName, strTem, (int)(strlen(strTem) - strlen(cloudDevicePwd) - 1));
        }
        
        bCloudDevice = true;
        if (playbcakInfo->magVersion == MAG_VERSION_0_1)
        {
            if(playbcakInfo->token != NULL)
            {
                if(strchr(pStr, '?') != NULL)
                {
                    sprintf(buf, "rtsp://%s:%s@%s:%d/%s&token=%s", cloudDeviceName, cloudDevicePwd, playbcakInfo->magIp, playbcakInfo->magPort, pStr + 1, playbcakInfo->token);
                }
                else
                {
                    sprintf(buf, "rtsp://%s:%s@%s:%d/%s?token=%s", cloudDeviceName, cloudDevicePwd, playbcakInfo->magIp, playbcakInfo->magPort, pStr + 1, playbcakInfo->token);
                }
            }
            else
            {
                sprintf(buf, "rtsp://%s:%s@%s:%d/%s", cloudDeviceName, cloudDevicePwd, playbcakInfo->magIp, playbcakInfo->magPort, pStr + 1);
            }
        }
        else
        {
            if(playbcakInfo->token != NULL)
            {
                if(strchr(pStr, '?') != NULL)
                {
                   sprintf(buf, "rtsp://%s:%s@%s:%d/%s&cnid=%d&token=%s&begin=%s&end=%s", cloudDeviceName, cloudDevicePwd, playbcakInfo->magIp, playbcakInfo->magPort, pStr + 1, playbcakInfo->mcuNetID,playbcakInfo->token, playbcakInfo->begin, playbcakInfo->end);
                }
                else
                {
                    sprintf(buf, "rtsp://%s:%s@%s:%d/%s?cnid=%d&token=%s&begin=%s&end=%s", cloudDeviceName, cloudDevicePwd, playbcakInfo->magIp, playbcakInfo->magPort, pStr + 1, playbcakInfo->mcuNetID,playbcakInfo->token, playbcakInfo->begin, playbcakInfo->end);
                }
            }
            else
            {
                if(strchr(pStr, '?') != NULL)
                {
                    sprintf(buf, "rtsp://%s:%s@%s:%d/%s&cnid=%d&begin=%s&end=%s", cloudDeviceName, cloudDevicePwd, playbcakInfo->magIp, playbcakInfo->magPort, pStr + 1, playbcakInfo->mcuNetID,playbcakInfo->begin, playbcakInfo->end);
                }
                else
                {
                    sprintf(buf, "rtsp://%s:%s@%s:%d/%s?cnid=%d&begin=%s&end=%s", cloudDeviceName, cloudDevicePwd, playbcakInfo->magIp, playbcakInfo->magPort, pStr + 1, playbcakInfo->mcuNetID,playbcakInfo->begin, playbcakInfo->end);
                }
            }
        }
    }
    else
    {
        // 非云存储
        bCloudDevice = false;
        if (playbcakInfo->magVersion == MAG_VERSION_0_1)
        {
            /**为了兼容MAG V1.0，从MAGV2.2版本应用层需要判断MAG版本来支持本级录像回放URL*/
            if (playbcakInfo->token != NULL)
            {
                if(strchr(playbcakInfo->playbackUrl, '?') != NULL)
                {
                    sprintf(buf, "rtsp://%s:%d/%s&token=%s", playbcakInfo->magIp, playbcakInfo->magPort, playbcakInfo->playbackUrl+7, playbcakInfo->token);
                }
                else
                {
                    sprintf(buf, "rtsp://%s:%d/%s?token=%s", playbcakInfo->magIp, playbcakInfo->magPort, playbcakInfo->playbackUrl+7, playbcakInfo->token);
                }
            }
            else
            {
                sprintf(buf, "rtsp://%s:%d/%s", playbcakInfo->magIp, playbcakInfo->magPort, playbcakInfo->playbackUrl+7);
            }
        }
        else
        {
            if (playbcakInfo->token != NULL)
            {
                if(strchr(playbcakInfo->playbackUrl, '?') != NULL)
                {
                    sprintf(buf, "rtsp://%s:%d/%s&cnid=%d&token=%s&begin=%s&end=%s", playbcakInfo->magIp, playbcakInfo->magPort, playbcakInfo->playbackUrl+7, playbcakInfo->mcuNetID, playbcakInfo->token, playbcakInfo->begin, playbcakInfo->end);
                }
                else
                {
                    sprintf(buf, "rtsp://%s:%d/%s?cnid=%d&token=%s&begin=%s&end=%s", playbcakInfo->magIp, playbcakInfo->magPort, playbcakInfo->playbackUrl+7, playbcakInfo->mcuNetID, playbcakInfo->token, playbcakInfo->begin, playbcakInfo->end);
                }
            }
            else
            {
                if (strchr(playbcakInfo->playbackUrl, '?') != NULL)
                {
                    sprintf(buf, "rtsp://%s:%d/%s&cnid=%d&begin=%s&end=%s", playbcakInfo->magIp, playbcakInfo->magPort, playbcakInfo->playbackUrl+7, playbcakInfo->mcuNetID, playbcakInfo->begin, playbcakInfo->end);
                }
                else
                {
                    sprintf(buf, "rtsp://%s:%d/%s?cnid=%d&begin=%s&end=%s", playbcakInfo->magIp, playbcakInfo->magPort, playbcakInfo->playbackUrl+7, playbcakInfo->mcuNetID, playbcakInfo->begin, playbcakInfo->end);
                }
            }
        }
    }

	LOGD(buf);
	return true;
}

/** @fn bool GenerateNCGPlaybackUrl
 *   @brief 生成级联回放地址
 *   @param [in] playbcakInfo - 级联远程回放信息
 *   @param [out] buf - 生成的远程回放地址
 *   @param [out] 无
 *   @return true - 播放成功, false - 播放失败
 */
RTSPCLIENTSDKAPI bool GenerateNCGPlaybackUrl(PNCGPlaybcakInfo playbcakInfo, char* buf)
{
	if(playbcakInfo == NULL || buf == NULL) {
		LOGD("RtspClient->GenerateNCGPlaybackUrl() pszRtspUrl == NULL\n");
		return false;
	}

	if(playbcakInfo->magIp == NULL) {
		LOGD("RtspClient->GenerateNCGPlaybackUrl() magIp == null\n");
		return false;
	}

	if(playbcakInfo->cameraIndexCode == NULL) {
		LOGD("RtspClient->GenerateNCGPlaybackUrl() cameraIndexCode == null\n");
		return false;
	}

	if(playbcakInfo->begin == NULL) {
		LOGD("RtspClient->GenerateNCGPlaybackUrl() begin == null\n");
		return false;
	}

	if(playbcakInfo->end == NULL) {
		LOGD("RtspClient->GenerateNCGPlaybackUrl() end == null\n");
		return false;
	}


	if(playbcakInfo->recordType == 0) {
		LOGD("RtspClient->GenerateNCGPlaybackUrl() recordType == 0\n");
		playbcakInfo->recordType = 23;
	}

	LOGD("RtspClient->GeneratePlaybackUrl() cameraIndexCode:%s\n", playbcakInfo->magIp);
	LOGD("RtspClient->GeneratePlaybackUrl() cameraIndexCode:%d\n", playbcakInfo->magPort);
	LOGD("RtspClient->GeneratePlaybackUrl() cameraIndexCode:%s\n", playbcakInfo->cameraIndexCode);
	LOGD("RtspClient->GeneratePlaybackUrl() token:%s\n", playbcakInfo->token);


    memset(buf, 0, MAX_URL_LEN);
    if(playbcakInfo->token != NULL) {
        sprintf(buf, "rtsp://%s:%d/ncg_playback://%s:%s:%s:1?cnid=%d&token=%s&filename=%s&recordPos=%d&recordType=%d", playbcakInfo->magIp, playbcakInfo->magPort,
        		playbcakInfo->cameraIndexCode, playbcakInfo->begin, playbcakInfo->end, playbcakInfo->mcuNetID, playbcakInfo->token, playbcakInfo->fileName, playbcakInfo->recordPos, playbcakInfo->recordType);
    }
    else {
        sprintf(buf, "rtsp://%s:%d/ncg_playback://%s:%s:%s:1?cnid=%d&filename=%s&recordPos=%d&recordType=%d", playbcakInfo->magIp, playbcakInfo->magPort,
        		playbcakInfo->cameraIndexCode, playbcakInfo->begin, playbcakInfo->end, playbcakInfo->mcuNetID, playbcakInfo->fileName, playbcakInfo->recordPos, playbcakInfo->recordType);
    }

	LOGD(buf);
	return true;
}

/** @fn bool RtspClientPlaybackByTime
 *   @brief 按时间回放
 *   @param [in] handle - 引擎句柄
 *   @param [in] pszRtspUrl - 回放地址
 *   @param [in] deviceName - 设备名称
 *   @param [in] devicePsw - 设备密码
 *   @param [out] 无
 *   @param [out] 无
 *   @return true - 播放成功, false - 播放失败
 */
RTSPCLIENTSDKAPI bool RtspClientPlaybackByTime( int engineId, const char* pszRtspUrl, const char* deviceName, const char* devicePsw, pABS_TIME from, pABS_TIME to )
{
	if(!gInitLib)
	{
		LOGD("RtspClient->RtspClientPlaybackByTime() MCRSDK not init\n");
		return false;
	}

    HPR_Guard g_guard(&oEngineLock[engineId]);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientPlaybackByTime() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        return false;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
    	LOGD("RtspClientSDK::RtspClientPlaybackByTime->engine[%d] engineId invalid", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return false;
    }

    if(from == NULL)
    {
    	LOGD("RtspClientSDK::RtspClientPlaybackByTime->engine[%d] from time is null", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_START_TIME_NULL;
		return false;
    }

    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        LOGD("RtspClient->RtspClientPlaybackByTime() not find rtsp engine\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine == NULL)
    {
        LOGD("RtspClient->RtspClientPlaybackByTime() rtsp engine is null\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
        return false;
    }

    if (bCloudDevice)
    {
        // 云存储设备获取录像流需要传入访问的用户名、密码
        if(pRtspClientEngine->PlaybackByTime(pszRtspUrl, cloudDeviceName, cloudDevicePwd, from, to) == HPR_FALSE)
        {
            LOGD("RtspClient->RtspClientPlaybackByTime() start play back fail\n");
            return false;
        }
    }
    else
    {
        if(pRtspClientEngine->PlaybackByTime(pszRtspUrl, deviceName, devicePsw, from, to) == HPR_FALSE)
        {
            LOGD("RtspClient->RtspClientPlaybackByTime() start play back fail\n");
            return false;
        }
    }
    bBackFinish[engineId] = false;
    return true;
}

/** @fn bool RtspClientSetPlaybackPos(void* handle)
 *   @brief 在绝对时间范围内，随机定播放
 *   @param [in] handle - 引擎句柄
 *   @param [in] pfrom - 回放开始时间
 *   @param [in] pto - 回放结束时间，默认为空
 *   @param [out] 无
 *   @return true - 成功, false - 失败
 */
RTSPCLIENTSDKAPI bool RtspClientSetPlaybackPos( int engineId, pABS_TIME pfrom, pABS_TIME pto)
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientSetPlaybackPos() MCRSDK not init\n");
		return false;
	}

    HPR_Guard g_guard(&oEngineLock[engineId]);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientSetPlaybackPos() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        return false;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
    	LOGD("RtspClientSDK::RtspClientSetPlaybackPos->engine[%d] engineId invalid", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return false;
    }

    if(pfrom == NULL )
    {
    	LOGD("RtspClientSDK::RtspClientSetPlaybackPos->engine[%d] from time is null", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_START_TIME_NULL;
		return false;
    }

    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        LOGD("RtspClient->RtspClientSetPlaybackPos() not find rtsp engine\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine == NULL)
    {
        LOGD("RtspClient->RtspClientSetPlaybackPos() rtsp engine is null\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
        return false;
    }

    if(pRtspClientEngine->RandomPlay(pfrom, pto) == HPR_FALSE)
    {
        LOGD("RtspClient->RtspClientSetPlaybackPos() set play pos fail\n");
        return false;
    }

    bBackFinish[engineId] = false;
    return true;
}

/** @fn bool RtspClientPlaybackSlow
 *   @brief 设置回放速率
 *   @param [in] handle - 引擎句柄
 *   @param [in] scale为速率改变的方向, >1 播放速率加倍, <1 播放速率减半, =1 恢复正常播放速率
 *   @param [out] 无
 *   @return true - 暂停成功, false - 暂停失败
 */
RTSPCLIENTSDKAPI bool RtspClientPlaybackSlow( int engineId)
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientPlaybackSlow() MCRSDK not init\n");
		return false;
	}

    HPR_Guard g_guard(&oEngineLock[engineId]);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientPlaybackSlow() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        return false;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
    	LOGD("RtspClientSDK::RtspClientPlaybackSlow->engine[%d] engineId invalid", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return false;
    }

    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        LOGD("RtspClient->RtspClientPlaybackSlow() not find rtsp engine\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine == NULL)
    {
        LOGD("RtspClient->RtspClientPlaybackSlow() rtsp engine is null\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
        return false;
    }

    if(iScales[engineId] > MIN_SCALE)
    {
    	iScales[engineId]--;
		if(pRtspClientEngine->ChangeRate(iScales[engineId]) == HPR_FALSE)
		{
			LOGD("RtspClient->RtspClientPlaybackSlow() engine[%d] change from %d rate to %d fail\n", engineId, iScales[engineId], iScales[engineId]);
			return false;
		}
    }

    return true;
}

/** @fn bool RtspClientPlaybackFast
 *   @brief 设置回放速率
 *   @param [in] handle - 引擎句柄
 *   @param [in] scale为速率改变的方向, >1 播放速率加倍, <1 播放速率减半, =1 恢复正常播放速率
 *   @param [out] 无
 *   @return true - 暂停成功, false - 暂停失败
 */
RTSPCLIENTSDKAPI bool RtspClientPlaybackFast( int engineId)
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientPlaybackFast() MCRSDK not init\n");
		return false;
	}

    HPR_Guard g_guard(&oEngineLock[engineId]);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientPlaybackFast() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        return false;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
    	LOGD("RtspClientSDK::RtspClientPlaybackFast->engine[%d] engineId invalid", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return false;
    }

    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        LOGD("RtspClient->RtspClientPlaybackFast() not find rtsp engine\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine == NULL)
    {
        LOGD("RtspClient->RtspClientPlaybackFast() rtsp engine is null\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
        return false;
    }

    if(iScales[engineId] < MAX_SCALE)
	{
    	iScales[engineId]++;
    	if(pRtspClientEngine->ChangeRate(iScales[engineId]) == HPR_FALSE)
		{
			LOGD("RtspClient->RtspClientPlaybackFast() engine[%d] change rate to %d fail\n", engineId, iScales[engineId]);
			return false;
		}
	}

    return true;
}

/** @fn bool RtspClientPlaybackNormal
 *   @brief 设置回放速率
 *   @param [in] handle - 引擎句柄
 *   @param [in] scale为速率改变的方向, >1 播放速率加倍, <1 播放速率减半, =1 恢复正常播放速率
 *   @param [out] 无
 *   @return true - 暂停成功, false - 暂停失败
 */
RTSPCLIENTSDKAPI bool RtspClientPlaybackNormal( int engineId)
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientPlaybackNormal() MCRSDK not init\n");
		return false;
	}

    HPR_Guard g_guard(&oEngineLock[engineId]);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientPlaybackNormal() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        return false;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
    	LOGD("RtspClientSDK::RtspClientPlaybackNormal->engine[%d] engineId invalid", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return false;
    }

    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        LOGD("RtspClient->RtspClientPlaybackNormal() not find rtsp engine\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine == NULL)
    {
        LOGD("RtspClient->RtspClientPlaybackNormal() rtsp engine is null\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
        return false;
    }

    if(pRtspClientEngine->ChangeRate(NORMAL_SCALE) == HPR_FALSE)
    {
        LOGD("RtspClient->RtspClientPlaybackNormal() engine[%d] change rate fail\n", engineId);
        return false;
    }

    iScales[engineId] = NORMAL_SCALE;

    return true;
}

/** @fn bool RtspClientPause(void* handle)
 *   @brief 暂停
 *   @param [in] handle - 引擎句柄
 *   @param [out] 无
 *   @return true - 成功, false - 失败
 */
RTSPCLIENTSDKAPI bool RtspClientPause( int engineId )
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientPause() MCRSDK not init\n");
		return false;
	}

	HPR_Guard g_guard(&oEngineLock[engineId]);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientPause() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        return false;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
    	LOGD("RtspClientSDK::RtspClientPause->engine[%d] engineId invalid", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return false;
    }

    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        LOGD("RtspClient->RtspClientPause() not find rtsp engine\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine == NULL)
    {
        LOGD("RtspClient->RtspClientPause() rtsp engine is null\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
        return false;
    }

    if(pRtspClientEngine->PlayBackPause() == HPR_FALSE)
    {
        LOGD("RtspClient->RtspClientPause() pause fail\n");
        return false;
    }

    LOGD("RtspClient->RtspClientPause() success\n");
    return true;
}

/** @fn bool RtspClientResume(void* handle)
 *   @brief  恢复
 *   @param [in] handle - 引擎句柄
 *   @param [out] 无
 *   @return true - 成功, false - 失败
 */
RTSPCLIENTSDKAPI bool RtspClientResume( int engineId )
{
	if(!gInitLib) {
		LOGD("RtspClient->RtspClientResume() MCRSDK not init\n");
		return false;
	}

    HPR_Guard g_guard(&oEngineLock[engineId]);

    if(!bInitLib)
    {
        LOGD("RtspClient->RtspClientResume() not init lib\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_INIT;
        return false;
    }

    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
    	LOGD("RtspClientSDK::RtspClientResume->engine[%d] engineId invalid", engineId);
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return false;
    }

    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        LOGD("RtspClient->RtspClientResume() not find rtsp engine\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return false;
    }

    CRtspClientEngine* pRtspClientEngine = iter->second;
    if (pRtspClientEngine == NULL)
    {
        LOGD("RtspClient->RtspClientResume() rtsp engine is null\n");
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NULL;
        return false;
    }

    if(pRtspClientEngine->PlayBackResume() == HPR_FALSE)
    {
        LOGD("RtspClient->RtspClientResume() resume fail\n");
        return false;
    }

    bBackFinish[engineId] = false;
    return true;
}

/** @fn int RtspClientGetLastError()
 *  @brief  获取错误码
 *  @param [in] 无
 *  @param [out] 无
 *  @return 错误码
 */
RTSPCLIENTSDKAPI int RtspClientGetLastError()
{
	int errorCode = g_nRtspClientErrorCode;
//	g_nRtspClientErrorCode = 0;
    return errorCode;
}

/** @fn int GetRtspClientVersion()
 *  @brief  获取版本号
 *  @param [in] 无
 *  @param [out] 无
 *  @return 错误码
 */
RTSPCLIENTSDKAPI char* GetRtspClientVersion()
{
    return (char*)"RtspClientSDK V1.0.0(build20170523)";
}


RTSPCLIENTSDKAPI const char * RtspClientGetMediaHeader( int engineId, int *len)
{
    if(!gInitLib)
    {
        CMCRPrint::Print("RtspClient->RtspClientReleaseEngine() MCRSDK not init\n");
        return NULL;
    }
    
    HPR_Guard g_guard(&oClientLock);
    
    if(!bInitLib)
    {
        CMCRPrint::Print("RtspClient->RtspClientReleaseEngine() not init lib\n");
        return NULL;
    }
    
    if(engineId < 0 || engineId >= ENGINE_MAX_NUM)
    {
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
        return NULL;
    }
    
    //查找指向键engineId的元素，返回指向元素的指针，如果元素不存在，则指向g_RtspEngine_Map.end()
    map<int, CRtspClientEngine*>::iterator iter = oRtspEngine_Map.find(engineId);
    if(iter == oRtspEngine_Map.end())
    {
        g_nRtspClientErrorCode = RTSPCLIENT_ENGINE_NOT_EXSIT;
        return NULL;
    }
    
    CRtspClientEngine* pRtspClientEngine = iter->second;
    
    return pRtspClientEngine->fetchMediaHeader(len);
}
