/*******************************************************************************
 Copyright 2007-2008 Hikvision Digital Technology Co., Ltd.
 FileName:		RtspParser.cpp
 Description:	RtspParser.h Rtsp信令解析
 Author:	    weilinfeng
 Date:			20013-06-28
 Modification History:
 *******************************************************************************/

#include "RtspParser.h"
#include "MCRPrint.h"
#include "base64.h"
#include "CString.h"

CRtspParser::CRtspParser()
{
}

CRtspParser::~CRtspParser()
{
}

/*******************************************************************************
 Function: IsEnd()
 Description: 判断Rtsp信令是否完整
 Input: buf - 数据buf
 buflen	- buf长度
 Output:      	
 Return: true - 已接收完， false - 未接收完
 *******************************************************************************/
bool CRtspParser::IsEnd(char* buf, int buflen)
{
	bool bRet = false;
	char sContentLenBuf[32];
	memset(sContentLenBuf, 0, 32);
	int len = strlen(buf);
	if (len < 4)
	{
		LOGD("CRtspParser::IsEnd() recv buf len:%d < 4\n", len);
		return false;
	}	
	
	bRet = GetValueIngoreCase(buf, (char*)"Content-length", sContentLenBuf, 32);
	if (bRet)
	{
        LOGD("CRtspParser::IsEnd() Find Value of Key 'Content-length'\n");
		HPR_INT32 packetContentLen = HPR_Atoi32(sContentLenBuf);
		char* packetHeaderEndPos = HPR_Strstr(buf, "\r\n\r\n");
		if (!packetHeaderEndPos)
		{
			LOGD("CRtspParser::IsEnd() Not Find Packet Header End Pos\n");
			return false;
		}
		
		int packetHeaderLen = packetHeaderEndPos + 4 - buf;
		LOGD("\n CRtspParser::IsEnd()  packetHeaderLen=%d, packetContentLen=%d, buflen=%d\n", packetHeaderLen, packetContentLen, buflen);
		if ((buflen + 5) >= packetHeaderLen + packetContentLen && (buflen - 5) <= packetHeaderLen + packetContentLen)
		{ // 在有些网络状态下，某些网关会改变content-length值，这里这种相等判断有问题，需要改变判断方式
			return true;
		}
		else 
		{
			LOGD("CRtspParser::IsEnd() packetHeaderLen incorrect\n");
			return false;
		}
	}
	else
	{
		char* packetEndPos = HPR_Strstr(buf, "\r\n\r\n");
		if (packetEndPos == NULL){
			LOGD("CRtspParser::IsEnd() Not Find Packet End Pos\n");
			return false;
		}

		if ((packetEndPos + 4 - buf) == buflen){
			return true;
		}
		LOGD("CRtspParser::IsEnd() buflen is wrong\n");
	}
	// 报文中有可能会收到视频数据，接收长度与实际报文长度不同时，依然认为成功。hyf 2016-9-8
    return true;
}

/**	@fn GetRedirectUrl
 * 	@author	weilinfeng
 *	@brief  获取Rtsp重定向地址
 *	@param srcBuf - 输入buf [in]
 *	@param strRedirectUrl - 重定向的rtsp地址 [out]
 *	@return	true - 解析成功
 *			false - 解析失败
 */
bool CRtspParser::GetRedirectUrl(char* srcBuf, char* strRedirectUrl)
{
	if (srcBuf == NULL || strRedirectUrl == NULL)
	{
		LOGD("CRtspParser::GetRedirectUrl() input param error\n");
		return false;
	}

	char chRedirectUrl[MAX_URL_LEN];
	memset(chRedirectUrl, 0, MAX_URL_LEN);

	bool bRet = GetValueIngoreCase(srcBuf, (char*)"Location", chRedirectUrl, MAX_URL_LEN);
	if (!bRet)
	{
		LOGD("CRtspParser::GetRedirectUrl() get redirect url fail\n");
		return false;
	}

	memcpy(strRedirectUrl, chRedirectUrl, MAX_URL_LEN);

	return true;
}

/*******************************************************************************
 Function:    	GetSdpUrl()
 Description:	解析sdp信息
 buf  -	输入buf
 buflen	- 输入buf长度
 sdpinfo - sdp信息结构buf[out]
 urlbuf	- DESCRIBE URL
 Output:      	
 Return: true - 解析成功, false - 解析失败
 *******************************************************************************/
bool CRtspParser::GetSdpUrl(char* buf, int buflen, PSDP_INFO sdpinfo, char *urlbuf)
{
	LOGD("CRtspParser::GetSdpUrl()\n");
	
	if(!buf || buflen <= 0 ) {
		LOGD("CRtspParser::GetSdpUrl() can not find 'Content-Base\n");
		return false;
	}

	memset(sdpinfo, 0, sizeof(SDP_INFO));
	sdpinfo->mediacount = 0;

	char contentLengthBuf[32];
	char contentBaseBuf[MAX_URL_LEN+1];
	memset(contentBaseBuf, 0, MAX_URL_LEN+1);
	memset(contentLengthBuf, 0, 32);

	// 获取SDP中的Content-Base值
	bool bRet = GetValueIngoreCase(buf, (char*)"Content-Base", contentBaseBuf, MAX_URL_LEN+1);
	if (!bRet)
	{
		LOGD("CRtspParser::GetSdpUrl() can not find 'Content-Base\n");
	}

	// 获取报文体的长度
	bRet = GetValueIngoreCase(buf, (char*)"Content-Length", contentLengthBuf, 32);
	if (!bRet)
	{
		LOGD("CRtspParser::GetSdpUrl() can not find 'Content-Length', fail\n");
		return false;
	}

	int iContentLen = atoi(contentLengthBuf);
	char* posbody = buf + buflen - iContentLen;

	 ///////////////////////////////////会话级////////////////////////////////////////////
	// 从报文体的开始位置查找"m="
	char* posmedia = HPR_Strcasestr(posbody, "m=");
	if (!posmedia)
	{
		LOGD("CRtspParser::GetSdpUrl() can not find 'm=', fail\n");
		return false;
	}

	char* pos1 = NULL;
	char* pos2 = NULL;
	char* pos3 = NULL;
	char* pos4 = NULL;

	char baseurl[MAX_URL_LEN+1] = {0};

	int baseurllen = strlen(contentBaseBuf);
	if(baseurllen != 0)
	{
		strcpy(baseurl, contentBaseBuf);
		LOGD(contentBaseBuf);
	}


	/////////////////////媒体级数据//////////////////////////////////////////////////

	pos1 = HPR_Strcasestr(posmedia, "a=Media_header");
	if(!pos1)
	{
		LOGD("CRtspParser::GetSdpUrl() parse a=Media_header begin fail\n");
		return false;
	}

	pos2 = HPR_Strstr(pos1, ":");
	if(!pos2)
	{
		LOGD("CRtspParser::GetSdpUrl() parse a=Media_header : fail\n");
		return false;
	}

	pos2 = HPR_Strstr(pos2+1, "MEDIAINFO=");
	if(!pos2)
	{
		LOGD("CRtspParser::GetSdpUrl() parse MEDIAINFO= begin fail\n");
		return false;
	}

	pos3 = HPR_Strstr(pos2 + 10, "\r\n");
	if(!pos3)
	{
		LOGD("CRtspParser::GetSdpUrl() parse MEDIAINFO= end fail\n");
		return false;
	}

	pos4 = HPR_Strstr(pos2 + 10, ",");
	char encHeader[MAX_HEAD_LEN+1];
	memset(encHeader, 0, MAX_HEAD_LEN+1);
	int encHeaderlen;
	if ((!pos4) || (pos4 >= pos3))
	{

		encHeaderlen = (int)(pos3 - pos2 - 10);
		if ((encHeaderlen <= 0) || (encHeaderlen > MAX_HEAD_LEN))
		{
			LOGD("CRtspParser::GetSdpUrl() parse encHeaderlen incorrect1\n");
			return false;
		}
		memcpy(encHeader, pos2 + 10, encHeaderlen);

		char* temPos = NULL;
		temPos = HPR_Strstr(encHeader, ";");
		if (temPos)
		{
			encHeaderlen = temPos - encHeader;
			memset(encHeader+encHeaderlen, 0, MAX_HEAD_LEN+1 - encHeaderlen);
		}

		LOGD(encHeader);

		bool ret = GetHeadInfoBySDP(encHeader, encHeaderlen, sdpinfo->header, sdpinfo->headlen);
		if (!ret)
		{
			LOGD("CRtspParser::GetSdpUrl() GetHeadInfoBySDP fail\n");
			return false;
		}

		LOGD("CRtspParser::GetSdpUrl() media header: \n");
		LOGD(sdpinfo->header);
	}
	else {
		int encStrlen = pos4 - pos2 - 10;
		if ((encStrlen<=0) || (encStrlen> MAX_ENCRYPTDES_LEN)) {
			LOGD("CRtspParser::GetSdpUrl() parse encStrlen incorrect\n");
			return false;
		}

		if(HPR_Strncasecmp(pos2 + 10, "base64", 6) != 0) {
			LOGD("CRtspParser::GetSdpUrl() parse not base64\n");
			return false;
		}

		encHeaderlen = pos3- pos4 - 1;
		if ((encHeaderlen <= 0)||(encHeaderlen > MAX_HEAD_LEN)) {
			LOGD("CRtspParser::GetSdpUrl() encHeaderlen  incorrect2\n");
			return false;
		}

		strncpy(encHeader, pos4 + 1, encHeaderlen);
		memset(sdpinfo->header, 0, MAX_HEAD_LEN);
		sdpinfo->headlen = decode_base64((unsigned char *)sdpinfo->header, encHeader);
		if(!sdpinfo->headlen) {
			LOGD("CRtspParser::GetSdpUrl() decode_base64  incorrect2\n");
			return false;
		}

		LOGD("CRtspParser::GetSdpUrl() media header: \n");
		LOGD(sdpinfo->header);
	}

	while (posmedia)
	{
		char* beginpos = HPR_Strcasestr(posmedia, "a=control");
		if(!beginpos)
		{
			LOGD("CRtspParser::GetSdpUrl() posmedia, a=control beginpos fail\n");
			//return false;
			break;
		}

		char* endpos = HPR_Strstr(beginpos, "\r\n");
		if (!endpos)
		{
			LOGD("CRtspParser::GetSdpUrl() posmedia, a=control endpos fail\n");
			//return false;
			break;
		}

		int urllen = endpos - beginpos - 10;
		if ((urllen <= 0) || (urllen > MAX_URL_LEN))
		{
			LOGD("CRtspParser::GetSdpUrl() posmedia, urllen incorrect\n");
			//return false;
			break;
		}

		if (sdpinfo->mediacount > MAX_SINGLERTSP_RTP_COUNT)
		{
			LOGD("CRtspParser::GetSdpUrl() posmedia, sdpinfo.mediacount too big\n");
			//return false;
			break;
		}

		memset(sdpinfo->mediaurl[sdpinfo->mediacount], 0, MAX_URL_LEN+1);

		if (baseurllen > 0)
		{
			if ((urllen < 7) || (HPR_Strncasecmp(beginpos+10, "rtsp://", 7) != 0))
			{
				memcpy(sdpinfo->mediaurl[sdpinfo->mediacount], baseurl, baseurllen);
				if((sdpinfo->mediaurl[sdpinfo->mediacount][baseurllen-1]) != '/') {
					memcpy(sdpinfo->mediaurl[sdpinfo->mediacount] + baseurllen, "/", 1);
				}
				memcpy(sdpinfo->mediaurl[sdpinfo->mediacount] + baseurllen + 1, beginpos + 10, urllen);
			}
			else
			{
				strncpy(sdpinfo->mediaurl[sdpinfo->mediacount], beginpos + 10, urllen);
			}
		}
		else
		{
			if((urllen < 7) || (HPR_Strncasecmp(beginpos+10, "rtsp://", 7) != 0))
			{
				int len = strlen(urlbuf);
				memcpy(sdpinfo->mediaurl[sdpinfo->mediacount], urlbuf, len);
				if(sdpinfo->mediaurl[sdpinfo->mediacount][len-1] != '/') {
					memcpy(sdpinfo->mediaurl[sdpinfo->mediacount] + len, "/", 1);
				}
				memcpy(sdpinfo->mediaurl[sdpinfo->mediacount] + len + 1, beginpos + 10, urllen);
			}
			else
			{
				memcpy(sdpinfo->mediaurl[sdpinfo->mediacount], beginpos + 10, urllen);
			}

		}

		sdpinfo->mediacount++;
		if(sdpinfo->mediacount > MEDIA_INFO_MAX_NUM) {
			return true;
		}

		posmedia = HPR_Strcasestr(posmedia+1, "m=");
	}

	if (sdpinfo->mediacount <= 0)
	{
		return false;
	}

	return true;
}

/*******************************************************************************
 Function:    	GetValueIngoreCase()
 Description:	解析键对应的值，键为字符串（忽略大小写）
 Input:			srcBuf		-	输入buf
 strTarget	-	键值
 Output:      	dstBuf - 输出buf
 dstBufLen	-	输出内容长度
 Return:		true - 解析成功, false - 解析失败
 *******************************************************************************/
bool CRtspParser::GetValueIngoreCase(char* srcBuf, char* strTarget, char* dstBuf, int dstBufLen)
{
	if (!srcBuf || !strTarget || !dstBuf)
	{
		return false;
	}
	memset(dstBuf, 0, dstBufLen);

	// Key的长度小于0
	int targetLen = strlen(strTarget);
	if (targetLen <= 0)
	{
		return false;
	}

	// 查找Key, 忽略大小写
	char* keyPos = HPR_Strcasestr(srcBuf, strTarget);
	if(!keyPos)
	{
		//CRtspClientPrint::Print("CRtspParser::GetValueIngoreCase() failed(don't find key)\n");
		return false;
	}

	// 查找key后面的冒号
	char* colonPos = HPR_Strstr(keyPos, ":");
	if (!colonPos)
	{
		LOGD("CRtspParser::GetValueIngoreCase() failed(don't find :)\n");
		return false;
	}

	// 查找key这一行的结束位置, 若没有找到\r\n, 则认为剩余的部分都是value值
	char* endPos = HPR_Strstr(colonPos, "\r\n");
	if (!endPos)
	{
		endPos = srcBuf + strlen(srcBuf);
	}

	// 若key对应的value长度大于缓冲区长度，则返回false
	if ((endPos - colonPos) + 1 > dstBufLen)
	{
		LOGD("CRtspParser::GetValueIngoreCase() failed(the len of value > dstBufLen)\n");
		return false;
	}

	// 获取Value值
	memcpy(dstBuf, colonPos + 1, endPos - colonPos - 1);
	CCString::trim_lc(dstBuf);
	return true;
}

/*******************************************************************************
 Function:    	GetCodeIndex()
 Description:	解析状态码
 Input:			buf		-	输入buf
 Output:      	
 Return:		true - 解析成功, false - 解析失败
 *******************************************************************************/
status_code CRtspParser::GetStatusCodeIndex(char* buf)
{
	int len = strlen(buf);
	
	if (len < 9)
	{
		return RTSP_STATUS_UNK;
	}
	
	if (HPR_Strncasecmp(buf, "rtsp/", 5) != 0)
	{
		return RTSP_STATUS_UNK;
	}
	
	char* pos = HPR_Strstr(buf, "\r\n");
	if (!pos) 
	{
		return RTSP_STATUS_UNK;
	}
	
	char stauscode[32];
	memset(stauscode, 0, 32);
	memcpy(stauscode, buf + 9, 3);
	stauscode[3] = '\0';
	
	return m_oRtspCommon.ConvertRtspStatus2Index(stauscode);
}

/*******************************************************************************
 Function:    	GetSession()
 Description:	解析session值
 Input:			srcBuf		-	输入buf
 strTarget	-	键值
 Output:      	dstBuf		-	输出buf
 dstBufLen	-	输出部分长度
 Return:		true - 解析成功, false - 解析失败
 *******************************************************************************/
bool CRtspParser::GetSession(char* srcBuf, char* strTarget, char* dstBuf, int dstBufLen)
{
	LOGD("CRtspParser::GetSession()\n");
	
	int targetLen = 0;
	char* keyPos = NULL;
	char* colonPos = NULL;
	char* endPos = NULL;
	char* fhPos = NULL;
	
	if (!srcBuf) 
	{
		return false;
	}
	if (!strTarget) 
	{
		return false;
	}
	if (!dstBuf) 
	{
		return false;
	}
	
	memset(dstBuf, 0, dstBufLen);

	targetLen = strlen(strTarget);
	if (targetLen <= 0) 
	{
		return false;
	}
	
	keyPos = HPR_Strcasestr(srcBuf, strTarget);
	if(!keyPos) 
	{
		LOGD("CRtspParser::GetSession() failed(don't find key)\n");
		return false;
	}
	
	colonPos = HPR_Strstr(keyPos, ":");
	if (!colonPos) 
	{
		LOGD("CRtspParser::GetSession() failed(don't find :)\n");
		return false;
	}
	
	endPos = HPR_Strstr(colonPos, "\r\n");
	if (!endPos)
	{
		LOGD("CRtspParser::GetSession() failed(don't find end)\n");
		return false;
	}

	colonPos++;
	if ((endPos - colonPos) + 1 > dstBufLen) 
	{
		LOGD("CRtspParser::GetSession() failed(the len of value > dstBufLen)\n");
		return false;
	}
	
	memcpy(dstBuf, colonPos, endPos - colonPos);
	dstBuf[endPos - colonPos] = '\0';
	
	fhPos = strstr(dstBuf, ";");
	if (fhPos)
	{
		memcpy(dstBuf, colonPos, fhPos - dstBuf);
		dstBuf[fhPos - dstBuf] = '\0';
	}
	else
	{
		memcpy(dstBuf, colonPos, endPos - colonPos);
		dstBuf[endPos - colonPos] = '\0';
	}
	
	CCString::trim_lc(dstBuf);

	return true;
}

/*******************************************************************************
 Function:    	GetServerPort()
 Description:	解析服务器rtp、rtcp端口
 Input:			srcBuf		-	输入buf
				strTarget	-	键值
 Output:      	dstBuf		-	输出buf
				dstBufLen	-	输出内容长度
				iServerport	-	RTP SERVER PORT
 Return:		true - 解析成功, false - 解析失败
 *******************************************************************************/
bool CRtspParser::GetServerPort(char* srcBuf, char* strTarget, char* dstBuf, int dstBufLen, int& iServerport)
{
	LOGD("CRtspParser::GetServerPort()\n");
	
	int targetLen = 0;
	char* keyPos = NULL;
	char* hxPos = NULL;
	char* fhPos = NULL;
	
	if (!srcBuf || !strTarget || !dstBuf) 
	{
		return false;
	}
	
	memset(dstBuf, 0, dstBufLen);

	targetLen = (size_t)strlen(strTarget);
	if (targetLen <= 0) 
	{
		return false;
	}
	
	keyPos = HPR_Strcasestr(srcBuf, strTarget);
	if(!keyPos) 
	{
		LOGD("CRtspParser::GetServerPort() failed(don't find key)\n");
		return false;
	}
	
	keyPos += strlen(strTarget) + 1;
	fhPos = HPR_Strstr(keyPos, ";");
	if (!fhPos) 
	{
		memcpy(dstBuf, keyPos, (srcBuf + strlen(srcBuf) - keyPos));
		dstBuf[srcBuf + strlen(srcBuf) - keyPos] = '\0';
	}
	else
	{
		if (fhPos - keyPos > dstBufLen)
		{
			LOGD("CRtspParser::GetServerPort() len > dstBufLen\n");
			return false;
		}
		
		memcpy(dstBuf, keyPos, fhPos - keyPos);
		dstBuf[fhPos - keyPos] = '\0';
	}

	CCString::trim_lc(dstBuf);

	hxPos = HPR_Strstr(dstBuf, "-");
	if (!hxPos)
	{
		iServerport = HPR_Atoi32(dstBuf);
	}
	else
	{
#ifdef HK_APPLE
        hxPos = (char *)"";
#else
		hxPos = '\0';
#endif
		iServerport = HPR_Atoi32(dstBuf);
	}
	
	LOGD("CRtspParser::GetServerPort() rtp iServerport:%d \n", iServerport);

	return true;
}

/*******************************************************************************
 Function:    	GetInterleaved()
 Description:	解析Rtp or Rtsp模式先的服务器端口
 Input:			srcBuf		-	输出buf
 strTarget	-	键值
 Output:      	dstBuf		-	输出buf
 dstBufLen	-	输出内容长度
 iInterleaved-	RTP
 Return:		true - 解析成功, false - 解析失败
 *******************************************************************************/
bool CRtspParser::GetInterleaved(char* srcBuf, char* strTarget, char* dstBuf, int dstBufLen, int& iInterleaved)
{
	LOGD("CRtspParser::GetInterleaved()\n");
	
	int targetLen = 0;
	char* keyPos = NULL;
	char* hxPos = NULL;
	char* fhPos = NULL;
	
	if (!srcBuf || !strTarget || !dstBuf) 
	{
		return false;
	}
	
	memset(dstBuf, 0, dstBufLen);

	targetLen = strlen(strTarget);
	if (targetLen <= 0) 
	{
		return false;
	}
	
	keyPos = HPR_Strcasestr(srcBuf, strTarget);
	if(!keyPos) 
	{
		LOGD("CRtspParser::GetInterleaved() failed(don't find key)\n");
		return false;
	}
	
	keyPos += strlen(strTarget) + 1;
	fhPos = strstr(keyPos, ";");
	if (!fhPos) 
	{
		memcpy(dstBuf, keyPos, (srcBuf + strlen(srcBuf) - keyPos));
		dstBuf[(srcBuf + strlen(srcBuf) - keyPos)] = '\0';
	}
	else
	{
		if (fhPos - keyPos > dstBufLen)
		{
			LOGD("CRtspParser::GetInterleaved() len > dstBufLen\n");
			return false;
		}
		
		memcpy(dstBuf, keyPos, fhPos - keyPos);
		dstBuf[fhPos - keyPos] = '\0';
	}

	CCString::trim_lc(dstBuf);

	hxPos = strstr(dstBuf, "-");
	if (!hxPos)
	{
		iInterleaved = atoi(dstBuf);
	}
	else
	{
#ifdef HK_APPLE
        hxPos = (char *)"";
#else
        hxPos = '\0';
#endif
		iInterleaved = atoi(dstBuf);
	}
	
	return true;
}

/*******************************************************************************
 Function:    	GetServerSsrc()
 Description:	解析服务器 ssrc
 Input:			srcBuf		-	shurubuf
 strTarget	-	键值
 Output:      	dstBuf		-	输出buf
 dstBufLen	-   输出内容长度
 Return:		true - 解析成功, false - 解析失败
 *******************************************************************************/
bool CRtspParser::GetServerSsrc(char* srcBuf, char* strTarget, char* dstBuf, int dstBufLen)
{
	LOGD("CRtspParser::GetServerSsrc()\n");
	
	int targetLen = 0;
	char* keyPos = NULL;
	char* fhPos = NULL;
	
	if (!srcBuf || !strTarget || !dstBuf) 
	{
		return false;
	}
	memset(dstBuf, 0, dstBufLen);
	
	targetLen = strlen(strTarget);
	if (targetLen <= 0) 
	{
		return false;
	}
	
	keyPos = HPR_Strcasestr(srcBuf, strTarget);
	if (!keyPos) 
	{
		LOGD("CRtspParser::GetServerSsrc() failed(don't find key)\n");
		return false;
	}
	
	keyPos += strlen(strTarget) + 1;
	fhPos = strstr(keyPos, ";");
	if (!fhPos) 
	{
		memcpy(dstBuf, keyPos, (srcBuf + strlen(srcBuf) - keyPos));
		dstBuf[(srcBuf + strlen(srcBuf) - keyPos)] = '\0';
	}
	else
	{
		if (fhPos - keyPos > dstBufLen)
		{
			LOGD("CRtspParser::GetServerSsrc() len > dstBufLen\n");
			return false;
		}
		
		memcpy(dstBuf, keyPos, fhPos - keyPos);
		dstBuf[fhPos - keyPos] = '\0';
	}
	
	CCString::trim_lc(dstBuf);

	return true;
	
}


/*******************************************************************************
 Function:    	GetHeadInfoBySDP
 Description:	解析Stream Header
 Input:			enchead		-	输入buf
 srcLen	-		输入内容长度
 Output:      	dst		-	输出buf
 dstBufLen	-   输出内容长度
 Return:		true - 解析成功, false - 解析失败
 *******************************************************************************/
HPR_BOOL CRtspParser::GetHeadInfoBySDP(const char* enchead, int srcLen, char* dst, int& dstLen)
{
    if ((NULL == enchead) || (NULL == dst) || (80 != srcLen))
    {
    	LOGD("CRtspParser::GetHeadInfoBySDP srcLen is %d \n", srcLen);
        return HPR_FALSE;
    }

    memset(dst, 0, dstLen);

    int len = 0;
    for (int i = 0; i < srcLen; i += 2)
    {
        if (enchead[i] >= '0' && enchead[i] <= '9')
        {
            dst[i/2] += enchead[i] - '0';
        }
        else if(enchead[i] >= 'a' && enchead[i] <= 'f')
        {
            dst[i/2] += enchead[i] - 'a' + 10;
        }
        else if(enchead[i] >= 'A' && enchead[i] <= 'F')
        {
            dst[i/2] += enchead[i] - 'A' + 10;
        }
        else
        {
            return HPR_FALSE;
        }
        dst[i/2] *= 16;
        if (enchead[i+1] >= '0' && enchead[i+1] <= '9')
        {
            dst[i/2] += enchead[i+1] - '0';
        }
        else if(enchead[i+1] >='a' && enchead[i+1] <= 'f')
        {
            dst[i/2] += enchead[i+1] - 'a' + 10;
        }
        else if(enchead[i+1] >='A' && enchead[i+1] <= 'F')
        {
            dst[i/2] += enchead[i+1] - 'A' + 10;
        }
        else
        {
            return HPR_FALSE;
        }
        len++;
    }

    dstLen = len;
    if (len != dstLen)
    {
    	LOGD("parse headLen[%d] is error \n", len);
        return HPR_FALSE;
    }
    return HPR_TRUE;
}

/*******************************************************************************
 Function:    	GetExtErrCode()
 Description:	解析状态码
 Input:			buf		-	输入buf
 Output:
 Return:		ExtErrCode
 *******************************************************************************/
long CRtspParser::GetExtErrCode(char* buf)
{
	int len = strlen(buf);

	if (len < 9)
	{
		return RTSP_STATUS_UNK;
	}

	// 查找Key, 忽略大小写
	char* keyPos = HPR_Strcasestr(buf, "ExtErrInfo");
	if(!keyPos)
	{
		//CRtspClientPrint::Print("CRtspParser::GetValueIngoreCase() failed(don't find key)\n");
		return RTSP_STATUS_UNK;
	}

	char* pos = HPR_Strstr(buf, "errcode=");
	if (!pos)
	{
		return RTSP_STATUS_UNK;
	}

	char* pos1 = HPR_Strstr(pos, ";");
	if (!pos1)
	{
		return RTSP_STATUS_UNK;
	}

	char errcode[32];
	memset(errcode, 0, 32);
	memcpy(errcode, pos + 8, pos1 - pos - 8);
	errcode[pos1 - pos - 7] = '\0';
	LOGD("errcode is :%s \n", errcode);
	return atol(errcode);
}
