/*
 * TalkClientSDK.h
 *
 *  Created on: 2014-6-6
 *      Author: weilinfeng
 */

#ifndef TALKCLIENTSDK_H_
#define TALKCLIENTSDK_H_

#ifdef __cpluscplus
#define TALKCLIENTSDK_API extern
#else
#define  TALKCLIENTSDKAPI
#endif

#include "Define.h"

/*****************状态码*****************************/
#ifndef AUDIOCODEC_ERR_SUCCESS
# define AUDIOCODEC_ERR_SUCCESS 0
#endif

#ifndef AUDIOCODEC_ERR_NOT_SUPPORT_ENCODE
# define AUDIOCODEC_ERR_NOT_SUPPORT_ENCODE 1
#endif

#ifndef AUDIOCODEC_ERR_NOT_SUPPORT_DECODE
# define AUDIOCODEC_ERR_NOT_SUPPORT_DECODE 2
#endif


#define REGISTER_FAIL 		100	//注册失败
#define KEEPALIVE_FAIL		150	// 心跳保活失败
#define SUCCESS				200	//请求命令已成功处理
#define ERROR				400	//通用错误码，服务器无法精确描述的错误均返回ERROR错误
#define ENOTEXIST			401	//参考REGISTER命令中返回码
#define EACCESS				402	//验证未通过，用户名或密码错误
#define ENOTREG				403	//参考KEEPALIVE命令中返回码
#define EREGONOTHERPU		404	//相同用户名在其他终端上注册
#define EINVALIDPARA		410	//非法参数，一般是缺少参数或参数值有误
#define ECHANNELNOTEXIST	411	//指定频道不存在
#define EALREADYINCHANNEL	412	//设备已经在其它频道内，需先退出频道
#define EDECLINE			413	//被叫方拒绝了呼叫
#define EUNREACHABLE		414	//目的地址不可达
#define EUSRISNOTINCHANNEL	415	//用户不在指定频道内
#define EALREADYINCALL		416	//用户已在其他单呼会话中
#define ENORESPONSE			417	//对方未响应
#define ECALLTERMINATE		418	//呼叫终止
#define EALLOCMEM			500	//内存分配失败

#define USER_NOT_REGISTER  	10000  //用户未注册或已经注销
#define STATE_ERROR        	10001  //当前状态不正确
#define GET_VAG_INFO_FAIL  	10002  //从CMS获取VAG信息失败
#define LOGIN_VAG_FAIL     	10003  // 登录VAG失败
#define START_VAG_TALK_FAIL	10004  //开启VAG语音对讲失败
#define HANDLEING_STATE    	10005  // 正在处理状态，用于开启和停止语音

#ifndef TYPE_G711_U
#define TYPE_G711_U 1
#endif

#ifndef TYPE_G711_A
#define TYPE_G711_A 2
#endif

#ifndef TYPE_G722
#define TYPE_G722 3
#endif

#ifndef TYPE_G726
#define TYPE_G726 4
#endif

#ifndef TYPE_AAC
#define TYPE_AAC 5
#endif

#ifndef TYPE_MP3
#define TYPE_MP3 6
#endif

// 登陆信息
typedef struct
{
	char puID[USER_ID_MAX_LEN];		// 设备PUID,如果是cu，则传cuUserID
    char servIP[IP_MAX_LEN];		// 服务器IP，用MAG的IP
    unsigned short servPort;    	// 注册服务器端口，MAG对讲端口
    char cuUserID[USER_ID_MAX_LEN]; // 客户端UserID，用登陆MSP的SessionID表示
    char password[PSW_MAX_LEN];		// 登录对讲服务器的密码，暂时不用，可以为空或者12345
    unsigned short type;   			// 终端类型，0：单兵；1：客户端（包括mpu）；2：车载设
    int codecType;					// 音频编码类型
}ST_LOGIN_INFO, *PST_LOGIN_INFO;

// 对讲信息
typedef struct
{
	char cuUserID[USER_ID_MAX_LEN]; 			// 客户端UserID，用登陆MSP的SessionID表示
	char toUserID[USER_ID_MAX_LEN]; 			// 接收方的UserID，使用对讲器的indexCode，用于对讲服务器对讲(目前接VAG可使用deviceIndexCode)
	char deviceIndexCode[DEVICE_INDEX_MAX_LEN];	// 设备序列号，用于VAG对讲
	unsigned int  deviceType;   						// 设备类型，用于VAG对讲30000
    unsigned int  channelNum;							// 对讲通道编号，用于VAG对讲，目前统一传0
}ST_TALK_INFO, *PST_TALK_INFO;

// 群组信息
typedef struct
{
	unsigned short vChannel; 							// 频道：范围在0~128（十进制）的整数，具有相同频道的终端属于同一个对讲组；
    char vChannelPsw[PSW_MAX_LEN];						// 原始密码长度不超过8位，可包含数字、字母、下划线；频道密码的32位MD5值；0频道无需密码验证；
}ST_GROUP_JOIN_INFO, *PST_GROUP_JOIN_INFO;

// 点对点信息
typedef struct
{
	char toUserID[USER_ID_MAX_LEN]; 		// 接收方的UserID，使用对讲器的indexCode
	char msgContent[CONTENT_MAX_LEN];					// 消息内容，长度暂定1024, 采用GB2312编码
}ST_MSG_INFO, *PST_MSG_INFO;

// 群组信息
typedef struct
{
	unsigned short  vChannel; 							// 频道：范围在0~128（十进制）的整数，具有相同频道的终端属于同一个对讲组；
	char msgContent[CONTENT_MAX_LEN];					// 消息内容，长度暂定1024, 采用GB2312编码
}ST_MSG_GROUP_INFO, *PST_MSG_GROUP_INFO;

// 对讲信息
typedef struct
{
	char deviceIndexCode[DEVICE_INDEX_MAX_LEN];			// 设备序列号
	unsigned int  deviceType;   						// 设备类型，用于VAG对讲30000
}ST_DEV_INFO, *PST_DEV_INFO;

typedef void (*pPCMDataListener)(char* buffer, unsigned int buflen, void* pUser);
typedef void (*pMessageListener)(unsigned short vChannel, char* content, unsigned int contentlen, char* fromUserID, unsigned int userIDLen, void* pUser);
typedef void (*pErrorListener)(int ErrorCode, char* pDescribe, unsigned int desLen, void* pUser);

/** @fn bool Init()
 *   @brief 初始化Talk库
 *   @return true - 初始化成功; false - 初始化库失败
 */
TALKCLIENTSDKAPI bool Init();

/** @fn bool Fini()
 *   @brief 反初始化push库
 *   @return 无
 */
TALKCLIENTSDKAPI void Fini();

/** @fn bool Login(void* handle)
 *   @brief 登陆服务
 *   @param [in] mesageLister - 消息回调函数指针
 *   @return true - 成功, false - 失败
 */
TALKCLIENTSDKAPI bool Login(ST_LOGIN_INFO &loginInfo, void* pUser);

/** @fn bool Logout(void* handle)
 *   @brief 登出服务
 *   @return true - 成功, false - 失败
 */
TALKCLIENTSDKAPI void Logout();

/** @fn bool SetErrorListener(pErrorLister errorListener)
 *   @brief 设置 消息回调
 *   @param [in] errorListener - 错误回调函数指针
 *   @return true - 成功, false - 失败
 */
TALKCLIENTSDKAPI bool SetErrorListener(pErrorListener errorListener);

/** @fn bool SetMessageListener(pMessageListener mesageListener)
 *   @brief 设置 消息回调
 *   @param [in] mesageListener - 消息回调函数指针
 *   @return true - 成功, false - 失败
 */
TALKCLIENTSDKAPI bool SetMessageListener(pMessageListener mesageListener);

/** @fn bool SetPCMDataListener(pPCMDataListener pcmDataListener)
 *   @brief 设置数据包回调
 *   @param [in] pcmDataListener - 数据回调函数指针
 *   @return true - 成功, false - 失败
 */
TALKCLIENTSDKAPI bool SetPCMDataListener(pPCMDataListener pcmDataListener);

/** @fn bool InputAudioData(const char* pData, HPR_INT32 nDataSize)
 *   @brief 塞入音频数据
 *   @param [in] InputAudioData - PCM音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
TALKCLIENTSDKAPI bool InputAudioData(const char* pData, unsigned int nDataSize);

/** @fn bool StartTalk(TALK_INFO &talkInfo)
 *   @brief 开始单呼
 *   @param [in] talkInfo - 点对点对讲信息
 *   @param [out] 无
 *   @return  true - 呼叫成功; false - 呼叫失败
 */
TALKCLIENTSDKAPI bool StartTalk(ST_TALK_INFO &talkInfo);

/** @fn bool StopTalk()
 *   @brief 停止点对点对讲
 *   @param [in] 无
 *   @param [out] 无
 *   @return  无
 */
TALKCLIENTSDKAPI void StopTalk();

/** @fn bool StopTalk()
 *   @brief 停止点对点对讲
 *   @param [in] 无
 *   @param [out] 无
 *   @return  无
 */
TALKCLIENTSDKAPI int GetDeviceEncodeType();

/** @fn int JoinGroup(GROUP_JOIN_INFO &groupJoinInfo)
 *   @brief 申请进入对讲组
 *   @param [in] groupInfo - 群组信息
 *   @param [out] 无
 *   @return  -1-加入群组指定群组失败， 0-代表终端在改组无权限，1~16的整数，代表终端在该组的权限
 */
TALKCLIENTSDKAPI bool JoinGroup(ST_GROUP_JOIN_INFO &groupJoinInfo);

/** @fn bool QuitGroup()
 *   @brief 退出对讲组
 *   @param [in] 无
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
TALKCLIENTSDKAPI bool QuitGroup();

/** @fn bool StartGroupTalk()
 *   @brief	开始群对讲
 *   @param [in] 无
 *   @return  true - 开始群对讲成功; false - 开始群对讲失败
 */
TALKCLIENTSDKAPI bool StartGroupTalk();

/** @fn bool StopGroupTalk()
 *   @brief	停止群对讲
 *   @param [in] 无
 *   @return  无
 */
TALKCLIENTSDKAPI void StopGroupTalk();

/** @fn bool SendMsg(void* handle)
 *   @brief 点对点消息发送
 *   @param [in] InputAudioData - PCM音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
TALKCLIENTSDKAPI bool SendMsg(ST_MSG_INFO &msgInfo);

/** @fn bool SendGroupMsg(void* handle)
 *   @brief 向群组发送消息
 *   @param [in] InputAudioData - PCM音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
TALKCLIENTSDKAPI bool SendGroupMsg(ST_MSG_GROUP_INFO &msgGroupInfo);

/** @fn int GetErrorCode()
 *  @brief  获取错误码
 *  @param [in] 无
 *  @param [out] 无
 *  @return 错误码
 */
TALKCLIENTSDKAPI int GetErrorCode();

#endif /* TALKCLIENTSDK_H_ */
