/*
 * TalkDefine.h
 *
 *  Created on: 2014-6-6
 *      Author: weilinfeng
 */

#ifndef TALKDEFINE_H_
#define TALKDEFINE_H_

#include "Define.h"
#include "hpr2_types.h"

#define TALK_PACKET_SIZE 1024
#define TALK_PACKET_MAX_SIZE 1024*2

#define VALUE_G711_A "G711_A"
#define VALUE_G711_A_LEN 6

#define VALUE_G711_U "G711_U"
#define VALUE_G711_U_LEN 6

#define VALUE_G722 "G722"
#define VALUE_G722_LEN 4

#define VALUE_G726 "G726"
#define VALUE_G726_LEN 4

#define VALUE_AAC "AAC"
#define VALUE_AAC_LEN 3

#define VALUE_MP3 "MP3"
#define VALUE_MP3_LEN 3

// 版本号
#define TALK_VERSION		0x01
//消息体类型： 1—二进制报文体 */
#define TALK_BODYTYPE		0x01
//消息体类型： 1—二进制报文体 */
#define CMD_REQUEST			0x00
//消息体类型： 1—二进制报文体 */
#define CMD_RESPONSE		0x01

// 信令魔术字段
#define TALK_CMD_MAGIC 0XA5B5
#define AUDIO_MAGIC 0x4101
#define GROUP_AUDIO_MAGIC 0x4100

/**************二进制命令码定义************************/
//		命令码				值		含义
#define REGISTER			0x0001	//设备注册包
#define UNREGISTER			0x0002	//设备注销
#define KEEPALIVE			0x0010	//设备心跳包
#define JOINGROUP			0x0011	//加入对讲组
#define SERVERNOTIFY		0x0021	//服务器下发通知
#define QUITGROUP			0x0022	//退出对讲组
#define INDIVIDUALCALL		0x0031	//请求语音单呼
#define QUITINDIVIDUALCALL	0x0032	//退出语音单呼
#define INDIVIDUALINFO		0x0041	//点对点消息
#define GROUPINFO			0x0042	//群组消息
#define AUDIO				0x0143	//语音
#define GROUPAUDIO			0x0144	//群组语音
#define GET_INCODETYPE		0x0051	// 获取编码类型


/**************TLV类型码定义************************/
//		类型码					值		含义
#define TYPE_PUID				0x0100	//设备的PUID
#define TYPE_ENCODETYPE			0x0101	//支持的语音编码类型
#define TYPE_INTERVAL			0x0102	//设备心跳间隔
#define TYPE_USRID				0x0103	//对讲服务器登录用户ID
#define TYPE_REGTYPE			0x0104	//群组参与者注册类型
#define TYPE_USRPSW				0x0105	//对讲服务器登录用户密码
#define TYPE_VCHANNEL			0x0200	//对讲频道
#define TYPE_VCHANNELPSW		0x0201	//对讲频道密码
#define TYPE_RANK				0x0202	//设备权限级别
#define TYPE_IP					0x0203	//IP地址
#define TYPE_PORT				0x0204	//端口
#define TYPE_LOWRANK			0x0206	//权限过低
#define TYPE_OCCUPIED			0x0207	//有其他组员在发言
#define TYPE_GROUPSCHANGED		0x0208	//后台分组已变更，PU应重新下载分组
#define TYPE_USRID_FROM			0x0209	//源USRID
#define TYPE_USRID_TO			0x020A	//目的USRID
#define TYPE_QUITINDIVIDUALCALL	0x020B	//对端已退出单呼会话
#define TYPE_INFO				0x020C	//文本消息内容
#define TYPE_SESSION			0x0302	//会话标识
/*****************接入VAG新增的字段*****************************/
#define TYPE_DEVICE_INDEX_CODE	0x1B01	//设备编号
#define TYPE_DEVICE_TYPE		0x1B02	//设备类型，需要MCU从MSP获取
#define TYPE_CHANNEL_NUM		0x1B03	//对讲通道号

typedef struct _ST_LOGIN_RES_INFO {
	unsigned short interval;
	char redirectIp[IP_MAX_LEN];
	unsigned short port;
}ST_LOGIN_RES_INFO;

typedef struct _ST_TALK_RES_INFO {
	unsigned int session;
	char fromUserID[USER_ID_MAX_LEN];
	char toUserID[USER_ID_MAX_LEN];
}ST_TALK_RES_INFO;


#define lit_endian		0x0123
#define big_endian		0x3210
#define vod_byte_order	lit_endian


/*
* talk packet header
*/
typedef struct
{
	hpr2_uint16 	magic ;			/* 报文魔术字，2字节整数，为固定值：0XA5B5 */
	hpr2_uint8  	version;		/* 协议版本号，为0x01 */
	hpr2_uint8  	bodytype;		/* 消息体类型： 1—二进制报文体 */
	hpr2_uint32  	bodylen;		/* 消息体长度 */
#if vod_byte_order == big_endian
	hpr2_uint16 	cmdtype : 2;	/* 命令类型 */
	hpr2_uint16 	reserve : 4;	/* 保留位 */
	hpr2_uint16 	cmd	   	: 10;	/* 命令码 */
#elif vod_byte_order == lit_endian
	hpr2_uint16 	cmd	   	: 10;	/* 命令码 */
	hpr2_uint16 	reserve : 4;	/* 保留位 */
	hpr2_uint16 	cmdtype : 2;	/* 命令类型 */
#endif
	hpr2_uint16 	status;			/* 状态码，仅在命令类型为回应时有意义 */
	hpr2_uint32 	sequence;		/* 报文序号，4字节整数，应答报文需填写与请求报文一致的序号 */
} talk_hdr, *ptalk_hdr;
#define talk_hdr_len sizeof(talk_hdr)

/*
* tlv item header
*/
typedef struct
{
	hpr2_uint16 	tlvtype ;		/* 消息体中每个子项的类型 */
	hpr2_uint16  datalen;		/* 消息体中每个子项携带的数据 */
} tlv_item_hdr, *ptlv_item_hdr;
#define tlv_item_hdr_len sizeof(tlv_item_hdr)

/*
* audio header
*/
typedef struct
{
	hpr2_uint16 	magic ;			/*2字节网络字节序整数，为固定值：0x4101 */
	hpr2_uint8  	offset;			/* 1字节整数 */
} audio_hdr, *paudio_hdr;
#define audio_hdr_len 3


/*
* group audio header
*/
typedef struct
{
	hpr2_uint16 	magic ;			/*2字节网络字节序整数，为固定值：0x4100 */
	hpr2_uint16  vChannel;		/* 2字节网络字节序整数，频道号； */
	hpr2_uint8  	rank;			/* 1字节整数，权限等级 */
	hpr2_uint8  	offset;			/* 1字节整数 */
} group_audio_hdr, *pgroup_audio_hdr;

#define group_audio_hdr_len 6

#endif /* TALKDEFINE_H_ */
