/*
 * TalkEngine.h
 *
 *  Created on: 2014-6-6
 *      Author: weilinfeng
 */

#ifndef TALKENGINE_H_
#define TALKENGINE_H_

#include "TalkClientSDK.h"
#include "TalkNetSDK.h"
#include "AudioCodecInferface.h"

typedef void (*PCMDataCallbackPtr)(char* pData, unsigned int nDataSize, void* pUser);
typedef void (*MessageCallbackPtr)(unsigned short vChannel, char* content, unsigned int contentlen, char* fromUserID, unsigned int userIDLen, void* pUser);
typedef void (*ErrorCallbackPtr)(unsigned int nMsgCode, char* pDescribe, unsigned int desLen, void* pUser);

class CTalkEngine: public TalkNetSDKObserver {
public:
	CTalkEngine();

	~CTalkEngine();
	// 初始化
	bool Init(void* pUser);
	// 反初始化
	void Fini();
	// 设置回调函数
	void SetCallback(PCMDataCallbackPtr pcmCallbackPtr, MessageCallbackPtr msgCallbackPtr, ErrorCallbackPtr errorCallbackPtr);
	//注册服务器
	bool Login(ST_LOGIN_INFO &loginInfo);
	// 登出服务器
	bool Logout();
	// 塞入音频数据
	bool InputAudioData(const char* pData, unsigned int nDataSize);
	// 开始点对点对讲
	bool StartTalk(ST_TALK_INFO &talkInfo);
	// 停止点对点对讲
	bool StopTalk();
	// 申请进入群组
	int GetDeviceEncodeType();
	// 申请进入群组
	bool JoinGroup(ST_GROUP_JOIN_INFO &groupJoinInfo);
	// 退出对讲组
	bool QuitGroup();
	// 开始群对讲
	bool StartGroupTalk();
	// 停止群对讲
	void StopGroupTalk();
	// 点对点消息发送
	bool SendMsg(ST_MSG_INFO &msgInfo);
	// 向群组发送消息
	bool SendGroupMsg(ST_MSG_GROUP_INFO &msgGroupInfo);
	// 获取错误码
	int GetErrorCode();

	//
	PCMDataCallbackPtr mPcmCallbackPtr;
	//
	MessageCallbackPtr mMsgCallbackPtr;
	//
	ErrorCallbackPtr mErrorCallbackPtr;

	// 对讲网络SDK对讲
	CTalkNetSDK* mCTalkNetSDK;

	//
	void* mUser;
private:
	// 网路SDK回调
	void NetSDKG7XXDataCallBack(unsigned short vChannel, char* pData, unsigned int nDataSize, char* fromUserID, unsigned int userIDLen);
	void NetSDKGMsgCallBack(unsigned short vChannel, char* content, unsigned int contentlen, char* fromUserID, unsigned int userIDLen);
	void NetSDKGErrorCallBack(unsigned int nMsgCode, char* pDescribe, unsigned int desLen);
private:
	// 初始化标识
	bool bInit;
	// 登录标识
	bool bLogin;
	// 登录标识
	bool bTalk;
	// 登录标识
	bool bGroupTalk;
	// 音频编解码类型
	int mCodecType;
	// 错误码
	int mErrorCode;
	// 编码类型
	int mEncType;
/********* 测试代码*************/
	// 录文件
//	FILE * m_file_pcm;
//	// 录文件
//	FILE * m_file_g711;
//	// 录文件
//	FILE * m_file_g711_s;
};


#endif /* TALKENGINE_H_ */
