/*
 * TalkNetSDK.h
 *
 *  Created on: 2014-6-6
 *      Author: weilinfeng
 */

#ifndef TALKNETSDK_H_
#define TALKNETSDK_H_

#include "TalkClientSDK.h"
#include "HPR_Socket.h"
#include "HPR_Mutex.h"
#include "TalkDefine.h"
#include "TalkPacket.h"
#include "UDPSer.h"


class CTalkPacket;

// 对讲服务SDK监听类
class TalkNetSDKObserver
{
public:
	virtual ~TalkNetSDKObserver() = 0;
    virtual void NetSDKG7XXDataCallBack(unsigned short vChannel, char* pData, unsigned int nDataSize, char* fromUserID, unsigned int userIDLen) = 0;
    virtual void NetSDKGMsgCallBack(unsigned short vChannel, char* content, unsigned int contentlen, char* fromUserID, unsigned int userIDLen) = 0;
    virtual void NetSDKGErrorCallBack(unsigned int nMsgCode, char* pDescribe, unsigned int desLen) = 0;
};

class CTalkNetSDK {
public:
	// 构造函数
	CTalkNetSDK();
	// 析构函数
	~CTalkNetSDK();
	// 初始化
	bool Init(TalkNetSDKObserver * observer, int codecType = TYPE_G711_A);
	// 反初始化
	void Fini();
	//注册服务器
	bool Login(ST_LOGIN_INFO &loginInfo);
	// 登出服务器
	bool Logout();

	// 获取设备的编码类型
	bool GetDivAEncType(ST_DEV_INFO &devInfo, int& encType);

	// 开始点对点对讲
	bool StartTalk(ST_TALK_INFO &talkInfo);
	// 停止点对点对讲
	bool StopTalk();
	// 发送编码后的音频数据
	bool SendG7XXData(const char* pData, HPR_INT32 nDataSize);
	// 处理回复的报文
	void HandleReqPacket(const char* pPackBuffer, unsigned int nLen, const char* szFrom, unsigned short int nPort, unsigned long dwUser);
	// 获取错误码
	int GetErrorCode();
	// 保活线程执行函数，定时发心跳
	static HPR_VOIDPTR KeepAliveProc(HPR_VOIDPTR pParam);

	/*****************对讲协议未完曾部分********************************************/
	// 申请进入群组
	bool JoinGroup(ST_GROUP_JOIN_INFO &groupJoinInfo);
	// 开始群对讲
	bool StartGroupTalk();
	// 停止群对讲
	void StopGroupTalk();
	// 退出对讲组
	bool QuitGroup(const char* cuUserID);
	// 点对点消息发送
	bool SendMsg(ST_MSG_INFO &msgInfo);
	// 向群组发送消息
	bool SendGroupMsg(ST_MSG_GROUP_INFO &msgGroupInfo);
public:
	// 网络SDK监听对象
	TalkNetSDKObserver* mObserver;
	// 对讲包处理对象
	CTalkPacket* mCTalkPacket;
	// 互斥锁
	HPR_MUTEX_T mLock;

private:
	// 初始化标识
	bool bInit;
	// 登陆标识
	bool bLogin;
	// 点对点对讲标识
	bool bTalk;
	// 群组对讲标识
	bool bGroupTalk;
	// 进入对讲组标识
	bool bJoin;
	// 服务端的地址
	HPR_ADDR_T mServAddr;

	// UDP服务对象
	CUDPSer* mCUDPSer;
	// 心跳结束标识
	bool bKeepAliveFinish;
	// 心跳保活成
	HPR_HANDLE mKeepAliveThreadHanlde;
	// 音频编解码
	int mCodecType;
    // 序号
	unsigned int mSequence;
    // 保活请求时间
    time_t mKeepAliveReqTime;
    // 保活回应时间
    time_t mKeepAliveRespTime;
    // 注册结构体
	ST_LOGIN_INFO mRegisterInfo;
	//
	ST_TALK_INFO mTalkInfo;
	//
	ST_LOGIN_RES_INFO mLoginResInfo;
	//
	ST_TALK_RES_INFO mTalkResInfo;
	//
	hpr2_uint16 mErrorCode;

	int mEncType;
private:
	// 分配内存
	bool Molloc();
	// 释放内存
	void Free();
	// 验证登陆参数
	bool IsLoginInfoValid(ST_LOGIN_INFO &loginInfo);
	// 验证对讲参数
	bool IsTalkInfoValid(ST_TALK_INFO &talkInfo);
	// 注册对讲服务器
	bool Register(ST_LOGIN_INFO &loginInfo);
	// 注册对讲服务器
	bool UnRegister();
	// 发送心跳
	bool KeepAlive();
	// Sequence自增
	void IncreaseSequence(unsigned int& nSequence);
	// 处理注册回应信令
	bool HandleRegisterRespone(const char* pPacket, unsigned int nPacketLen, ST_LOGIN_RES_INFO* stLoginResInfo);
	// 处理注销回应信令
	bool HandleUnRegisterRespone(const char* pPacket, unsigned int nPacketLen);

	bool HandleGetEncTypeRespone(const char* pPacket, unsigned int nPacketLen, int& encType);
	// 处理单呼回应信令
	bool HandleTalkRespone(const char* pPacket, unsigned int nPacketLen, _ST_TALK_RES_INFO* sttalkResInfo);
	// 处理退出单呼回应信令
	bool HandleQuitTalkRespone(const char* pPacket, unsigned int nPacketLen);
	// 处理心跳回应信令
	bool HandleKeepAliveRespone(const char* pPacket, unsigned int nPacketLen, talk_hdr talkHdr);
	// 处理音频回复
	bool HandleAudioRespone(const char* pPacket, unsigned int nPacketLen);
};


#endif /* TALKNETSDK_H_ */
