/*
 * UDPSer.h
 *
 *  Created on: 2014-6-7
 *      Author: weilinfeng
 */

#ifndef UDPSER_H_
#define UDPSER_H_
#include <queue>
#include "HPR_Thread.h"
#include "HPR_Mutex.h"
#include "HPR_Socket.h"
#include <map>
#include "hpr2_types.h"
#include "TalkPacket.h"


// 最大包长度
#define MAX_PACKET_LEN      1024
// 发送失败循环发送的次数
#define  SEND_CMD_COUNT	3
#define MAX_DATA_LEN	(1024*2)

typedef struct _ST_DATA_UNIT
{
	char				szFromIP[IP_MAX_LEN];			//服务器IP
	unsigned short int	nFromPort;						//服务端口
	unsigned short int	nReserved;						//保留字段
	char				szDataBuf[MAX_DATA_LEN];		//数据缓冲
	unsigned			int nDataBufLen;				//数据长度
	_ST_DATA_UNIT() {
		memset(szFromIP, 0, IP_MAX_LEN);
		nFromPort = 0;
		nReserved = 0;
		memset(szDataBuf, 0, MAX_DATA_LEN);
		nDataBufLen = 0;
	}
}ST_DATA_UNIT;

typedef struct
{
	// 信令ID
	hpr2_uint16	cmd;
	// 信令回复时间点
	time_t reqTime;
	bool bReply;
	ST_DATA_UNIT stCmdUnit;
}ST_CMD_TIME;


typedef void (*pfnDataListener)(const char* pPackBuffer, unsigned int nLen, const char* szFrom, unsigned short int nPort, unsigned long dwUser);

class CUDPSer {
public:
	// 构造函数
	CUDPSer();
	// 虚的析构函数
	virtual ~CUDPSer();
	// 发送数据,不等待回应
	bool SendPacket(const char* szToIP, unsigned short port, const void* sendBuf, unsigned int contentLen);
	// 发送数据,不等待回应
	bool SendPacketWaitReply(const char* szIP, unsigned short port, const void* contentBuf, unsigned int contentLen, hpr2_uint16 cmd, char* reply, unsigned int* replyLen, unsigned int waitSec=3);
	// 创建本地套接字
	bool Create(pfnDataListener pfn, unsigned long dwUser, const char* szSrvIP=NULL, unsigned short port = 0);
	// 停止数据接收
	void Stop();
	// 开始数据接收
	bool Run();
	// 获取本地使用的IP和端口
	int GetSockAddr(char * pszIP, unsigned short * pnPort);
    // 服务器响应线程执行函数
    static HPR_VOIDPTR RecvPacketProc(HPR_VOIDPTR pParam);
    // 数据处理线程
   static HPR_VOIDPTR impDataHandler(HPR_VOIDPTR pParam);
private:
	// 创建标识
	bool bCreate;
	// 监听套接字
	HPR_SOCK_T mListenSocket;
	// 回调函数指针
	pfnDataListener pfnListener;
	// 服务器地址
	HPR_ADDR_T mServAddr;
	// 数据接收线程句柄
	HPR_HANDLE mRecvThreadHanlde;
	// 数据包处理线程
	HPR_HANDLE mHandThreadHanlde;
	// 接收数据结束标识
	bool mRecvFinish;
	// 数据队列
	typedef std::queue<ST_DATA_UNIT> StDataQuque;
	StDataQuque	m_DataQueue;
	// 互斥锁
	HPR_MUTEX_T mQueueLock;
	// 数据队列
	typedef std::map<hpr2_uint16, ST_CMD_TIME> StTimeMap;
	StTimeMap	m_TimeMap;
	// map锁
	HPR_MUTEX_T mMapLock;
	// 链表中的数据
	unsigned int m_iMsgCount;
	// 用户数据
	unsigned long m_dwUser;
	// 对讲包处理对象
	CTalkPacket* m_pCTalkPacket;
	// 等待响应报文
	bool bWaitReply;
private:
	// 接收数据函数
	bool RecvData();
	// 接收数据函数
	bool HandleData();
	// UDP接收数据
	unsigned int RecvUDPPacket(HPR_ADDR_T* fromAddr, char* pBuf, int waitSec=1);
};

void GuessLocalIP(char* pszLocalIP );

#endif /* UDPSER_H_ */
