/*
 * TalkClientSDK.cpp
 *
 *  Created on: 2014-6-6
 *      Author: weilinfeng
 */
#include "TalkClientSDK.h"
#include "TalkEngine.h"
#include "MCRPrint.h"

// MCRSDK初始化标识
extern bool gInitLib;
// 对讲引擎对象
CTalkEngine * pCTalkEngine = NULL;
// 对讲客户端初始化标识
bool bInit = false;
// 登陆标识
bool bLogin = false;

bool bTalk = false;

pPCMDataListener pTalkPCMDataCallback;

pMessageListener pTalkMsgCallback;

pErrorListener pTalkErrorCallback;

void PCMDataCallback(char* pData, unsigned int nDataSize, void* pUser){
	//LOGD("TalkClientSDK->PCMDataCallback pData len:%d", nDataSize);
	if(pTalkPCMDataCallback == NULL) {
		LOGD("TalkClientSDK->PCMDataCallback pTalkPCMDataCallback == NULL");
		return;
	}
	(*pTalkPCMDataCallback)(pData, nDataSize, pUser);
}

void MessageCallback(unsigned short vChannel, char* content, unsigned int contentlen, char* fromUserID, unsigned int userIDLen, void* pUser){
	//LOGD("TalkClientSDK->MessageCallback content len:%d", contentlen);
	if(pTalkMsgCallback == NULL) {
		LOGD("TalkClientSDK->MessageCallback pTalkMsgCallback == NULL");
		return;
	}
	(*pTalkMsgCallback)(vChannel, content, contentlen, fromUserID, userIDLen, pUser);
}

void ErrorCallback(unsigned int nMsgCode, char* pDescribe, unsigned int desLen, void* pUser){
	//LOGD("TalkClientSDK->ErrorCallback nMsgCode:%d", nMsgCode);
	if(pTalkErrorCallback != NULL) {
		LOGD("TalkClientSDK->ErrorCallback pTalkErrorCallback == NULL");
		return;
	}
	(*pTalkErrorCallback)(nMsgCode, pDescribe, desLen, pUser);
}

/** @fn bool Init()
 *   @brief 初始化Talk库
 *   @return true - 初始化成功; false - 初始化库失败
 */
TALKCLIENTSDKAPI bool Init() {
	if(!gInitLib) {
		LOGD("TalkClientSDK->Init() MCRSDK not init\n");
		return false;
	}

	if(bInit) {
		LOGD("TalkClientSDK->Init() have been init\n");
		return true;
	}

	if(pCTalkEngine == NULL) {
		pCTalkEngine = new CTalkEngine();
	}

	bInit = true;

	LOGD("TalkClientSDK->Init() success\n");
	return true;
}

/** @fn bool Fini()
 *   @brief 反初始化push库
 *   @return 无
 */
TALKCLIENTSDKAPI void Fini() {
	if(!gInitLib) {
		LOGD("TalkClientSDK->Fini() MCRSDK not init\n");
		return;
	}

	if(pCTalkEngine != NULL) {
		delete pCTalkEngine;
		pCTalkEngine = NULL;
	}

	bInit = false;

	LOGD("TalkClientSDK->Fini() success\n");
}

/** @fn bool Login(void* handle)
 *   @brief 登陆服务
 *   @param [in] mesageLister - 消息回调函数指针
 *   @return true - 成功, false - 失败
 */
TALKCLIENTSDKAPI bool Login(ST_LOGIN_INFO &loginInfo, void* pUser) {
	if(!bInit) {
		LOGD("TalkClientSDK->Login() not init\n");
		return false;
	}

	if(bLogin) {
		LOGD("TalkClientSDK->Login() have been login\n");
		return true;
	}

	// 设置回调
	pCTalkEngine->SetCallback(PCMDataCallback, MessageCallback, ErrorCallback);

	bool bRet = pCTalkEngine->Init(pUser);
	if(!bRet) {
		LOGD("TalkClientSDK->Login() have been init\n");
		return false;
	}

	bRet = pCTalkEngine->Login(loginInfo);
	if(!bRet) {
		LOGD("TalkClientSDK->Login() Login fail\n");
		pCTalkEngine->Fini();
		return false;
	}

	bLogin = true;

	LOGD("TalkClientSDK->Login() Login success\n");
	return true;
}

/** @fn bool Logout(void* handle)
 *   @brief 登出服务
 *   @return true - 成功, false - 失败
 */
TALKCLIENTSDKAPI void Logout() {
	if(!bInit) {
		LOGD("TalkClientSDK->Logout() not init\n");
		return;
	}

	if(bTalk) {
		StopTalk();
	}

	// 登出
	pCTalkEngine->Logout();

	// 释放引擎内存
	pCTalkEngine->SetCallback(NULL, NULL, NULL);
	// 释放引擎
	pCTalkEngine->Fini();

	bLogin = false;
	LOGD("TalkClientSDK->Logout() success\n");
}

/** @fn bool SetErrorListener(pErrorLister errorListener)
 *   @brief 设置 消息回调
 *   @param [in] errorListener - 错误回调函数指针
 *   @return true - 成功, false - 失败
 */
TALKCLIENTSDKAPI bool SetErrorListener(pErrorListener errorListener) {
	if(errorListener == NULL) {
		LOGD("TalkClientSDK->SetErrorListener() errorListener == NULL\n");
		return false;
	}

	pTalkErrorCallback = errorListener;
	LOGD("TalkClientSDK->SetErrorListener() success\n");
	return true;
}

/** @fn bool SetMessageListener(pMessageListener mesageListener)
 *   @brief 设置 消息回调
 *   @param [in] mesageListener - 消息回调函数指针
 *   @return true - 成功, false - 失败
 */
TALKCLIENTSDKAPI bool SetMessageListener(pMessageListener mesageListener) {
	if(mesageListener == NULL) {
		LOGD("TalkClientSDK->SetMessageListener() mesageListener == NULL\n");
		return false;
	}

	pTalkMsgCallback = mesageListener;
	LOGD("TalkClientSDK->SetMessageListener() success\n");
	return true;
}

/** @fn bool SetPCMDataListener(pPCMDataListener pcmDataListener)
 *   @brief 设置数据包回调
 *   @param [in] pcmDataListener - 数据回调函数指针
 *   @return true - 成功, false - 失败
 */
TALKCLIENTSDKAPI bool SetPCMDataListener(pPCMDataListener pcmDataListener) {
	if(pcmDataListener == NULL) {
		//LOGD("TalkClientSDK->SetPCMDataListener() pcmDataListener == NULL\n");
		return false;
	}

	pTalkPCMDataCallback = pcmDataListener;
	return true;
}

/** @fn bool InputAudioData(const char* pData, HPR_INT32 nDataSize)
 *   @brief 塞入音频数据
 *   @param [in] InputAudioData - PCM音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
TALKCLIENTSDKAPI bool InputAudioData(const char* pData, unsigned int nDataSize) {
	if(!bInit) {
		LOGD("TalkClientSDK->InputAudioData() not init\n");
		return false;
	}

	if(!bLogin) {
		LOGD("TalkClientSDK->InputAudioData() not login\n");
		return false;
	}

	//LOGD("TalkClientSDK->InputAudioData()\n");
	return pCTalkEngine->InputAudioData(pData, nDataSize);
}

/** @fn bool StartTalk(TALK_INFO &talkInfo)
 *   @brief 开始单呼
 *   @param [in] talkInfo - 点对点对讲信息
 *   @param [out] 无
 *   @return  true - 呼叫成功; false - 呼叫失败
 */
TALKCLIENTSDKAPI bool StartTalk(ST_TALK_INFO &talkInfo) {
	if(!bInit) {
		LOGD("TalkClientSDK->StartTalk() not init\n");
		return false;
	}

	if(!bLogin) {
		LOGD("TalkClientSDK->StartTalk() not login\n");
		return false;
	}

	if(bTalk) {
		LOGD("TalkClientSDK->StartTalk() have start talk\n");
		return true;
	}

	bool bRet = pCTalkEngine->StartTalk(talkInfo);
	if(!bRet) {
		LOGD("TalkClientSDK->StartTalk() StartTalk fail\n");
		return false;
	}

	bTalk = true;
	LOGD("TalkClientSDK->StartTalk() StartTalk success\n");
	return true;
}

/** @fn bool StopTalk()
 *   @brief 停止点对点对讲
 *   @param [in] 无
 *   @param [out] 无
 *   @return  无
 */
TALKCLIENTSDKAPI void StopTalk() {
	if(!bInit) {
		LOGD("TalkClientSDK->StopTalk() not init\n");
		return;
	}

	if(!bLogin) {
		LOGD("TalkClientSDK->StopTalk() not login\n");
		return;
	}

	if(!bTalk) {
		LOGD("TalkClientSDK->StopTalk() have Stop Talk\n");
		return;
	}

	pCTalkEngine->StopTalk();

	bTalk = false;
	LOGD("TalkClientSDK->StopTalk() StopTalk success\n");
}

TALKCLIENTSDKAPI int GetDeviceEncodeType() {
	if(!bInit) {
		LOGD("TalkClientSDK->GetDeviceEncodeType() not init\n");
		return -1;
	}

	if(!bLogin) {
		LOGD("TalkClientSDK->GetDeviceEncodeType() not login\n");
		return -1;
	}

	return pCTalkEngine->GetDeviceEncodeType();
}

/** @fn int JoinGroup(GROUP_JOIN_INFO &groupJoinInfo)
 *   @brief 申请进入对讲组
 *   @param [in] groupInfo - 群组信息
 *   @param [out] 无
 *   @return  -1-加入群组指定群组失败， 0-代表终端在改组无权限，1~16的整数，代表终端在该组的权限
 */
TALKCLIENTSDKAPI bool JoinGroup(ST_GROUP_JOIN_INFO &groupJoinInfo) {
	if(!bInit) {
		LOGD("TalkClientSDK->JoinGroup() not init\n");
		return false;
	}

	if(!bLogin) {
		LOGD("TalkClientSDK->JoinGroup() not login\n");
		return false;
	}

	bool bRet = pCTalkEngine->JoinGroup(groupJoinInfo);
	if(!bRet) {
		LOGD("TalkClientSDK->JoinGroup() JoinGroup fail\n");
		return false;
	}

	LOGD("TalkClientSDK->JoinGroup() JoinGroup success\n");
	return true;
}

/** @fn bool QuitGroup(const char* cuUserID)
 *   @brief 退出对讲组
 *   @param [in] cuUserID - 客户端UserID
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
TALKCLIENTSDKAPI bool QuitGroup() {
	if(!bInit) {
		LOGD("TalkClientSDK->QuitGroup() not init\n");
		return false;
	}

	if(!bLogin) {
		LOGD("TalkClientSDK->QuitGroup() not login\n");
		return false;
	}

	bool bRet = pCTalkEngine->QuitGroup();
	if(!bRet) {
		LOGD("TalkClientSDK->QuitGroup() QuitGroup fail\n");
		return false;
	}

	LOGD("TalkClientSDK->QuitGroup() QuitGroup success\n");
	return true;
}

/** @fn bool StartGroupTalk()
 *   @brief	开始群对讲
 *   @param [in] 无
 *   @return  true - 开始群对讲成功; false - 开始群对讲失败
 */
TALKCLIENTSDKAPI bool StartGroupTalk() {
	if(!bInit) {
		LOGD("TalkClientSDK->StartGroupTalk() not init\n");
		return false;
	}

	if(!bLogin) {
		LOGD("TalkClientSDK->StartGroupTalk() not login\n");
		return false;
	}

	bool bRet = pCTalkEngine->StartGroupTalk();
	if(!bRet) {
		LOGD("TalkClientSDK->StartGroupTalk() StartGroupTalk fail\n");
		return false;
	}

	LOGD("TalkClientSDK->StartGroupTalk() StartGroupTalk success\n");
	return true;
}

/** @fn bool StopGroupTalk()
 *   @brief	停止群对讲
 *   @param [in] 无
 *   @return  无
 */
TALKCLIENTSDKAPI void StopGroupTalk() {
	if(!bInit) {
		LOGD("TalkClientSDK->StopGroupTalk() not init\n");
		return;
	}

	if(!bLogin) {
		LOGD("TalkClientSDK->StopGroupTalk() not login\n");
		return;
	}

	pCTalkEngine->StopGroupTalk();

	LOGD("TalkClientSDK->StopGroupTalk() StopGroupTalk success\n");
}

/** @fn bool sendMessage(void* handle)
 *   @brief 点对点消息发送
 *   @param [in] InputAudioData - PCM音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
TALKCLIENTSDKAPI bool SendMsg(ST_MSG_INFO &msgInfo) {
	if(!bInit) {
		LOGD("TalkClientSDK->SendMsg() not init\n");
		return false;
	}

	if(!bLogin) {
		LOGD("TalkClientSDK->SendMsg() not login\n");
		return false;
	}

	bool bRet = pCTalkEngine->SendMsg(msgInfo);
	if(!bRet) {
		LOGD("TalkClientSDK->SendMsg() SendMsg fail\n");
		return false;
	}

	LOGD("TalkClientSDK->SendMsg() SendMsg success\n");
	return true;
}

/** @fn bool SendGroupMsg(void* handle)
 *   @brief 向群组发送消息
 *   @param [in] InputAudioData - PCM音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return  true - 塞入音频数据成功; false - 塞入音频数据失败
 */
TALKCLIENTSDKAPI bool SendGroupMsg(ST_MSG_GROUP_INFO &SendGroupMsg){
	if(!bInit) {
		LOGD("TalkClientSDK->SendGroupMsg() not init\n");
		return false;
	}

	if(!bLogin) {
		LOGD("TalkClientSDK->SendGroupMsg() not login\n");
		return false;
	}

	bool bRet = pCTalkEngine->SendGroupMsg(SendGroupMsg);
	if(!bRet) {
		LOGD("TalkClientSDK->SendGroupMsg() SendMsg fail\n");
		return false;
	}

	LOGD("TalkClientSDK->SendGroupMsg() SendGroupMsg success\n");
	return true;
}

/** @fn int GetErrorCode()
 *  @brief  获取错误码
 *  @param [in] 无
 *  @param [out] 无
 *  @return 错误码
 */
TALKCLIENTSDKAPI int GetErrorCode() {
	if(!bInit) {
		LOGD("TalkClientSDK->SendGroupMsg() not init\n");
		return SUCCESS;
	}

	return pCTalkEngine->GetErrorCode();
}

