/*
 * TalkEngine.cpp
 *
 *  Created on: 2014-6-6
 *      Author: weilinfeng
 */

#include "TalkEngine.h"
#include "MCRPrint.h"

CTalkEngine::CTalkEngine()
: bInit(false)
, bLogin(false)
, bTalk(false)
//, mAudioCodec(NULL)
, mCTalkNetSDK(NULL)
//, mCodecType(TYPE_G711)
, mUser(NULL)
, mErrorCode(SUCCESS)
, mEncType(-1)
//, m_file_pcm(NULL)
//, m_file_g711(NULL)
//, m_file_g711_s(NULL)
{

}

void PCMAudioDataListener(char* pPCMData, unsigned int nDataSize, void* pUser) {

	CTalkEngine* engine = (CTalkEngine*)pUser;
	if(engine == NULL) {
		return;
	}

	if(engine->mPcmCallbackPtr != NULL) {
		(*engine->mPcmCallbackPtr)(pPCMData, nDataSize, engine->mUser);
	}

}

void XXXAudioDataListener(char* pXXXData, unsigned int nDataSize, void* pUser) {
	LOGD("CTalkEngine->XXXAudioDataListener pG7XXData len:%d", nDataSize);
	CTalkEngine* engine = (CTalkEngine*)pUser;
	if(engine == NULL) {
		return;
	}

	if(engine->mCTalkNetSDK != NULL) {
		engine->mCTalkNetSDK->SendG7XXData(pXXXData, nDataSize);
	}

	LOGD("CTalkEngine->XXXAudioDataListener() SendG7XXData success");
}


CTalkEngine::~CTalkEngine() {

}

TalkNetSDKObserver::~TalkNetSDKObserver() {

}

// 初始化
bool CTalkEngine::Init(void* pUser){
	if(bInit) {
		LOGD("CTalkEngine->Init() have been init");
		return true;
	}

	// 初始化
	bool bRet = AudioInit(this);
	if(!bRet) {
		LOGD("CTalkEngine->Init() AudioInit fail");
		Fini();
		return false;
	}

	bRet = SetPCMAudioDataListener(PCMAudioDataListener);
	if(!bRet) {
		LOGD("CTalkEngine->Init() SetPCMAudioDataListener fail");
		Fini();
		return false;
	}

	bRet = SetXXXAudioDataListener(XXXAudioDataListener);
	if(!bRet) {
		LOGD("CTalkEngine->Init() SetPCMAudioDataListener fail");
		Fini();
		return false;
	}

	mCTalkNetSDK = new CTalkNetSDK();
	if(mCTalkNetSDK == NULL) {
		LOGD("CTalkEngine->Init() m_pCTalkNetSDK == NULL");
		Fini();
		return false;
	}

	bRet = mCTalkNetSDK->Init(this, mCodecType);
	if(!bRet) {
		LOGD("CTalkEngine->Init() CTalkNetSDK Init fail");
		Fini();
		return false;
	}

	mUser = pUser;
	bInit = true;
	return true;
}
// 反初始化
void CTalkEngine::Fini(){
//	if(mAudioCodec != NULL) {
//		mAudioCodec->Fini();
//		delete mAudioCodec;
//		mAudioCodec = NULL;
//	}

	// 反初始化音频编码库
	AudioFini();

	if(mCTalkNetSDK != NULL) {
		mCTalkNetSDK->Fini();
		delete mCTalkNetSDK;
		mCTalkNetSDK = NULL;
	}

	bInit = false;
}

/** @fn void SetCallback
 *   @brief 设置回调
 *   @param [in] InputAudioData - PCM音频数据
 *   @param [in] nDataSize - 音频数据的长度
 *   @param [out] 无
 *   @return 无
 */
void CTalkEngine::SetCallback(PCMDataCallbackPtr pcmCallbackPtr, MessageCallbackPtr msgCallbackPtr, ErrorCallbackPtr errorCallbackPtr){
	mPcmCallbackPtr = pcmCallbackPtr;
	mMsgCallbackPtr = msgCallbackPtr;
	mErrorCallbackPtr = errorCallbackPtr;
}

//注册服务器
bool CTalkEngine::Login(ST_LOGIN_INFO &loginInfo) {
	if(!bInit) {
		LOGD("CTalkEngine->Login() CTalkNetSDK no Init");
		return false;
	}

	if(bLogin) {
		LOGD("CTalkEngine->Login() have been login");
		return true;
	}

	LOGD("*********CTalkEngine->Login() loginInfo*********");
	LOGD(loginInfo.puID);
	LOGD("CTalkEngine->Login() puID len:%d", strlen(loginInfo.puID));
	LOGD(loginInfo.cuUserID);
	LOGD("CTalkEngine->Login() UserID len:%d", strlen(loginInfo.cuUserID));
	LOGD(loginInfo.password);
	LOGD("CTalkEngine->Login() password len:%d", strlen(loginInfo.password));
	LOGD(loginInfo.servIP);
	LOGD("CTalkEngine->Login() servIP len:%d", strlen(loginInfo.servIP));
	LOGD("CTalkEngine->Login() servPort:%d", loginInfo.servPort);
	LOGD("CTalkEngine->Login() type:%d", loginInfo.type);
	LOGD("CTalkEngine->Login() codecType:%d", loginInfo.codecType);

	bool bRet = mCTalkNetSDK->Login(loginInfo);
	if(!bRet) {
		LOGD("CTalkEngine->Login() CTalkNetSDK Login fail");
		mErrorCode = mCTalkNetSDK->GetErrorCode();
		return false;
	}

	bLogin = true;

	LOGD("CTalkEngine->Login() success");
	return true;
}

// 登出服务器
bool CTalkEngine::Logout() {
	if(!bInit) {
		LOGD("CTalkEngine->Logout() CTalkNetSDK no Init");
		return false;
	}

	if(bTalk) {
		StopTalk();
	}

	bool bRet = mCTalkNetSDK->Logout();
	if(!bRet) {
		mErrorCode = mCTalkNetSDK->GetErrorCode();
		LOGD("CTalkEngine->Logout() CTalkNetSDK Logout fail");
	}

	bLogin = false;

	LOGD("CTalkEngine->Logout() success");
	return bRet;
}

// 塞入音频数据
bool CTalkEngine::InputAudioData(const char* pData, unsigned int nDataSize) {
	if(!bInit) {
		LOGD("CTalkEngine->InputAudioData() CTalkNetSDK no Init");
		return false;
	}

	if(!bLogin) {
		LOGD("CTalkEngine->InputAudioData() no login");
		return true;
	}

	if(!bTalk && !bGroupTalk) {
		LOGD("CTalkEngine->InputAudioData() no start talk");
		return false;
	}

//	if (m_file_pcm != NULL) {
//		LOGD("CTalkEngine->InputAudioData() m_file_pcm data\n");
//		fwrite(pData, 1, nDataSize, m_file_pcm);
//		fflush(m_file_pcm);
//	}

	//mAudioCodec->InputPCMData(pData, nDataSize);

	InputPCMAudioData(pData, nDataSize);

	LOGD("CTalkEngine->InputAudioData()\n");
	return true;
}

// 开始点对点对讲
bool CTalkEngine::StartTalk(ST_TALK_INFO &talkInfo) {
	if(!bInit) {
		LOGD("CTalkEngine->StartTalk() no Init");
		return false;
	}

	if(!bLogin) {
		LOGD("CTalkEngine->StartTalk() no login");
		return false;
	}

	if(bTalk) {
		LOGD("CTalkEngine->StartTalk() have been start talk");
		return true;
	}

	// 获取获取设备的编码类型
	int encType = 0;
	ST_DEV_INFO devInfo;
	memset(&devInfo, 0, sizeof(ST_DEV_INFO));
	memcpy(devInfo.deviceIndexCode, talkInfo.deviceIndexCode, DEVICE_INDEX_MAX_LEN);
	devInfo.deviceType = talkInfo.deviceType;
	bool bRet = mCTalkNetSDK->GetDivAEncType(devInfo, encType);
	if(!bRet) {
		LOGD("CTalkEngine->StartTalk() GetDivAEncType fail");
		return true;
	}
	mEncType = encType;

	LOGD("CTalkEngine->StartTalk() GetDivAEncType encType:%d", encType);

	DecodeParams decodeParams;
	decodeParams.bitRate = 16000;
	bRet = StartAudioDecode(encType, decodeParams);
	if(!bRet) {
		LOGD("CTalkEngine->StartTalk() StartAudioDecode fail");
		mErrorCode = GetErrorCode();
		return false;
	}

	EncodeParams encodeParams;
	if(encType == TYPE_G711_A || encType == TYPE_G711_U || encType == TYPE_G726) {
		encodeParams.sampleRate = 8000;
	}
	else if(encType == TYPE_G722){
		encodeParams.sampleRate = 16000;
	}
	else
	{ // 其他格式不支持
		return false;
	}

	encodeParams.bitRate = 16000;
	encodeParams.channels = 1;
	bRet = StartAudioEncode(encType, encodeParams);
	if(!bRet) {
		LOGD("CTalkEngine->StartTalk() StartAudioEncode fail");
		mErrorCode = GetErrorCode();
		StopAudioDecode();
		return false;
	}

	bRet = mCTalkNetSDK->StartTalk(talkInfo);
	if(!bRet) {
		LOGD("CTalkEngine->StartTalk() StartTalk start");
		StopAudioEncode();
		StopAudioDecode();
		mErrorCode = mCTalkNetSDK->GetErrorCode();
		return false;
	}

//	m_file_g711 = fopen("/mnt/extSdCard/g711.g7", "wb");
//	if (m_file_g711 == NULL) {
//		LOGD("CTalkEngine->StartTalk() m_file_g711 fail");
//		m_pAudioCodec->StopEncode();
//		m_pAudioCodec->StopDecode();
//		m_pCTalkNetSDK->StopTalk();
//		return false;
//	}
//
//	m_file_pcm = fopen("/mnt/extSdCard/pcm.p", "wb");
//	if (m_file_pcm == NULL) {
//		LOGD("CTalkEngine->StartTalk() m_file_pcm fail");
//		m_pAudioCodec->StopEncode();
//		m_pAudioCodec->StopDecode();
//		m_pCTalkNetSDK->StopTalk();
//		fclose(m_file_pcm);
//		m_file_pcm = NULL;
//		return false;
//	}
//
//	m_file_g711_s = fopen("/mnt/extSdCard/g711_s.g7", "wb");
//	if (m_file_g711_s == NULL) {
//		LOGD("CTalkEngine->StartTalk() m_file_g711_s fail");
//		m_pAudioCodec->StopEncode();
//		m_pAudioCodec->StopDecode();
//		m_pCTalkNetSDK->StopTalk();
//		fclose(m_file_pcm);
//		m_file_pcm = NULL;
//		fclose(m_file_g711);
//		m_file_g711 = NULL;
//		return false;
//	}

	bTalk = true;

	LOGD("CTalkEngine->StartTalk() StartTalk success");
	return true;
}
// 停止点对点对讲
bool CTalkEngine::StopTalk() {
	if(!bInit) {
		LOGD("CTalkEngine->StopTalk() no Init");
		return false;
	}

	if(!bLogin) {
		LOGD("CTalkEngine->StopTalk() no login");
		return false;
	}

	if(!bTalk) {
		LOGD("CTalkEngine->StopTalk() have Stop talk");
		return true;
	}

	StopAudioEncode();

	StopAudioDecode();

	bool bRet = mCTalkNetSDK->StopTalk();
	if(!bRet) {
		LOGD("CTalkEngine->StopTalk() StopTalk fail");
		mErrorCode = mCTalkNetSDK->GetErrorCode();
		//return false;
	}

//	if (m_file_pcm != NULL) {
//		fclose(m_file_pcm);
//		m_file_pcm = NULL;
//	}
//
//	if (m_file_g711 != NULL) {
//		fclose(m_file_g711);
//		m_file_g711 = NULL;
//	}
//
//	if (m_file_g711_s != NULL) {
//		fclose(m_file_g711_s);
//		m_file_g711_s = NULL;
//	}

	bTalk = false;
	LOGD("CTalkEngine->StopTalk() StopTalk complete");
	return bRet;
}

int CTalkEngine::GetDeviceEncodeType() {
	return mEncType;
}

// 申请进入群组
bool CTalkEngine::JoinGroup(ST_GROUP_JOIN_INFO &groupJoinInfo) {
	return true;
}

// 退出对讲组
bool CTalkEngine::QuitGroup() {
	return true;
}

// 开始群对讲
bool CTalkEngine::StartGroupTalk() {

	bGroupTalk = true;
	return true;
}

// 停止群对讲
void CTalkEngine::StopGroupTalk() {

	bGroupTalk = false;
}

// 点对点消息发送
bool CTalkEngine::SendMsg(ST_MSG_INFO &msgInfo) {
	return true;
}

// 向群组发送消息
bool CTalkEngine::SendGroupMsg(ST_MSG_GROUP_INFO &msgGroupInfo) {
	return true;
}

// 网路SDK回调
void CTalkEngine::NetSDKG7XXDataCallBack(unsigned short vChannel, char* pData, unsigned int nDataSize, char* fromUserID, unsigned int userIDLen) {
	//LOGD("CTalkEngine->NetSDKG7XXDataCallBack pData len:%d", nDataSize);
	if(!bInit) {
		LOGD("CTalkEngine->NetSDKG7XXDataCallBack() no init");
		return;
	}

	if(!bLogin) {
		LOGD("CTalkEngine->NetSDKG7XXDataCallBack() no login");
		return;
	}

	if(!bTalk && !bGroupTalk) {
		LOGD("CTalkEngine->NetSDKG7XXDataCallBack() no start talk");
		return;
	}

//	if (m_file_g711_s != NULL) {
//		LOGD("CTalkEngine->NetSDKG7XXDataCallBack() m_file_g711_s data\n");
//		fwrite(pData, 1, nDataSize, m_file_g711_s);
//		fflush(m_file_g711_s);
//	}
	InputXXXAudioData(pData, nDataSize);
}

void CTalkEngine::NetSDKGMsgCallBack(unsigned short vChannel, char* content, unsigned int contentlen, char* fromUserID, unsigned int userIDLen) {
	LOGD("CTalkEngine->NetSDKGMsgCallBack content len:%d", contentlen);
}

void CTalkEngine::NetSDKGErrorCallBack(unsigned int nMsgCode, char* pDescribe, unsigned int desLen) {
	LOGD("CTalkEngine->NetSDKGErrorCallBack nMsgCode:%d", nMsgCode);
	LOGD("CTalkEngine->NetSDKGErrorCallBack pDescribe:%s", pDescribe);
}

/** @fn void getErrorCode()
 *   @brief 获取错误码
 *   @param [int] 无
 *   @return 错误码
 */
int CTalkEngine::GetErrorCode() {
	return mErrorCode;
}
