/*
 * TalkNetSDK.cpp
 *
 *  Created on: 2014-6-6
 *      Author: weilinfeng
 */

#include "TalkNetSDK.h"
#include "MCRPrint.h"
#include "hpr2_sleep.h"

void DataListener(const char* pPackBuffer, unsigned int nLen, const char* szFrom, unsigned short int nPort, unsigned long dwUser);
void DataListener(const char* pPackBuffer, unsigned int nLen, const char* szFrom, unsigned short int nPort, unsigned long dwUser) {
	//LOGD("CTalkNetSDK->DataListener pPack len:%d", nLen);
	if (pPackBuffer == NULL) {
		LOGD("CTalkNetSDK->DataListener pPackBuffer == NULL");
		return;
	}

	CTalkNetSDK *pCTalkNetSDK = (CTalkNetSDK *)dwUser;
	if (pCTalkNetSDK == NULL) {
		LOGD("CTalkNetSDK->DataListener pCTalkNetSDK == NULL");
		return;
	}

	pCTalkNetSDK->HandleReqPacket(pPackBuffer, nLen, szFrom, nPort, dwUser);
}


/** @fn CTalkNetSDK
 *   @brief	构造函数
 */
CTalkNetSDK::CTalkNetSDK()
: bInit(false)
, bLogin(false)
, bTalk(false)
, bGroupTalk(false)
, bJoin(false)
, mObserver(NULL)
, mCTalkPacket(NULL)
, mCUDPSer(NULL)
, mKeepAliveThreadHanlde(HPR_INVALID_HANDLE)
, bKeepAliveFinish(false)
, mSequence(1)
, mErrorCode(SUCCESS)
, mEncType(-1)
{
	memset(&mRegisterInfo, 0, sizeof(ST_LOGIN_INFO));
	memset(&mTalkInfo, 0, sizeof(ST_TALK_INFO));
	memset(&mLoginResInfo, 0, sizeof(ST_LOGIN_RES_INFO));
	memset(&mTalkResInfo, 0, sizeof(ST_TALK_RES_INFO));

}
// 析构函数
CTalkNetSDK::~CTalkNetSDK()
{

}

/** @fn bool Init()
 *   @brief	初始化
 *   @param [in] observer - 数据回调
 *   @param [in] codecType - 编码类型
 *   @param [out] 无
 *   @return true - 成功; false - 失败
 */
bool CTalkNetSDK::Init(TalkNetSDKObserver * observer, int codecType)
{
	if(bInit) {
		LOGD("CTalkNetSDK->Init() have been inited");
		return true;
	}

	bool bRet = Molloc();
	if(!bRet) {
		LOGD("CTalkNetSDK->Init() Molloc fail");
		Fini();
		return false;
	}

	// 开启UDPser
	bRet = mCUDPSer->Create(DataListener, (unsigned long) this);
	if(!bRet) {
		LOGD("CTalkNetSDK->Init() m_pCUDPSer Create fail");
		Fini();
		return false;
	}

	mCodecType = codecType;
	mObserver = observer;
	bInit = true;

	LOGD("CTalkNetSDK->Init() success");
	return true;
}

/** @fn bool Fini()
 *   @brief	反初始化
 *   @param [out] 无
 *   @return
 */
void CTalkNetSDK::Fini()
{
	// 关闭UDPser
	if(mCUDPSer != NULL) {
		mCUDPSer->Stop();
	}

	// 释放内存
	Free();

	bInit = false;
	LOGD("CTalkNetSDK->Fini() success");
}

/** @fn bool Login()
 *   @brief	注册服务器
 *   @param [in] loginInfo - 登陆信息
 *   @param [out] 无
 *   @return  true - 成功; false - 失败
 */
bool CTalkNetSDK::Login(ST_LOGIN_INFO &loginInfo)
{
	if(!bInit) {
		LOGD("CTalkNetSDK->Login() no init");
		return false;
	}

	if(bLogin) {
		LOGD("CTalkNetSDK->Login() have been logined");
		return true;
	}

	bool bRet = mCUDPSer->Run();
	if(!bRet) {
		LOGD("CTalkNetSDK->Login() m_pCUDPSer Run fail");
		return false;
	}

	// 发送注册信令
	bRet = Register(loginInfo);
	if(!bRet) {
		LOGD("CTalkNetSDK->Login() Register fail");
		mCUDPSer->Stop();
		return false;
	}

	// 开启心跳线程
	bKeepAliveFinish = false;
	mKeepAliveThreadHanlde = HPR_Thread_Create(KeepAliveProc, this, 0);
	if (mKeepAliveThreadHanlde == NULL)
	{
		LOGD("CTalkNetSDK->Login() create KeepAlive thread fail");
		mCUDPSer->Stop();
		return false;
	}

	bLogin = true;

	LOGD("CTalkNetSDK->Login() Login success");
	return true;
}

/** @fn bool Login()
 *   @brief	登出服务器
 *   @param [out] 无
 *   @return  true - 成功; false - 失败
 */
bool CTalkNetSDK::Logout()
{
	if(!bInit) {
		LOGD("CTalkNetSDK->Logout() no init");
		return false;
	}

	if(bTalk) {
		StopTalk();
	}

	// 注销
	bool bRet = UnRegister();
	if(!bRet) {
		LOGD("CTalkNetSDK->Logout() UnRegister fail");
	}

	mCUDPSer->Stop();

	bKeepAliveFinish = true;
	if (mKeepAliveThreadHanlde != HPR_INVALID_HANDLE) {
		HPR_Thread_Wait(mKeepAliveThreadHanlde);
		LOGD("CTalkNetSDK->Logout() keep alive thread wait end\n");
		mKeepAliveThreadHanlde = HPR_INVALID_HANDLE;
	}

	bLogin = false;
	LOGD("CTalkNetSDK->Logout() Logout complete");

	return bRet;
}


bool CTalkNetSDK::GetDivAEncType(ST_DEV_INFO &devInfo, int& encType) {
	if(!bInit) {
		LOGD("CTalkNetSDK->GetDivAEncType() no init");
		return false;
	}

	char dataBuf[TALK_PACKET_SIZE];
	memset(dataBuf, 0, TALK_PACKET_SIZE);
	int dataLen  = mCTalkPacket->PackGetEncType(mSequence, devInfo, dataBuf, TALK_PACKET_SIZE);
	IncreaseSequence(mSequence);
	if(dataLen == 0) {
		LOGD("CTalkNetSDK->GetDivAEncType() pack get encode packet fail");
		return false;
	}

	char replyBuf[TALK_PACKET_SIZE];
	unsigned int replyLen = TALK_PACKET_SIZE;
	memset(replyBuf, 0, TALK_PACKET_SIZE);
	bool bRet = mCUDPSer->SendPacketWaitReply(mRegisterInfo.servIP, mRegisterInfo.servPort, dataBuf, dataLen, GET_INCODETYPE, replyBuf, &replyLen);
	if(!bRet) {
		LOGD("CTalkNetSDK->GetDivAEncType() send Wait Reply get Encode packet fail");
		return false;
	}

	bRet = HandleGetEncTypeRespone(replyBuf, replyLen, encType);
	if(!bRet) {
		LOGD("CTalkNetSDK->GetDivAEncType() HandleRegisterRespone fail");
		return false;
	}

	LOGD("CTalkNetSDK->GetDivAEncType() success encType:%d", encType);
	return true;
}

/** @fn bool StartTalk()
 *   @brief	开始点对点对讲
 *   @param [in] talkInfo - 对讲信息
 *   @param [out] 无
 *   @return  true - 成功; false - 失败
 */
bool CTalkNetSDK::StartTalk(ST_TALK_INFO &talkInfo)
{
	if(!bInit) {
		LOGD("CTalkNetSDK->StartTalk() no init");
		return false;
	}

	if(!bLogin) {
		LOGD("CTalkNetSDK->StartTalk() no login");
		return false;
	}

	if(bTalk) {
		LOGD("CTalkNetSDK->StartTalk() have start talk");
		return true;
	}

	// 验证传入的registerInfo参数的有效性
	bool bRet = IsTalkInfoValid(talkInfo);
	if (!bRet){
		LOGD("CTalkNetSDK->StartTalk() talkInfo invalid");
		return false;
	}

	memset(&mTalkInfo, 0, sizeof(ST_TALK_INFO));
	memcpy(mTalkInfo.cuUserID, talkInfo.cuUserID, strlen(talkInfo.cuUserID));
	memcpy(mTalkInfo.toUserID, talkInfo.toUserID, strlen(talkInfo.toUserID));
	memcpy(mTalkInfo.deviceIndexCode, talkInfo.deviceIndexCode, strlen(talkInfo.deviceIndexCode));
	mTalkInfo.deviceType = talkInfo.deviceType;
	mTalkInfo.channelNum = talkInfo.channelNum;

	char dataBuf[TALK_PACKET_SIZE];
	memset(dataBuf, 0, TALK_PACKET_SIZE);
	int dataLen  = mCTalkPacket->PackIndividualCall(mSequence, talkInfo, dataBuf, TALK_PACKET_SIZE);
	IncreaseSequence(mSequence);
	if(dataLen == 0) {
		LOGD("CTalkNetSDK->StartTalk() pack talk packet fail");
		return false;
	}

	char replyBuf[TALK_PACKET_SIZE];
	unsigned int replyLen = TALK_PACKET_SIZE;
	memset(replyBuf, 0, TALK_PACKET_SIZE);
	bRet = mCUDPSer->SendPacketWaitReply(mRegisterInfo.servIP, mRegisterInfo.servPort, dataBuf, dataLen, INDIVIDUALCALL, replyBuf, &replyLen);
	if(!bRet) {
		LOGD("CTalkNetSDK->StartTalk() send talk packet fail");
		return false;
	}

	bRet = HandleTalkRespone(replyBuf, replyLen, &mTalkResInfo);
	if(!bRet) {
		LOGD("CTalkNetSDK->StartTalk() HandleTalkRespone fail");
		return false;
	}

	bTalk = true;

	LOGD("CTalkNetSDK->StartTalk() StartTalk success");
	return true;
}

/** @fn bool StopTalk()
 *   @brief	停止点对点对讲
 *   @param [out] 无
 *   @return  true - 成功; false - 失败
 */
bool CTalkNetSDK::StopTalk() {
	if(!bInit) {
		LOGD("CTalkNetSDK->StopTalk() no init");
		return false;
	}

	if(!bLogin) {
		LOGD("CTalkNetSDK->StopTalk() no login");
		return false;
	}

	if(!bTalk) {
		LOGD("CTalkNetSDK->StopTalk() have stop talk");
		return true;
	}

	char dataBuf[TALK_PACKET_SIZE];
	memset(dataBuf, 0, TALK_PACKET_SIZE);
	int dataLen  = mCTalkPacket->PackQuitIndividualCall(mSequence, mRegisterInfo.cuUserID, mTalkResInfo.session, dataBuf, TALK_PACKET_SIZE);
	IncreaseSequence(mSequence);
	if(dataLen == 0) {
		LOGD("CTalkNetSDK->StopTalk() pack quit  individual call packet fail");
		// 服务器回复失败，任务成功，避免上层调用判断无法释放资源
		//return false;
	}

	char replyBuf[TALK_PACKET_SIZE];
	unsigned int replyLen = TALK_PACKET_SIZE;
	memset(replyBuf, 0, TALK_PACKET_SIZE);
	bool bRet = mCUDPSer->SendPacketWaitReply(mRegisterInfo.servIP, mRegisterInfo.servPort, dataBuf, dataLen, QUITINDIVIDUALCALL, replyBuf, &replyLen);
	if(!bRet) {
		LOGD("CTalkNetSDK->StopTalk() send quit individual call packet fail");
		// 服务器回复失败，任务成功，避免上层调用判断无法释放资源
		//return false;
	}

	bRet = HandleQuitTalkRespone(replyBuf, replyLen);
	if(!bRet) {
		LOGD("CTalkNetSDK->StopTalk() HandleTalkRespone fail");
		// 服务器回复失败，任务成功，避免上层调用判断无法释放资源
		//return false;
	}

	bTalk = false;
	LOGD("CTalkNetSDK->StopTalk() StopTalk success");
	return true;
}

/** @fn bool SendG7XXData()
 *   @brief	发送编码后的音频数据
 *   @param [in] pData - 数据缓冲
 *   @param [in] nDataSize - 数据长度
 *   @param [out] 无
 *   @return  true - 成功; false - 失败
 */
bool CTalkNetSDK::SendG7XXData(const char* pData, HPR_INT32 nDataSize) {
	//LOGD("CTalkNetSDK->SendG7XXData()");

	if(!bTalk && !bGroupTalk) {
		LOGD("CTalkNetSDK->SendG7XXData() no start");
		return false;
	}

	if(nDataSize > TALK_PACKET_SIZE) {
		LOGD("CTalkNetSDK->SendG7XXData() nDataSize > TALK_PACKET_SIZE");
		return false;
	}

	char dataBuf[TALK_PACKET_SIZE*2];
	memset(dataBuf, 0, TALK_PACKET_SIZE*2);
	int dataLen = mCTalkPacket->PackAudio(pData, nDataSize, mTalkInfo.cuUserID, mTalkInfo.toUserID, mTalkResInfo.session, dataBuf, TALK_PACKET_SIZE*2);
	if(dataLen == 0) {
		LOGD("CTalkNetSDK->SendG7XXData() pack talk packet fail");
		return false;
	}

	bool bRet = mCUDPSer->SendPacket(mRegisterInfo.servIP, mRegisterInfo.servPort, dataBuf, dataLen);
	if(!bRet) {
		LOGD("CTalkNetSDK->SendG7XXData() send talk packet fail");
		return false;
	}

	//LOGD("CTalkNetSDK->SendG7XXData() success");
	return true;
}

/** @fn bool Molloc()
 *   @brief	分配内存
 *   @param [in] 无
 *   @param [out] 无
 *   @return  true - 成功; false - 失败
 */
bool CTalkNetSDK::Molloc() {
	// 创建互斥对象
	HPR_INT32 iRet = HPR_MutexCreate(&mLock);
	if (iRet == HPR_ERROR) {
		LOGD("CTalkNetSDK->Molloc() create pack Lock fail");
		return false;
	}

	mCTalkPacket = new CTalkPacket();
	if(mCTalkPacket == NULL) {
		LOGD("CTalkNetSDK->Molloc() new m_pCTalkPacket fail");
		Fini();
		return false;
	}

	mCUDPSer = new CUDPSer();
	if(mCUDPSer == NULL) {
		Free();
		return false;
	}

	return true;
}

/** @fn Free()
 *   @brief	释放内存
 *   @param [in] 无
 *   @param [out] 无
 *   @return
 */
void CTalkNetSDK::Free() {
	if(mCTalkPacket != NULL) {
		delete mCTalkPacket;
		mCTalkPacket = NULL;
	}

	if(mCUDPSer != NULL) {
		delete mCUDPSer;
		mCUDPSer = NULL;
	}

	HPR_MutexDestroy(&mLock);
}

/** @fn VerifyLoginParam
 *  @brief 验证登陆参数
 *  @param loginInfo - 登陆信息(IN)
 *  @param 无(OUT)
 *  @return true - 成功
 *          false - 失败
 */
bool CTalkNetSDK::IsLoginInfoValid(ST_LOGIN_INFO &loginInfo) {
	if(loginInfo.cuUserID == NULL) {
		LOGD("CTalkNetSDK->IsLoginInfoValid() loginInfo.cuUserID == NULL");
		return false;
	}

	if(loginInfo.servIP == NULL) {
		LOGD("CTalkNetSDK->IsLoginInfoValid() loginInfo.servIP == NULL");
		return false;
	}

	return true;
}

/** @fn IsTalkInfoValid
 *  @brief 验证对讲参数
 *  @param loginInfo - 对讲信息(IN)
 *  @param 无(OUT)
 *  @return true - 成功   false - 失败
 */
bool CTalkNetSDK::IsTalkInfoValid(ST_TALK_INFO &talkInfo) {
	if(talkInfo.toUserID == NULL) {
		LOGD("CTalkNetSDK->IsTalkInfoValid() talkInfo.toUserID == NULL");
		return false;
	}

	if(talkInfo.deviceIndexCode == NULL) {
		LOGD("CTalkNetSDK->IsTalkInfoValid() talkInfo.deviceIndexCode == NULL");
		return false;
	}

	return true;
}

/** @fn bool Register()
 *   @brief	注册服务器
 *   @param [in] loginInfo - 登陆信息
 *   @param [out] 无
 *   @return  true - 成功; false - 失败
 */
bool CTalkNetSDK::Register(ST_LOGIN_INFO &loginInfo)
{
	// 验证传入的registerInfo参数的有效性
	bool bRet = IsLoginInfoValid(loginInfo);
	if (!bRet){
		LOGD("CTalkNetSDK->Register() loginInfo invalid");
		return false;
	}

	LOGD("*********CTalkNetSDK->Register() loginInfo*********");
	LOGD(loginInfo.puID);
	LOGD("CTalkNetSDK->Register() puID len:%d", strlen(loginInfo.puID));
	LOGD(loginInfo.cuUserID);
	LOGD("CTalkNetSDK->Register() UserID len:%d", strlen(loginInfo.cuUserID));
	LOGD(loginInfo.password);
	LOGD("CTalkNetSDK->Register() password len:%d", strlen(loginInfo.password));
	LOGD(loginInfo.servIP);
	LOGD("CTalkNetSDK->Register() servIP len:%d", strlen(loginInfo.servIP));
	LOGD("CTalkNetSDK->Register() servPort:%d", loginInfo.servPort);
	LOGD("CTalkNetSDK->Register() type:%d", loginInfo.type);
	LOGD("CTalkNetSDK->Register() codecType:%d", loginInfo.codecType);

	memset(&mRegisterInfo, 0, sizeof(ST_LOGIN_INFO));
	memcpy(mRegisterInfo.puID, loginInfo.puID, strlen(loginInfo.puID));
	memcpy(mRegisterInfo.cuUserID, loginInfo.cuUserID, strlen(loginInfo.cuUserID));
	memcpy(mRegisterInfo.password, loginInfo.password, strlen(loginInfo.password));
	memcpy(mRegisterInfo.servIP, loginInfo.servIP, strlen(loginInfo.servIP));
	mRegisterInfo.servPort = loginInfo.servPort;
	mRegisterInfo.type = loginInfo.type;
	mRegisterInfo.codecType = loginInfo.codecType;

	char dataBuf[TALK_PACKET_SIZE];
	memset(dataBuf, 0, TALK_PACKET_SIZE);
	int dataLen  = mCTalkPacket->PackRegister(mSequence, loginInfo, dataBuf, TALK_PACKET_SIZE);
	IncreaseSequence(mSequence);
	if(dataLen == 0) {
		LOGD("CTalkNetSDK->Register() pack register packet fail");
		return false;
	}

	char replyBuf[TALK_PACKET_SIZE];
	unsigned int replyLen = TALK_PACKET_SIZE;
	memset(replyBuf, 0, TALK_PACKET_SIZE);
	bRet = mCUDPSer->SendPacketWaitReply(mRegisterInfo.servIP, mRegisterInfo.servPort, dataBuf, dataLen, REGISTER, replyBuf, &replyLen);
	if(!bRet) {
		LOGD("CTalkNetSDK->Register() send Wait Reply register packet fail");
		return false;
	}

	bRet = HandleRegisterRespone(replyBuf, replyLen, &mLoginResInfo);
	if(!bRet) {
		LOGD("CTalkNetSDK->Register() HandleRegisterRespone fail");
		return false;
	}

	LOGD("CTalkNetSDK->Register() success");
	return true;
}

/** @fn bool UnRegister()
 *   @brief	注销服务器
 *   @param [in] 无
 *   @param [out] 无
 *   @return true - 成功; false - 失败
 */
bool CTalkNetSDK::UnRegister()
{
	char dataBuf[TALK_PACKET_SIZE];
	memset(dataBuf, 0, TALK_PACKET_SIZE);
	int dataLen  = mCTalkPacket->PackUnRegister(mSequence, mRegisterInfo.cuUserID, dataBuf, TALK_PACKET_SIZE);
	IncreaseSequence(mSequence);
	if(dataLen == 0) {
		LOGD("CTalkNetSDK->UnRegister() pack unregister packet fail");
		// 服务器回复失败，任务成功，避免上层调用判断无法释放资源
		//return false;
	}

	char replyBuf[TALK_PACKET_SIZE];
	unsigned int replyLen = TALK_PACKET_SIZE;
	memset(replyBuf, 0, TALK_PACKET_SIZE);
	bool bRet = mCUDPSer->SendPacketWaitReply(mRegisterInfo.servIP, mRegisterInfo.servPort, dataBuf, dataLen, UNREGISTER, replyBuf, &replyLen);
	if(!bRet) {
		LOGD("CTalkNetSDK->UnRegister() send Wait Reply unregister packet fail");
		// 服务器回复失败，任务成功，避免上层调用判断无法释放资源
		//return false;
	}

	bRet = HandleUnRegisterRespone(replyBuf, replyLen);
	if(!bRet) {
		LOGD("CTalkNetSDK->UnRegister() HandleUnRegisterRespone fail");
		// 服务器回复失败，任务成功，避免上层调用判断无法释放资源
		//return false;
	}

	LOGD("CTalkNetSDK->UnRegister() success");
	return true;
}

/** @fn bool KeepAliveProc()
 *   @brief	接收线程执行函数
 *   @param [in] pParam
 *   @param [out] 无
 *   @return
 */
HPR_VOIDPTR CTalkNetSDK::KeepAliveProc(HPR_VOIDPTR pParam)
{
	CTalkNetSDK* pCTalkNetSDK = (CTalkNetSDK*)pParam;
    if (pCTalkNetSDK != NULL)
    {
    	pCTalkNetSDK->KeepAlive();
    }

    return NULL;
}

/** @fn bool KeepAlive()
 *   @brief	接收线程执行函数
 *   @param [in] 无
 *   @param [out] 无
 *   @return true - 成功; false - 失败
 */
bool CTalkNetSDK::KeepAlive() {


	// 循环数
	HPR_INT32 nLoop = 0;
	char userID[USER_ID_MAX_LEN];
	memset(userID, 0, USER_ID_MAX_LEN);
	memcpy(userID, mRegisterInfo.cuUserID, USER_ID_MAX_LEN);

	// 时间初始化
	mKeepAliveReqTime = time(NULL);
	mKeepAliveRespTime = time(NULL);

	for (;;) {
		if (bKeepAliveFinish) {
			LOGD("CTalkNetSDK->KeepAlive() keep alve finish");
			break;
		}

		// 发送保活包和接受到的保活包时间超过30s时,认为掉线
		if (mKeepAliveReqTime - mKeepAliveRespTime  > 30) {
			LOGD("CTalkNetSDK->KeepAlive() ReqTime2:%ld", mKeepAliveReqTime);
			LOGD("CTalkNetSDK->KeepAlive() RespTime2:%ld", mKeepAliveRespTime);
			mKeepAliveRespTime = mKeepAliveReqTime;
			if (mObserver != NULL) {
				char errorDes[MAX_ERROR_LEN] = {0};
				sprintf(errorDes, "%s", "KeepAlive fail");
				mObserver->NetSDKGErrorCallBack(KEEPALIVE_FAIL, errorDes, strlen(errorDes));
			}
		}

		// 发送间隔时间到
		if (nLoop >= mLoginResInfo.interval * 2) {
			LOGD("CTalkNetSDK->KeepAlive() ReqTime:%ld", mKeepAliveReqTime);
			LOGD("CTalkNetSDK->KeepAlive() RespTime:%ld", mKeepAliveRespTime);
			// 生成保活报文
			char dataBuf[TALK_PACKET_SIZE];
			memset(dataBuf, 0, TALK_PACKET_SIZE);
			unsigned int dataLen  = mCTalkPacket->PackKeepAlive(mSequence, mRegisterInfo.cuUserID, dataBuf, TALK_PACKET_SIZE);
			IncreaseSequence(mSequence);
			if(dataLen <= 0) {
				LOGD("CTalkNetSDK->KeepAlive() pack keep alive packet fail");
				hpr2_sleep(20);
				continue;
			}

			// 发送保活报文
			bool bRet = mCUDPSer->SendPacket(mRegisterInfo.servIP, mRegisterInfo.servPort, dataBuf, dataLen);
			if(!bRet) {
				LOGD("CTalkNetSDK->KeepAlive() send keepalive packet fail");
			}

			// 保活请求时间
			mKeepAliveReqTime = time(NULL);

			nLoop = 0;
		}
		else
		{
			hpr2_sleep(500);
			nLoop += 1;
		}
	}

	return true;
}

/** @fn void HandleReqPacket()
 *   @brief	处理回复信令
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [in] szFrom
 *   @param [in] nPort
 *   @param [out] 无
 *   @return
 */
void CTalkNetSDK::HandleReqPacket(const char* pPacket, unsigned int nPacketLen, const char* szFrom, unsigned short int nPort, unsigned long dwUser) {
	if(pPacket == NULL || nPacketLen == 0) {
		LOGD("CTalkNetSDK->HandleReqPacket() pPacket == NULL");
		return;
	}

	talk_hdr talkHdr = {0};
	bool bRet = mCTalkPacket->UnpackTalkHdr(pPacket, nPacketLen, &talkHdr);
	if(!bRet) {
		LOGD("CTalkNetSDK->HandleReqPacket() UnpackTalkHdr fail");
		return;
	}

	switch(talkHdr.cmd) {
		case REGISTER:
			LOGD("CTalkNetSDK->HandleReqPacket() REGISTER SUCCESS");
			break;
		case UNREGISTER:
			break;
		case KEEPALIVE:
			HandleKeepAliveRespone(pPacket, nPacketLen, talkHdr);
			break;
		case AUDIO:
			HandleAudioRespone(pPacket, nPacketLen);
			break;
		case GROUPAUDIO:
			break;
		case INDIVIDUALCALL:
			break;
		case QUITINDIVIDUALCALL:
			break;
		case JOINGROUP:
			break;
		case SERVERNOTIFY:
			break;
		case QUITGROUP:
			break;
		case INDIVIDUALINFO:
			break;
		case GROUPINFO:
			break;
		default:
			break;
	}
}

/** @fn void HandleRegisterRespone()
 *   @brief	处理注册回复
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [out] stLoginResInfo
 *   @param [out] 无
 *   @return true - 成功; false - 失败
 */
bool CTalkNetSDK::HandleRegisterRespone(const char* pPacket, unsigned int nPacketLen, ST_LOGIN_RES_INFO* stLoginResInfo) {
	if(pPacket == NULL || nPacketLen == 0) {
		LOGD("CTalkNetSDK->HandleRegisterRespone pPacket == NULL");
		return false;
	}

	if(stLoginResInfo == NULL) {
		LOGD("CTalkNetSDK->HandleRegisterRespone stLoginResInfo == NULL");
		return false;
	}

	talk_hdr talkHdr = {0};
	bool bRet = mCTalkPacket->UnpackTalkHdr(pPacket, nPacketLen, &talkHdr);
	if(!bRet) {
		LOGD("CTalkNetSDK->HandleRegisterRespone UnpackTalkHdr fail");
		return false;
	}

	if(talkHdr.status != SUCCESS) {
		mErrorCode = talkHdr.status;
		LOGD("CTalkNetSDK->HandleRegisterRespone talkHdr.status != SUCCESS");
		return false;
	}

	bRet = mCTalkPacket->UnPackRegister(pPacket, nPacketLen, stLoginResInfo);
	if(!bRet) {
		LOGD("CTalkNetSDK->HandleRegisterRespone UnPackRegister fail");
		return false;
	}

	LOGD("CTalkNetSDK->HandleRegisterRespone success");
	return true;
}

/** @fn void HandleUnRegisterRespone()
 *   @brief	处理注销回应信令
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [out] 无
 *   @return true - 成功; false - 失败
 */
bool CTalkNetSDK::HandleUnRegisterRespone(const char* pPacket, unsigned int nPacketLen) {
	if(pPacket == NULL || nPacketLen == 0) {
		LOGD("CTalkNetSDK->HandleUnRegisterRespone pPacket == NULL");
		return false;
	}

	talk_hdr talkHdr = {0};
	bool bRet = mCTalkPacket->UnpackTalkHdr(pPacket, nPacketLen, &talkHdr);
	if(!bRet) {
		LOGD("CTalkNetSDK->HandleUnRegisterRespone UnpackTalkHdr fail");
		return false;
	}

	if(talkHdr.status != SUCCESS) {
		mErrorCode = talkHdr.status;
		LOGD("CTalkNetSDK->HandleUnRegisterRespone talkHdr.status != SUCCESS");
		return false;
	}

	LOGD("CTalkNetSDK->HandleUnRegisterRespone success");
	return true;
}

bool CTalkNetSDK::HandleGetEncTypeRespone(const char* pPacket, unsigned int nPacketLen, int& encType) {
	if(pPacket == NULL || nPacketLen == 0) {
		LOGD("CTalkNetSDK->HandleGetEncTypeRespone pPacket == NULL");
		return false;
	}

	talk_hdr talkHdr = {0};
	bool bRet = mCTalkPacket->UnpackTalkHdr(pPacket, nPacketLen, &talkHdr);
	if(!bRet) {
		LOGD("CTalkNetSDK->HandleGetEncTypeRespone UnpackTalkHdr fail");
		return false;
	}

	if(talkHdr.status != SUCCESS) {
		mErrorCode = talkHdr.status;
		LOGD("CTalkNetSDK->HandleGetEncTypeRespone talkHdr.status != SUCCESS");
		return false;
	}

	bRet = mCTalkPacket->UnPackGetEncType(pPacket, nPacketLen, encType);
	if(!bRet) {
		LOGD("CTalkNetSDK->HandleGetEncTypeRespone UnPackGetEncType fail");
		return false;
	}

	LOGD("CTalkNetSDK->HandleGetEncTypeRespone success");
	return true;
}

/** @fn void HandleTalkRespone()
 *   @brief	处理单呼回应信令
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [in] sttalkResInfo
 *   @param [out] 无
 *   @return true - 成功; false - 失败
 */
bool CTalkNetSDK::HandleTalkRespone(const char* pPacket, unsigned int nPacketLen, _ST_TALK_RES_INFO* sttalkResInfo) {
	if(pPacket == NULL || nPacketLen == 0) {
		LOGD("CTalkNetSDK->HandleTalkRespone pPacket == NULL");
		return false;
	}

	if(sttalkResInfo == NULL) {
		LOGD("CTalkNetSDK->HandleRegisterRespone stLoginResInfo == NULL");
		return false;
	}

	talk_hdr talkHdr = {0};
	bool bRet = mCTalkPacket->UnpackTalkHdr(pPacket, nPacketLen, &talkHdr);
	if(!bRet) {
		LOGD("CTalkNetSDK->HandleTalkRespone UnpackTalkHdr fail");
		return false;
	}

	if(talkHdr.status != SUCCESS) {
		mErrorCode = talkHdr.status;
		LOGD("CTalkNetSDK->HandleTalkRespone talkHdr.status != SUCCESS");
		return false;
	}

	bRet = mCTalkPacket->UnPackIndividualCall(pPacket, nPacketLen, sttalkResInfo);
	if(!bRet) {
		LOGD("CTalkNetSDK->HandleTalkRespone UnPackRegister fail");
		return false;
	}

	LOGD("CTalkNetSDK->HandleTalkRespone success");
	return true;
}

/** @fn void HandleQuitTalkRespone()
 *   @brief 处理退出单呼回应信令
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [out] 无
 *   @return true - 成功; false - 失败
 */
bool CTalkNetSDK::HandleQuitTalkRespone(const char* pPacket, unsigned int nPacketLen) {
	if (pPacket == NULL || nPacketLen <= 0) {
		LOGD("CTalkNetSDK->HandleQuitTalkRespone() pPacket == NULL");
		return HPR_FALSE;
	}

	talk_hdr talkHdr = {0};
	bool bRet = mCTalkPacket->UnpackTalkHdr(pPacket, nPacketLen, &talkHdr);
	if(!bRet) {
		LOGD("CTalkNetSDK->HandleQuitTalkRespone UnpackTalkHdr fail");
		return false;
	}

	// 判断返回值是否是200
	if (talkHdr.status != SUCCESS){
		mErrorCode = talkHdr.status;
		LOGD("CTalkNetSDK->HandleQuitTalkRespone talkHdr.status != SUCCESS");
		return false;
	}

	LOGD("CTalkNetSDK->HandleQuitTalkRespone success");
	return true;
}

/** @fn void HandleQuitTalkRespone()
 *   @brief 处理心跳回应信令
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [in] talkHdr
 *   @param [out] 无
 *   @return true - 成功; false - 失败
 */
bool CTalkNetSDK::HandleKeepAliveRespone(const char* pPacket, unsigned int nPacketLen, talk_hdr talkHdr) {
	// 保活回应时间
	mKeepAliveRespTime = time(NULL);

    if (pPacket == NULL || nPacketLen <= 0) {
    	LOGD("CTalkNetSDK->HandleKeepAliveRespone() pPacket == NULL");
        return HPR_FALSE;
    }

	// 判断返回值是否是200
	if (talkHdr.status != SUCCESS){
		// 注册失败消息回调
		if (mObserver != NULL) {
			char errorDes[MAX_ERROR_LEN] = {0};
			sprintf(errorDes, "%s", "KeepAlive fail");
			mObserver->NetSDKGErrorCallBack(KEEPALIVE_FAIL, errorDes, strlen(errorDes));
		}

		return false;
	}

	return true;
}

/** @fn void HandleAudioRespone()
 *   @brief 处理音频回复
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [out] 无
 *   @return true - 成功; false - 失败
 */
bool CTalkNetSDK::HandleAudioRespone(const char* pPacket, unsigned int nPacketLen) {
	if(!bTalk && !bGroupTalk) {
		LOGD("CTalkNetSDK->HandleAudioRespone() no start");
		return false;
	}

	if (pPacket == NULL || nPacketLen <= 0) {
		LOGD("CTalkNetSDK->HandleAudioRespone() pPacket == NULL");
		return false;
	}

	unsigned int offset = 0;
	bool bRet = mCTalkPacket->UnPackAudio(pPacket, nPacketLen, &offset);
	if(!bRet) {
		LOGD("CTalkNetSDK->HandleTalkRespone UnPackRegister fail");
		return false;
	}

	if(mObserver != NULL) {
		mObserver->NetSDKG7XXDataCallBack(-1, (char*)(pPacket+offset), nPacketLen-offset, mTalkResInfo.fromUserID, strlen(mTalkResInfo.fromUserID));
	}

	//LOGD("CTalkNetSDK->HandleAudioRespone() success");
	return true;
}

/** @fn void IncreaseSequence()
 *   @brief 序列号自增
 *   @param [out] nSequence
 *   @return
 */
void CTalkNetSDK::IncreaseSequence(unsigned int & nSequence)
{
    nSequence++;
    nSequence = (nSequence >= 65535) ? 1 : nSequence;
}

/** @fn void getErrorCode()
 *   @brief 获取错误码
 *   @param [int] 无
 *   @return 错误码
 */
int CTalkNetSDK::GetErrorCode() {
	return mErrorCode;
}

/*****************对讲协议未完曾部分********************************************/
// 申请进入群组
bool CTalkNetSDK::JoinGroup(ST_GROUP_JOIN_INFO &groupJoinInfo)
{
	return true;
}
// 开始群对讲
bool CTalkNetSDK::StartGroupTalk()
{
	return true;
}
// 停止群对讲
void CTalkNetSDK::StopGroupTalk()
{

}
// 退出对讲组
bool CTalkNetSDK::QuitGroup(const char* cuUserID)
{
	return true;
}

// 点对点消息发送
bool CTalkNetSDK::SendMsg(ST_MSG_INFO &msgInfo)
{
	return true;
}
// 向群组发送消息
bool CTalkNetSDK::SendGroupMsg(ST_MSG_GROUP_INFO &msgGroupInfo)
{
	return true;
}


