/*
 * TalkPacket.cpp
 *
 *  Created on: 2014-6-9
 *      Author: weilinfeng
 */
#include "TalkPacket.h"
#include "MCRPrint.h"
#include "hpr2_byte_order.h"

//构造函数
CTalkPacket::CTalkPacket() {
}

// 析构函数
CTalkPacket::~CTalkPacket() {
}

/** @fn void PackRegister()
 *   @brief 生成注册包
 *   @param [in] sequence
 *   @param [in] loginInfo
 *   @param [out] dstBuf
 *   @param [in] dstBufLen
 *   @return datalen > 0 - 成功; datalen < 0 - 失败
 */
int CTalkPacket::PackRegister(unsigned int sequence, ST_LOGIN_INFO loginInfo,
		char* dstBuf, int dstBufLen) {
	if (dstBuf == NULL || dstBufLen < TALK_PACKET_SIZE) {
		LOGD("CTalkPacket::PackRegister() dstBuf == NULL\n");
		return 0;
	}

	memset(dstBuf, 0, dstBufLen);
	int dataLen = talk_hdr_len;
	// TYPE_PUID
	tlv_item_hdr tlvItemHdr = { 0 };
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_PUID);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(loginInfo.puID));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, loginInfo.puID, strlen(loginInfo.puID));
	dataLen = dataLen + strlen(loginInfo.puID);

	// TYPE_USRID
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_USRID);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(loginInfo.cuUserID));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, loginInfo.cuUserID, strlen(loginInfo.cuUserID));
	dataLen = dataLen + strlen(loginInfo.cuUserID);

	// TYPE_USRPSW
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_USRPSW);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(loginInfo.password));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, loginInfo.password, strlen(loginInfo.password));
	dataLen = dataLen + strlen(loginInfo.password);

	// TYPE_REGTYPE
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_REGTYPE);
	tlvItemHdr.datalen = hpr2_htons_c(2);
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, &loginInfo.type, 2);
	dataLen = dataLen + 2;

	// TYPE_ENCODETYPE
	const char* encode = NULL;
	hpr2_uint16 encodelen = 0;
	if (loginInfo.codecType == TYPE_G711_A) {
		encode = VALUE_G711_A;
		encodelen = VALUE_G711_A_LEN;
	} else if (loginInfo.codecType == TYPE_G711_U) {
		encode = VALUE_G711_U;
		encodelen = VALUE_G711_U_LEN;
	} else if (loginInfo.codecType == TYPE_G722) {
		encode = VALUE_G722;
		encodelen = VALUE_G722_LEN;
	} else if (loginInfo.codecType == TYPE_G726) {
		encode = VALUE_G726;
		encodelen = VALUE_G726_LEN;
	} else if (loginInfo.codecType == TYPE_AAC) {
		encode = VALUE_AAC;
		encodelen = VALUE_AAC_LEN;
	} else if (loginInfo.codecType == TYPE_MP3) {
		encode = VALUE_MP3;
		encodelen = VALUE_MP3_LEN;
	}

	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_ENCODETYPE);
	tlvItemHdr.datalen = hpr2_htons_c(encodelen);
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, encode, encodelen);
	dataLen = dataLen + encodelen;

	talk_hdr talkhdr = { 0 };
	talkhdr.magic = hpr2_htons_c(TALK_CMD_MAGIC);
	talkhdr.version = TALK_VERSION;
	talkhdr.bodytype = TALK_BODYTYPE;
	talkhdr.bodylen = hpr2_htonl_c((dataLen - talk_hdr_len));
	talkhdr.cmdtype |= CMD_REQUEST;
	talkhdr.reserve |= 0;
	talkhdr.cmd |= REGISTER;
	talkhdr.status = hpr2_htons_c(SUCCESS);
	talkhdr.sequence = hpr2_htonl_c(sequence);

	// 将命令码和命令类型转字节序
	unsigned char* pCur = (unsigned char*) &talkhdr + 2 * sizeof(hpr2_uint32);
	hpr2_htons_v(pCur, 0, 1);
	memcpy(dstBuf, &talkhdr, talk_hdr_len);

	LOGD("CTalkPacket::PackRegister() success\n");
	return dataLen;
}

/** @fn void PackUnRegister()
 *   @brief 生成注销包
 *   @param [in] sequence
 *   @param [in] userId
 *   @param [out] dstBuf
 *   @param [in] dstBufLen
 *   @return datalen > 0 - 成功; datalen < 0 - 失败
 */
int CTalkPacket::PackUnRegister(unsigned int sequence, const char* userId,
		char* dstBuf, int dstBufLen) {
	if (dstBuf == NULL || dstBufLen < TALK_PACKET_SIZE) {
		LOGD("CTalkPacket::PackKeepAlive() dstBuf == NULL\n");
		return 0;
	}

	memset(dstBuf, 0, dstBufLen);

	int dataLen = talk_hdr_len;

	// TYPE_USRID
	tlv_item_hdr tlvItemHdr = { 0 };
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_USRID);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(userId));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, userId, strlen(userId));
	dataLen = dataLen + strlen(userId);

	talk_hdr talkhdr = { 0 };
	talkhdr.magic = hpr2_htons_c(TALK_CMD_MAGIC);
	talkhdr.version = TALK_VERSION;
	talkhdr.bodytype = TALK_BODYTYPE;
	talkhdr.bodylen = hpr2_htonl_c((dataLen - talk_hdr_len));
	talkhdr.cmdtype |= CMD_REQUEST;
	talkhdr.reserve |= 0;
	talkhdr.cmd |= UNREGISTER;
	talkhdr.status = hpr2_htons_c(SUCCESS);
	talkhdr.sequence = hpr2_htonl_c(sequence);

	// 将命令码和命令类型转字节序
	unsigned char* pCur = (unsigned char*) &talkhdr + 2 * sizeof(hpr2_uint32);
	hpr2_htons_v(pCur, 0, 1);
	memcpy(dstBuf, &talkhdr, talk_hdr_len);

	LOGD("CTalkPacket::PackUnRegister() success\n");
	return dataLen;
}

/** @fn void PackIndividualCall()
 *   @brief 生成单呼包
 *   @param [in] sequence
 *   @param [in] talkInfo
 *   @param [out] dstBuf
 *   @param [in] dstBufLen
 *   @return datalen > 0 - 成功; datalen < 0 - 失败
 */
int CTalkPacket::PackIndividualCall(unsigned int sequence,
		ST_TALK_INFO talkInfo, char* dstBuf, int dstBufLen) {
	if (dstBuf == NULL || dstBufLen < TALK_PACKET_SIZE) {
		LOGD("CTalkPacket::PackTalk() dstBuf == NULL\n");
		return 0;
	}

	memset(dstBuf, 0, dstBufLen);

	int dataLen = talk_hdr_len;

	// TYPE_USRID_FROM
	tlv_item_hdr tlvItemHdr = { 0 };
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_USRID_FROM);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(talkInfo.cuUserID));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, talkInfo.cuUserID, strlen(talkInfo.cuUserID));
	dataLen = dataLen + strlen(talkInfo.cuUserID);

	// TYPE_USRID_TO
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_USRID_TO);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(talkInfo.toUserID));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, talkInfo.toUserID, strlen(talkInfo.toUserID));
	dataLen = dataLen + strlen(talkInfo.toUserID);

	// TYPE_DEVICE_INDEX_CODE
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_DEVICE_INDEX_CODE);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(talkInfo.deviceIndexCode));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, talkInfo.deviceIndexCode,
			strlen(talkInfo.deviceIndexCode));
	dataLen = dataLen + strlen(talkInfo.deviceIndexCode);

	// TYPE_DEVICE_TYPE
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_DEVICE_TYPE);
	tlvItemHdr.datalen = hpr2_htons_c(4);
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, &talkInfo.deviceType, 4);
	dataLen = dataLen + 4;

	// TYPE_CHANNEL_NUM
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_CHANNEL_NUM);
	tlvItemHdr.datalen = hpr2_htons_c(4);
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, &talkInfo.channelNum, 4);
	dataLen = dataLen + 4;

	talk_hdr talkhdr = { 0 };
	talkhdr.magic = hpr2_htons_c(TALK_CMD_MAGIC);
	talkhdr.version = TALK_VERSION;
	talkhdr.bodytype = TALK_BODYTYPE;
	talkhdr.bodylen = hpr2_htonl_c((dataLen - talk_hdr_len));
	talkhdr.cmdtype |= CMD_REQUEST;
	talkhdr.reserve |= 0;
	talkhdr.cmd |= INDIVIDUALCALL;
	talkhdr.status = hpr2_htons_c(SUCCESS);
	talkhdr.sequence = hpr2_htonl_c(sequence);

	// 将命令码和命令类型转字节序
	unsigned char* pCur = (unsigned char*) &talkhdr + 2 * sizeof(hpr2_uint32);
	hpr2_htons_v(pCur, 0, 1);
	memcpy(dstBuf, &talkhdr, talk_hdr_len);

	LOGD("CTalkPacket::PackIndividualCall() success\n");
	return dataLen;
}

/** @fn void PackIndividualCall()
 *   @brief 生成退出单呼包
 *   @param [in] sequence
 *   @param [in] userId
 *   @param [in] session
 *   @param [out] dstBuf
 *   @param [in] dstBufLen
 *   @return datalen > 0 - 成功; datalen < 0 - 失败
 */
int CTalkPacket::PackQuitIndividualCall(unsigned int sequence,
		const char* userId, unsigned int session, char* dstBuf, int dstBufLen) {
	if (dstBuf == NULL || dstBufLen < TALK_PACKET_SIZE) {
		LOGD("CTalkPacket::PackKeepAlive() dstBuf == NULL\n");
		return 0;
	}

	memset(dstBuf, 0, dstBufLen);

	int dataLen = talk_hdr_len;

	// TYPE_USRID
	tlv_item_hdr tlvItemHdr = { 0 };
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_USRID);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(userId));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, userId, strlen(userId));
	dataLen = dataLen + strlen(userId);

	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_SESSION);
	tlvItemHdr.datalen = hpr2_htons_c(4);
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, &session, 4);
	dataLen = dataLen + 4;

	talk_hdr talkhdr = { 0 };
	talkhdr.magic = hpr2_htons_c(TALK_CMD_MAGIC);
	talkhdr.version = TALK_VERSION;
	talkhdr.bodytype = TALK_BODYTYPE;
	talkhdr.bodylen = hpr2_htonl_c((dataLen - talk_hdr_len));
	talkhdr.cmdtype |= CMD_REQUEST;
	talkhdr.reserve |= 0;
	talkhdr.cmd |= QUITINDIVIDUALCALL;
	talkhdr.status = hpr2_htons_c(SUCCESS);
	talkhdr.sequence = hpr2_htonl_c(sequence);

	// 将命令码和命令类型转字节序
	unsigned char* pCur = (unsigned char*) &talkhdr + 2 * sizeof(hpr2_uint32);
	hpr2_htons_v(pCur, 0, 1);
	memcpy(dstBuf, &talkhdr, talk_hdr_len);

	LOGD("CTalkPacket::PackQuitIndividualCall() success\n");
	return dataLen;
}

/** @fn void PackKeepAlive()
 *   @brief 生成心跳包
 *   @param [in] sequence
 *   @param [in] userId
 *   @param [out] dstBuf
 *   @param [in] dstBufLen
 *   @return datalen > 0 - 成功; datalen < 0 - 失败
 */
int CTalkPacket::PackKeepAlive(unsigned int sequence, const char* userId,
		char* dstBuf, int dstBufLen) {
	if (dstBuf == NULL || dstBufLen < TALK_PACKET_SIZE) {
		LOGD("CTalkPacket::PackKeepAlive() dstBuf == NULL\n");
		return 0;
	}

	memset(dstBuf, 0, dstBufLen);

	int dataLen = talk_hdr_len;

	// TYPE_USRID
	tlv_item_hdr tlvItemHdr = { 0 };
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_USRID);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(userId));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, userId, strlen(userId));
	dataLen = dataLen + strlen(userId);

	talk_hdr talkhdr = { 0 };
	talkhdr.magic = hpr2_htons_c(TALK_CMD_MAGIC);
	talkhdr.version = TALK_VERSION;
	talkhdr.bodytype = TALK_BODYTYPE;
	talkhdr.bodylen = hpr2_htonl_c((dataLen - talk_hdr_len));
	talkhdr.cmdtype |= CMD_REQUEST;
	talkhdr.reserve |= 0;
	talkhdr.cmd |= KEEPALIVE;
	talkhdr.status = hpr2_htons_c(SUCCESS);
	talkhdr.sequence = hpr2_htonl_c(sequence);

	// 将命令码和命令类型转字节序
	unsigned char* pCur = (unsigned char*) &talkhdr + 2 * sizeof(hpr2_uint32);
	hpr2_htons_v(pCur, 0, 1);
	memcpy(dstBuf, &talkhdr, talk_hdr_len);

	//LOGD("CTalkPacket::PackKeepAlive() success\n");
	return dataLen;
}

// 生成获取编码类型的包
int CTalkPacket::PackGetEncType(unsigned int sequence, ST_DEV_INFO devInfo, char* dstBuf, int dstBufLen) {
	if (dstBuf == NULL || dstBufLen < TALK_PACKET_SIZE) {
		LOGD("CTalkPacket::PackGetEncType() dstBuf == NULL\n");
		return 0;
	}

	memset(dstBuf, 0, dstBufLen);

	int dataLen = talk_hdr_len;

	// TYPE_DEVICE_INDEX_CODE
	tlv_item_hdr tlvItemHdr = { 0 };
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_DEVICE_INDEX_CODE);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(devInfo.deviceIndexCode));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, devInfo.deviceIndexCode,
			strlen(devInfo.deviceIndexCode));
	dataLen = dataLen + strlen(devInfo.deviceIndexCode);

	// TYPE_DEVICE_TYPE
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_DEVICE_TYPE);
	tlvItemHdr.datalen = hpr2_htons_c(4);
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, &devInfo.deviceType, 4);
	dataLen = dataLen + 4;

	talk_hdr talkhdr = { 0 };
	talkhdr.magic = hpr2_htons_c(TALK_CMD_MAGIC);
	talkhdr.version = TALK_VERSION;
	talkhdr.bodytype = TALK_BODYTYPE;
	talkhdr.bodylen = hpr2_htonl_c((dataLen - talk_hdr_len));
	talkhdr.cmdtype |= CMD_REQUEST;
	talkhdr.reserve |= 0;
	talkhdr.cmd |= GET_INCODETYPE;
	talkhdr.status = hpr2_htons_c(SUCCESS);
	talkhdr.sequence = hpr2_htonl_c(sequence);

	// 将命令码和命令类型转字节序
	unsigned char* pCur = (unsigned char*) &talkhdr + 2 * sizeof(hpr2_uint32);
	hpr2_htons_v(pCur, 0, 1);
	memcpy(dstBuf, &talkhdr, talk_hdr_len);

	LOGD("CTalkPacket::PackGetEncType() success\n");
	return dataLen;
}

/** @fn void PackAudio()
 *   @brief 生成语音数据包
 *   @param [in] pData
 *   @param [in] nDataSize
 *   @param [in] fromUserId
 *   @param [in] toUserId
 *   @param [in] session
 *   @param [out] dstBuf
 *   @param [in] dstBufLen
 *   @return datalen > 0 - 成功; datalen < 0 - 失败
 */
int CTalkPacket::PackAudio(const char* pData, unsigned int nDataSize,
		const char* fromUserId, const char* toUserId, unsigned int session,
		char* dstBuf, int dstBufLen) {
	if (dstBuf == NULL || dstBufLen < TALK_PACKET_SIZE) {
		LOGD("CTalkPacket::PackAudio() dstBuf == NULL\n");
		return 0;
	}

	memset(dstBuf, 0, dstBufLen);

	// 初始书长度为音频数据头长度
	int dataLen = audio_hdr_len;

	// TYPE_USRID_FROM
	tlv_item_hdr tlvItemHdr = { 0 };
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_USRID_FROM);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(fromUserId));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, fromUserId, strlen(fromUserId));
	dataLen = dataLen + strlen(fromUserId);

	// TYPE_USRID_TO
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_USRID_TO);
	tlvItemHdr.datalen = hpr2_htons_c(strlen(toUserId));
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, toUserId, strlen(toUserId));
	dataLen = dataLen + strlen(toUserId);

	// TYPE_SESSION
	tlvItemHdr.tlvtype = hpr2_htons_c(TYPE_SESSION);
	tlvItemHdr.datalen = hpr2_htons_c(4);
	memcpy(dstBuf + dataLen, &tlvItemHdr, tlv_item_hdr_len);
	dataLen = dataLen + tlv_item_hdr_len;
	memcpy(dstBuf + dataLen, &session, 4);
	dataLen = dataLen + 4;

	audio_hdr audiohdr = { 0 };
	audiohdr.magic = hpr2_htons_c(AUDIO_MAGIC);
	audiohdr.offset = dataLen;
	// 单呼数据头
	// 由于 audio_hdr结构体占了3个字节，使用sizeof获取长度时值为4，由于sizeof()四字节对齐导致
	memcpy(dstBuf, &audiohdr, audio_hdr_len);

	// 音频数据data
	memcpy(dstBuf + dataLen, pData, nDataSize);
	dataLen = dataLen + nDataSize;

	//LOGD("CTalkPacket::PackKeepAlive() success\n");
	return dataLen;
}

/** @fn void UnpackTalkHdr()
 *   @brief 拆数据头包
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [out] talkHdr
 *   @param [out] vchan
 *   @param [out] rank
 *   @return true - 成功; false - 失败
 */
bool CTalkPacket::UnpackTalkHdr(const char* pPacket, hpr2_uint32 nPacketLen,
		talk_hdr* talkHdr, hpr2_uint16* vchan, hpr2_uint8 *rank) {
	if (pPacket == NULL || nPacketLen == 0) {
		LOGD("CTalkPacket::UnpackTalkHdr() pPacket == NULL\n");
		return false;
	}

	if (talkHdr == NULL) {
		LOGD("CTalkPacket::UnpackTalkHdr() talkHdr == NULL\n");
		return false;
	}

	if (nPacketLen < talk_hdr_len) {
		LOGD(
				"CTalkPacket::unpackcmd() nPacketLen < talk_hdr_len, len:%d\n",
				nPacketLen);
		return false;
	}

	memset(talkHdr, 0, talk_hdr_len);
	char temBuf[talk_hdr_len] = { 0 };
	memcpy(temBuf, pPacket, talk_hdr_len);

	hpr2_uint16 magic;
	unsigned char* pCur = (unsigned char*) temBuf;
	hpr2_ntohs_v(pCur, 0, 1);
	// 将报文魔术字段保存成员变量
	memcpy(&magic, temBuf, sizeof(hpr2_uint16));

	if (magic == AUDIO_MAGIC) {
		audio_hdr* audioHdr = reinterpret_cast<audio_hdr*>(temBuf);
		talkHdr->magic = audioHdr->magic;
		talkHdr->cmd = AUDIO;
		talkHdr->cmdtype = 1;
	} else if (magic == GROUP_AUDIO_MAGIC) {
		group_audio_hdr* groupAudioHdr =
				reinterpret_cast<group_audio_hdr*>(temBuf);
		talkHdr->magic = groupAudioHdr->magic;
		*vchan = hpr2_ntohs_c(groupAudioHdr->vChannel);
		*rank = groupAudioHdr->rank;
		talkHdr->cmd = GROUPAUDIO;
		talkHdr->cmdtype = 1;
	} else {
		unsigned char* pCur = (unsigned char*) temBuf + 2 * sizeof(hpr2_uint32);
		hpr2_ntohs_v(pCur, 0, 1);

		talk_hdr* temptalkhdr = reinterpret_cast<talk_hdr*>(temBuf);
		talkHdr->magic = temptalkhdr->magic;
		talkHdr->version = temptalkhdr->version;
		talkHdr->bodytype = temptalkhdr->bodytype;
		talkHdr->bodylen = hpr2_htonl_c(temptalkhdr->bodylen);
		talkHdr->cmdtype = temptalkhdr->cmdtype;
		talkHdr->cmd = temptalkhdr->cmd;
		talkHdr->status = hpr2_ntohs_c(temptalkhdr->status);
		talkHdr->sequence = hpr2_ntohl_c(temptalkhdr->sequence);

//		LOGD("CTalkPacket::UnpackTalkHdr() magic:%d\n",
//				talkHdr->magic);
//		LOGD("CTalkPacket::UnpackTalkHdr() version:%d\n",
//				talkHdr->version);
//		LOGD("CTalkPacket::UnpackTalkHdr() bodytype:%d\n",
//				talkHdr->bodytype);
//		LOGD("CTalkPacket::UnpackTalkHdr() bodylen:%d\n",
//				talkHdr->bodylen);
//		LOGD("CTalkPacket::UnpackTalkHdr() cmdtype:%d\n",
//				talkHdr->cmdtype);
//		LOGD("CTalkPacket::UnpackTalkHdr() reserve:%d\n",
//				talkHdr->reserve);
//		LOGD("CTalkPacket::UnpackTalkHdr() cmd:%d\n", talkHdr->cmd);
//		LOGD("CTalkPacket::UnpackTalkHdr() status:%d\n",
//				talkHdr->status);
//		LOGD("CTalkPacket::UnpackTalkHdr() sequence:%d\n",
//				talkHdr->sequence);
	}

	//LOGD("CTalkPacket::UnpackTalkHdr() success\n");
	return true;
}

/** @fn void UnPackRegister()
 *   @brief 拆注册回复包
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [out] loginResInfo
 *   @return true - 成功; false - 失败
 */
bool CTalkPacket::UnPackRegister(const char* pPacket, unsigned int nPacketLen,
		ST_LOGIN_RES_INFO* loginResInfo) {
	if (pPacket == NULL || nPacketLen == 0) {
		LOGD("CTalkPacket::UnPackRegister() pPacket == NULL\n");
		return false;
	}

	if (loginResInfo == NULL) {
		LOGD(
				"CTalkPacket::UnPackRegister() loginResInfo == NULL\n");
		return false;
	}

	// 去掉包头
	pPacket = pPacket + talk_hdr_len;
	nPacketLen = nPacketLen - talk_hdr_len;

	if (nPacketLen < tlv_item_hdr_len) {
		LOGD(
				"CTalkPacket::UnPackRegister() nPacketLen too short\n");
		return false;
	}

	while (true) {
		if (nPacketLen < tlv_item_hdr_len) {
			LOGD("CTalkPacket::UnPackRegister() parse finish\n");
			return true;
		}

		tlv_item_hdr tlvHdr = { 0 };
		memcpy(&tlvHdr, pPacket, tlv_item_hdr_len);
		tlvHdr.tlvtype = hpr2_ntohs_c(tlvHdr.tlvtype);
		tlvHdr.datalen = hpr2_ntohs_c(tlvHdr.datalen);
		// 数据向后移tlv_item_hdr_len长度
		pPacket = pPacket + tlv_item_hdr_len;
		nPacketLen = nPacketLen - tlv_item_hdr_len;

		switch (tlvHdr.tlvtype) {
		case TYPE_INTERVAL:
			memcpy(&loginResInfo->interval, pPacket, sizeof(hpr2_uint16));
			LOGD("CTalkPacket::UnPackRegister() interval:%d\n",
					loginResInfo->interval);
			break;
		case TYPE_IP:
			memset(loginResInfo->redirectIp, 0, IP_MAX_LEN);
			memcpy(loginResInfo->redirectIp, pPacket, tlvHdr.datalen);
			LOGD("CTalkPacket::UnPackRegister() redirect Ip:%s",
					(char*) loginResInfo->redirectIp);
			break;
		case TYPE_PORT:
			memcpy(&loginResInfo->port, pPacket, sizeof(hpr2_uint16));
			LOGD("CTalkPacket::UnPackRegister() redirectPort:%d\n",
					loginResInfo->port);
			break;
		default:
			break;
		}
		// 数据向后移datalen长度
		pPacket = pPacket + tlvHdr.datalen;
		nPacketLen = nPacketLen - tlvHdr.datalen;
	}

	LOGD("CTalkPacket::UnPackRegister() success\n");
	return true;
}

bool CTalkPacket::UnPackGetEncType(const char* pPacket, unsigned int nPacketLen,
		int& encType) {
//	LOGD(pPacket);
	if (pPacket == NULL || nPacketLen == 0) {
		LOGD("CTalkPacket::UnPackGetEncType() pPacket == NULL\n");
		return false;
	}

	// 去掉包头
	pPacket = pPacket + talk_hdr_len;
	nPacketLen = nPacketLen - talk_hdr_len;

	if (nPacketLen < tlv_item_hdr_len) {
		LOGD(
				"CTalkPacket::UnPackGetEncType() nPacketLen too short\n");
		return false;
	}

	while (true) {
		if (nPacketLen < tlv_item_hdr_len) {
			LOGD("CTalkPacket::UnPackGetEncType() parse finish\n");
			return true;
		}

		tlv_item_hdr tlvHdr = { 0 };
		memcpy(&tlvHdr, pPacket, tlv_item_hdr_len);
		tlvHdr.tlvtype = hpr2_ntohs_c(tlvHdr.tlvtype);
		tlvHdr.datalen = hpr2_ntohs_c(tlvHdr.datalen);
		// 数据向后移tlv_item_hdr_len长度
		pPacket = pPacket + tlv_item_hdr_len;
		nPacketLen = nPacketLen - tlv_item_hdr_len;

		switch (tlvHdr.tlvtype) {
		case TYPE_ENCODETYPE:
			LOGD("tlvHdr.datalen\n");
			char encTypes[32];
			memset(encTypes, 0, 32);
			memcpy(encTypes, pPacket, tlvHdr.datalen);
			encType = getEncType(encTypes);
			LOGD("CTalkPacket::UnPackGetEncType() encType:%d\n", encType);
			break;
		default:
			break;
		}
		// 数据向后移datalen长度
		pPacket = pPacket + tlvHdr.datalen;
		nPacketLen = nPacketLen - tlvHdr.datalen;
	}

	LOGD("CTalkPacket::UnPackGetEncType() success\n");
	return true;
}

/** @fn void UnPackRegister()
 *   @brief 拆单呼回复包
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [out] talkResInfo
 *   @return true - 成功; false - 失败
 */
bool CTalkPacket::UnPackIndividualCall(const char* pPacket,
		unsigned int nPacketLen, ST_TALK_RES_INFO* talkResInfo) {
	if (pPacket == NULL || nPacketLen == 0) {
		LOGD(
				"CTalkPacket::UnPackIndividualCall() pPacket == NULL\n");
		return false;
	}

	if (talkResInfo == NULL) {
		LOGD(
				"CTalkPacket::UnPackIndividualCall() talkResInfo == NULL\n");
		return false;
	}

	// 去掉包头
	pPacket = pPacket + talk_hdr_len;
	nPacketLen = nPacketLen - talk_hdr_len;

	if (nPacketLen < tlv_item_hdr_len) {
		LOGD(
				"CTalkPacket::UnPackIndividualCall() nPacketLen too short\n");
		return false;
	}

	while (true) {
		if (nPacketLen < tlv_item_hdr_len) {
			LOGD(
					"CTalkPacket::UnPackIndividualCall() parse finish\n");
			return true;
		}

		tlv_item_hdr tlvHdr = { 0 };
		memcpy(&tlvHdr, pPacket, tlv_item_hdr_len);
		tlvHdr.tlvtype = hpr2_ntohs_c(tlvHdr.tlvtype);
		tlvHdr.datalen = hpr2_ntohs_c(tlvHdr.datalen);
		// 数据向后移tlv_item_hdr_len长度
		pPacket = pPacket + tlv_item_hdr_len;
		nPacketLen = nPacketLen - tlv_item_hdr_len;

		switch (tlvHdr.tlvtype) {
		case TYPE_USRID_FROM:
			memset(talkResInfo->fromUserID, 0, USER_ID_MAX_LEN);
			memcpy(talkResInfo->fromUserID, pPacket, tlvHdr.datalen);
			LOGD(
					"CTalkPacket::UnPackIndividualCall() from userID:%s",
					talkResInfo->fromUserID);
			break;
		case TYPE_SESSION:
			memcpy(&talkResInfo->session, pPacket, sizeof(hpr2_uint32));
			LOGD("CTalkPacket::UnPackIndividualCall() session:%d\n",
					talkResInfo->session);
			break;
		case TYPE_USRID_TO:
			memset(talkResInfo->toUserID, 0, USER_ID_MAX_LEN);
			memcpy(talkResInfo->toUserID, pPacket, tlvHdr.datalen);
			LOGD("CTalkPacket::UnPackIndividualCall() to userID:%s",
					talkResInfo->toUserID);
			break;
		default:
			break;
		}

		// 数据向后移datalen长度
		pPacket = pPacket + tlvHdr.datalen;
		nPacketLen = nPacketLen - tlvHdr.datalen;
	}

	LOGD("CTalkPacket::UnPackIndividualCall() success\n");
	return true;
}

/** @fn void UnPackAudio()
 *   @brief 拆语音回复包
 *   @param [in] pPacket
 *   @param [in] nPacketLen
 *   @param [out] offset
 *   @return true - 成功; false - 失败
 */
bool CTalkPacket::UnPackAudio(const char* pPacket, unsigned int nPacketLen,
		unsigned int* offset) {

	if (pPacket == NULL || nPacketLen == 0) {
		LOGD("CTalkPacket::UnPackAudio() pPacket == NULL\n");
		return false;
	}

	// 去掉包头
	audio_hdr audioHdr = { 0 };
	memcpy(&audioHdr, pPacket, audio_hdr_len);
	audioHdr.magic = hpr2_ntohs_c(audioHdr.magic);
	*offset = audioHdr.offset;

	//LOGD("CTalkPacket::UnPackAudio() success\n");
	return true;
}

int CTalkPacket::getEncType(const char* encTypes) {
	int encType = 0;
	if (memcmp(encTypes, VALUE_G711_A, VALUE_G711_A_LEN)  == 0) {
		encType = TYPE_G711_A;
	}
	else if (memcmp(encTypes, VALUE_G711_U, VALUE_G711_U_LEN)  == 0) {
		encType = TYPE_G711_U;
	}
	else if (memcmp(encTypes, VALUE_G722, VALUE_G722_LEN)  == 0) {
		encType = TYPE_G722;
	}
	else if (memcmp(encTypes, VALUE_G726, VALUE_G726_LEN)  == 0) {
		encType = TYPE_G726;
	}
	else if (memcmp(encTypes, VALUE_AAC, VALUE_AAC_LEN)  == 0) {
		encType = TYPE_AAC;
	}
	else if (memcmp(encTypes, VALUE_MP3, VALUE_MP3_LEN)  == 0) {
		encType = TYPE_MP3;
	}

	return encType;
}



