/*
 * UDPSer.cpp
 *
 *  Created on: 2014-6-7
 *      Author: weilinfeng
 */

#include "UDPSer.h"
#include "MCRPrint.h"
#include "HPR_Netdb.h"
#include "hpr2_sleep.h"
#include "HPR_Select.h"
#include "HPR_Types.h"


// 构造函数
CUDPSer::CUDPSer()
: bCreate(false)
, mRecvFinish(false)
, mRecvThreadHanlde(HPR_INVALID_HANDLE)
, mHandThreadHanlde(HPR_INVALID_HANDLE)
, pfnListener(NULL)
, m_iMsgCount(0)
, m_dwUser(0)
, bWaitReply(false)
{
	m_pCTalkPacket = new CTalkPacket();
}

CUDPSer:: ~CUDPSer()
{
	if(m_pCTalkPacket != NULL) {
		delete m_pCTalkPacket;
		m_pCTalkPacket = NULL;
	}
}

/** @fn SendPacketWaitReply
 *  @brief 创建任意端口本地套接字
 *  @param 无
 *  @return true - 成功， false - false
 */
bool CUDPSer::Create(pfnDataListener pfn, unsigned long dwUser, const char* szSrvIP, unsigned short port)
{
	if(bCreate) {
		LOGD("CUDPSer->Create() have been created");
		return true;
	}

	if(pfn == NULL) {
		LOGD("CUDPSer->Create() pfn == NULL");
		return false;
	}

	// 创建互斥对象
	HPR_INT32 iRet = HPR_MutexCreate(&mQueueLock);
	if (iRet == HPR_ERROR) {
		LOGD("CTalkNetSDK->Init() create Queue Lock fail");
		return false;
	}

	iRet = HPR_MutexCreate(&mMapLock);
	if (iRet == HPR_ERROR) {
		LOGD("CTalkNetSDK->Init() create map Lock fail");
		HPR_MutexDestroy(&mQueueLock);
		return false;
	}

	// 创建对讲套接字
	HPR_SOCK_T listenSocket = HPR_CreateSocket(AF_INET, SOCK_DGRAM, 0);
	if (listenSocket == HPR_INVALID_SOCKET) {
		LOGD("CUDPSer->Create() create socket fail");
		HPR_MutexDestroy(&mMapLock);
		HPR_MutexDestroy(&mQueueLock);
		return false;
	}

	HPR_ADDR_T localAddr;
	HPR_INT32 ret = HPR_MakeAddrByString(AF_INET, szSrvIP, port, &localAddr);
	if (ret != HPR_OK) {
		LOGD("CUDPSer->Create() HPR_MakeAddrByString fail:%d", HPR_GetSystemLastError());
		HPR_MutexDestroy(&mMapLock);
		HPR_MutexDestroy(&mQueueLock);
		HPR_CloseSocket(listenSocket);
		listenSocket = HPR_INVALID_SOCKET;
		return false;
	}

	if (HPR_Bind(listenSocket, (HPR_ADDR_T *)&localAddr) == -1) {
		LOGD("CUDPSer->Create() HPR_Bind fail:%d", HPR_GetSystemLastError());
		HPR_MutexDestroy(&mMapLock);
		HPR_MutexDestroy(&mQueueLock);
		HPR_CloseSocket(listenSocket);
		listenSocket = HPR_INVALID_SOCKET;
		return false;
	}

	mListenSocket = listenSocket;
	pfnListener = pfn;
	m_dwUser = dwUser;
	bCreate = true;

	return true;
}

/** @fn SendPacketWaitReply
 *  @brief 发送数据,不等待回应
 *  @param 无
 *  @return true - 成功， false - false
 */
bool CUDPSer::SendPacket(const char* szToIP, unsigned short port, const void* sendBuf, unsigned int dataLen){
	if(sendBuf == NULL || dataLen == 0) {
		LOGD("CUDPSer->SendPacket() sendBuf == NULL");
		return false;
	}

	HPR_ADDR_T remoteAddr;
	HPR_INT32 iret = HPR_MakeAddrByString(AF_INET, szToIP, port, &remoteAddr);
	if (iret != HPR_OK) {
		LOGD("CUDPSer->SendPacket() HPR_MakeAddrByString fail");
		return false;
	}

	iret = HPR_SendTo(mListenSocket, (HPR_VOIDPTR)sendBuf, dataLen, &remoteAddr);
	if (iret == HPR_ERROR) {
		char szTmp[1024 * 2] = {0};
		sprintf(szTmp,"CUDPSer->SendPacket() HPR_SendTo failed, IP:%s, port:%d, data:%s, socket:%d, err=%d\n",szToIP, port, (const char*) sendBuf, mListenSocket, HPR_GetLastError());
		LOGD(szTmp);
		return false;
	}

	LOGD("CUDPSer->SendPacket() success dataLen:%d", dataLen);
	return true;
}

/** @fn SendPacketWaitReply
 *  @brief 发送数据,等待回应
 *  @param 无
 *  @return true - 成功， false - false
 */
bool CUDPSer::SendPacketWaitReply(const char* szIP, unsigned short port, const void* contentBuf, unsigned int contentLen, hpr2_uint16 cmd, char* reply, unsigned int* replyLen, unsigned int waitSec)
{
	if(reply == NULL) {
		LOGD("CUDPSer->SendPacketWaitReply()  reply buffer == NULL");
		return false;
	}

	if(contentBuf == NULL || contentLen <= 0) {
		LOGD("CUDPSer->SendPacketWaitReply()  contentBuf == NULL");
		return false;
	}

	// 等待响应
	bWaitReply = true;
	bool  bRet = false;

	ST_CMD_TIME stCmdTime = {0};
	stCmdTime.cmd = cmd;
	stCmdTime.bReply = false;
	stCmdTime.reqTime = time(NULL);
	LOGD("CUDPSer->SendPacketWaitReply()  SendPacketWaitReply1 stCmdTime.reqTime:%ld", stCmdTime.reqTime);

	//需要加锁
	HPR_MutexLock(&mMapLock);
	LOGD("CUDPSer->SendPacketWaitReply() m_TimeMap cmd:%d\n", cmd);
	std::pair<std::map<hpr2_uint16, ST_CMD_TIME>::iterator, bool> ret = m_TimeMap.insert(std::map<hpr2_uint16, ST_CMD_TIME>::value_type(cmd,stCmdTime));
	if(!ret.second) {
		LOGD("CUDPSer->SendPacketWaitReply()  m_TimeMap.insert fail");
		HPR_MutexUnlock(&mMapLock);
		bWaitReply = false;
		return false;
	}
	HPR_MutexUnlock(&mMapLock);

	// 获取当前时间
	time_t nowTime = time(NULL);
	// 循环技计数
	int count = 0;
	while(true) {
		// 每秒发送一个包
		if((count % 1000) == 0) {
			bRet = SendPacket(szIP, port, contentBuf, contentLen);
			if(!bRet) {
				LOGD("CUDPSer->SendPacketWaitReply() SendPacket fail");
			}
			LOGD("CUDPSer->SendPacketWaitReply() SendPacket success");
		}

		// 需要加锁
		HPR_MutexLock(&mMapLock);
		std::map<hpr2_uint16, ST_CMD_TIME>::iterator iter = m_TimeMap.find(cmd);
		if(iter == m_TimeMap.end()){
			LOGD("CUDPSer->SendPacketWaitReply()  m_TimeMap.find fail");
			bRet = false;
			HPR_MutexUnlock(&mMapLock);
			break;
		}
		LOGD("CUDPSer->SendPacketWaitReply()  m_TimeMap.find success");
		stCmdTime = iter->second;

		// 如果收到回复则直接返回
		if(stCmdTime.bReply) {
			LOGD("CUDPSer->SendPacketWaitReply()  SendPacketWaitReply success");
			memset(reply, 0, *replyLen);
			*replyLen = stCmdTime.stCmdUnit.nDataBufLen;
			memcpy(reply, stCmdTime.stCmdUnit.szDataBuf, stCmdTime.stCmdUnit.nDataBufLen);
			HPR_MutexUnlock(&mMapLock);
			bRet = true;
			break;
		}

		HPR_MutexUnlock(&mMapLock);

		// 获取当前时间
		nowTime = time(NULL);
		LOGD("CUDPSer->SendPacketWaitReply()  SendPacketWaitReply2 nowTime:%ld", nowTime);
		LOGD("CUDPSer->SendPacketWaitReply()  SendPacketWaitReply2 stCmdTime.reqTime:%ld", stCmdTime.reqTime);
		// 发送包时间超过waitSecs时,认为失败
		if (nowTime - stCmdTime.reqTime > waitSec) {
			LOGD("CUDPSer->SendPacketWaitReply()  SendPacketWaitReply (nowTime - stCmdTime.reqTime > %d)", waitSec);
			bRet = false;
			break;
		}

		hpr2_sleep(100);
		count = count + 100;
	}

	HPR_MutexLock(&mMapLock);
	// 需要加锁
	m_TimeMap.erase(cmd);
	HPR_MutexUnlock(&mMapLock);

	bWaitReply = false;
	return bRet;
}

/** @fn Stop
 *  @brief 停止数据接收
 *  @param 无
 *  @return
 */
void CUDPSer::Stop()
{
	mRecvFinish = true;

	if (mHandThreadHanlde != HPR_INVALID_HANDLE) {
		HPR_Thread_Wait(mHandThreadHanlde);
		LOGD("CUDPSer->Stop() handle data thread wait end\n");
		mHandThreadHanlde = HPR_INVALID_HANDLE;
	}

	if (mRecvThreadHanlde != HPR_INVALID_HANDLE) {
		HPR_Thread_Wait(mRecvThreadHanlde);
		LOGD("CUDPSer->Stop() recv thread wait end\n");
		mRecvThreadHanlde = HPR_INVALID_HANDLE;
	}

	if (HPR_INVALID_SOCKET != mListenSocket) {
		HPR_CloseSocket(mListenSocket);
		mListenSocket = HPR_INVALID_SOCKET;
	}

	HPR_MutexDestroy(&mMapLock);
	HPR_MutexDestroy(&mQueueLock);

	bCreate = false;

	LOGD("CUDPSer->Stop() success\n");
}

/** @fn Run
 *  @brief 开始数据接收
 *  @param 无
 *  @return
 */
bool CUDPSer::Run()
{
	mRecvFinish = false;

	mRecvThreadHanlde = HPR_Thread_Create(RecvPacketProc, this, 0);
	if (mRecvThreadHanlde == NULL)
	{
		LOGD("CUDPSer->Create() create thread fail");
		Stop();
		return false;
	}

	mHandThreadHanlde = HPR_Thread_Create(impDataHandler, this, 0);
	if (mHandThreadHanlde == NULL)
	{
		LOGD("CUDPSer->Create() create data handle thread fail");
		Stop();
		return false;
	}

	LOGD("CUDPSer->Run() success\n");
	return true;
}

/** @fn impDataHandler
 *  @brief 接收线程
 *  @param 无
 *  @return
 */
HPR_VOIDPTR CUDPSer::RecvPacketProc(HPR_VOIDPTR pParam)
{
	CUDPSer* pCUDPSer = (CUDPSer*)pParam;
    if (pCUDPSer != NULL)
    {
    	pCUDPSer->RecvData();
    }

    return NULL;
}

/** @fn impDataHandler
 *  @brief 接收线程执行函数
 *  @param 无
 *  @return
 */
HPR_VOIDPTR CUDPSer::impDataHandler(HPR_VOIDPTR pParam)
{
	CUDPSer* pCUDPSer = (CUDPSer*)pParam;
    if (pCUDPSer != NULL)
    {
    	pCUDPSer->HandleData();
    }

    return NULL;
}

/** @fn RecvData
 *  @brief 接收数据
 *  @param 无
 *  @return true - 成功， false - false
 */
bool CUDPSer::RecvData()
{
	// 循环接受报文
	for (;;) {
		if (mRecvFinish) {
			LOGD("CUDPSer->RecvData() recv finish");
			break;
		}

		// 接受数据
		char czRecvBuf[MAX_PACKET_LEN] = {0};
		unsigned int  recvSize = RecvUDPPacket(&mServAddr, czRecvBuf, 1);
		if (recvSize > 0) {
			if(bWaitReply) {
				talk_hdr talkHdr = {0};
				if(m_pCTalkPacket->UnpackTalkHdr(czRecvBuf, recvSize, &talkHdr)) {
					// 需要加锁
					HPR_MutexLock(&mMapLock);
					LOGD("CUDPSer->RecvData() m_TimeMap cmd:%d\n", talkHdr.cmd);
					std::map<hpr2_uint16, ST_CMD_TIME>::iterator iter = m_TimeMap.find(talkHdr.cmd);
					if(iter != m_TimeMap.end()){
						// 如果MAG回复的包的状态为正在处理，则继续等待直到回复正确或者超时
						if(talkHdr.status != HANDLEING_STATE) {
							iter->second.bReply = true;
							//const char* serverIp = HPR_GetAddrString(&mServAddr);
							//memcpy(stCmdTime.stCmdUnit.szFromIP, serverIp, IP_MAX_LEN);
							iter->second.stCmdUnit.nFromPort = HPR_GetAddrPort(&mServAddr);
							memcpy(iter->second.stCmdUnit.szDataBuf, czRecvBuf, recvSize);
							iter->second.stCmdUnit.nDataBufLen = recvSize;
						}
					}
					HPR_MutexUnlock(&mMapLock);
				}
				else {
					LOGD("CUDPSer->RecvData() UnpackTalkHdr fail\n");
				}
			}

			ST_DATA_UNIT stDataUnit;
//			const char* serverIp = HPR_GetAddrString(&mServAddr);
//			memcpy(stDataUnit.szFromIP, serverIp, IP_MAX_LEN);
			stDataUnit.nFromPort = HPR_GetAddrPort(&mServAddr);
			memcpy(stDataUnit.szDataBuf, czRecvBuf, recvSize);
			stDataUnit.nDataBufLen = recvSize;

			HPR_MutexLock(&mQueueLock);
			// 当队列中数据超过50个时，删除首部元素，然后在插入元素
			if(m_iMsgCount > 500) {
				LOGD("CUDPSer->RecvData() push3 m_iMsgCount > 500\n");
				m_DataQueue.pop();
			}
			m_DataQueue.push(stDataUnit);
			m_iMsgCount = (unsigned int)m_DataQueue.size();
			//LOGD("CUDPSer->RecvData() m_iMsgCount:%d",m_iMsgCount);
			HPR_MutexUnlock(&mQueueLock);
		}
		else {
			//LOGD("CUDPSer->RecvData() fail\n");
		}
	}

	LOGD("CUDPSer->RecvData() finish\n");
	return true;
}

/** @fn HandleData
 *  @brief 处理接收到的数据
 *  @param 无
 *  @return true - 成功， false - false
 */
bool CUDPSer::HandleData()
{
	// 循环接受报文
	for (;;){
		if (mRecvFinish){
			LOGD("CUDPSer->HandleData() recv finish");
			break;
		}

		HPR_MutexLock(&mQueueLock);
		if(m_DataQueue.empty()) {
			//LOGD("CUDPSer->HandleData() DataQueue is empty");
			HPR_MutexUnlock(&mQueueLock);
			hpr2_sleep(500);
			continue;
		}
		ST_DATA_UNIT stDataUnit = m_DataQueue.front();
		//LOGD("CUDPSer->HandleData() DataQueue front");
		m_DataQueue.pop();
		//LOGD("CUDPSer->HandleData() DataQueue pop");
		HPR_MutexUnlock(&mQueueLock);

		// 如果内存中的数据为空时等待50毫秒
		if(m_iMsgCount == 0) {
			hpr2_sleep(50);
		}

		if(pfnListener != NULL) {
			(*pfnListener)(stDataUnit.szDataBuf, stDataUnit.nDataBufLen, stDataUnit.szFromIP, stDataUnit.nFromPort, m_dwUser);
		}
	}

	return true;
}

/** @fn RecvUDPPacket
 *  @brief 接受UDP报文
 *  @param fromAddr - 对方地址(IN)
 *  @param pBuf - 接受的内容(IN)
 *  @return 成功返回报文长度
 */
unsigned int CUDPSer::RecvUDPPacket(HPR_ADDR_T* fromAddr, char* pBuf, int waitSec)
{
    unsigned int recvSize = 0;
    struct timeval selectTimeout = {0, 0};
    selectTimeout.tv_sec = waitSec;

    fd_set readSet;
    FD_ZERO(&readSet);
    FD_SET(mListenSocket, &readSet);
    if (HPR_Select(mListenSocket + 1, &readSet, NULL, NULL, &selectTimeout) > 0) {
        if (HPR_FdIsSet(mListenSocket, &readSet)) {
            recvSize = HPR_RecvFrom(mListenSocket, pBuf, MAX_PACKET_LEN, (HPR_ADDR_T*)&fromAddr);
        }
    }
    //LOGD("CUDPSer->RecvUDPPacket() data size:%d", recvSize);
    return recvSize;
}

// 获取本地使用的IP和端口
int CUDPSer::GetSockAddr(char * pszIP, unsigned short * pnPort)
{
	LOGD("CUDPSer->GetSockAddr()");
	struct sockaddr_in name;
	socklen_t namelen = sizeof(name);
	getsockname(mListenSocket, (struct sockaddr*) &name, &namelen);
	*pnPort = ntohs(name.sin_port);
	GuessLocalIP(pszIP);
	strcpy(pszIP, inet_ntoa(name.sin_addr));
	return 0;
}

void GuessLocalIP(char* pszLocalIP) {
	char Hostname[100];
	hostent *pHostEnt;
	int nAdapter = 0;
	HPR_GetHostName(Hostname, sizeof(Hostname));
	pHostEnt = HPR_GetAddressByName(Hostname);

	while (pHostEnt->h_addr_list[nAdapter]) {
		char* p = inet_ntoa(*(in_addr*) pHostEnt->h_addr_list[nAdapter]);
		if (p != NULL) {
			strcpy(pszLocalIP, p);
			break;
		}
		nAdapter++;
	}
}
