/*
 * AECSpeaker.cpp
 *
 *  Created on: 2016-2-24
 *      Author: chifengliang
 */

#include "AECSpeaker.h"
#include <stdlib.h>
#include <stdio.h>
#include "KLog.h"
#include "ByteBuffer.h"
#include "hik_malloc.h"
#include "aec_error_code.h"
#include "datatypedef.h"

//#define TEST_AUDIO 1

AECSpeaker::AECSpeaker():
isCreate(false),
aec_handle(NULL),
aec_param(NULL)
{
	// TODO Auto-generated constructor stub
}

AECSpeaker::~AECSpeaker()
{
	// TODO Auto-generated destructor stub
	destroy();
}

bool AECSpeaker::create(int sample_rate, int bits_per_sample)
{
	if (isCreate)
	{
		return true;
	}
	try
	{
		byteBuf.allocate(FRAME_SIZE);
		consultBuf.allocate(FRAME_SIZE);
	}
	catch(char* ex) {
		//LOGD((const char*)ex);
		return false;
	}
	LOGD("the AEC version:%d", (int)HIKAEC_GetVersion());

	int ret = HPR_MutexCreate(&aecLock, HPR_MUTEX_FAST);
	if (ret != HPR_OK)
	{
		isCreate = false;
		return false;
	}

	S32 memsize;

	// 一次只处理一帧数据
	int h_result = HIKAEC_GetMemSize(&memsize);
	if (h_result != 1)
	{
		LOGE("getMemSize fail.result=%d", h_result);
		isCreate = false;
		HPR_MutexDestroy(&aecLock);
		return false;
	}
	aec_param = malloc(memsize);
	if (NULL == aec_param)
	{
		LOGE("outOfMemory could not malloc .");
		isCreate = false;
		HPR_MutexDestroy(&aecLock);
		return false;
	}

	h_result = HIKAEC_Create(aec_param, sample_rate);
	if (h_result != 1)
	{
		if (NULL != aec_param)
		{
			free(aec_param);
		}
		aec_param = NULL;
		isCreate = false;
		LOGE("HIKAECSP_Create fail. result=%d", h_result);
		HPR_MutexDestroy(&aecLock);
		return false;
	}
	isCreate = true;
	return true;
}
void AECSpeaker::destroy()
{
	if (NULL != aec_param)
	{
		free(aec_param);
		aec_param = NULL;
	}
	byteBuf.clear();
	HPR_MutexLock(&aecLock);
	consultBuf.clear();
	while (!dqueue.empty())
	{
		void* tmpData = dqueue.front();
		if (NULL != tmpData)
		{
			dqueue.pop();
			free(tmpData);
			tmpData = NULL;
		}
	}
	HPR_MutexUnlock(&aecLock);
	HPR_MutexDestroy(&aecLock);
}
void AECSpeaker::inputConsultData(void* data, int len)
{
	if (NULL == data)
	{
		return;
	}
	if (!isCreate)
	{
		return;
	}
//	return;
	HPR_MutexLock(&aecLock);
	// 将数据存入缓存
	int srcDataOffset = 0;
CONTINUE:
	int position = consultBuf.getPosition();
	int total = position + len;
	if (total >= FRAME_SIZE)
	{// 存入最后满一帧数据
		void* tmpData = NULL;
		if (dqueue.size() <= 40)
		{ // 如果存入的缓存大于40帧 丢弃
			tmpData = malloc(FRAME_SIZE);
			if (position > 0)
			{ // 如果缓存中有数据 将缓存中的数据拿出
				consultBuf.rewind();
				consultBuf.get(tmpData, position);
				consultBuf.clear();
			}
			memcpy(tmpData + position, (const void*)data + srcDataOffset, FRAME_SIZE - position);
			dqueue.push(tmpData);
		}
		len = len - (FRAME_SIZE - position);
		srcDataOffset += FRAME_SIZE - position;
		goto CONTINUE;
	}
	else if (total > 0)
	{ // 不满一帧数据 存入缓存
		consultBuf.put(data + srcDataOffset, len);
	}
	HPR_MutexUnlock(&aecLock);
}

bool AECSpeaker::proAEC(void* src, int srclen, void* dst, int& dstlen)
{
	if (!isCreate)
	{
		LOGD("has not create aec.");
		return false;
	}
	if (NULL == src || NULL == dst)
	{
//		LOGD("input src or dst data is NULL.");
		return false;
	}

	char consultframe[FRAME_SIZE] = {0};
	char oneframe[FRAME_SIZE] = {0};
	bool isFinish = true;
	int proCount = 0;
	int proSrcLen = 0;
	dstlen = 0;
	bool hasFarData = true;
	int totalSize = srclen + byteBuf.getPosition(); // 此次的数据 + 缓存数据
CONTINUE: // 如果数据没有处理完继续处理
// 获取参考数据
	HPR_MutexLock(&aecLock);
	if (!dqueue.empty())
	{
		hasFarData = true;
		void* tmpData = dqueue.front();
		dqueue.pop();
		if (NULL != tmpData)
		{
			memcpy(consultframe, tmpData, FRAME_SIZE);
			free(tmpData);
			tmpData = NULL;
		}
#ifdef TEST_AUDIO
	FILE* fp3 = fopen("/mnt/sdcard/const.pcm", "ab+");
	fwrite(consultframe, FRAME_SIZE, 1, fp3);
	fclose(fp3);
#endif
	}
	else
	{
		hasFarData = false;
	}
	HPR_MutexUnlock(&aecLock);


	memset(oneframe, 0, FRAME_SIZE);
	if (totalSize == FRAME_SIZE)
	{ // 整好一帧数据
		int bufSize = byteBuf.getPosition();
		if (bufSize > 0)
		{
			byteBuf.rewind();
			byteBuf.get(oneframe, bufSize);
			byteBuf.clear();
		}
		if (proCount > 0)
		{ // 此时src已经移动
			memcpy(oneframe, (const void*)src, FRAME_SIZE);
		}
		else
		{ // 第一次处理
			memcpy(oneframe + bufSize, (const void*)src, totalSize - bufSize);
		}
		isFinish = true;
	}
	else if (totalSize > FRAME_SIZE)
	{// 大于一帧数据
		int bufSize = byteBuf.getPosition();
		if (bufSize > 0)
		{
			byteBuf.rewind();
			byteBuf.get(oneframe, bufSize);
			byteBuf.clear();
		}
		memcpy(oneframe + bufSize, src, FRAME_SIZE - bufSize);
		totalSize -= FRAME_SIZE; // 总长减处理过的长度
		src += FRAME_SIZE - bufSize;
		proCount++; // 处理次数
		isFinish = false;
	}
	else if (totalSize < FRAME_SIZE)
	{// 数据不足， 存入缓存
		byteBuf.put(src, totalSize - byteBuf.getPosition());
		isFinish = true;
		if (proCount > 0)
		{ // 已经处理过数据
			return true;
		}
		else
		{
			return false;
		}
	}
	register int offset = 0;
	int ret = 0;

#ifdef TEST_AUDIO
	FILE* fp = fopen("/mnt/sdcard/src.pcm", "ab+");
	fwrite(oneframe, FRAME_SIZE, 1, fp);
	fclose(fp);
#endif

	if (hasFarData)
	{
		for (offset = 0; offset < FRAME_SIZE;)
		{
			ret = HIKAEC_Process(aec_param,
					(short*)(consultframe + offset), (const short*)(oneframe + offset),
					(short*)(dst + dstlen + offset), 64);
			if (ret != 1)
			{
				LOGD("aec process error.ret=%d", ret);
				memcpy(dst + dstlen + offset, oneframe + offset, 64 * 2);
			}
			offset += 64 * 2;
		}
	}
	else
	{ // 没有远端数据， 直接返回原始数据
		memcpy(dst + dstlen, oneframe, FRAME_SIZE);
	}

#ifdef TEST_AUDIO
	FILE* fp2 = fopen("/mnt/sdcard/src_aec.pcm", "ab+");
	fwrite(dst + dstlen, FRAME_SIZE, 1, fp2);
	fclose(fp2);
#endif
	dstlen += FRAME_SIZE;


	if (!isFinish)
	{// 还未处理完， 继续处理数据
//		LOGD("proAECis finish: %d", isFinish);
		goto CONTINUE;
	}
	else
	{
		return true;
	}
	return false;
}
