/*
 * ByteBuffer.h
 *
 *  Created on: 2016-2-3
 *      Author: chifengliang
 */

#ifndef BYTEBUFFER_H_
#define BYTEBUFFER_H_

class ByteBuffer {
public:
	ByteBuffer();
	virtual ~ByteBuffer();
public:
	/**
	 * 分配固定存储空间
	 */
	ByteBuffer* allocate(int capacity);

	/**
	 * 获取存储空间中的数据
	 * dst[out] 要获取数据存储位置
	 * dst[in] 需要获取的长度
	 */
	ByteBuffer* get(void* dst, int dstlen);

	/**
	 * 获取存储空间中的数据
	 * dst[out] 要获取数据存储位置
	 * dstOffset[in] 要存储的偏移量
	 * dst[in] 需要获取的长度
	 */
	ByteBuffer* get(void* dst, int dstOffset, int byteCount);

	/**
	 * 存入数据
	 * src[in] 要存储的数据
	 * srclen[in] 要存储的数据长度
	 */
	ByteBuffer* put(void* src, int srclen);

	/**
	 * 存入数据
	 * src[in] 要存储的数据
	 * srcOffset[in] 要存储的数据的偏移量
	 * byteCount[in] 要存储的数据的长度
	 */
	ByteBuffer* put(void* src, int srcOffset, int byteCount);

	/**
	 * 获取存储的数据
	 */
	void* array();

	/**
	 * 剩余数据量
	 */
	int remaining();

	/**
	 * 重置当前位置
	 */
	ByteBuffer* rewind();

	/**
	 * 获取当前位置
	 */
	int getPosition();

	/**
	 * 清除数据
	 */
	ByteBuffer* clear();

private:
	void* data;
	int position;
	int capacity;
	int limit;

};

#endif /* BYTEBUFFER_H_ */
