﻿/*******************************************************************************************************************************
** 版权信息：Copyright (c) 2008, 杭州海康威视数字技术有限公司
**
** 文件名称：audio_de_noise.h
**
** 文件标识：HIKVISION_AUDIO_DE_NOISE_H
**
** 文件摘要：声学去噪模块
**
** 当前版本：1.4.1
** 作    者：胡林艳
** 日    期：2015.04.15
** 备    注：修改fft的命名，和其他的冲突
**
** 当前版本：1.4.0
** 作    者：胡林艳
** 日    期：2014.12.01
** 备    注：增加48k，44.1k采样率
**
**
** 当前版本：1.3.2
** 作    者：张一凡
** 日    期：2014.09.24
** 备    注：修改算法库在R2平台的加密方式
**
** 当前版本：1.3.2
** 作    者：胡林艳
** 日    期：2014.07.4
** 备    注：问题单 PJ01D2014032703修改，在process时判断设置的采样点数和实际的采样点数是否相同
**
** 当前版本：1.3.1
** 作    者：胡林艳
** 日    期：2014.04.11
** 备    注：修改输入为320导致挂死的问题
**
**
** 当前版本：1.3.0
** 作    者：胡林艳
** 日    期：2014.03.23
** 备    注：增加32k采样率的算法
**
**
** 当前版本：1.2.0
** 作    者：胡林艳
** 日    期：2013.06.13
** 备    注：在ANR进行优化，修改可支持的帧长
**
** 当前版本：1.1.0
** 作    者：陈扬坤
** 日    期：2012.10.18
** 备    注：在AMR进行优化，修改接口
**
** 当前版本：1.0
** 作    者：沈林杰
** 日    期：2010.7.10
** 备    注：修正部分内存没有初始化
**
** 历史版本：0.8
** 作    者：沈林杰
** 日    期：2010.7.6
** 备    注：
**
*******************************************************************************************************************************/
#ifndef _HIK_AUDIO_DE_NOISE_H
#define _HIK_AUDIO_DE_NOISE_H

#include "datatypedef.h"
#include "mem_tab.h"

# ifdef __cplusplus
extern "C" {
# endif

/*******************************************************************************************************************************
//文件版本和时间宏声明
*******************************************************************************************************************************/
/* 当前版本号*/
#define HIK_DENOISE_MAJOR_VERSION       1
/* 子版本号，性能优化、局部结构调整、模块内集成其他库的主版本提升时递增，最大31 */
#define HIK_DENOISE_SUB_VERSION         4
/* 修正版本号，修正bug后递增，最大31 */
#define HIK_DENOISE_REVISION_VERSION    1
/* 版本日期*/
#define HIK_DENOISE_VER_YEAR            15                  /* 年                                  */
#define HIK_DENOISE_VER_MONTH           04                  /* 月                                  */
#define HIK_DENOISE_VER_DAY             15                  /* 日                                  */

#define HIK_DENOISE_FRAME_SIZE		    1280
typedef enum _De_Level_
{	
	DENOISE_LEVEL1 = 1,
	DENOISE_LEVEL2 = 2, 
	DENOISE_LEVEL3 = 3
}DENOISE_LEVEL;

//参数配置索引
typedef enum _DENOISE_CFG_ENUM_KEY_PARAM_
{
	DENOISE_CFG_PARAM_START               = 0x0000, 
	DENOISE_CFG_PARAM_ANR_LEVEL           = 0x0001,
	DENOISE_CFG_PARAM_ANR_SWITCH          = 0x0002,
	DENOISE_CFG_PARAM_END
}DENOISE_CFG_ENUM_KEY_PARAM;

/*******************************************************************************************************************************
* 状态码
*******************************************************************************************************************************/
#define HIK_DENOISE_LIB_S_FAIL              0x00000000      /* 函数调用失败                        */
#define HIK_DENOISE_LIB_S_OK                0x00000001      /* 函数调用成功                        */


#define HIK_DENOISE_LIB_E_PARAM_NULL        0x80000001      /* 输入指针参数为空                    */
#define HIK_DENOISE_LIB_E_FRAME_SIZE        0x80000002      /* 帧长错误                            */
#define HIK_DENOISE_LIB_E_BUF_NULL          0x80000003      /* 输入缓存的指针为空                  */
#define HIK_DENOISE_LIB_E_SAMPLE_RATE       0x80000004      /* 采样率不能为零                      */
#define HIK_DENOISE_LIB_E_DE_LEVEL          0x80000005      /* Level设置错误                       */

/*******************************************************************************************************************************
*******************************************************************************************************************************/

#ifndef _HRESULT_DEFINED
#define _HRESULT_DEFINED
typedef  S32 HRESULT;
#endif 



/* 声明去噪接口参数结构体 */
typedef struct _DENOISE_STATE_PARAM_STRU
{
    U32           frame_size;                               /* 每帧输入采样点个数                  */
    U32           sample_rate;                              /* 输入：采样率，支持8000，16000,32000 */
	DENOISE_LEVEL de_level;                                 /* 去噪Level, 默认level1,              */
	S32           reserved[16];                             /* 保留                                */
}DENOISE_PARAM;

typedef struct _DENOISE_PROCESS_PARAM_
{
	S16            *in_buf;                                 /* 输入数据buf                         */
	U32            in_buf_size;                             /* 输入数据buf采样点个数               */
	S16            *out_buf;                                /* 输出数据buf                         */
	U32            out_buf_size;                            /* 输出数据buf采样点个数               */
	U32            out_data_size;                           /* 输出数据实际采样点个数              */
	S32            reserved[16];                            /* 保留                                */
}DENOISE_PROCESS_PARAM;

// 算法一次处理的样点数
/* 可支持的输入采样点个数枚举 */
typedef enum _DENOISE_FRAME_LEN_TYPE_
{
	DENOISE_FRAME_LEN_TYPE_160       = 160,                 /* 160点，采样点个数                   */
	DENOISE_FRAME_LEN_TYPE_320       = 320,                 /* 320点，采样点个数                   */
	DENOISE_FRAME_LEN_TYPE_640       = 640,                 /* 640点，采样点个数                   */
	DENOISE_FRAME_LEN_TYPE_1024      = 1024,                /* 1024点，采样点个数                  */
	DENOISE_FRAME_LEN_TYPE_1152      = 1152,                /* 1152点，采样点个数                  */
	DENOISE_FRAME_LEN_TYPE_1280      = 1280,                /* 1280点，采样点个数                  */
}DENOISE_FRAME_LEN_TYPE;

/* 可支持采样率*/
typedef enum _DENOISE_SAMPLERATE_TYPE_
{
    DENOISE_SAMPLERATE_TYPE_8k       = 8000,                /* 8k采样率                            */
	DENOISE_SAMPLERATE_TYPE_16k      = 16000,               /* 16k采样率                           */
	DENOISE_SAMPLERATE_TYPE_32k      = 32000,               /* 32k采样率                           */
	DENOISE_SAMPLERATE_TYPE_48k      = 48000,               /* 48k采样率                           */
	DENOISE_SAMPLERATE_TYPE_44k      = 44100,               /* 44.1k采样率                         */
}DENOISE_SAMPLERATE_TYP;

/*******************************************************************************************************************************
* 接口函数声明
*******************************************************************************************************************************/

/*******************************************************************************************************************************
* 函数名: DENOISE_GetMemSize
*
* 功  能：获取去噪所需的内存大小
*
* 参  数：param      -I 去噪接口参数结构体指针
*         mem_tab    -I 内存申请表
*
* 返回值：返回状态码
*
* 备  注：

*******************************************************************************************************************************/
HRESULT HIK_DENOISE_GetMemSize(DENOISE_PARAM * param, MEM_TAB *mem_tab);

/*******************************************************************************************************************************
* 函数名: DENOISE_Create
*
* 功  能：去噪处理器创建
*
* 参  数： param      -I 接口参数结构体指针
*          mem_tab    -I 内存申请表
*          handle     -O 处理器句柄
*
* 返回值：返回状态码
*
* 备  注：

*******************************************************************************************************************************/
HRESULT HIK_DENOISE_Create(DENOISE_PARAM *param, MEM_TAB *mem_tab, void **handle);

/*******************************************************************************************************************************
* 函数名: DENOISE_Run
*
* 功  能：去噪处理
*
* 参  数：handle                  -I    处理器句柄
*         proc_param              -IO   处理参数
*
* 返回值：返回状态码
*
* 备  注：

*******************************************************************************************************************************/
HRESULT HIK_DENOISE_Proc(void * handle, DENOISE_PROCESS_PARAM *proc_param);

/*******************************************************************************************************************************
* 功  能：获取当前编码版本信息
*
* 参  数：无
*
* 返回值：返回版本信息
*
* 备  注：版本信息格式为：版本号＋年（7位）＋月（4位）＋日（5位）
*         其中版本号为：主版本号（6位）＋子版本号（5位）＋修正版本号（5位）
*******************************************************************************************************************************/
U32 HIK_DENOISE_GetVersion();

#ifdef __cplusplus
}
#endif

#endif 





















