/*
 * ByteBuffer.cpp
 *
 *  Created on: 2016-2-3
 *      Author: chifengliang
 */

#include "ByteBuffer.h"
#include <jni.h>
#include <stdlib.h>
#include <string>
#include "KLog.h"
ByteBuffer::ByteBuffer():
data(NULL),
position(0),
limit(0),
capacity(0)
{
	// TODO Auto-generated constructor stub

}

ByteBuffer::~ByteBuffer()
{
	// TODO Auto-generated destructor stub
	if (NULL != data)
	{
		free(data);
		data = NULL;
	}
	limit = 0;
	capacity = 0;
	position = 0;
}

ByteBuffer* ByteBuffer::allocate(int capacity)
{
	if (NULL != data)
	{
		LOGD("has already allocate.");
		return this;
	}
	if (capacity < 0)
	{
		throw capacity;
	}
	data = malloc(capacity);
	if (NULL == data)
	{
		LOGD("throw an Exception memory overflow.");
		throw "memory overflow. allocate fail.";
	}
	memset(data, 0, capacity);
	this->capacity = capacity;
	this->limit = capacity;
	return this;
}

/**
 * 获取存储空间中的数据
 * dst[out] 要获取数据存储位置
 * dst[in] 需要获取的长度
 */
ByteBuffer* ByteBuffer::get(void* dst, int dstlen)
{
	if (NULL == dst)
	{
		LOGD("thow an Exception dst is NULL.");
		throw "dst is NULL.";
	}

	if (dstlen > remaining())
	{ // 需要取的数据比目前存的数据多
		LOGD("thow an Exception dstlen > has datalen.[remain=%d][dstlen=%d]", remaining(), dstlen);
		throw "dstlen > has datalen";
	}
	if (NULL == data)
	{
		LOGD("thow an Exception NULL pointer.");
		throw "this has not allocated.";
	}
	memcpy(dst, data + position, dstlen);
	position += dstlen;
	return this;
}

/**
 * 获取存储空间中的数据
 * dst[out] 要获取数据存储位置
 * dstOffset[in] 要存储的偏移量
 * dst[in] 需要获取的长度
 */
ByteBuffer* ByteBuffer::get(void* dst, int dstOffset, int byteCount)
{
	if (NULL == dst)
	{
		LOGD("thow an Exception dst is NULL.");
		throw "dst is NULL.";
	}
	if (byteCount > remaining())
	{ // 需要取的数据比目前存的数据多
		LOGD("thow an Exception get too much.[remain=%d][byteCount=%d]", remaining(), byteCount);
		throw "dstlen > has datalen";
	}
	if (NULL == data)
	{
		LOGD("thow an Exception NULL pointer.");
		throw "this has not allocated.";
	}
	memcpy(dst + dstOffset, data, byteCount);
	position += byteCount;
	return this;
}

/**
 * 存入数据
 * src[in] 要存储的数据
 * srclen[in] 要存储的数据长度
 */
ByteBuffer* ByteBuffer::put(void* src, int srclen)
{
	if (NULL == src)
	{
		LOGD("thow an Exception src is NULL.");
		throw "src is NULL.";
	}
	if (srclen > remaining())
	{
		LOGD("thow an Exception overflow.[remain=%d][srclen=%d]", remaining(), srclen);
		throw "buffer overflow error.remaining=%d", remaining();
	}
	if (NULL == data)
	{
		LOGD("thow an Exception NULL pointer.");
		throw "this has not allocated.";
	}
	memcpy(data + position, src, srclen);
	position += srclen;
	return this;
}

/**
 * 存入数据
 * src[in] 要存储的数据
 * srcOffset[in] 要存储的数据的偏移量
 * byteCount[in] 要存储的数据的长度
 */
ByteBuffer* ByteBuffer::put(void* src, int srcOffset, int byteCount)
{
	if (NULL == src)
	{
		LOGD("thow an Exception dst is NULL.");
		throw "src is NULL.";
	}
	if (byteCount > remaining())
	{
		LOGD("thow an Exception overflow.[remain=%d][byteCount=%d]", remaining(), byteCount);
		throw "buffer overflow error.remaining=%d", remaining();
	}
	if (NULL == data)
	{
		LOGD("thow an Exception NULL pointer.");
		throw "this has not allocated.";
	}
	memcpy(data + position, src + srcOffset, byteCount);
	position += byteCount;
	return this;
}

/**
 * 获取存储的数据
 */
void* ByteBuffer::array()
{
	return data;
}

/**
 * 剩余数据量
 */
int ByteBuffer::remaining()
{
	return limit - position;
}

/**
 * 重置当前位置
 */
ByteBuffer* ByteBuffer::rewind()
{
	position = 0;
	return this;
}

/**
 * 获取当前位置
 */
int ByteBuffer::getPosition()
{
	return position;
}

/**
 * 清除数据
 */
ByteBuffer* ByteBuffer::clear()
{
	position = 0;
	limit = capacity;
	if (NULL != data)
	{
		memset(data, 0, capacity);
	}
	return this;
}
