/*
 * Denoise.cpp
 *
 *  Created on: 2015-11-20
 *      Author: chifengliang
 */

#include "Denoise.h"
#include "KLog.h"
Denoise::Denoise(int rate /** = 16000*/):lockRet(HPR_FALSE)
{
	// TODO Auto-generated constructor stub
	lockRet = HPR_MutexCreate(&noiseLock, HPR_MUTEX_FAST);
	mem_tab->base = NULL;
	param.sample_rate = rate;
	try
	{
		byteBuf.allocate(AUDIO_FRAME_SIZE);
	}
	catch(char* ex)
	{
		LOGD(ex);
	}
//	param.frame_size = -1;
}

jbyteArray Denoise::proNoise(JNIEnv * env, jbyteArray jsrcdata, jint jsrclen)
{
	if (NULL == env)
	{
		LOGE("JNIEnv is NULL.");
		return NULL;
	}

	if (lockRet == HPR_OK)
	{
		HPR_MutexLock(&noiseLock);
	}
	jbyte* byteData = env->GetByteArrayElements(jsrcdata, NULL);
	char out_buf[(int)jsrclen + AUDIO_FRAME_SIZE];
	memset(out_buf, 0, (int)jsrclen + AUDIO_FRAME_SIZE);
	int out_buf_len = 0;
	bool ret = proNoisePri(byteData, (int)jsrclen, out_buf, out_buf_len);
	env->ReleaseByteArrayElements(jsrcdata, byteData, 0);
	if (!ret)
	{
		if (lockRet == HPR_OK)
		{
			HPR_MutexUnlock(&noiseLock);
		}
//		LOGE("proNoise:get src data error return NULL.");
		return NULL;
	}

	jbyteArray data_array = env->NewByteArray(out_buf_len);

	if (data_array == NULL)
	{
		LOGE("Denoise::proNoise-> dstdata is NULL.");
		if (lockRet == HPR_OK)
		{
			HPR_MutexUnlock(&noiseLock);
		}
		return NULL;
	}
	env->SetByteArrayRegion(data_array, 0, out_buf_len, (const jbyte*)out_buf);
	if (lockRet == HPR_OK)
	{
		HPR_MutexUnlock(&noiseLock);
	}
	return data_array;
}

bool Denoise::proNoisePri(void* src, int srclen, void* dst, int& dstlen)
{
	if (srclen <= 0)
	{
		LOGD("srclen <= 0.");
		return false;
	}

	int totalSize = srclen + byteBuf.getPosition(); // 此次的数据 + 缓存数据
	char oneframe[AUDIO_FRAME_SIZE] = {0};
	bool isFinish = true;
	int proCount = 0;
	int proSrcLen = 0;
	dstlen = 0;
CONTINUE: // 如果数据没有处理完继续处理
	memset(oneframe, 0, AUDIO_FRAME_SIZE);
//	LOGI("totalSize=%d, proCount=%d, srclen=%d, position=%d", totalSize, proCount, srclen, byteBuf.getPosition());
	if (totalSize == AUDIO_FRAME_SIZE)
	{ // 整好一帧数据
		int bufSize = byteBuf.getPosition();
		if (bufSize > 0)
		{
			byteBuf.rewind();
			byteBuf.get(oneframe, bufSize);
			byteBuf.clear();
		}
		if (proCount > 0)
		{ // 此时src已经移动
			memcpy(oneframe, (const void*)src, AUDIO_FRAME_SIZE);
		}
		else
		{ // 第一次处理
			memcpy(oneframe + bufSize, (const void*)src, totalSize - bufSize);
		}
		isFinish = true;
	}
	else if (totalSize > AUDIO_FRAME_SIZE)
	{// 大于一帧数据
		int bufSize = byteBuf.getPosition();
		if (bufSize > 0)
		{
			byteBuf.rewind();
			byteBuf.get(oneframe, bufSize);
			byteBuf.clear();
		}
		memcpy(oneframe + bufSize, src, AUDIO_FRAME_SIZE - bufSize);
		totalSize -= AUDIO_FRAME_SIZE; // 总长减处理过的长度
		src += AUDIO_FRAME_SIZE - bufSize;
		proCount++; // 处理次数
		isFinish = false;
	}
	else if (totalSize < AUDIO_FRAME_SIZE)
	{// 数据不足， 存入缓存
		byteBuf.put(src, totalSize - byteBuf.getPosition());
		isFinish = true;
		if (proCount > 0)
		{ // 已经处理过数据
			return true;
		}
		else
		{
//			LOGI("this round has no enough data input to buffer.");
			return false;
		}
	}

	int size = (int)AUDIO_FRAME_SIZE / sizeof(short);
	int h_result = 0;

	if (size != param.frame_size)
	{
		param.frame_size = size;
		param.de_level = DENOISE_LEVEL1;
		h_result = HIK_DENOISE_GetMemSize(&param, mem_tab);
		if (h_result != HIK_DENOISE_LIB_S_OK)
		{
			LOGE("get memsize error.");
			return false;
		}
		if (NULL != mem_tab->base)
		{
			free(mem_tab->base);
			mem_tab->base = NULL;
		}
		mem_tab->base = (unsigned char*)malloc(mem_tab->size);
		if (mem_tab->base == NULL)
		{
			LOGE("memory calloc failed!\n");
			return false;
		}
		memset(mem_tab->base, 0, mem_tab->size);
		h_result = HIK_DENOISE_Create(&param, mem_tab, &handle);
		if (h_result != HIK_DENOISE_LIB_S_OK)
		{
			LOGE("create denoise error.%d", h_result);
			return false;
		}
	}
	char out_buf[AUDIO_FRAME_SIZE] = {0};
	proc_param.in_buf = (S16*)(oneframe);
	proc_param.in_buf_size = size;
	proc_param.out_buf = (S16*)(out_buf);
	proc_param.out_buf_size = size;
	int ret = HIK_DENOISE_Proc(handle, &proc_param); // 处理较长数据会崩溃，处理长度不一数据效果较差
	if (ret < 0)
	{
		return false;
	}
	memcpy(dst + dstlen, (void*)proc_param.out_buf, AUDIO_FRAME_SIZE);
	dstlen += AUDIO_FRAME_SIZE;
	if (!isFinish)
	{// 还未处理完， 继续处理数据
		goto CONTINUE;
	}
	else
	{
		return true;
	}
}

Denoise::~Denoise()
{
	// TODO Auto-generated destructor stub
	if (NULL != mem_tab->base)
	{
		free(mem_tab->base);
		mem_tab->base = NULL;
	}
	HPR_MutexDestroy(&noiseLock);
}

