/*
 * CopyRight:	HangZhou Hikvision System Technology Co., Ltd. All	Right Reserved.
 * FileName:	hpr2_rmm.h
 * Desc:		hpr2 reallocate memory(memory pool).
 * Author:		zhaoyiji
 * Date:		2012-11-12
 * Contact:		zhaoyiji@hikvision.com.cn
 * History:		Created By Zhaoyiji 2012-11-12 (V0.1)
 * */

#ifndef __HPR2_RMM_H__
#define __HPR2_RMM_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#if defined(_WIN32) | defined (_WIN64)
#include <windows.h>
#else
#include <pthread.h>
#endif

//#define HPR2_RMM_OK (0)
//#define HPR2_RMM_ERR (-1)

typedef struct hpr2_rmm_t hpr2_rmm_t;
typedef unsigned int hpr2_size_t;
typedef hpr2_size_t hpr2_rmm_off_t;

/**
 * hpr2_rmm_init : init hpr2 rmm pool.
 * @param rmm : (out) The relocatable memory block
 * @param lock: (in) An lock or NULL
 *              if no locking is required.
 * @param mem_buf: (in) 
 * @param mem_size: (in) 
 * @return HPR_OK success, else fail.
 * @sa hpr2_rmm_detroy(),hpr2_rmm_malloc(), hpr2_rmm_free(), hpr2_rmm_calloc(), hpr2_rmm_realloc(), hpr2_rmm_addr_get(), hpr2_rmm_offset_get().
 */
int hpr2_rmm_init(hpr2_rmm_t** rmm, void* lock, void* mem_buf, unsigned int mem_size);

/**
 * hpr2_rmm_detroy : destroy hpr2 rmm pool.
 * @param rmm : (in) The relocatable memory block
 * @return HPR_OK success, else fail.
 * @sa hpr2_rmm_init(), hpr2_rmm_malloc(), hpr2_rmm_free(), hpr2_rmm_calloc(), hpr2_rmm_realloc(), hpr2_rmm_addr_get(), hpr2_rmm_offset_get().
 */
int hpr2_rmm_destroy(hpr2_rmm_t* rmm);

/**
 * hpr2_rmm_malloc : malloc memory from rmm pool.
 * @param rmm : (in) The relocatable memory block
 * @param req_size : (in) How much memory to allocate
 * @return entity of rmm address bias, HPR_ERROR fail.
 * @sa hpr2_rmm_init(), hpr2_rmm_detroy(), hpr2_rmm_free(), hpr2_rmm_calloc(), hpr2_rmm_realloc(), hpr2_rmm_addr_get(), hpr2_rmm_offset_get().
 */
hpr2_rmm_off_t hpr2_rmm_malloc(hpr2_rmm_t* rmm, unsigned int req_size);

/**
 * hpr2_rmm_calloc: not implemented yet.
 */
/*
hpr2_rmm_off_t hpr2_rmm_calloc(hpr2_rmm_t* rmm, unsigned int req_size);
*/
/**
 * hpr2_rmm_realloc: not implemented yet.
 */
/*
hpr2_rmm_off_t hpr2_rmm_realloc(hpr2_rmm_t* rmm, void* entity, unsigned int req_size);
*/

/**
 * hpr2_rmm_free : free memory to rmm pool.
 * @param rmm : (in) The relocatable memory block
 * @param entity : (in) The memory allocation to free
 * @return HPR_OK success, else fail.
 * @sa hpr2_rmm_init(), hpr2_rmm_detroy(), hpr2_rmm_malloc(), hpr2_rmm_calloc(), hpr2_rmm_realloc(), hpr2_rmm_addr_get(), hpr2_rmm_offset_get().
 */
int hpr2_rmm_free(hpr2_rmm_t* rmm, hpr2_rmm_off_t entity);

/**
 * hpr2_rmm_addr_get : Retrieve the physical address of a relocatable allocation of memory.
 * @param rmm : (in) The relocatable memory block
 * @param entity : (in) The memory allocation to free
 * @return memory address, NULL fail.
 * @sa hpr2_rmm_init(), hpr2_rmm_detroy(), hpr2_rmm_malloc(), hpr2_rmm_calloc(), hpr2_rmm_realloc(), hpr2_rmm_free(), hpr2_rmm_offset_get().
 */
void* hpr2_rmm_addr_get(hpr2_rmm_t* rmm, hpr2_rmm_off_t entity);

/**
 * hpr2_rmm_offset_get : compute the offset of a relocatable allocation of memory.
 * @param rmm : (in) The relocatable memory block.
 * @param entity : (in) The physical address to convert to an offset.
 * @return the offset of a relocatable allocation of memory. 
 * @sa hpr2_rmm_init(), hpr2_rmm_detroy(), hpr2_rmm_malloc(), hpr2_rmm_calloc(), hpr2_rmm_realloc(), hpr2_rmm_free(), hpr2_rmm_offset_get().
 */
hpr2_rmm_off_t hpr2_rmm_offset_get(hpr2_rmm_t* rmm, void* entity);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
